/********************************************************************************
** Form generated from reading UI file 'dlg_records_management_check_msgs.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_RECORDS_MANAGEMENT_CHECK_MSGS_H
#define UI_DLG_RECORDS_MANAGEMENT_CHECK_MSGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgRecordsManagementCheckMsgs
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *updateUploadedMsgListCheckBox;
    QGroupBox *accountGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *allCheckBox;
    QSpacerItem *horizontalSpacer;
    QLabel *accountFilterLabel;
    QLineEdit *accountFilterLine;
    LoweredTableView *accountTableView;
    QFrame *dateFrame;
    QHBoxLayout *horizontalLayout_6;
    QGridLayout *dateGridLayout;
    QFormLayout *formLayout;
    QLabel *fromDateLabel;
    QDateEdit *fromDateEdit;
    QFormLayout *formLayout_2;
    QLabel *toDateLabel;
    QDateEdit *toDateEdit;
    QCheckBox *noTimeConstraintCheckBox;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *checkMsgsButton;
    QProgressBar *progressBar;
    QGroupBox *msgGroupBox;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *downloadMissingAttachCheckBox;
    QSpacerItem *horizontalSpacer_3;
    QLabel *messageFilterLabel;
    QLineEdit *messageFilterLine;
    LoweredTableView *msgTableView;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *exportPushButton;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *uploadPushButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgRecordsManagementCheckMsgs)
    {
        if (DlgRecordsManagementCheckMsgs->objectName().isEmpty())
            DlgRecordsManagementCheckMsgs->setObjectName("DlgRecordsManagementCheckMsgs");
        DlgRecordsManagementCheckMsgs->resize(820, 680);
        verticalLayout = new QVBoxLayout(DlgRecordsManagementCheckMsgs);
        verticalLayout->setObjectName("verticalLayout");
        updateUploadedMsgListCheckBox = new QCheckBox(DlgRecordsManagementCheckMsgs);
        updateUploadedMsgListCheckBox->setObjectName("updateUploadedMsgListCheckBox");

        verticalLayout->addWidget(updateUploadedMsgListCheckBox);

        accountGroupBox = new QGroupBox(DlgRecordsManagementCheckMsgs);
        accountGroupBox->setObjectName("accountGroupBox");
        verticalLayout_2 = new QVBoxLayout(accountGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        allCheckBox = new QCheckBox(accountGroupBox);
        allCheckBox->setObjectName("allCheckBox");

        horizontalLayout->addWidget(allCheckBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        accountFilterLabel = new QLabel(accountGroupBox);
        accountFilterLabel->setObjectName("accountFilterLabel");

        horizontalLayout->addWidget(accountFilterLabel);

        accountFilterLine = new QLineEdit(accountGroupBox);
        accountFilterLine->setObjectName("accountFilterLine");

        horizontalLayout->addWidget(accountFilterLine);


        verticalLayout_2->addLayout(horizontalLayout);

        accountTableView = new LoweredTableView(accountGroupBox);
        accountTableView->setObjectName("accountTableView");
        accountTableView->setShowGrid(false);
        accountTableView->setWordWrap(false);
        accountTableView->horizontalHeader()->setHighlightSections(false);
        accountTableView->horizontalHeader()->setStretchLastSection(true);
        accountTableView->verticalHeader()->setVisible(false);
        accountTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout_2->addWidget(accountTableView);

        dateFrame = new QFrame(accountGroupBox);
        dateFrame->setObjectName("dateFrame");
        dateFrame->setFrameShape(QFrame::StyledPanel);
        dateFrame->setFrameShadow(QFrame::Raised);
        horizontalLayout_6 = new QHBoxLayout(dateFrame);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        dateGridLayout = new QGridLayout();
        dateGridLayout->setObjectName("dateGridLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        fromDateLabel = new QLabel(dateFrame);
        fromDateLabel->setObjectName("fromDateLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, fromDateLabel);

        fromDateEdit = new QDateEdit(dateFrame);
        fromDateEdit->setObjectName("fromDateEdit");
        fromDateEdit->setDisplayFormat(QString::fromUtf8("dd.MM.yyyy"));

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, fromDateEdit);


        dateGridLayout->addLayout(formLayout, 0, 0, 1, 1);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        toDateLabel = new QLabel(dateFrame);
        toDateLabel->setObjectName("toDateLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, toDateLabel);

        toDateEdit = new QDateEdit(dateFrame);
        toDateEdit->setObjectName("toDateEdit");
        toDateEdit->setDisplayFormat(QString::fromUtf8("dd.MM.yyyy"));

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, toDateEdit);


        dateGridLayout->addLayout(formLayout_2, 0, 1, 1, 1);


        horizontalLayout_6->addLayout(dateGridLayout);

        noTimeConstraintCheckBox = new QCheckBox(dateFrame);
        noTimeConstraintCheckBox->setObjectName("noTimeConstraintCheckBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(noTimeConstraintCheckBox->sizePolicy().hasHeightForWidth());
        noTimeConstraintCheckBox->setSizePolicy(sizePolicy);

        horizontalLayout_6->addWidget(noTimeConstraintCheckBox);


        verticalLayout_2->addWidget(dateFrame);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        checkMsgsButton = new QPushButton(accountGroupBox);
        checkMsgsButton->setObjectName("checkMsgsButton");

        horizontalLayout_2->addWidget(checkMsgsButton);


        verticalLayout_2->addLayout(horizontalLayout_2);


        verticalLayout->addWidget(accountGroupBox);

        progressBar = new QProgressBar(DlgRecordsManagementCheckMsgs);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(0);

        verticalLayout->addWidget(progressBar);

        msgGroupBox = new QGroupBox(DlgRecordsManagementCheckMsgs);
        msgGroupBox->setObjectName("msgGroupBox");
        verticalLayout_3 = new QVBoxLayout(msgGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        downloadMissingAttachCheckBox = new QCheckBox(msgGroupBox);
        downloadMissingAttachCheckBox->setObjectName("downloadMissingAttachCheckBox");

        horizontalLayout_3->addWidget(downloadMissingAttachCheckBox);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        messageFilterLabel = new QLabel(msgGroupBox);
        messageFilterLabel->setObjectName("messageFilterLabel");

        horizontalLayout_3->addWidget(messageFilterLabel);

        messageFilterLine = new QLineEdit(msgGroupBox);
        messageFilterLine->setObjectName("messageFilterLine");

        horizontalLayout_3->addWidget(messageFilterLine);


        verticalLayout_3->addLayout(horizontalLayout_3);

        msgTableView = new LoweredTableView(msgGroupBox);
        msgTableView->setObjectName("msgTableView");
        msgTableView->setShowGrid(false);
        msgTableView->setWordWrap(false);
        msgTableView->horizontalHeader()->setHighlightSections(false);
        msgTableView->horizontalHeader()->setStretchLastSection(true);
        msgTableView->verticalHeader()->setVisible(false);
        msgTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout_3->addWidget(msgTableView);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        exportPushButton = new QPushButton(msgGroupBox);
        exportPushButton->setObjectName("exportPushButton");

        horizontalLayout_4->addWidget(exportPushButton);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        uploadPushButton = new QPushButton(msgGroupBox);
        uploadPushButton->setObjectName("uploadPushButton");

        horizontalLayout_4->addWidget(uploadPushButton);


        verticalLayout_3->addLayout(horizontalLayout_4);


        verticalLayout->addWidget(msgGroupBox);

        buttonBox = new QDialogButtonBox(DlgRecordsManagementCheckMsgs);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgRecordsManagementCheckMsgs);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgRecordsManagementCheckMsgs, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgRecordsManagementCheckMsgs, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgRecordsManagementCheckMsgs);
    } // setupUi

    void retranslateUi(QDialog *DlgRecordsManagementCheckMsgs)
    {
        DlgRecordsManagementCheckMsgs->setWindowTitle(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Check Messages Against Records Management", nullptr));
#if QT_CONFIG(tooltip)
        updateUploadedMsgListCheckBox->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Update local information about already uploaded messages\n"
"from records management service before checking\n"
"whether to upload the messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        updateUploadedMsgListCheckBox->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Update list of uploaded messages from the service.", nullptr));
        accountGroupBox->setTitle(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Select data boxes to be checked.", nullptr));
        allCheckBox->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Select all", nullptr));
        accountFilterLabel->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Filter:", nullptr));
        fromDateLabel->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "From date:", nullptr));
#if QT_CONFIG(tooltip)
        fromDateEdit->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Check only messages with the specified delivery date and later.", nullptr));
#endif // QT_CONFIG(tooltip)
        toDateLabel->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "To date:", nullptr));
#if QT_CONFIG(tooltip)
        toDateEdit->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Check only messages with the specified delivery date and earlier.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        noTimeConstraintCheckBox->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "If checked then all messages in selected data boxes are going to be checked.\n"
"No delivery time constraints are going to be imposed.", nullptr));
#endif // QT_CONFIG(tooltip)
        noTimeConstraintCheckBox->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Without time costraints.", nullptr));
#if QT_CONFIG(tooltip)
        checkMsgsButton->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Generates a list of all messages from the selected data boxes\n"
"which have not been uploaded into the records management service.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkMsgsButton->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Check Missing Messages", nullptr));
        msgGroupBox->setTitle(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "List of messages not present in the records management service", nullptr));
#if QT_CONFIG(tooltip)
        downloadMissingAttachCheckBox->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Try automatically downloading missing attachments\n"
"before uploading into records management service.", nullptr));
#endif // QT_CONFIG(tooltip)
        downloadMissingAttachCheckBox->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Download missing attachments.", nullptr));
        messageFilterLabel->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Filter:", nullptr));
#if QT_CONFIG(tooltip)
        exportPushButton->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Export displayed message list into a CSV file.", nullptr));
#endif // QT_CONFIG(tooltip)
        exportPushButton->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Export CSV", nullptr));
#if QT_CONFIG(tooltip)
        uploadPushButton->setToolTip(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Tries finding a match between available message identifiers\n"
"(e.g. file marks, reference numbers) and data found\n"
"in the records management hierarchy.\n"
"\n"
"If there is a unique match then an automatic message upload\n"
"into the records management service is performed.", nullptr));
#endif // QT_CONFIG(tooltip)
        uploadPushButton->setText(QCoreApplication::translate("DlgRecordsManagementCheckMsgs", "Try Automatic Upload", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgRecordsManagementCheckMsgs: public Ui_DlgRecordsManagementCheckMsgs {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_RECORDS_MANAGEMENT_CHECK_MSGS_H
