/********************************************************************************
** Form generated from reading UI file 'dlg_view_log.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_VIEW_LOG_H
#define UI_DLG_VIEW_LOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgViewLog
{
public:
    QVBoxLayout *verticalLayout;
    QPlainTextEdit *logTextEdit;

    void setupUi(QDialog *DlgViewLog)
    {
        if (DlgViewLog->objectName().isEmpty())
            DlgViewLog->setObjectName("DlgViewLog");
        DlgViewLog->resize(400, 300);
        verticalLayout = new QVBoxLayout(DlgViewLog);
        verticalLayout->setObjectName("verticalLayout");
        logTextEdit = new QPlainTextEdit(DlgViewLog);
        logTextEdit->setObjectName("logTextEdit");
        logTextEdit->setReadOnly(true);

        verticalLayout->addWidget(logTextEdit);


        retranslateUi(DlgViewLog);

        QMetaObject::connectSlotsByName(DlgViewLog);
    } // setupUi

    void retranslateUi(QDialog *DlgViewLog)
    {
        DlgViewLog->setWindowTitle(QCoreApplication::translate("DlgViewLog", "Log content", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgViewLog: public Ui_DlgViewLog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_VIEW_LOG_H
