/****************************************************************************
** Meta object code from reading C++ file 'draft_db.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/datovka_shared/io/draft_db.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'draft_db.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7DraftDbE_t {};
} // unnamed namespace

template <> constexpr inline auto DraftDb::qt_create_metaobjectdata<qt_meta_tag_ZN7DraftDbE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DraftDb",
        "draftsInserted",
        "",
        "DraftEntryList",
        "entries",
        "draftsUpdated",
        "draftsDeleted",
        "Json::Int64StringList",
        "ids",
        "draftCountChanged",
        "DraftCounts",
        "draftCounts"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'draftsInserted'
        QtMocHelpers::SignalData<void(const DraftEntryList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'draftsUpdated'
        QtMocHelpers::SignalData<void(const DraftEntryList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'draftsDeleted'
        QtMocHelpers::SignalData<void(const Json::Int64StringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'draftCountChanged'
        QtMocHelpers::SignalData<void(const DraftCounts &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DraftDb, qt_meta_tag_ZN7DraftDbE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DraftDb::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7DraftDbE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7DraftDbE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7DraftDbE_t>.metaTypes,
    nullptr
} };

void DraftDb::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DraftDb *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->draftsInserted((*reinterpret_cast<std::add_pointer_t<DraftEntryList>>(_a[1]))); break;
        case 1: _t->draftsUpdated((*reinterpret_cast<std::add_pointer_t<DraftEntryList>>(_a[1]))); break;
        case 2: _t->draftsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 3: _t->draftCountChanged((*reinterpret_cast<std::add_pointer_t<DraftCounts>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftEntryList >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftEntryList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftCounts >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DraftDb::*)(const DraftEntryList & )>(_a, &DraftDb::draftsInserted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DraftDb::*)(const DraftEntryList & )>(_a, &DraftDb::draftsUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DraftDb::*)(const Json::Int64StringList & )>(_a, &DraftDb::draftsDeleted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DraftDb::*)(const DraftCounts & )>(_a, &DraftDb::draftCountChanged, 3))
            return;
    }
}

const QMetaObject *DraftDb::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DraftDb::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7DraftDbE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "SQLiteDbSingle"))
        return static_cast< SQLiteDbSingle*>(this);
    return QObject::qt_metacast(_clname);
}

int DraftDb::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void DraftDb::draftsInserted(const DraftEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DraftDb::draftsUpdated(const DraftEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DraftDb::draftsDeleted(const Json::Int64StringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DraftDb::draftCountChanged(const DraftCounts & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
