/********************************************************************************
** Form generated from reading UI file 'dlg_tag.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TAG_H
#define UI_DLG_TAG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgTag
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *tagNamelineEdit;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *currentColor;
    QPushButton *changeColorPushButton;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgTag)
    {
        if (DlgTag->objectName().isEmpty())
            DlgTag->setObjectName("DlgTag");
        DlgTag->resize(337, 115);
        verticalLayout = new QVBoxLayout(DlgTag);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        label = new QLabel(DlgTag);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        tagNamelineEdit = new QLineEdit(DlgTag);
        tagNamelineEdit->setObjectName("tagNamelineEdit");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, tagNamelineEdit);

        label_2 = new QLabel(DlgTag);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        currentColor = new QPushButton(DlgTag);
        currentColor->setObjectName("currentColor");
        currentColor->setEnabled(true);
        currentColor->setText(QString::fromUtf8(""));
        currentColor->setAutoDefault(false);

        horizontalLayout->addWidget(currentColor);

        changeColorPushButton = new QPushButton(DlgTag);
        changeColorPushButton->setObjectName("changeColorPushButton");

        horizontalLayout->addWidget(changeColorPushButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        formLayout->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(DlgTag);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(tagNamelineEdit, currentColor);
        QWidget::setTabOrder(currentColor, changeColorPushButton);

        retranslateUi(DlgTag);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgTag, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgTag, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgTag);
    } // setupUi

    void retranslateUi(QDialog *DlgTag)
    {
        DlgTag->setWindowTitle(QCoreApplication::translate("DlgTag", "Tag properties", nullptr));
        label->setText(QCoreApplication::translate("DlgTag", "Tag name:", nullptr));
        label_2->setText(QCoreApplication::translate("DlgTag", "Tag color:", nullptr));
        changeColorPushButton->setText(QCoreApplication::translate("DlgTag", "Change", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTag: public Ui_DlgTag {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TAG_H
