/********************************************************************************
** Form generated from reading UI file 'dlg_update_portable.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_UPDATE_PORTABLE_H
#define UI_DLG_UPDATE_PORTABLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgUpdatePortable
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *descriptionLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *pathEdit;
    QPushButton *changePathButton;
    QRadioButton *storageDontMoveButton;
    QRadioButton *storageCopyButton;
    QLabel *fsTypeNotificationLabel;
    QLabel *freeSpaceNotificationLabel;
    QProgressBar *progress;
    QLabel *progressLabel;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgUpdatePortable)
    {
        if (DlgUpdatePortable->objectName().isEmpty())
            DlgUpdatePortable->setObjectName("DlgUpdatePortable");
        DlgUpdatePortable->resize(420, 240);
        verticalLayout = new QVBoxLayout(DlgUpdatePortable);
        verticalLayout->setObjectName("verticalLayout");
        descriptionLabel = new QLabel(DlgUpdatePortable);
        descriptionLabel->setObjectName("descriptionLabel");

        verticalLayout->addWidget(descriptionLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        pathEdit = new QLineEdit(DlgUpdatePortable);
        pathEdit->setObjectName("pathEdit");

        horizontalLayout->addWidget(pathEdit);

        changePathButton = new QPushButton(DlgUpdatePortable);
        changePathButton->setObjectName("changePathButton");

        horizontalLayout->addWidget(changePathButton);


        verticalLayout->addLayout(horizontalLayout);

        storageDontMoveButton = new QRadioButton(DlgUpdatePortable);
        storageDontMoveButton->setObjectName("storageDontMoveButton");

        verticalLayout->addWidget(storageDontMoveButton);

        storageCopyButton = new QRadioButton(DlgUpdatePortable);
        storageCopyButton->setObjectName("storageCopyButton");

        verticalLayout->addWidget(storageCopyButton);

        fsTypeNotificationLabel = new QLabel(DlgUpdatePortable);
        fsTypeNotificationLabel->setObjectName("fsTypeNotificationLabel");

        verticalLayout->addWidget(fsTypeNotificationLabel);

        freeSpaceNotificationLabel = new QLabel(DlgUpdatePortable);
        freeSpaceNotificationLabel->setObjectName("freeSpaceNotificationLabel");

        verticalLayout->addWidget(freeSpaceNotificationLabel);

        progress = new QProgressBar(DlgUpdatePortable);
        progress->setObjectName("progress");
        progress->setValue(24);

        verticalLayout->addWidget(progress);

        progressLabel = new QLabel(DlgUpdatePortable);
        progressLabel->setObjectName("progressLabel");

        verticalLayout->addWidget(progressLabel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgUpdatePortable);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgUpdatePortable);

        QMetaObject::connectSlotsByName(DlgUpdatePortable);
    } // setupUi

    void retranslateUi(QDialog *DlgUpdatePortable)
    {
        descriptionLabel->setText(QCoreApplication::translate("DlgUpdatePortable", "n/a", nullptr));
        changePathButton->setText(QCoreApplication::translate("DlgUpdatePortable", "Change", nullptr));
#if QT_CONFIG(tooltip)
        storageDontMoveButton->setToolTip(QCoreApplication::translate("DlgUpdatePortable", "Leaves the configuration and all stored application data in the currently running\n"
"application version. No data will be copied into the newly extracted directory.\n"
"You'll need to copy or move the data manually in order to access it from\n"
"the newly extracted application package.", nullptr));
#endif // QT_CONFIG(tooltip)
        storageDontMoveButton->setText(QCoreApplication::translate("DlgUpdatePortable", "Leave current storage in place. I'll move it manually later.", nullptr));
#if QT_CONFIG(tooltip)
        storageCopyButton->setToolTip(QCoreApplication::translate("DlgUpdatePortable", "The application will attempt to copy all the configuration and stored\n"
"application data into the newly extracted directory.", nullptr));
#endif // QT_CONFIG(tooltip)
        storageCopyButton->setText(QCoreApplication::translate("DlgUpdatePortable", "Copy current storage to new place.", nullptr));
        fsTypeNotificationLabel->setText(QCoreApplication::translate("DlgUpdatePortable", "n/a", nullptr));
        freeSpaceNotificationLabel->setText(QCoreApplication::translate("DlgUpdatePortable", "n/a", nullptr));
        progressLabel->setText(QCoreApplication::translate("DlgUpdatePortable", "n/a", nullptr));
        (void)DlgUpdatePortable;
    } // retranslateUi

};

namespace Ui {
    class DlgUpdatePortable: public Ui_DlgUpdatePortable {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_UPDATE_PORTABLE_H
