/********************************************************************************
** Form generated from reading UI file 'dlg_backup.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_BACKUP_H
#define UI_DLG_BACKUP_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgBackup
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoLabel;
    QFrame *line;
    QGroupBox *whereGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *dirLabel;
    QLineEdit *directoryLine;
    QPushButton *chooseDirButton;
    QLabel *availableSpaceLabel;
    QGroupBox *whatGroupBox;
    QVBoxLayout *verticalLayout_3;
    QLabel *noCredentialsLabel;
    QCheckBox *allCheckBox;
    LoweredTableView *accountView;
    QCheckBox *tagsCheckBox;
    QLabel *requiredSpaceLabel;
    QLabel *warningLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgBackup)
    {
        if (DlgBackup->objectName().isEmpty())
            DlgBackup->setObjectName("DlgBackup");
        DlgBackup->resize(415, 560);
        verticalLayout = new QVBoxLayout(DlgBackup);
        verticalLayout->setObjectName("verticalLayout");
        infoLabel = new QLabel(DlgBackup);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        infoLabel->setWordWrap(true);

        verticalLayout->addWidget(infoLabel);

        line = new QFrame(DlgBackup);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        whereGroupBox = new QGroupBox(DlgBackup);
        whereGroupBox->setObjectName("whereGroupBox");
        verticalLayout_2 = new QVBoxLayout(whereGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        dirLabel = new QLabel(whereGroupBox);
        dirLabel->setObjectName("dirLabel");

        horizontalLayout->addWidget(dirLabel);

        directoryLine = new QLineEdit(whereGroupBox);
        directoryLine->setObjectName("directoryLine");

        horizontalLayout->addWidget(directoryLine);

        chooseDirButton = new QPushButton(whereGroupBox);
        chooseDirButton->setObjectName("chooseDirButton");

        horizontalLayout->addWidget(chooseDirButton);


        verticalLayout_2->addLayout(horizontalLayout);

        availableSpaceLabel = new QLabel(whereGroupBox);
        availableSpaceLabel->setObjectName("availableSpaceLabel");

        verticalLayout_2->addWidget(availableSpaceLabel);


        verticalLayout->addWidget(whereGroupBox);

        whatGroupBox = new QGroupBox(DlgBackup);
        whatGroupBox->setObjectName("whatGroupBox");
        verticalLayout_3 = new QVBoxLayout(whatGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        noCredentialsLabel = new QLabel(whatGroupBox);
        noCredentialsLabel->setObjectName("noCredentialsLabel");
        noCredentialsLabel->setAlignment(Qt::AlignCenter);
        noCredentialsLabel->setWordWrap(true);

        verticalLayout_3->addWidget(noCredentialsLabel);

        allCheckBox = new QCheckBox(whatGroupBox);
        allCheckBox->setObjectName("allCheckBox");

        verticalLayout_3->addWidget(allCheckBox);

        accountView = new LoweredTableView(whatGroupBox);
        accountView->setObjectName("accountView");
        accountView->setShowGrid(false);
        accountView->setWordWrap(false);
        accountView->horizontalHeader()->setHighlightSections(false);
        accountView->horizontalHeader()->setStretchLastSection(true);
        accountView->verticalHeader()->setVisible(false);
        accountView->verticalHeader()->setHighlightSections(false);

        verticalLayout_3->addWidget(accountView);

        tagsCheckBox = new QCheckBox(whatGroupBox);
        tagsCheckBox->setObjectName("tagsCheckBox");

        verticalLayout_3->addWidget(tagsCheckBox);

        requiredSpaceLabel = new QLabel(whatGroupBox);
        requiredSpaceLabel->setObjectName("requiredSpaceLabel");

        verticalLayout_3->addWidget(requiredSpaceLabel);


        verticalLayout->addWidget(whatGroupBox);

        warningLabel = new QLabel(DlgBackup);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(warningLabel);

        buttonBox = new QDialogButtonBox(DlgBackup);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgBackup);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgBackup, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgBackup);
    } // setupUi

    void retranslateUi(QDialog *DlgBackup)
    {
        DlgBackup->setWindowTitle(QCoreApplication::translate("DlgBackup", "Back Up Data", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgBackup", "n/a", nullptr));
        whereGroupBox->setTitle(QCoreApplication::translate("DlgBackup", "Where do you want to back up your data?", nullptr));
        dirLabel->setText(QCoreApplication::translate("DlgBackup", "Directory", nullptr));
        chooseDirButton->setText(QCoreApplication::translate("DlgBackup", "Choose", nullptr));
        availableSpaceLabel->setText(QCoreApplication::translate("DlgBackup", "n/a", nullptr));
        whatGroupBox->setTitle(QCoreApplication::translate("DlgBackup", "What do you want to back up?", nullptr));
#if QT_CONFIG(tooltip)
        noCredentialsLabel->setToolTip(QCoreApplication::translate("DlgBackup", "Keep a copy of you login credentials and preserve it in a safe place\n"
"where only allowed persons have access.", nullptr));
#endif // QT_CONFIG(tooltip)
        noCredentialsLabel->setText(QCoreApplication::translate("DlgBackup", "Because of security reasons the application does not allow you to make back-ups of login credentials.", nullptr));
        allCheckBox->setText(QCoreApplication::translate("DlgBackup", "Select all", nullptr));
        tagsCheckBox->setText(QCoreApplication::translate("DlgBackup", "Tag database", nullptr));
        requiredSpaceLabel->setText(QCoreApplication::translate("DlgBackup", "n/a", nullptr));
        warningLabel->setText(QCoreApplication::translate("DlgBackup", "n/a", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgBackup: public Ui_DlgBackup {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_BACKUP_H
