/********************************************************************************
** Form generated from reading UI file 'dlg_backup_progress.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_BACKUP_PROGRESS_H
#define UI_DLG_BACKUP_PROGRESS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgBackupProgress
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *taskLabel;
    QProgressBar *taskProgress;
    QLabel *notificationLabel;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgBackupProgress)
    {
        if (DlgBackupProgress->objectName().isEmpty())
            DlgBackupProgress->setObjectName("DlgBackupProgress");
        DlgBackupProgress->resize(400, 120);
        verticalLayout = new QVBoxLayout(DlgBackupProgress);
        verticalLayout->setObjectName("verticalLayout");
        taskLabel = new QLabel(DlgBackupProgress);
        taskLabel->setObjectName("taskLabel");

        verticalLayout->addWidget(taskLabel);

        taskProgress = new QProgressBar(DlgBackupProgress);
        taskProgress->setObjectName("taskProgress");
        taskProgress->setValue(24);

        verticalLayout->addWidget(taskProgress);

        notificationLabel = new QLabel(DlgBackupProgress);
        notificationLabel->setObjectName("notificationLabel");

        verticalLayout->addWidget(notificationLabel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgBackupProgress);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgBackupProgress);

        QMetaObject::connectSlotsByName(DlgBackupProgress);
    } // setupUi

    void retranslateUi(QDialog *DlgBackupProgress)
    {
        taskLabel->setText(QCoreApplication::translate("DlgBackupProgress", "n/a", nullptr));
        notificationLabel->setText(QCoreApplication::translate("DlgBackupProgress", "n/a", nullptr));
        (void)DlgBackupProgress;
    } // retranslateUi

};

namespace Ui {
    class DlgBackupProgress: public Ui_DlgBackupProgress {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_BACKUP_PROGRESS_H
