/********************************************************************************
** Form generated from reading UI file 'dlg_email_content.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_EMAIL_CONTENT_H
#define UI_DLG_EMAIL_CONTENT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgEmailContent
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *messageZfoCheck;
    QCheckBox *deliveryInfoZfoCheck;
    QCheckBox *allAttachmentsCheck;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgEmailContent)
    {
        if (DlgEmailContent->objectName().isEmpty())
            DlgEmailContent->setObjectName("DlgEmailContent");
        DlgEmailContent->resize(400, 180);
        verticalLayout = new QVBoxLayout(DlgEmailContent);
        verticalLayout->setObjectName("verticalLayout");
        groupBox = new QGroupBox(DlgEmailContent);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        messageZfoCheck = new QCheckBox(groupBox);
        messageZfoCheck->setObjectName("messageZfoCheck");

        verticalLayout_2->addWidget(messageZfoCheck);

        deliveryInfoZfoCheck = new QCheckBox(groupBox);
        deliveryInfoZfoCheck->setObjectName("deliveryInfoZfoCheck");

        verticalLayout_2->addWidget(deliveryInfoZfoCheck);

        allAttachmentsCheck = new QCheckBox(groupBox);
        allAttachmentsCheck->setObjectName("allAttachmentsCheck");

        verticalLayout_2->addWidget(allAttachmentsCheck);


        verticalLayout->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(DlgEmailContent);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(messageZfoCheck, deliveryInfoZfoCheck);
        QWidget::setTabOrder(deliveryInfoZfoCheck, allAttachmentsCheck);

        retranslateUi(DlgEmailContent);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgEmailContent, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgEmailContent, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgEmailContent);
    } // setupUi

    void retranslateUi(QDialog *DlgEmailContent)
    {
        DlgEmailContent->setWindowTitle(QCoreApplication::translate("DlgEmailContent", "Select E-mail Content", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgEmailContent", "What content do you want to append into the created e-mail?", nullptr));
        messageZfoCheck->setText(QCoreApplication::translate("DlgEmailContent", "whole data message in ZFO format", nullptr));
        deliveryInfoZfoCheck->setText(QCoreApplication::translate("DlgEmailContent", "delivery info in ZFO format", nullptr));
        allAttachmentsCheck->setText(QCoreApplication::translate("DlgEmailContent", "all attachments of the messages", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgEmailContent: public Ui_DlgEmailContent {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_EMAIL_CONTENT_H
