/********************************************************************************
** Form generated from reading UI file 'dlg_prefs.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_PREFS_H
#define UI_DLG_PREFS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgPrefs
{
public:
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer;
    QLabel *warningLabel;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *warningAcceptButton;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QWidget *page_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    LoweredTableView *prefTableView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgPrefs)
    {
        if (DlgPrefs->objectName().isEmpty())
            DlgPrefs->setObjectName("DlgPrefs");
        DlgPrefs->resize(800, 600);
        verticalLayout_3 = new QVBoxLayout(DlgPrefs);
        verticalLayout_3->setObjectName("verticalLayout_3");
        stackedWidget = new QStackedWidget(DlgPrefs);
        stackedWidget->setObjectName("stackedWidget");
        page = new QWidget();
        page->setObjectName("page");
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        warningLabel = new QLabel(page);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setAlignment(Qt::AlignCenter);
        warningLabel->setWordWrap(true);

        verticalLayout_2->addWidget(warningLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        warningAcceptButton = new QPushButton(page);
        warningAcceptButton->setObjectName("warningAcceptButton");

        horizontalLayout->addWidget(warningAcceptButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        verticalLayout = new QVBoxLayout(page_2);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        filterLabel = new QLabel(page_2);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout_2->addWidget(filterLabel);

        filterLine = new QLineEdit(page_2);
        filterLine->setObjectName("filterLine");

        horizontalLayout_2->addWidget(filterLine);


        verticalLayout->addLayout(horizontalLayout_2);

        prefTableView = new LoweredTableView(page_2);
        prefTableView->setObjectName("prefTableView");
        prefTableView->setShowGrid(false);
        prefTableView->setWordWrap(false);
        prefTableView->horizontalHeader()->setHighlightSections(false);
        prefTableView->horizontalHeader()->setStretchLastSection(true);
        prefTableView->verticalHeader()->setVisible(false);
        prefTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout->addWidget(prefTableView);

        stackedWidget->addWidget(page_2);

        verticalLayout_3->addWidget(stackedWidget);

        buttonBox = new QDialogButtonBox(DlgPrefs);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_3->addWidget(buttonBox);

        QWidget::setTabOrder(warningAcceptButton, filterLine);
        QWidget::setTabOrder(filterLine, prefTableView);

        retranslateUi(DlgPrefs);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgPrefs, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgPrefs, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgPrefs);
    } // setupUi

    void retranslateUi(QDialog *DlgPrefs)
    {
        DlgPrefs->setWindowTitle(QCoreApplication::translate("DlgPrefs", "All preferences", nullptr));
        warningLabel->setText(QCoreApplication::translate("DlgPrefs", "Changing these settings directly can be harmful to the stability, security and performance of this application. You should only continue if you are sure of what you are doing.", nullptr));
        warningAcceptButton->setText(QCoreApplication::translate("DlgPrefs", "I accept the risk.", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgPrefs", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPrefs: public Ui_DlgPrefs {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_PREFS_H
