/********************************************************************************
** Form generated from reading UI file 'dlg_view_changelog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_VIEW_CHANGELOG_H
#define UI_DLG_VIEW_CHANGELOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgViewChangeLog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *headerLabel;
    QTextEdit *changeLogTextEdit;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgViewChangeLog)
    {
        if (DlgViewChangeLog->objectName().isEmpty())
            DlgViewChangeLog->setObjectName("DlgViewChangeLog");
        DlgViewChangeLog->resize(400, 300);
        verticalLayout = new QVBoxLayout(DlgViewChangeLog);
        verticalLayout->setObjectName("verticalLayout");
        headerLabel = new QLabel(DlgViewChangeLog);
        headerLabel->setObjectName("headerLabel");

        verticalLayout->addWidget(headerLabel);

        changeLogTextEdit = new QTextEdit(DlgViewChangeLog);
        changeLogTextEdit->setObjectName("changeLogTextEdit");
        changeLogTextEdit->setReadOnly(true);

        verticalLayout->addWidget(changeLogTextEdit);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonBox = new QDialogButtonBox(DlgViewChangeLog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(DlgViewChangeLog);

        QMetaObject::connectSlotsByName(DlgViewChangeLog);
    } // setupUi

    void retranslateUi(QDialog *DlgViewChangeLog)
    {
        DlgViewChangeLog->setWindowTitle(QCoreApplication::translate("DlgViewChangeLog", "What's new?", nullptr));
        headerLabel->setText(QCoreApplication::translate("DlgViewChangeLog", "Version:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgViewChangeLog: public Ui_DlgViewChangeLog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_VIEW_CHANGELOG_H
