/****************************************************************************
** Meta object code from reading C++ file 'dlg_timestamp_renewal.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/gui/dlg_timestamp_renewal.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlg_timestamp_renewal.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18ScanAccountsThreadE_t {};
} // unnamed namespace

template <> constexpr inline auto ScanAccountsThread::qt_create_metaobjectdata<qt_meta_tag_ZN18ScanAccountsThreadE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ScanAccountsThread",
        "started",
        "",
        "stopped",
        "interrupted",
        "processed",
        "AcntIdDb",
        "acntIdDb",
        "QList<TstEntry>",
        "tstEntries",
        "progress",
        "num",
        "total"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'started'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stopped'
        QtMocHelpers::SignalData<void(bool)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Signal 'processed'
        QtMocHelpers::SignalData<void(const AcntIdDb &, const QList<TstEntry> &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(const AcntIdDb &, int, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScanAccountsThread, qt_meta_tag_ZN18ScanAccountsThreadE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ScanAccountsThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ScanAccountsThreadE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ScanAccountsThreadE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ScanAccountsThreadE_t>.metaTypes,
    nullptr
} };

void ScanAccountsThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScanAccountsThread *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->started(); break;
        case 1: _t->stopped((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->processed((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<TstEntry>>>(_a[2]))); break;
        case 3: _t->progress((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<TstEntry> >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ScanAccountsThread::*)()>(_a, &ScanAccountsThread::started, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScanAccountsThread::*)(bool )>(_a, &ScanAccountsThread::stopped, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScanAccountsThread::*)(const AcntIdDb & , const QList<TstEntry> & )>(_a, &ScanAccountsThread::processed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ScanAccountsThread::*)(const AcntIdDb & , int , int )>(_a, &ScanAccountsThread::progress, 3))
            return;
    }
}

const QMetaObject *ScanAccountsThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScanAccountsThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ScanAccountsThreadE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int ScanAccountsThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ScanAccountsThread::started()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ScanAccountsThread::stopped(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ScanAccountsThread::processed(const AcntIdDb & _t1, const QList<TstEntry> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void ScanAccountsThread::progress(const AcntIdDb & _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN19DlgTimestampRenewalE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgTimestampRenewal::qt_create_metaobjectdata<qt_meta_tag_ZN19DlgTimestampRenewalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgTimestampRenewal",
        "focusSelectedMsg",
        "",
        "AcntId",
        "acntId",
        "dmId",
        "year",
        "msgType",
        "done",
        "resultCode",
        "renewSelectedTrackedTimestamps",
        "renewSelectedAllStoredTimestamps",
        "renewListedAllStoredTimestamps",
        "updateTrackedMessageSelection",
        "QItemSelection",
        "selected",
        "deselected",
        "updateAllStoredMessageSelection",
        "watchAllStoredMessageList",
        "toggleTrackedTimestampTracking",
        "enable",
        "toggleAllStoredTimestampTracking",
        "viewTrackedMessagesListContextMenu",
        "QPoint",
        "point",
        "trackedMessageItemDoubleClicked",
        "QModelIndex",
        "index",
        "viewAllStoredMessagesListContextMenu",
        "allStoredMessageItemDoubleClicked",
        "filterTrackedMessages",
        "text",
        "filterAllMessages",
        "hideTrackedMessagesFilter",
        "hideAllMessagesFilter",
        "watchIdentificationsInserted",
        "Json::MsgId2List",
        "msgIds",
        "watchIdentificationsDeleted",
        "watchMsgTimestampDataUpdated",
        "TstValidityHash",
        "values",
        "watchPrefsModification",
        "valueType",
        "name",
        "QVariant",
        "value",
        "applyAllSelection",
        "state",
        "applyRestampableFiltering",
        "modifiedAccountSelection",
        "startAccountScan",
        "interruptAccountScan",
        "watchScanStopped",
        "interrupted",
        "watchScanProcessed",
        "AcntIdDb",
        "acntIdDb",
        "QList<TstEntry>",
        "tstEntries",
        "watchScanProgress",
        "num",
        "total",
        "selectExternalSrcZfo",
        "viewExternalSrcZfo",
        "selectExternalTgtZfo",
        "renewExternalTimestamps",
        "watchUploadProgress",
        "transactId",
        "uploadTotal",
        "uploadCurrent",
        "watchUploadProgressFinished",
        "result",
        "resultDesc",
        "resultVal",
        "watchDownloadProgress",
        "MsgId",
        "msgId",
        "downloadTotal",
        "downloadCurrent",
        "watchDownloadProgressFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusSelectedMsg'
        QtMocHelpers::SignalData<void(AcntId, qint64, QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::LongLong, 5 }, { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'done'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'renewSelectedTrackedTimestamps'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renewSelectedAllStoredTimestamps'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renewListedAllStoredTimestamps'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateTrackedMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 14, 16 },
        }}),
        // Slot 'updateTrackedMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(13, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'updateAllStoredMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 14, 16 },
        }}),
        // Slot 'updateAllStoredMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(17, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'watchAllStoredMessageList'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleTrackedTimestampTracking'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'toggleAllStoredTimestampTracking'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'viewTrackedMessagesListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'trackedMessageItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'viewAllStoredMessagesListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'allStoredMessageItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'filterTrackedMessages'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'filterAllMessages'
        QtMocHelpers::SlotData<void(const QString &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'hideTrackedMessagesFilter'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideAllMessagesFilter'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchIdentificationsInserted'
        QtMocHelpers::SlotData<void(const Json::MsgId2List &, const AcntId &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'watchIdentificationsDeleted'
        QtMocHelpers::SlotData<void(const Json::MsgId2List &)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Slot 'watchMsgTimestampDataUpdated'
        QtMocHelpers::SlotData<void(const TstValidityHash &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'watchPrefsModification'
        QtMocHelpers::SlotData<void(int, const QString &, const QVariant &)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 43 }, { QMetaType::QString, 44 }, { 0x80000000 | 45, 46 },
        }}),
        // Slot 'applyAllSelection'
        QtMocHelpers::SlotData<void(int)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'applyRestampableFiltering'
        QtMocHelpers::SlotData<void(int)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 48 },
        }}),
        // Slot 'modifiedAccountSelection'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'startAccountScan'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'interruptAccountScan'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchScanStopped'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 54 },
        }}),
        // Slot 'watchScanProcessed'
        QtMocHelpers::SlotData<void(const AcntIdDb &, const QList<TstEntry> &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 }, { 0x80000000 | 58, 59 },
        }}),
        // Slot 'watchScanProgress'
        QtMocHelpers::SlotData<void(const AcntIdDb &, int, int)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 56, 57 }, { QMetaType::Int, 61 }, { QMetaType::Int, 62 },
        }}),
        // Slot 'selectExternalSrcZfo'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewExternalSrcZfo'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectExternalTgtZfo'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renewExternalTimestamps'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchUploadProgress'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, qint64, qint64)>(67, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 68 }, { QMetaType::LongLong, 69 }, { QMetaType::LongLong, 70 },
        }}),
        // Slot 'watchUploadProgressFinished'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, int, const QString &, const QVariant &)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 68 }, { QMetaType::Int, 72 }, { QMetaType::QString, 73 },
            { 0x80000000 | 45, 74 },
        }}),
        // Slot 'watchDownloadProgress'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, const MsgId &, qint64, qint64)>(75, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 68 }, { 0x80000000 | 76, 77 }, { QMetaType::LongLong, 78 },
            { QMetaType::LongLong, 79 },
        }}),
        // Slot 'watchDownloadProgressFinished'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, const MsgId &, int, const QString &)>(80, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 68 }, { 0x80000000 | 76, 77 }, { QMetaType::Int, 72 },
            { QMetaType::QString, 73 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgTimestampRenewal, qt_meta_tag_ZN19DlgTimestampRenewalE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgTimestampRenewal::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgTimestampRenewalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgTimestampRenewalE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19DlgTimestampRenewalE_t>.metaTypes,
    nullptr
} };

void DlgTimestampRenewal::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgTimestampRenewal *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusSelectedMsg((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->done((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->renewSelectedTrackedTimestamps(); break;
        case 3: _t->renewSelectedAllStoredTimestamps(); break;
        case 4: _t->renewListedAllStoredTimestamps(); break;
        case 5: _t->updateTrackedMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 6: _t->updateTrackedMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 7: _t->updateAllStoredMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 8: _t->updateAllStoredMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 9: _t->watchAllStoredMessageList(); break;
        case 10: _t->toggleTrackedTimestampTracking((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->toggleAllStoredTimestampTracking((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->viewTrackedMessagesListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 13: _t->trackedMessageItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 14: _t->viewAllStoredMessagesListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 15: _t->allStoredMessageItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 16: _t->filterTrackedMessages((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->filterAllMessages((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->hideTrackedMessagesFilter(); break;
        case 19: _t->hideAllMessagesFilter(); break;
        case 20: _t->watchIdentificationsInserted((*reinterpret_cast<std::add_pointer_t<Json::MsgId2List>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[2]))); break;
        case 21: _t->watchIdentificationsDeleted((*reinterpret_cast<std::add_pointer_t<Json::MsgId2List>>(_a[1]))); break;
        case 22: _t->watchMsgTimestampDataUpdated((*reinterpret_cast<std::add_pointer_t<TstValidityHash>>(_a[1]))); break;
        case 23: _t->watchPrefsModification((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 24: _t->applyAllSelection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->applyRestampableFiltering((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->modifiedAccountSelection(); break;
        case 27: _t->startAccountScan(); break;
        case 28: _t->interruptAccountScan(); break;
        case 29: _t->watchScanStopped((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->watchScanProcessed((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<TstEntry>>>(_a[2]))); break;
        case 31: _t->watchScanProgress((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 32: _t->selectExternalSrcZfo(); break;
        case 33: _t->viewExternalSrcZfo(); break;
        case 34: _t->selectExternalTgtZfo(); break;
        case 35: _t->renewExternalTimestamps(); break;
        case 36: _t->watchUploadProgress((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4]))); break;
        case 37: _t->watchUploadProgressFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[5]))); break;
        case 38: _t->watchDownloadProgress((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[5]))); break;
        case 39: _t->watchDownloadProgressFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::MsgId2List >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::MsgId2List >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TstValidityHash >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<TstEntry> >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 37:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 39:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgTimestampRenewal::*)(AcntId , qint64 , QString , int )>(_a, &DlgTimestampRenewal::focusSelectedMsg, 0))
            return;
    }
}

const QMetaObject *DlgTimestampRenewal::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgTimestampRenewal::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19DlgTimestampRenewalE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgTimestampRenewal::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void DlgTimestampRenewal::focusSelectedMsg(AcntId _t1, qint64 _t2, QString _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
