/********************************************************************************
** Form generated from reading UI file 'dlg_survey.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_SURVEY_H
#define UI_DLG_SURVEY_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgSurvey
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *labelMessage;
    QHBoxLayout *horizontalLayout_01;
    QSpacerItem *horizontalSpacer_01;
    QPushButton *fillSurveyPushButton;
    QSpacerItem *horizontalSpacer_02;
    QSpacerItem *verticalSpacer_01;
    QFrame *line;
    QHBoxLayout *horizontalLayout_02;
    QSpacerItem *horizontalSpacer_03;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgSurvey)
    {
        if (DlgSurvey->objectName().isEmpty())
            DlgSurvey->setObjectName("DlgSurvey");
        DlgSurvey->resize(400, 300);
        verticalLayout_3 = new QVBoxLayout(DlgSurvey);
        verticalLayout_3->setObjectName("verticalLayout_3");
        labelMessage = new QLabel(DlgSurvey);
        labelMessage->setObjectName("labelMessage");
        labelMessage->setAlignment(Qt::AlignCenter);
        labelMessage->setWordWrap(true);

        verticalLayout_3->addWidget(labelMessage);

        horizontalLayout_01 = new QHBoxLayout();
        horizontalLayout_01->setObjectName("horizontalLayout_01");
        horizontalSpacer_01 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_01->addItem(horizontalSpacer_01);

        fillSurveyPushButton = new QPushButton(DlgSurvey);
        fillSurveyPushButton->setObjectName("fillSurveyPushButton");
        QFont font;
        font.setBold(true);
        fillSurveyPushButton->setFont(font);

        horizontalLayout_01->addWidget(fillSurveyPushButton);

        horizontalSpacer_02 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_01->addItem(horizontalSpacer_02);


        verticalLayout_3->addLayout(horizontalLayout_01);

        verticalSpacer_01 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_01);

        line = new QFrame(DlgSurvey);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        horizontalLayout_02 = new QHBoxLayout();
        horizontalLayout_02->setObjectName("horizontalLayout_02");
        horizontalSpacer_03 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_02->addItem(horizontalSpacer_03);

        buttonBox = new QDialogButtonBox(DlgSurvey);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Discard|QDialogButtonBox::Ignore|QDialogButtonBox::Yes);
        buttonBox->setCenterButtons(true);

        horizontalLayout_02->addWidget(buttonBox);


        verticalLayout_3->addLayout(horizontalLayout_02);


        retranslateUi(DlgSurvey);

        QMetaObject::connectSlotsByName(DlgSurvey);
    } // setupUi

    void retranslateUi(QDialog *DlgSurvey)
    {
        DlgSurvey->setWindowTitle(QCoreApplication::translate("DlgSurvey", "Survey", nullptr));
        labelMessage->setText(QCoreApplication::translate("DlgSurvey", "n/a", nullptr));
        fillSurveyPushButton->setText(QCoreApplication::translate("DlgSurvey", "Fill out Survey", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgSurvey: public Ui_DlgSurvey {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_SURVEY_H
