/********************************************************************************
** Form generated from reading UI file 'dlg_gov_service.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_GOV_SERVICE_H
#define UI_DLG_GOV_SERVICE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgGovService
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *accountNameLabel;
    QLabel *boxIdLabel;
    QLabel *accountLabel;
    QLabel *boxLabel;
    QLabel *requestNameLabel;
    QLabel *requestLabel;
    QFrame *line;
    QLabel *promptLabel;
    QFormLayout *formGovLayout;
    QLabel *invalidValueLabel;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *sendButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *DlgGovService)
    {
        if (DlgGovService->objectName().isEmpty())
            DlgGovService->setObjectName("DlgGovService");
        DlgGovService->resize(480, 180);
        verticalLayout = new QVBoxLayout(DlgGovService);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        accountNameLabel = new QLabel(DlgGovService);
        accountNameLabel->setObjectName("accountNameLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, accountNameLabel);

        boxIdLabel = new QLabel(DlgGovService);
        boxIdLabel->setObjectName("boxIdLabel");
        QFont font;
        font.setBold(false);
        boxIdLabel->setFont(font);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, boxIdLabel);

        accountLabel = new QLabel(DlgGovService);
        accountLabel->setObjectName("accountLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, accountLabel);

        boxLabel = new QLabel(DlgGovService);
        boxLabel->setObjectName("boxLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, boxLabel);

        requestNameLabel = new QLabel(DlgGovService);
        requestNameLabel->setObjectName("requestNameLabel");
        QFont font1;
        font1.setBold(true);
        requestNameLabel->setFont(font1);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, requestNameLabel);

        requestLabel = new QLabel(DlgGovService);
        requestLabel->setObjectName("requestLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, requestLabel);


        verticalLayout->addLayout(formLayout);

        line = new QFrame(DlgGovService);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        promptLabel = new QLabel(DlgGovService);
        promptLabel->setObjectName("promptLabel");
        promptLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(promptLabel);

        formGovLayout = new QFormLayout();
        formGovLayout->setObjectName("formGovLayout");
        formGovLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formGovLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        verticalLayout->addLayout(formGovLayout);

        invalidValueLabel = new QLabel(DlgGovService);
        invalidValueLabel->setObjectName("invalidValueLabel");
        invalidValueLabel->setEnabled(false);
        invalidValueLabel->setFont(font1);
        invalidValueLabel->setText(QString::fromUtf8(""));
        invalidValueLabel->setAlignment(Qt::AlignCenter);
        invalidValueLabel->setWordWrap(true);

        verticalLayout->addWidget(invalidValueLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        sendButton = new QPushButton(DlgGovService);
        sendButton->setObjectName("sendButton");
        sendButton->setEnabled(false);

        horizontalLayout->addWidget(sendButton);

        cancelButton = new QPushButton(DlgGovService);
        cancelButton->setObjectName("cancelButton");

        horizontalLayout->addWidget(cancelButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(DlgGovService);

        QMetaObject::connectSlotsByName(DlgGovService);
    } // setupUi

    void retranslateUi(QDialog *DlgGovService)
    {
        DlgGovService->setWindowTitle(QCoreApplication::translate("DlgGovService", "E-Government Request", nullptr));
        accountNameLabel->setText(QCoreApplication::translate("DlgGovService", "n/a", nullptr));
        boxIdLabel->setText(QCoreApplication::translate("DlgGovService", "n/a", nullptr));
        accountLabel->setText(QCoreApplication::translate("DlgGovService", "Data box:", nullptr));
        boxLabel->setText(QCoreApplication::translate("DlgGovService", "Recipient:", nullptr));
        requestNameLabel->setText(QCoreApplication::translate("DlgGovService", "n/a", nullptr));
        requestLabel->setText(QCoreApplication::translate("DlgGovService", "Request:", nullptr));
        promptLabel->setText(QCoreApplication::translate("DlgGovService", "The service requires these fields to be filled in:", nullptr));
        sendButton->setText(QCoreApplication::translate("DlgGovService", "Send", nullptr));
        cancelButton->setText(QCoreApplication::translate("DlgGovService", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgGovService: public Ui_DlgGovService {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_GOV_SERVICE_H
