/********************************************************************************
** Form generated from reading UI file 'dlg_proxysets.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_PROXYSETS_H
#define UI_DLG_PROXYSETS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgProxysets
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *mainLabel;
    QGroupBox *httpsGroupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *httpsNoProxyRadioButton;
    QRadioButton *httpsAutoProxyRadioButton;
    QLabel *httpsProxyDetectionLabel;
    QRadioButton *httpsManualProxyRadioButton;
    QHBoxLayout *httpsHorizontalLayou;
    QLabel *httpsHostnameLabel;
    QLineEdit *httpsHostnameLine;
    QLabel *httpsPortLabel;
    QLineEdit *httpsPortLine;
    QFrame *httpsLine;
    QCheckBox *httpsAuthCheckBox;
    QWidget *httpsProxyAuth;
    QHBoxLayout *horizontalLayout;
    QLabel *httpsUnameLabel;
    QLineEdit *httpsUnameLine;
    QLabel *httpsPwdLabel;
    QLineEdit *httpsPwdLine;
    QGroupBox *httpGroupBox;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *httpNoProxyRadioButton;
    QRadioButton *httpAutoProxyRadioButton;
    QLabel *httpProxyDetectionLabel;
    QRadioButton *httpManualProxyRadioButton;
    QHBoxLayout *httpHorizontalLayout;
    QLabel *httpHostnameLabel;
    QLineEdit *httpHostnameLine;
    QLabel *httpPortLabel;
    QLineEdit *httpPortLine;
    QFrame *httpLine;
    QCheckBox *httpAuthCheckBox;
    QWidget *httpProxyAuth;
    QHBoxLayout *horizontalLayout_2;
    QLabel *httpUnameLabel;
    QLineEdit *httpUnameLine;
    QLabel *httpPwdLabel;
    QLineEdit *httpPwdLine;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *proxyButtonBox;

    void setupUi(QDialog *DlgProxysets)
    {
        if (DlgProxysets->objectName().isEmpty())
            DlgProxysets->setObjectName("DlgProxysets");
        DlgProxysets->resize(452, 643);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgProxysets->sizePolicy().hasHeightForWidth());
        DlgProxysets->setSizePolicy(sizePolicy);
        DlgProxysets->setMinimumSize(QSize(400, 400));
        DlgProxysets->setSizeIncrement(QSize(0, 0));
        verticalLayout_2 = new QVBoxLayout(DlgProxysets);
        verticalLayout_2->setObjectName("verticalLayout_2");
        mainLabel = new QLabel(DlgProxysets);
        mainLabel->setObjectName("mainLabel");
        mainLabel->setLineWidth(1);
        mainLabel->setAlignment(Qt::AlignCenter);
        mainLabel->setWordWrap(true);

        verticalLayout_2->addWidget(mainLabel);

        httpsGroupBox = new QGroupBox(DlgProxysets);
        httpsGroupBox->setObjectName("httpsGroupBox");
        QFont font;
        font.setBold(true);
        httpsGroupBox->setFont(font);
        verticalLayout = new QVBoxLayout(httpsGroupBox);
        verticalLayout->setObjectName("verticalLayout");
        httpsNoProxyRadioButton = new QRadioButton(httpsGroupBox);
        httpsNoProxyRadioButton->setObjectName("httpsNoProxyRadioButton");
        QFont font1;
        font1.setBold(false);
        httpsNoProxyRadioButton->setFont(font1);

        verticalLayout->addWidget(httpsNoProxyRadioButton);

        httpsAutoProxyRadioButton = new QRadioButton(httpsGroupBox);
        httpsAutoProxyRadioButton->setObjectName("httpsAutoProxyRadioButton");
        httpsAutoProxyRadioButton->setFont(font1);
        httpsAutoProxyRadioButton->setChecked(true);

        verticalLayout->addWidget(httpsAutoProxyRadioButton);

        httpsProxyDetectionLabel = new QLabel(httpsGroupBox);
        httpsProxyDetectionLabel->setObjectName("httpsProxyDetectionLabel");
        QFont font2;
        font2.setBold(false);
        font2.setItalic(true);
        httpsProxyDetectionLabel->setFont(font2);

        verticalLayout->addWidget(httpsProxyDetectionLabel);

        httpsManualProxyRadioButton = new QRadioButton(httpsGroupBox);
        httpsManualProxyRadioButton->setObjectName("httpsManualProxyRadioButton");
        httpsManualProxyRadioButton->setFont(font1);

        verticalLayout->addWidget(httpsManualProxyRadioButton);

        httpsHorizontalLayou = new QHBoxLayout();
        httpsHorizontalLayou->setObjectName("httpsHorizontalLayou");
        httpsHostnameLabel = new QLabel(httpsGroupBox);
        httpsHostnameLabel->setObjectName("httpsHostnameLabel");
        httpsHostnameLabel->setFont(font1);

        httpsHorizontalLayou->addWidget(httpsHostnameLabel);

        httpsHostnameLine = new QLineEdit(httpsGroupBox);
        httpsHostnameLine->setObjectName("httpsHostnameLine");
        httpsHostnameLine->setFont(font1);

        httpsHorizontalLayou->addWidget(httpsHostnameLine);

        httpsPortLabel = new QLabel(httpsGroupBox);
        httpsPortLabel->setObjectName("httpsPortLabel");
        httpsPortLabel->setFont(font1);

        httpsHorizontalLayou->addWidget(httpsPortLabel);

        httpsPortLine = new QLineEdit(httpsGroupBox);
        httpsPortLine->setObjectName("httpsPortLine");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(httpsPortLine->sizePolicy().hasHeightForWidth());
        httpsPortLine->setSizePolicy(sizePolicy1);
        httpsPortLine->setFont(font1);

        httpsHorizontalLayou->addWidget(httpsPortLine);


        verticalLayout->addLayout(httpsHorizontalLayou);

        httpsLine = new QFrame(httpsGroupBox);
        httpsLine->setObjectName("httpsLine");
        httpsLine->setFrameShape(QFrame::Shape::HLine);
        httpsLine->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(httpsLine);

        httpsAuthCheckBox = new QCheckBox(httpsGroupBox);
        httpsAuthCheckBox->setObjectName("httpsAuthCheckBox");
        httpsAuthCheckBox->setFont(font1);

        verticalLayout->addWidget(httpsAuthCheckBox);

        httpsProxyAuth = new QWidget(httpsGroupBox);
        httpsProxyAuth->setObjectName("httpsProxyAuth");
        horizontalLayout = new QHBoxLayout(httpsProxyAuth);
        horizontalLayout->setObjectName("horizontalLayout");
        httpsUnameLabel = new QLabel(httpsProxyAuth);
        httpsUnameLabel->setObjectName("httpsUnameLabel");
        httpsUnameLabel->setFont(font1);

        horizontalLayout->addWidget(httpsUnameLabel);

        httpsUnameLine = new QLineEdit(httpsProxyAuth);
        httpsUnameLine->setObjectName("httpsUnameLine");

        horizontalLayout->addWidget(httpsUnameLine);

        httpsPwdLabel = new QLabel(httpsProxyAuth);
        httpsPwdLabel->setObjectName("httpsPwdLabel");
        httpsPwdLabel->setFont(font1);

        horizontalLayout->addWidget(httpsPwdLabel);

        httpsPwdLine = new QLineEdit(httpsProxyAuth);
        httpsPwdLine->setObjectName("httpsPwdLine");
        httpsPwdLine->setEchoMode(QLineEdit::Password);

        horizontalLayout->addWidget(httpsPwdLine);


        verticalLayout->addWidget(httpsProxyAuth);


        verticalLayout_2->addWidget(httpsGroupBox);

        httpGroupBox = new QGroupBox(DlgProxysets);
        httpGroupBox->setObjectName("httpGroupBox");
        httpGroupBox->setFont(font);
        verticalLayout_3 = new QVBoxLayout(httpGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        httpNoProxyRadioButton = new QRadioButton(httpGroupBox);
        httpNoProxyRadioButton->setObjectName("httpNoProxyRadioButton");
        httpNoProxyRadioButton->setFont(font1);

        verticalLayout_3->addWidget(httpNoProxyRadioButton);

        httpAutoProxyRadioButton = new QRadioButton(httpGroupBox);
        httpAutoProxyRadioButton->setObjectName("httpAutoProxyRadioButton");
        httpAutoProxyRadioButton->setFont(font1);
        httpAutoProxyRadioButton->setChecked(true);

        verticalLayout_3->addWidget(httpAutoProxyRadioButton);

        httpProxyDetectionLabel = new QLabel(httpGroupBox);
        httpProxyDetectionLabel->setObjectName("httpProxyDetectionLabel");
        httpProxyDetectionLabel->setFont(font2);

        verticalLayout_3->addWidget(httpProxyDetectionLabel);

        httpManualProxyRadioButton = new QRadioButton(httpGroupBox);
        httpManualProxyRadioButton->setObjectName("httpManualProxyRadioButton");
        httpManualProxyRadioButton->setFont(font1);

        verticalLayout_3->addWidget(httpManualProxyRadioButton);

        httpHorizontalLayout = new QHBoxLayout();
        httpHorizontalLayout->setObjectName("httpHorizontalLayout");
        httpHostnameLabel = new QLabel(httpGroupBox);
        httpHostnameLabel->setObjectName("httpHostnameLabel");
        httpHostnameLabel->setFont(font1);

        httpHorizontalLayout->addWidget(httpHostnameLabel);

        httpHostnameLine = new QLineEdit(httpGroupBox);
        httpHostnameLine->setObjectName("httpHostnameLine");
        httpHostnameLine->setFont(font1);

        httpHorizontalLayout->addWidget(httpHostnameLine);

        httpPortLabel = new QLabel(httpGroupBox);
        httpPortLabel->setObjectName("httpPortLabel");
        httpPortLabel->setFont(font1);

        httpHorizontalLayout->addWidget(httpPortLabel);

        httpPortLine = new QLineEdit(httpGroupBox);
        httpPortLine->setObjectName("httpPortLine");
        sizePolicy1.setHeightForWidth(httpPortLine->sizePolicy().hasHeightForWidth());
        httpPortLine->setSizePolicy(sizePolicy1);
        httpPortLine->setMaximumSize(QSize(16777215, 16777215));
        httpPortLine->setBaseSize(QSize(0, 0));
        httpPortLine->setFont(font1);

        httpHorizontalLayout->addWidget(httpPortLine);


        verticalLayout_3->addLayout(httpHorizontalLayout);

        httpLine = new QFrame(httpGroupBox);
        httpLine->setObjectName("httpLine");
        httpLine->setFrameShape(QFrame::Shape::HLine);
        httpLine->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(httpLine);

        httpAuthCheckBox = new QCheckBox(httpGroupBox);
        httpAuthCheckBox->setObjectName("httpAuthCheckBox");
        httpAuthCheckBox->setFont(font1);

        verticalLayout_3->addWidget(httpAuthCheckBox);

        httpProxyAuth = new QWidget(httpGroupBox);
        httpProxyAuth->setObjectName("httpProxyAuth");
        horizontalLayout_2 = new QHBoxLayout(httpProxyAuth);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        httpUnameLabel = new QLabel(httpProxyAuth);
        httpUnameLabel->setObjectName("httpUnameLabel");
        httpUnameLabel->setFont(font1);

        horizontalLayout_2->addWidget(httpUnameLabel);

        httpUnameLine = new QLineEdit(httpProxyAuth);
        httpUnameLine->setObjectName("httpUnameLine");

        horizontalLayout_2->addWidget(httpUnameLine);

        httpPwdLabel = new QLabel(httpProxyAuth);
        httpPwdLabel->setObjectName("httpPwdLabel");
        httpPwdLabel->setFont(font1);

        horizontalLayout_2->addWidget(httpPwdLabel);

        httpPwdLine = new QLineEdit(httpProxyAuth);
        httpPwdLine->setObjectName("httpPwdLine");
        httpPwdLine->setEchoMode(QLineEdit::Password);

        horizontalLayout_2->addWidget(httpPwdLine);


        verticalLayout_3->addWidget(httpProxyAuth);


        verticalLayout_2->addWidget(httpGroupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        proxyButtonBox = new QDialogButtonBox(DlgProxysets);
        proxyButtonBox->setObjectName("proxyButtonBox");
        proxyButtonBox->setOrientation(Qt::Horizontal);
        proxyButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(proxyButtonBox);

        QWidget::setTabOrder(httpsNoProxyRadioButton, httpsAutoProxyRadioButton);
        QWidget::setTabOrder(httpsAutoProxyRadioButton, httpsManualProxyRadioButton);
        QWidget::setTabOrder(httpsManualProxyRadioButton, httpsHostnameLine);
        QWidget::setTabOrder(httpsHostnameLine, httpsPortLine);
        QWidget::setTabOrder(httpsPortLine, httpsAuthCheckBox);
        QWidget::setTabOrder(httpsAuthCheckBox, httpsUnameLine);
        QWidget::setTabOrder(httpsUnameLine, httpsPwdLine);
        QWidget::setTabOrder(httpsPwdLine, httpNoProxyRadioButton);
        QWidget::setTabOrder(httpNoProxyRadioButton, httpAutoProxyRadioButton);
        QWidget::setTabOrder(httpAutoProxyRadioButton, httpManualProxyRadioButton);
        QWidget::setTabOrder(httpManualProxyRadioButton, httpHostnameLine);
        QWidget::setTabOrder(httpHostnameLine, httpPortLine);
        QWidget::setTabOrder(httpPortLine, httpAuthCheckBox);
        QWidget::setTabOrder(httpAuthCheckBox, httpUnameLine);
        QWidget::setTabOrder(httpUnameLine, httpPwdLine);

        retranslateUi(DlgProxysets);
        QObject::connect(proxyButtonBox, &QDialogButtonBox::accepted, DlgProxysets, qOverload<>(&QDialog::accept));
        QObject::connect(proxyButtonBox, &QDialogButtonBox::rejected, DlgProxysets, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgProxysets);
    } // setupUi

    void retranslateUi(QDialog *DlgProxysets)
    {
        DlgProxysets->setWindowTitle(QCoreApplication::translate("DlgProxysets", "Proxy Settings", nullptr));
        mainLabel->setText(QCoreApplication::translate("DlgProxysets", "The HTTPS protocol is used when accessing the ISDS server while HTTP is used to download the certificate revocation list and information about new application versions.", nullptr));
        httpsGroupBox->setTitle(QCoreApplication::translate("DlgProxysets", "HTTPS proxy", nullptr));
#if QT_CONFIG(tooltip)
        httpsNoProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "No proxy", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsNoProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "No proxy", nullptr));
#if QT_CONFIG(tooltip)
        httpsAutoProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "Automatic proxy detection", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsAutoProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "Automatic proxy detection", nullptr));
        httpsProxyDetectionLabel->setText(QString());
#if QT_CONFIG(tooltip)
        httpsManualProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "Manual proxy settings", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsManualProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "Manual proxy settings", nullptr));
        httpsHostnameLabel->setText(QCoreApplication::translate("DlgProxysets", "Proxy hostname:", nullptr));
#if QT_CONFIG(tooltip)
        httpsHostnameLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your proxy hostname", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsPortLabel->setText(QCoreApplication::translate("DlgProxysets", "Port:", nullptr));
#if QT_CONFIG(tooltip)
        httpsPortLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter port", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        httpsAuthCheckBox->setToolTip(QCoreApplication::translate("DlgProxysets", "Show HTTPS proxy authentication", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsAuthCheckBox->setText(QCoreApplication::translate("DlgProxysets", "Show HTTPS proxy authentication", nullptr));
        httpsUnameLabel->setText(QCoreApplication::translate("DlgProxysets", "Username:", nullptr));
#if QT_CONFIG(tooltip)
        httpsUnameLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your username", nullptr));
#endif // QT_CONFIG(tooltip)
        httpsPwdLabel->setText(QCoreApplication::translate("DlgProxysets", "Password:", nullptr));
#if QT_CONFIG(tooltip)
        httpsPwdLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your password", nullptr));
#endif // QT_CONFIG(tooltip)
        httpGroupBox->setTitle(QCoreApplication::translate("DlgProxysets", "HTTP proxy", nullptr));
#if QT_CONFIG(tooltip)
        httpNoProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "No proxy", nullptr));
#endif // QT_CONFIG(tooltip)
        httpNoProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "No proxy", nullptr));
#if QT_CONFIG(tooltip)
        httpAutoProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "Automatic proxy detection", nullptr));
#endif // QT_CONFIG(tooltip)
        httpAutoProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "Automatic proxy detection", nullptr));
        httpProxyDetectionLabel->setText(QString());
#if QT_CONFIG(tooltip)
        httpManualProxyRadioButton->setToolTip(QCoreApplication::translate("DlgProxysets", "Manual proxy settings", nullptr));
#endif // QT_CONFIG(tooltip)
        httpManualProxyRadioButton->setText(QCoreApplication::translate("DlgProxysets", "Manual proxy settings", nullptr));
        httpHostnameLabel->setText(QCoreApplication::translate("DlgProxysets", "Proxy hostname:", nullptr));
#if QT_CONFIG(tooltip)
        httpHostnameLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your proxy hostname", nullptr));
#endif // QT_CONFIG(tooltip)
        httpPortLabel->setText(QCoreApplication::translate("DlgProxysets", "Port:", nullptr));
#if QT_CONFIG(tooltip)
        httpPortLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter port", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        httpAuthCheckBox->setToolTip(QCoreApplication::translate("DlgProxysets", "Show HTTP proxy authentication", nullptr));
#endif // QT_CONFIG(tooltip)
        httpAuthCheckBox->setText(QCoreApplication::translate("DlgProxysets", "Show HTTP proxy authentication", nullptr));
        httpUnameLabel->setText(QCoreApplication::translate("DlgProxysets", "Username:", nullptr));
#if QT_CONFIG(tooltip)
        httpUnameLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your username", nullptr));
#endif // QT_CONFIG(tooltip)
        httpPwdLabel->setText(QCoreApplication::translate("DlgProxysets", "Password:", nullptr));
#if QT_CONFIG(tooltip)
        httpPwdLine->setToolTip(QCoreApplication::translate("DlgProxysets", "Enter your password", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class DlgProxysets: public Ui_DlgProxysets {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_PROXYSETS_H
