/********************************************************************************
** Form generated from reading UI file 'dlg_gov_services.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_GOV_SERVICES_H
#define UI_DLG_GOV_SERVICES_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgGovServices
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QFormLayout *formLayout;
    QLabel *accountLabel;
    QComboBox *fromComboBox;
    QLabel *cannotSendLabel;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    QListView *govServiceListView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgGovServices)
    {
        if (DlgGovServices->objectName().isEmpty())
            DlgGovServices->setObjectName("DlgGovServices");
        DlgGovServices->resize(480, 360);
        verticalLayout = new QVBoxLayout(DlgGovServices);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(DlgGovServices);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(label);

        line = new QFrame(DlgGovServices);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        accountLabel = new QLabel(DlgGovServices);
        accountLabel->setObjectName("accountLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, accountLabel);

        fromComboBox = new QComboBox(DlgGovServices);
        fromComboBox->setObjectName("fromComboBox");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, fromComboBox);


        verticalLayout->addLayout(formLayout);

        cannotSendLabel = new QLabel(DlgGovServices);
        cannotSendLabel->setObjectName("cannotSendLabel");
        QFont font;
        font.setBold(true);
        cannotSendLabel->setFont(font);
        cannotSendLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(cannotSendLabel);

        line_2 = new QFrame(DlgGovServices);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        filterLabel = new QLabel(DlgGovServices);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgGovServices);
        filterLine->setObjectName("filterLine");

        horizontalLayout->addWidget(filterLine);


        verticalLayout->addLayout(horizontalLayout);

        govServiceListView = new QListView(DlgGovServices);
        govServiceListView->setObjectName("govServiceListView");

        verticalLayout->addWidget(govServiceListView);

        buttonBox = new QDialogButtonBox(DlgGovServices);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(govServiceListView, fromComboBox);
        QWidget::setTabOrder(fromComboBox, filterLine);

        retranslateUi(DlgGovServices);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgGovServices, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgGovServices, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgGovServices);
    } // setupUi

    void retranslateUi(QDialog *DlgGovServices)
    {
        DlgGovServices->setWindowTitle(QCoreApplication::translate("DlgGovServices", "E-government Requests", nullptr));
        label->setText(QCoreApplication::translate("DlgGovServices", "Select the e-gov request form you want to fill in and send:", nullptr));
        accountLabel->setText(QCoreApplication::translate("DlgGovServices", "Data box:", nullptr));
        cannotSendLabel->setText(QCoreApplication::translate("DlgGovServices", "You cannot send any e-gov requests from this data box.", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgGovServices", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgGovServices: public Ui_DlgGovServices {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_GOV_SERVICES_H
