/********************************************************************************
** Form generated from reading UI file 'dlg_records_management_upload.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_RECORDS_MANAGEMENT_UPLOAD_H
#define UI_DLG_RECORDS_MANAGEMENT_UPLOAD_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_tree_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgRecordsManagementUpload
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    QLabel *pixmapLabel;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *appealLabel;
    QSpacerItem *horizontalSpacer;
    QFrame *line;
    QHBoxLayout *horizontalLayout;
    QPushButton *reloadButton;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    LoweredTreeView *uploadView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgRecordsManagementUpload)
    {
        if (DlgRecordsManagementUpload->objectName().isEmpty())
            DlgRecordsManagementUpload->setObjectName("DlgRecordsManagementUpload");
        DlgRecordsManagementUpload->resize(400, 300);
        verticalLayout = new QVBoxLayout(DlgRecordsManagementUpload);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        pixmapLabel = new QLabel(DlgRecordsManagementUpload);
        pixmapLabel->setObjectName("pixmapLabel");

        horizontalLayout_3->addWidget(pixmapLabel);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        appealLabel = new QLabel(DlgRecordsManagementUpload);
        appealLabel->setObjectName("appealLabel");

        horizontalLayout_2->addWidget(appealLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout_2);

        line = new QFrame(DlgRecordsManagementUpload);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        reloadButton = new QPushButton(DlgRecordsManagementUpload);
        reloadButton->setObjectName("reloadButton");

        horizontalLayout->addWidget(reloadButton);

        filterLabel = new QLabel(DlgRecordsManagementUpload);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgRecordsManagementUpload);
        filterLine->setObjectName("filterLine");

        horizontalLayout->addWidget(filterLine);


        verticalLayout_2->addLayout(horizontalLayout);


        horizontalLayout_3->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout_3);

        uploadView = new LoweredTreeView(DlgRecordsManagementUpload);
        uploadView->setObjectName("uploadView");

        verticalLayout->addWidget(uploadView);

        buttonBox = new QDialogButtonBox(DlgRecordsManagementUpload);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgRecordsManagementUpload);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgRecordsManagementUpload, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgRecordsManagementUpload, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgRecordsManagementUpload);
    } // setupUi

    void retranslateUi(QDialog *DlgRecordsManagementUpload)
    {
        DlgRecordsManagementUpload->setWindowTitle(QCoreApplication::translate("DlgRecordsManagementUpload", "Upload Message into Records Management Service", nullptr));
        pixmapLabel->setText(QString());
        appealLabel->setText(QCoreApplication::translate("DlgRecordsManagementUpload", "Appeal", nullptr));
        reloadButton->setText(QCoreApplication::translate("DlgRecordsManagementUpload", "Reload", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgRecordsManagementUpload", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgRecordsManagementUpload: public Ui_DlgRecordsManagementUpload {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_RECORDS_MANAGEMENT_UPLOAD_H
