/********************************************************************************
** Form generated from reading UI file 'dlg_view_filename_param_list.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_VIEW_FILENAME_PARAM_LIST_H
#define UI_DLG_VIEW_FILENAME_PARAM_LIST_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgViewFilenameParamList
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label_42;
    QHBoxLayout *horizontalLayout_5;
    QVBoxLayout *verticalLayout_24;
    QGroupBox *groupBox_12;
    QFormLayout *formLayout_5;
    QLabel *label_36;
    QLabel *label_37;
    QLabel *label_14;
    QLabel *label_25;
    QLabel *label_33;
    QLabel *label_34;
    QGroupBox *groupBox_13;
    QFormLayout *formLayout_4;
    QLabel *label_18;
    QLabel *label_67;
    QLabel *label_17;
    QLabel *label_13;
    QLabel *label_32;
    QLabel *label_12;
    QLabel *label_19;
    QLabel *label_68;
    QLabel *label_20;
    QLabel *label_21;
    QLabel *label_31;
    QLabel *label_22;
    QGroupBox *groupBox;
    QFormLayout *formLayout;
    QLabel *label_43;
    QLabel *label_69;
    QLabel *label_45;
    QLabel *label_47;
    QLabel *label_49;
    QLabel *label_51;
    QLabel *label_44;
    QLabel *label_70;
    QLabel *label_46;
    QLabel *label_48;
    QLabel *label_50;
    QLabel *label_52;
    QGroupBox *groupBox_2;
    QFormLayout *formLayout_2;
    QLabel *label_11;
    QLabel *label_26;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_14;
    QFormLayout *formLayout_3;
    QLabel *label_16;
    QLabel *label_23;
    QLabel *label_15;
    QLabel *label_24;
    QLabel *label_5;
    QLabel *label_4;
    QLabel *label_35;
    QLabel *label_101;
    QLabel *label_59;
    QLabel *label_60;
    QLabel *label_3;
    QLabel *label_6;
    QLabel *label;
    QLabel *label_2;
    QLabel *label_61;
    QLabel *label_62;
    QLabel *label_53;
    QLabel *label_55;
    QLabel *label_41;
    QLabel *label_56;
    QLabel *label_54;
    QLabel *label_57;
    QLabel *label_7;
    QLabel *label_58;
    QLabel *label_40;
    QLabel *label_8;
    QGroupBox *groupBox_3;
    QFormLayout *formLayout_6;
    QLabel *label_9;
    QLabel *label_10;
    QLabel *label_63;
    QLabel *label_39;
    QGroupBox *groupBox_4;
    QFormLayout *formLayout_8;
    QLabel *label_65;
    QLabel *label_66;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout_7;
    QLabel *label_28;
    QLabel *label_30;
    QLabel *label_29;
    QLabel *label_27;

    void setupUi(QDialog *DlgViewFilenameParamList)
    {
        if (DlgViewFilenameParamList->objectName().isEmpty())
            DlgViewFilenameParamList->setObjectName("DlgViewFilenameParamList");
        DlgViewFilenameParamList->resize(790, 806);
        verticalLayout = new QVBoxLayout(DlgViewFilenameParamList);
        verticalLayout->setObjectName("verticalLayout");
        label_42 = new QLabel(DlgViewFilenameParamList);
        label_42->setObjectName("label_42");
        label_42->setAlignment(Qt::AlignCenter);
        label_42->setWordWrap(true);

        verticalLayout->addWidget(label_42);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        verticalLayout_24 = new QVBoxLayout();
        verticalLayout_24->setObjectName("verticalLayout_24");
        groupBox_12 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_12->setObjectName("groupBox_12");
        QFont font;
        font.setBold(true);
        groupBox_12->setFont(font);
        formLayout_5 = new QFormLayout(groupBox_12);
        formLayout_5->setObjectName("formLayout_5");
        label_36 = new QLabel(groupBox_12);
        label_36->setObjectName("label_36");
        QFont font1;
        font1.setBold(false);
        font1.setItalic(true);
        label_36->setFont(font1);
        label_36->setText(QString::fromUtf8("%<AN>"));
        label_36->setTextFormat(Qt::PlainText);
        label_36->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_5->setWidget(0, QFormLayout::ItemRole::LabelRole, label_36);

        label_37 = new QLabel(groupBox_12);
        label_37->setObjectName("label_37");
        label_37->setFont(font1);

        formLayout_5->setWidget(0, QFormLayout::ItemRole::FieldRole, label_37);

        label_14 = new QLabel(groupBox_12);
        label_14->setObjectName("label_14");
        label_14->setFont(font1);
        label_14->setText(QString::fromUtf8("%<AB>"));
        label_14->setTextFormat(Qt::PlainText);
        label_14->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_5->setWidget(1, QFormLayout::ItemRole::LabelRole, label_14);

        label_25 = new QLabel(groupBox_12);
        label_25->setObjectName("label_25");
        label_25->setFont(font1);

        formLayout_5->setWidget(1, QFormLayout::ItemRole::FieldRole, label_25);

        label_33 = new QLabel(groupBox_12);
        label_33->setObjectName("label_33");
        label_33->setFont(font1);
        label_33->setText(QString::fromUtf8("%<AU>"));
        label_33->setTextFormat(Qt::PlainText);
        label_33->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_5->setWidget(2, QFormLayout::ItemRole::LabelRole, label_33);

        label_34 = new QLabel(groupBox_12);
        label_34->setObjectName("label_34");
        label_34->setFont(font1);

        formLayout_5->setWidget(2, QFormLayout::ItemRole::FieldRole, label_34);


        verticalLayout_24->addWidget(groupBox_12);

        groupBox_13 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_13->setObjectName("groupBox_13");
        groupBox_13->setFont(font);
        formLayout_4 = new QFormLayout(groupBox_13);
        formLayout_4->setObjectName("formLayout_4");
        label_18 = new QLabel(groupBox_13);
        label_18->setObjectName("label_18");
        label_18->setFont(font1);
        label_18->setText(QString::fromUtf8("%<AY>"));
        label_18->setTextFormat(Qt::PlainText);
        label_18->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(0, QFormLayout::ItemRole::LabelRole, label_18);

        label_67 = new QLabel(groupBox_13);
        label_67->setObjectName("label_67");
        label_67->setFont(font1);
        label_67->setText(QString::fromUtf8("%<Ay>"));
        label_67->setTextFormat(Qt::PlainText);
        label_67->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(2, QFormLayout::ItemRole::LabelRole, label_67);

        label_17 = new QLabel(groupBox_13);
        label_17->setObjectName("label_17");
        label_17->setFont(font1);
        label_17->setText(QString::fromUtf8("%<AM>"));
        label_17->setTextFormat(Qt::PlainText);
        label_17->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(4, QFormLayout::ItemRole::LabelRole, label_17);

        label_13 = new QLabel(groupBox_13);
        label_13->setObjectName("label_13");
        label_13->setFont(font1);
        label_13->setText(QString::fromUtf8("%<AD>"));
        label_13->setTextFormat(Qt::PlainText);
        label_13->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(6, QFormLayout::ItemRole::LabelRole, label_13);

        label_32 = new QLabel(groupBox_13);
        label_32->setObjectName("label_32");
        label_32->setFont(font1);
        label_32->setText(QString::fromUtf8("%<Ah>"));
        label_32->setTextFormat(Qt::PlainText);
        label_32->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(8, QFormLayout::ItemRole::LabelRole, label_32);

        label_12 = new QLabel(groupBox_13);
        label_12->setObjectName("label_12");
        label_12->setFont(font1);
        label_12->setText(QString::fromUtf8("%<Am>"));
        label_12->setTextFormat(Qt::PlainText);
        label_12->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_4->setWidget(10, QFormLayout::ItemRole::LabelRole, label_12);

        label_19 = new QLabel(groupBox_13);
        label_19->setObjectName("label_19");
        label_19->setFont(font1);

        formLayout_4->setWidget(0, QFormLayout::ItemRole::FieldRole, label_19);

        label_68 = new QLabel(groupBox_13);
        label_68->setObjectName("label_68");
        label_68->setFont(font1);

        formLayout_4->setWidget(2, QFormLayout::ItemRole::FieldRole, label_68);

        label_20 = new QLabel(groupBox_13);
        label_20->setObjectName("label_20");
        label_20->setFont(font1);

        formLayout_4->setWidget(4, QFormLayout::ItemRole::FieldRole, label_20);

        label_21 = new QLabel(groupBox_13);
        label_21->setObjectName("label_21");
        label_21->setFont(font1);

        formLayout_4->setWidget(6, QFormLayout::ItemRole::FieldRole, label_21);

        label_31 = new QLabel(groupBox_13);
        label_31->setObjectName("label_31");
        label_31->setFont(font1);

        formLayout_4->setWidget(8, QFormLayout::ItemRole::FieldRole, label_31);

        label_22 = new QLabel(groupBox_13);
        label_22->setObjectName("label_22");
        label_22->setFont(font1);

        formLayout_4->setWidget(10, QFormLayout::ItemRole::FieldRole, label_22);


        verticalLayout_24->addWidget(groupBox_13);

        groupBox = new QGroupBox(DlgViewFilenameParamList);
        groupBox->setObjectName("groupBox");
        groupBox->setFont(font);
        formLayout = new QFormLayout(groupBox);
        formLayout->setObjectName("formLayout");
        label_43 = new QLabel(groupBox);
        label_43->setObjectName("label_43");
        label_43->setFont(font1);
        label_43->setText(QString::fromUtf8("%<DY>"));
        label_43->setTextFormat(Qt::PlainText);
        label_43->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_43);

        label_69 = new QLabel(groupBox);
        label_69->setObjectName("label_69");
        label_69->setFont(font1);
        label_69->setText(QString::fromUtf8("%<Dy>"));
        label_69->setTextFormat(Qt::PlainText);
        label_69->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_69);

        label_45 = new QLabel(groupBox);
        label_45->setObjectName("label_45");
        label_45->setFont(font1);
        label_45->setText(QString::fromUtf8("%<DM>"));
        label_45->setTextFormat(Qt::PlainText);
        label_45->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_45);

        label_47 = new QLabel(groupBox);
        label_47->setObjectName("label_47");
        label_47->setFont(font1);
        label_47->setText(QString::fromUtf8("%<DD>"));
        label_47->setTextFormat(Qt::PlainText);
        label_47->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, label_47);

        label_49 = new QLabel(groupBox);
        label_49->setObjectName("label_49");
        label_49->setFont(font1);
        label_49->setText(QString::fromUtf8("%<Dh>"));
        label_49->setTextFormat(Qt::PlainText);
        label_49->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(8, QFormLayout::ItemRole::LabelRole, label_49);

        label_51 = new QLabel(groupBox);
        label_51->setObjectName("label_51");
        label_51->setFont(font1);
        label_51->setText(QString::fromUtf8("%<Dm>"));
        label_51->setTextFormat(Qt::PlainText);
        label_51->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout->setWidget(10, QFormLayout::ItemRole::LabelRole, label_51);

        label_44 = new QLabel(groupBox);
        label_44->setObjectName("label_44");
        label_44->setFont(font1);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, label_44);

        label_70 = new QLabel(groupBox);
        label_70->setObjectName("label_70");
        label_70->setFont(font1);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, label_70);

        label_46 = new QLabel(groupBox);
        label_46->setObjectName("label_46");
        label_46->setFont(font1);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, label_46);

        label_48 = new QLabel(groupBox);
        label_48->setObjectName("label_48");
        label_48->setFont(font1);

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, label_48);

        label_50 = new QLabel(groupBox);
        label_50->setObjectName("label_50");
        label_50->setFont(font1);

        formLayout->setWidget(8, QFormLayout::ItemRole::FieldRole, label_50);

        label_52 = new QLabel(groupBox);
        label_52->setObjectName("label_52");
        label_52->setFont(font1);

        formLayout->setWidget(10, QFormLayout::ItemRole::FieldRole, label_52);


        verticalLayout_24->addWidget(groupBox);

        groupBox_2 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setFont(font);
        formLayout_2 = new QFormLayout(groupBox_2);
        formLayout_2->setObjectName("formLayout_2");
        label_11 = new QLabel(groupBox_2);
        label_11->setObjectName("label_11");
        label_11->setFont(font1);
        label_11->setText(QString::fromUtf8("%<FN>"));
        label_11->setTextFormat(Qt::PlainText);
        label_11->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, label_11);

        label_26 = new QLabel(groupBox_2);
        label_26->setObjectName("label_26");
        label_26->setFont(font1);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, label_26);


        verticalLayout_24->addWidget(groupBox_2);


        horizontalLayout_5->addLayout(verticalLayout_24);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_14 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_14->setObjectName("groupBox_14");
        groupBox_14->setFont(font);
        formLayout_3 = new QFormLayout(groupBox_14);
        formLayout_3->setObjectName("formLayout_3");
        label_16 = new QLabel(groupBox_14);
        label_16->setObjectName("label_16");
        label_16->setFont(font1);
        label_16->setText(QString::fromUtf8("%<EMID>"));
        label_16->setTextFormat(Qt::PlainText);
        label_16->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, label_16);

        label_23 = new QLabel(groupBox_14);
        label_23->setObjectName("label_23");
        label_23->setFont(font1);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, label_23);

        label_15 = new QLabel(groupBox_14);
        label_15->setObjectName("label_15");
        label_15->setFont(font1);
        label_15->setText(QString::fromUtf8("%<EMS>"));
        label_15->setTextFormat(Qt::PlainText);
        label_15->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, label_15);

        label_24 = new QLabel(groupBox_14);
        label_24->setObjectName("label_24");
        label_24->setFont(font1);

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, label_24);

        label_5 = new QLabel(groupBox_14);
        label_5->setObjectName("label_5");
        label_5->setFont(font1);
        label_5->setText(QString::fromUtf8("%<ESB>"));
        label_5->setTextFormat(Qt::PlainText);
        label_5->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(2, QFormLayout::ItemRole::LabelRole, label_5);

        label_4 = new QLabel(groupBox_14);
        label_4->setObjectName("label_4");
        label_4->setFont(font1);

        formLayout_3->setWidget(2, QFormLayout::ItemRole::FieldRole, label_4);

        label_35 = new QLabel(groupBox_14);
        label_35->setObjectName("label_35");
        label_35->setFont(font1);
        label_35->setText(QString::fromUtf8("%<ESN>"));
        label_35->setTextFormat(Qt::PlainText);
        label_35->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(3, QFormLayout::ItemRole::LabelRole, label_35);

        label_101 = new QLabel(groupBox_14);
        label_101->setObjectName("label_101");
        label_101->setFont(font1);

        formLayout_3->setWidget(3, QFormLayout::ItemRole::FieldRole, label_101);

        label_59 = new QLabel(groupBox_14);
        label_59->setObjectName("label_59");
        label_59->setFont(font1);
        label_59->setText(QString::fromUtf8("%<ESA>"));
        label_59->setTextFormat(Qt::PlainText);
        label_59->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(4, QFormLayout::ItemRole::LabelRole, label_59);

        label_60 = new QLabel(groupBox_14);
        label_60->setObjectName("label_60");
        label_60->setFont(font1);

        formLayout_3->setWidget(4, QFormLayout::ItemRole::FieldRole, label_60);

        label_3 = new QLabel(groupBox_14);
        label_3->setObjectName("label_3");
        label_3->setFont(font1);
        label_3->setText(QString::fromUtf8("%<ERB>"));
        label_3->setTextFormat(Qt::PlainText);
        label_3->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(5, QFormLayout::ItemRole::LabelRole, label_3);

        label_6 = new QLabel(groupBox_14);
        label_6->setObjectName("label_6");
        label_6->setFont(font1);

        formLayout_3->setWidget(5, QFormLayout::ItemRole::FieldRole, label_6);

        label = new QLabel(groupBox_14);
        label->setObjectName("label");
        label->setFont(font1);
        label->setText(QString::fromUtf8("%<ERN>"));
        label->setTextFormat(Qt::PlainText);
        label->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(6, QFormLayout::ItemRole::LabelRole, label);

        label_2 = new QLabel(groupBox_14);
        label_2->setObjectName("label_2");
        label_2->setFont(font1);

        formLayout_3->setWidget(6, QFormLayout::ItemRole::FieldRole, label_2);

        label_61 = new QLabel(groupBox_14);
        label_61->setObjectName("label_61");
        label_61->setFont(font1);
        label_61->setText(QString::fromUtf8("%<ERA>"));
        label_61->setTextFormat(Qt::PlainText);
        label_61->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(7, QFormLayout::ItemRole::LabelRole, label_61);

        label_62 = new QLabel(groupBox_14);
        label_62->setObjectName("label_62");
        label_62->setFont(font1);

        formLayout_3->setWidget(7, QFormLayout::ItemRole::FieldRole, label_62);

        label_53 = new QLabel(groupBox_14);
        label_53->setObjectName("label_53");
        label_53->setFont(font1);
        label_53->setText(QString::fromUtf8("%<ESR>"));
        label_53->setTextFormat(Qt::PlainText);
        label_53->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(8, QFormLayout::ItemRole::LabelRole, label_53);

        label_55 = new QLabel(groupBox_14);
        label_55->setObjectName("label_55");
        label_55->setFont(font1);

        formLayout_3->setWidget(8, QFormLayout::ItemRole::FieldRole, label_55);

        label_41 = new QLabel(groupBox_14);
        label_41->setObjectName("label_41");
        label_41->setFont(font1);
        label_41->setText(QString::fromUtf8("%<ESF>"));
        label_41->setTextFormat(Qt::PlainText);
        label_41->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(9, QFormLayout::ItemRole::LabelRole, label_41);

        label_56 = new QLabel(groupBox_14);
        label_56->setObjectName("label_56");
        label_56->setFont(font1);

        formLayout_3->setWidget(9, QFormLayout::ItemRole::FieldRole, label_56);

        label_54 = new QLabel(groupBox_14);
        label_54->setObjectName("label_54");
        label_54->setFont(font1);
        label_54->setText(QString::fromUtf8("%<ERR>"));
        label_54->setTextFormat(Qt::PlainText);
        label_54->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(10, QFormLayout::ItemRole::LabelRole, label_54);

        label_57 = new QLabel(groupBox_14);
        label_57->setObjectName("label_57");
        label_57->setFont(font1);

        formLayout_3->setWidget(10, QFormLayout::ItemRole::FieldRole, label_57);

        label_7 = new QLabel(groupBox_14);
        label_7->setObjectName("label_7");
        label_7->setFont(font1);
        label_7->setText(QString::fromUtf8("%<ERF>"));
        label_7->setTextFormat(Qt::PlainText);
        label_7->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(11, QFormLayout::ItemRole::LabelRole, label_7);

        label_58 = new QLabel(groupBox_14);
        label_58->setObjectName("label_58");
        label_58->setFont(font1);

        formLayout_3->setWidget(11, QFormLayout::ItemRole::FieldRole, label_58);

        label_40 = new QLabel(groupBox_14);
        label_40->setObjectName("label_40");
        label_40->setFont(font1);
        label_40->setText(QString::fromUtf8("%<ETH>"));
        label_40->setTextFormat(Qt::PlainText);
        label_40->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_3->setWidget(12, QFormLayout::ItemRole::LabelRole, label_40);

        label_8 = new QLabel(groupBox_14);
        label_8->setObjectName("label_8");
        label_8->setFont(font1);

        formLayout_3->setWidget(12, QFormLayout::ItemRole::FieldRole, label_8);


        verticalLayout_3->addWidget(groupBox_14);

        groupBox_3 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_3->setObjectName("groupBox_3");
        groupBox_3->setFont(font);
        formLayout_6 = new QFormLayout(groupBox_3);
        formLayout_6->setObjectName("formLayout_6");
        label_9 = new QLabel(groupBox_3);
        label_9->setObjectName("label_9");
        label_9->setFont(font1);
        label_9->setText(QString::fromUtf8("%<MO>"));
        label_9->setTextFormat(Qt::PlainText);
        label_9->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        formLayout_6->setWidget(0, QFormLayout::ItemRole::LabelRole, label_9);

        label_10 = new QLabel(groupBox_3);
        label_10->setObjectName("label_10");
        label_10->setFont(font1);

        formLayout_6->setWidget(0, QFormLayout::ItemRole::FieldRole, label_10);

        label_63 = new QLabel(groupBox_3);
        label_63->setObjectName("label_63");
        label_63->setFont(font1);

        formLayout_6->setWidget(2, QFormLayout::ItemRole::FieldRole, label_63);

        label_39 = new QLabel(groupBox_3);
        label_39->setObjectName("label_39");
        label_39->setFont(font1);

        formLayout_6->setWidget(3, QFormLayout::ItemRole::FieldRole, label_39);


        verticalLayout_3->addWidget(groupBox_3);

        groupBox_4 = new QGroupBox(DlgViewFilenameParamList);
        groupBox_4->setObjectName("groupBox_4");
        groupBox_4->setFont(font);
        formLayout_8 = new QFormLayout(groupBox_4);
        formLayout_8->setObjectName("formLayout_8");
        label_65 = new QLabel(groupBox_4);
        label_65->setObjectName("label_65");
        QFont font2;
        font2.setBold(false);
        font2.setItalic(false);
        label_65->setFont(font2);
        label_65->setText(QString::fromUtf8("/"));
        label_65->setTextFormat(Qt::PlainText);

        formLayout_8->setWidget(0, QFormLayout::ItemRole::LabelRole, label_65);

        label_66 = new QLabel(groupBox_4);
        label_66->setObjectName("label_66");
        label_66->setFont(font1);

        formLayout_8->setWidget(0, QFormLayout::ItemRole::FieldRole, label_66);


        verticalLayout_3->addWidget(groupBox_4);


        horizontalLayout_5->addLayout(verticalLayout_3);


        verticalLayout->addLayout(horizontalLayout_5);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout_7 = new QFormLayout();
        formLayout_7->setObjectName("formLayout_7");
        label_28 = new QLabel(DlgViewFilenameParamList);
        label_28->setObjectName("label_28");
        label_28->setFont(font2);

        formLayout_7->setWidget(0, QFormLayout::ItemRole::LabelRole, label_28);

        label_30 = new QLabel(DlgViewFilenameParamList);
        label_30->setObjectName("label_30");
        label_30->setFont(font2);

        formLayout_7->setWidget(1, QFormLayout::ItemRole::LabelRole, label_30);

        label_29 = new QLabel(DlgViewFilenameParamList);
        label_29->setObjectName("label_29");
        label_29->setFont(font1);
        label_29->setText(QString::fromUtf8("dboxid1/123456789/DDZ_[2023-05-25]_h4axdn8_CZ-NIC_Registrace-domeny.zfo"));
        label_29->setTextFormat(Qt::PlainText);

        formLayout_7->setWidget(1, QFormLayout::ItemRole::FieldRole, label_29);

        label_27 = new QLabel(DlgViewFilenameParamList);
        label_27->setObjectName("label_27");
        label_27->setFont(font1);
        label_27->setText(QString::fromUtf8("%<AB>/<%EMID>/%<MO>_[%<AY>-%<AM>-%<AD>]_%<ESB>_%<ESN>_%<EMS>"));
        label_27->setTextFormat(Qt::PlainText);

        formLayout_7->setWidget(0, QFormLayout::ItemRole::FieldRole, label_27);


        verticalLayout_6->addLayout(formLayout_7);


        verticalLayout->addLayout(verticalLayout_6);


        retranslateUi(DlgViewFilenameParamList);

        QMetaObject::connectSlotsByName(DlgViewFilenameParamList);
    } // setupUi

    void retranslateUi(QDialog *DlgViewFilenameParamList)
    {
        DlgViewFilenameParamList->setWindowTitle(QCoreApplication::translate("DlgViewFilenameParamList", "File Name Parameter List", nullptr));
        label_42->setText(QCoreApplication::translate("DlgViewFilenameParamList", "List of available parameters and their descriptions. These parameters can be used in names of exported files. Use these parameters to pass additional information from message envelopes or related data boxes into the generated file names.", nullptr));
        groupBox_12->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Data box", nullptr));
        label_37->setText(QCoreApplication::translate("DlgViewFilenameParamList", "data-box name (separated by '-'); formerly %n", nullptr));
        label_25->setText(QCoreApplication::translate("DlgViewFilenameParamList", "data-box ID; formerly %d", nullptr));
        label_34->setText(QCoreApplication::translate("DlgViewFilenameParamList", "user ID; formerly %u", nullptr));
        groupBox_13->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Acceptance Time", nullptr));
        label_19->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance year (YYYY); formerly %Y", nullptr));
        label_68->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance year short (YY)", nullptr));
        label_20->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance month (MM); formerly %M", nullptr));
        label_21->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance day (DD); formerly %D", nullptr));
        label_31->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance hour (hh); formerly %h", nullptr));
        label_22->setText(QCoreApplication::translate("DlgViewFilenameParamList", "acceptance minute (mm); formerly %m", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Delivery Time", nullptr));
        label_44->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery year (YYYY)", nullptr));
        label_70->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery year short (YY)", nullptr));
        label_46->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery month (MM)", nullptr));
        label_48->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery day (DD)", nullptr));
        label_50->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery hour (hh)", nullptr));
        label_52->setText(QCoreApplication::translate("DlgViewFilenameParamList", "delivery minute (mm)", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Attachment File", nullptr));
        label_26->setText(QCoreApplication::translate("DlgViewFilenameParamList", "attachment filename (including suffix); formerly %f", nullptr));
        groupBox_14->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Message Envelope", nullptr));
        label_23->setText(QCoreApplication::translate("DlgViewFilenameParamList", "message ID; formerly %i", nullptr));
        label_24->setText(QCoreApplication::translate("DlgViewFilenameParamList", "message subject (separated by '-'); formerly %s", nullptr));
        label_4->setText(QCoreApplication::translate("DlgViewFilenameParamList", "sender data-box ID", nullptr));
        label_101->setText(QCoreApplication::translate("DlgViewFilenameParamList", "sender name (separated by '-'); formerly %S", nullptr));
        label_60->setText(QCoreApplication::translate("DlgViewFilenameParamList", "sender address (separated by '-')", nullptr));
        label_6->setText(QCoreApplication::translate("DlgViewFilenameParamList", "recipient data-box ID", nullptr));
        label_2->setText(QCoreApplication::translate("DlgViewFilenameParamList", "recipient name (separated by '-'); formerly %R", nullptr));
        label_62->setText(QCoreApplication::translate("DlgViewFilenameParamList", "recipient address (separated by '-')", nullptr));
        label_55->setText(QCoreApplication::translate("DlgViewFilenameParamList", "sender ref. number (separated by '-')", nullptr));
        label_56->setText(QCoreApplication::translate("DlgViewFilenameParamList", "sender file mark (separated by '-')", nullptr));
        label_57->setText(QCoreApplication::translate("DlgViewFilenameParamList", "recipient ref. number (separated by '-')", nullptr));
        label_58->setText(QCoreApplication::translate("DlgViewFilenameParamList", "recipient file mark (separated by '-')", nullptr));
        label_8->setText(QCoreApplication::translate("DlgViewFilenameParamList", "to hands (separated by '-')", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Message Orientation", nullptr));
        label_10->setText(QCoreApplication::translate("DlgViewFilenameParamList", "message orientation", nullptr));
        label_63->setText(QCoreApplication::translate("DlgViewFilenameParamList", "ODZ - sent data message", nullptr));
        label_39->setText(QCoreApplication::translate("DlgViewFilenameParamList", "DDZ - received data message", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("DlgViewFilenameParamList", "Directories and File Paths", nullptr));
        label_66->setText(QCoreApplication::translate("DlgViewFilenameParamList", "means directory separator", nullptr));
        label_28->setText(QCoreApplication::translate("DlgViewFilenameParamList", "For example:", nullptr));
        label_30->setText(QCoreApplication::translate("DlgViewFilenameParamList", "can expand to", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgViewFilenameParamList: public Ui_DlgViewFilenameParamList {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_VIEW_FILENAME_PARAM_LIST_H
