/********************************************************************************
** Form generated from reading UI file 'dlg_timestamp_expir.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TIMESTAMP_EXPIR_H
#define UI_DLG_TIMESTAMP_EXPIR_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgTimestampExpir
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *radioFromCurrent;
    QRadioButton *radioFromAll;
    QLabel *noteLabel1;
    QFrame *line_2;
    QRadioButton *radioFromDir;
    QHBoxLayout *horizontalLayout;
    QCheckBox *includeSubDir;
    QSpacerItem *horizontalSpacer;
    QLabel *noteLabel2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgTimestampExpir)
    {
        if (DlgTimestampExpir->objectName().isEmpty())
            DlgTimestampExpir->setObjectName("DlgTimestampExpir");
        DlgTimestampExpir->resize(600, 290);
        DlgTimestampExpir->setMinimumSize(QSize(600, 290));
        verticalLayout_2 = new QVBoxLayout(DlgTimestampExpir);
        verticalLayout_2->setObjectName("verticalLayout_2");
        groupBox = new QGroupBox(DlgTimestampExpir);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        radioFromCurrent = new QRadioButton(groupBox);
        radioFromCurrent->setObjectName("radioFromCurrent");
        radioFromCurrent->setChecked(true);

        verticalLayout->addWidget(radioFromCurrent);

        radioFromAll = new QRadioButton(groupBox);
        radioFromAll->setObjectName("radioFromAll");

        verticalLayout->addWidget(radioFromAll);

        noteLabel1 = new QLabel(groupBox);
        noteLabel1->setObjectName("noteLabel1");
        QFont font;
        font.setItalic(true);
        noteLabel1->setFont(font);
        noteLabel1->setWordWrap(true);

        verticalLayout->addWidget(noteLabel1);

        line_2 = new QFrame(groupBox);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        radioFromDir = new QRadioButton(groupBox);
        radioFromDir->setObjectName("radioFromDir");
        radioFromDir->setChecked(false);

        verticalLayout->addWidget(radioFromDir);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        includeSubDir = new QCheckBox(groupBox);
        includeSubDir->setObjectName("includeSubDir");
        includeSubDir->setEnabled(false);
        includeSubDir->setChecked(false);

        horizontalLayout->addWidget(includeSubDir);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        noteLabel2 = new QLabel(groupBox);
        noteLabel2->setObjectName("noteLabel2");
        noteLabel2->setFont(font);
        noteLabel2->setWordWrap(true);

        verticalLayout->addWidget(noteLabel2);


        verticalLayout_2->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(DlgTimestampExpir);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(radioFromCurrent, radioFromAll);
        QWidget::setTabOrder(radioFromAll, radioFromDir);
        QWidget::setTabOrder(radioFromDir, includeSubDir);

        retranslateUi(DlgTimestampExpir);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgTimestampExpir, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgTimestampExpir, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgTimestampExpir);
    } // setupUi

    void retranslateUi(QDialog *DlgTimestampExpir)
    {
        DlgTimestampExpir->setWindowTitle(QCoreApplication::translate("DlgTimestampExpir", "Time stamp expiration checking", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgTimestampExpir", "What do you want to do?", nullptr));
        radioFromCurrent->setText(QCoreApplication::translate("DlgTimestampExpir", "Check for expiring time stamps in current data box", nullptr));
        radioFromAll->setText(QCoreApplication::translate("DlgTimestampExpir", "Check for expiring time stamps in all data boxes", nullptr));
        noteLabel1->setText(QCoreApplication::translate("DlgTimestampExpir", "Note: Checking in all data boxes can be slow. The action cannot be aborted.", nullptr));
        radioFromDir->setText(QCoreApplication::translate("DlgTimestampExpir", "Check for expiring time stamps in ZFO files in selected directory", nullptr));
        includeSubDir->setText(QCoreApplication::translate("DlgTimestampExpir", "Include subdirectories", nullptr));
        noteLabel2->setText(QCoreApplication::translate("DlgTimestampExpir", "Note: Checking many files can be slow. The action cannot be aborted.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTimestampExpir: public Ui_DlgTimestampExpir {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TIMESTAMP_EXPIR_H
