/****************************************************************************
** Meta object code from reading C++ file 'tag_client.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/io/tag_client.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tag_client.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9TagClientE_t {};
} // unnamed namespace

template <> constexpr inline auto TagClient::qt_create_metaobjectdata<qt_meta_tag_ZN9TagClientE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagClient",
        "connectionError",
        "",
        "message",
        "connected",
        "disconnected",
        "reset",
        "tagsInserted",
        "Json::TagEntryList",
        "entries",
        "tagsUpdated",
        "tagsDeleted",
        "Json::Int64StringList",
        "ids",
        "tagAssignmentChanged",
        "Json::TagAssignmentList",
        "assignments",
        "handleSslErrors",
        "QNetworkReply*",
        "reply",
        "QList<QSslError>",
        "errors",
        "wsConnected",
        "wsSslErrors",
        "wsDisconnected",
        "wsTextMessageReceived",
        "keepAlive",
        "reconnect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectionError'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reset'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tagsInserted'
        QtMocHelpers::SignalData<void(const Json::TagEntryList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tagsUpdated'
        QtMocHelpers::SignalData<void(const Json::TagEntryList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'tagsDeleted'
        QtMocHelpers::SignalData<void(const Json::Int64StringList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'tagAssignmentChanged'
        QtMocHelpers::SignalData<void(const Json::TagAssignmentList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'handleSslErrors'
        QtMocHelpers::SlotData<void(QNetworkReply *, const QList<QSslError> &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'wsConnected'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'wsSslErrors'
        QtMocHelpers::SlotData<void(const QList<QSslError> &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'wsDisconnected'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'wsTextMessageReceived'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'keepAlive'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reconnect'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagClient, qt_meta_tag_ZN9TagClientE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagClient::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagClientE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagClientE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9TagClientE_t>.metaTypes,
    nullptr
} };

void TagClient::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagClient *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectionError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->connected(); break;
        case 2: _t->disconnected(); break;
        case 3: _t->reset(); break;
        case 4: _t->tagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 5: _t->tagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 6: _t->tagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 7: _t->tagAssignmentChanged((*reinterpret_cast<std::add_pointer_t<Json::TagAssignmentList>>(_a[1]))); break;
        case 8: _t->handleSslErrors((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[2]))); break;
        case 9: _t->wsConnected(); break;
        case 10: _t->wsSslErrors((*reinterpret_cast<std::add_pointer_t<QList<QSslError>>>(_a[1]))); break;
        case 11: _t->wsDisconnected(); break;
        case 12: _t->wsTextMessageReceived((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->keepAlive(); break;
        case 14: _t->reconnect(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QSslError> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)(const QString & )>(_a, &TagClient::connectionError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)()>(_a, &TagClient::connected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)()>(_a, &TagClient::disconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)()>(_a, &TagClient::reset, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)(const Json::TagEntryList & )>(_a, &TagClient::tagsInserted, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)(const Json::TagEntryList & )>(_a, &TagClient::tagsUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)(const Json::Int64StringList & )>(_a, &TagClient::tagsDeleted, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagClient::*)(const Json::TagAssignmentList & )>(_a, &TagClient::tagAssignmentChanged, 7))
            return;
    }
}

const QMetaObject *TagClient::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagClient::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagClientE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagClient::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void TagClient::connectionError(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TagClient::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TagClient::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TagClient::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TagClient::tagsInserted(const Json::TagEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TagClient::tagsUpdated(const Json::TagEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TagClient::tagsDeleted(const Json::Int64StringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TagClient::tagAssignmentChanged(const Json::TagAssignmentList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
