/********************************************************************************
** Form generated from reading UI file 'dlg_records_management.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_RECORDS_MANAGEMENT_H
#define UI_DLG_RECORDS_MANAGEMENT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "src/records_management/widgets/svg_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgRecordsManagement
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *urlLabel;
    QLineEdit *urlLine;
    QLabel *tokenLabel;
    QLineEdit *tokenLine;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *infoButton;
    QPushButton *eraseButton;
    QHBoxLayout *horizontalLayout_2;
    SvgView *graphicsView;
    QFormLayout *formLayout_2;
    QLabel *nameLabel;
    QLineEdit *nameLine;
    QLabel *tokenNameLabel;
    QLineEdit *tokenNameLine;
    QFrame *line;
    QCheckBox *automaticUploadCheckBox;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgRecordsManagement)
    {
        if (DlgRecordsManagement->objectName().isEmpty())
            DlgRecordsManagement->setObjectName("DlgRecordsManagement");
        DlgRecordsManagement->resize(400, 340);
        verticalLayout = new QVBoxLayout(DlgRecordsManagement);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        urlLabel = new QLabel(DlgRecordsManagement);
        urlLabel->setObjectName("urlLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, urlLabel);

        urlLine = new QLineEdit(DlgRecordsManagement);
        urlLine->setObjectName("urlLine");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, urlLine);

        tokenLabel = new QLabel(DlgRecordsManagement);
        tokenLabel->setObjectName("tokenLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, tokenLabel);

        tokenLine = new QLineEdit(DlgRecordsManagement);
        tokenLine->setObjectName("tokenLine");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, tokenLine);


        verticalLayout->addLayout(formLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        infoButton = new QPushButton(DlgRecordsManagement);
        infoButton->setObjectName("infoButton");

        horizontalLayout->addWidget(infoButton);

        eraseButton = new QPushButton(DlgRecordsManagement);
        eraseButton->setObjectName("eraseButton");

        horizontalLayout->addWidget(eraseButton);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        graphicsView = new SvgView(DlgRecordsManagement);
        graphicsView->setObjectName("graphicsView");

        horizontalLayout_2->addWidget(graphicsView);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_2->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        nameLabel = new QLabel(DlgRecordsManagement);
        nameLabel->setObjectName("nameLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, nameLabel);

        nameLine = new QLineEdit(DlgRecordsManagement);
        nameLine->setObjectName("nameLine");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, nameLine);

        tokenNameLabel = new QLabel(DlgRecordsManagement);
        tokenNameLabel->setObjectName("tokenNameLabel");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, tokenNameLabel);

        tokenNameLine = new QLineEdit(DlgRecordsManagement);
        tokenNameLine->setObjectName("tokenNameLine");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, tokenNameLine);


        horizontalLayout_2->addLayout(formLayout_2);


        verticalLayout->addLayout(horizontalLayout_2);

        line = new QFrame(DlgRecordsManagement);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        automaticUploadCheckBox = new QCheckBox(DlgRecordsManagement);
        automaticUploadCheckBox->setObjectName("automaticUploadCheckBox");

        verticalLayout->addWidget(automaticUploadCheckBox);

        buttonBox = new QDialogButtonBox(DlgRecordsManagement);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgRecordsManagement);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgRecordsManagement, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgRecordsManagement, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgRecordsManagement);
    } // setupUi

    void retranslateUi(QDialog *DlgRecordsManagement)
    {
        DlgRecordsManagement->setWindowTitle(QCoreApplication::translate("DlgRecordsManagement", "Records Management Settings", nullptr));
#if QT_CONFIG(tooltip)
        urlLabel->setToolTip(QCoreApplication::translate("DlgRecordsManagement", "Service location URL.", nullptr));
#endif // QT_CONFIG(tooltip)
        urlLabel->setText(QCoreApplication::translate("DlgRecordsManagement", "URL:", nullptr));
#if QT_CONFIG(tooltip)
        tokenLabel->setToolTip(QCoreApplication::translate("DlgRecordsManagement", "Access token into the records management service.", nullptr));
#endif // QT_CONFIG(tooltip)
        tokenLabel->setText(QCoreApplication::translate("DlgRecordsManagement", "Token:", nullptr));
        infoButton->setText(QCoreApplication::translate("DlgRecordsManagement", "Service Info", nullptr));
        eraseButton->setText(QCoreApplication::translate("DlgRecordsManagement", "Erase", nullptr));
        nameLabel->setText(QCoreApplication::translate("DlgRecordsManagement", "Service name:", nullptr));
        tokenNameLabel->setText(QCoreApplication::translate("DlgRecordsManagement", "Token name:", nullptr));
#if QT_CONFIG(tooltip)
        automaticUploadCheckBox->setToolTip(QCoreApplication::translate("DlgRecordsManagement", "When downloading complete messages and the sender or recipient\n"
"reference number or file mark of a newly downloaded message\n"
"unambiguously matches a place in the records management\n"
"hierarchy then this message will be automatically uploaded into\n"
"the matching place of the records management service.", nullptr));
#endif // QT_CONFIG(tooltip)
        automaticUploadCheckBox->setText(QCoreApplication::translate("DlgRecordsManagement", "Upload new messages to locations with matching identifiers.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgRecordsManagement: public Ui_DlgRecordsManagement {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_RECORDS_MANAGEMENT_H
