/********************************************************************************
** Form generated from reading UI file 'dlg_signature_detail.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_SIGNATURE_DETAIL_H
#define UI_DLG_SIGNATURE_DETAIL_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgSignatureDetail
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *messageSingature;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout;
    QLabel *mSignatureStatus;
    QSpacerItem *mHorizontalSpacer;
    QPushButton *mSignatureImage;
    QGroupBox *signingCertificate;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_4;
    QLabel *cStatus;
    QSpacerItem *cHorizontalSpacer;
    QPushButton *cImage;
    QHBoxLayout *horizontalLayout_5;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *showVerifyDetail;
    QWidget *verifyWidget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_3;
    QLabel *vDetail;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *showCertDetail;
    QWidget *certDetailWidget;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_10;
    QLabel *label_5;
    QLabel *cDetail;
    QSpacerItem *horizontalSpacer;
    QGroupBox *timeStamps;
    QVBoxLayout *verticalLayout_4;
    QTabWidget *tabWidget;
    QWidget *tab_0_envelope;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_3;
    QLabel *tStatus;
    QSpacerItem *tHorizontalSpacer;
    QPushButton *tImage;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_4;
    QLabel *tDetail;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgSignatureDetail)
    {
        if (DlgSignatureDetail->objectName().isEmpty())
            DlgSignatureDetail->setObjectName("DlgSignatureDetail");
        DlgSignatureDetail->resize(400, 447);
        verticalLayout_2 = new QVBoxLayout(DlgSignatureDetail);
        verticalLayout_2->setObjectName("verticalLayout_2");
        messageSingature = new QGroupBox(DlgSignatureDetail);
        messageSingature->setObjectName("messageSingature");
        QFont font;
        font.setBold(true);
        messageSingature->setFont(font);
        verticalLayout_7 = new QVBoxLayout(messageSingature);
        verticalLayout_7->setObjectName("verticalLayout_7");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        mSignatureStatus = new QLabel(messageSingature);
        mSignatureStatus->setObjectName("mSignatureStatus");
        QFont font1;
        font1.setBold(false);
        mSignatureStatus->setFont(font1);

        horizontalLayout->addWidget(mSignatureStatus);

        mHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(mHorizontalSpacer);

        mSignatureImage = new QPushButton(messageSingature);
        mSignatureImage->setObjectName("mSignatureImage");
        mSignatureImage->setEnabled(true);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/png_app_logo/16x16/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        mSignatureImage->setIcon(icon);

        horizontalLayout->addWidget(mSignatureImage);


        verticalLayout_7->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(messageSingature);

        signingCertificate = new QGroupBox(DlgSignatureDetail);
        signingCertificate->setObjectName("signingCertificate");
        signingCertificate->setFont(font);
        verticalLayout_6 = new QVBoxLayout(signingCertificate);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        cStatus = new QLabel(signingCertificate);
        cStatus->setObjectName("cStatus");
        cStatus->setFont(font1);

        horizontalLayout_4->addWidget(cStatus);

        cHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(cHorizontalSpacer);

        cImage = new QPushButton(signingCertificate);
        cImage->setObjectName("cImage");
        cImage->setIcon(icon);

        horizontalLayout_4->addWidget(cImage);


        verticalLayout_6->addLayout(horizontalLayout_4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        showVerifyDetail = new QCheckBox(signingCertificate);
        showVerifyDetail->setObjectName("showVerifyDetail");
        showVerifyDetail->setFont(font1);

        verticalLayout_5->addWidget(showVerifyDetail);

        verifyWidget = new QWidget(signingCertificate);
        verifyWidget->setObjectName("verifyWidget");
        verticalLayout = new QVBoxLayout(verifyWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_3 = new QLabel(verifyWidget);
        label_3->setObjectName("label_3");
        label_3->setMinimumSize(QSize(2, 0));
        label_3->setFont(font1);

        horizontalLayout_7->addWidget(label_3);

        vDetail = new QLabel(verifyWidget);
        vDetail->setObjectName("vDetail");
        vDetail->setFont(font1);

        horizontalLayout_7->addWidget(vDetail);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_4);


        verticalLayout->addLayout(horizontalLayout_7);


        verticalLayout_5->addWidget(verifyWidget);

        showCertDetail = new QCheckBox(signingCertificate);
        showCertDetail->setObjectName("showCertDetail");
        showCertDetail->setFont(font1);

        verticalLayout_5->addWidget(showCertDetail);

        certDetailWidget = new QWidget(signingCertificate);
        certDetailWidget->setObjectName("certDetailWidget");
        verticalLayout_3 = new QVBoxLayout(certDetailWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        label_5 = new QLabel(certDetailWidget);
        label_5->setObjectName("label_5");
        label_5->setMinimumSize(QSize(2, 0));
        label_5->setFont(font1);

        horizontalLayout_10->addWidget(label_5);

        cDetail = new QLabel(certDetailWidget);
        cDetail->setObjectName("cDetail");
        cDetail->setFont(font1);

        horizontalLayout_10->addWidget(cDetail);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_10);


        verticalLayout_5->addWidget(certDetailWidget);


        horizontalLayout_5->addLayout(verticalLayout_5);


        verticalLayout_6->addLayout(horizontalLayout_5);


        verticalLayout_2->addWidget(signingCertificate);

        timeStamps = new QGroupBox(DlgSignatureDetail);
        timeStamps->setObjectName("timeStamps");
        timeStamps->setFont(font);
        verticalLayout_4 = new QVBoxLayout(timeStamps);
        verticalLayout_4->setObjectName("verticalLayout_4");
        tabWidget = new QTabWidget(timeStamps);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setFont(font1);
        tab_0_envelope = new QWidget();
        tab_0_envelope->setObjectName("tab_0_envelope");
        verticalLayout_8 = new QVBoxLayout(tab_0_envelope);
        verticalLayout_8->setObjectName("verticalLayout_8");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        tStatus = new QLabel(tab_0_envelope);
        tStatus->setObjectName("tStatus");
        tStatus->setFont(font1);

        horizontalLayout_3->addWidget(tStatus);

        tHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(tHorizontalSpacer);

        tImage = new QPushButton(tab_0_envelope);
        tImage->setObjectName("tImage");
        tImage->setIcon(icon);

        horizontalLayout_3->addWidget(tImage);


        verticalLayout_8->addLayout(horizontalLayout_3);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label_4 = new QLabel(tab_0_envelope);
        label_4->setObjectName("label_4");
        label_4->setMinimumSize(QSize(15, 0));
        label_4->setFont(font1);

        horizontalLayout_6->addWidget(label_4);

        tDetail = new QLabel(tab_0_envelope);
        tDetail->setObjectName("tDetail");
        tDetail->setFont(font1);

        horizontalLayout_6->addWidget(tDetail);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_3);


        verticalLayout_8->addLayout(horizontalLayout_6);

        tabWidget->addTab(tab_0_envelope, QString());

        verticalLayout_4->addWidget(tabWidget);


        verticalLayout_2->addWidget(timeStamps);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgSignatureDetail);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(showVerifyDetail, showCertDetail);

        retranslateUi(DlgSignatureDetail);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgSignatureDetail, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgSignatureDetail, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgSignatureDetail);
    } // setupUi

    void retranslateUi(QDialog *DlgSignatureDetail)
    {
        DlgSignatureDetail->setWindowTitle(QCoreApplication::translate("DlgSignatureDetail", "Signature detail for current message", nullptr));
        messageSingature->setTitle(QCoreApplication::translate("DlgSignatureDetail", "Message signature", nullptr));
        mSignatureStatus->setText(QCoreApplication::translate("DlgSignatureDetail", "n/a", nullptr));
        mSignatureImage->setText(QString());
        signingCertificate->setTitle(QCoreApplication::translate("DlgSignatureDetail", "Signing certificate", nullptr));
        cStatus->setText(QCoreApplication::translate("DlgSignatureDetail", "n/a", nullptr));
        cImage->setText(QString());
        showVerifyDetail->setText(QCoreApplication::translate("DlgSignatureDetail", "Show verification detail", nullptr));
        label_3->setText(QString());
        vDetail->setText(QCoreApplication::translate("DlgSignatureDetail", "n/a", nullptr));
        showCertDetail->setText(QCoreApplication::translate("DlgSignatureDetail", "Show certificate detail", nullptr));
        label_5->setText(QString());
        cDetail->setText(QCoreApplication::translate("DlgSignatureDetail", "n/a", nullptr));
        timeStamps->setTitle(QCoreApplication::translate("DlgSignatureDetail", "Timestamps", nullptr));
        tStatus->setText(QCoreApplication::translate("DlgSignatureDetail", "n/a", nullptr));
        tImage->setText(QString());
        label_4->setText(QString());
        tDetail->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_0_envelope), QCoreApplication::translate("DlgSignatureDetail", "Envelope", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgSignatureDetail: public Ui_DlgSignatureDetail {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_SIGNATURE_DETAIL_H
