/****************************************************************************
** Meta object code from reading C++ file 'datovka.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/gui/datovka.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'datovka.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "haltRequested",
        "",
        "viewAskCloseDlg",
        "closedAskCloseDlg",
        "result",
        "synchroniseAllAccounts",
        "viewAddAccountDlg",
        "viewRemoveAccountDlg",
        "viewManageAccountsDlg",
        "viewImportDatabaseDlg",
        "viewDataBackupDlg",
        "automaticNotification",
        "viewDataRestoreDlg",
        "viewProxySettingsDlg",
        "viewRecordsManagementDlg",
        "getStoredMsgInfoFromRecordsManagement",
        "checkUploadMessagesIntoRecordsManagement",
        "editTagContSettings",
        "viewPreferencesDlg",
        "synchroniseSelectedAccount",
        "synchroniseSelectedAccountReceived",
        "synchroniseSelectedAccountSent",
        "viewCreateMessageDlg",
        "viewCreateGovRequestDlg",
        "accountMarkReceivedRead",
        "accountMarkReceivedUnread",
        "accountMarkReceivedUnsettled",
        "accountMarkReceivedInProgress",
        "accountMarkReceivedSettled",
        "viewChangePasswordDlg",
        "viewManageSelectedAccountDlg",
        "moveSelectedAccountUp",
        "moveSelectedAccountDown",
        "viewChangeDataDirectoryDlg",
        "viewImportMessagesFromDatabaseDlg",
        "viewImportMessagesFromZfosDlg",
        "viewVacuumMessageDatabaseDlg",
        "viewSplitMessageDatabaseByYearsDlg",
        "downloadSelectedMessageAttachments",
        "viewReplyMessageDlg",
        "viewForwardMessageDlg",
        "viewCreateMessageFromMessageDlg",
        "viewSignatureDetailsDlg",
        "viewAuthenticateMessageDlg",
        "openSelectedMessageExternally",
        "openAcceptanceInfoExternally",
        "viewSendSelectedMessageToRecordsManagementDlg",
        "exportSelectedMessageZfos",
        "exportSelectedAcceptanceInfoZfos",
        "exportSelectedAcceptanceInfoPdfs",
        "exportSelectedEnvelopePdfs",
        "exportSelectedEnvelopeAttachments",
        "createEmailWithZfos",
        "createEmailWithAllAttachments",
        "createEmailContentSelection",
        "viewSaveAllAttachmentsDlg",
        "viewSaveSelectedAttachmentsDlgs",
        "openSelectedAttachment",
        "QModelIndex",
        "index",
        "deleteSelectedMessages",
        "viewFindDataBoxDlg",
        "viewAuthenticateMessageFileDlg",
        "viewMessageFromZfoDlg",
        "viewExportCorrespondenceOverviewDlg",
        "viewTimestampExpirationDlg",
        "viewTimestampRenewalDlg",
        "closeTimestampRenewalDlg",
        "viewMessageSearchDlg",
        "closeMessageSearchDlg",
        "viewAvailableTagsDlg",
        "viewConvertImportFromMobileAppDlg",
        "viewConvertDataForMobileAppDlg",
        "repairDatabaseFiles",
        "repairDatabaseFile",
        "QUrl",
        "link",
        "toolBarSettingsToggled",
        "checked",
        "topPanelBarsToggled",
        "viewCustomiseToolBar",
        "viewCustomiseAccountBar",
        "viewCustomiseMessageBar",
        "viewCustomiseDetailBar",
        "viewCustomiseAttachmentBar",
        "viewLogDlg",
        "closedLogDlg",
        "viewAllSettingsDlg",
        "viewAboutAppDlg",
        "openHomepageUrl",
        "openHelpUrl",
        "accountMarkRecentReceivedRead",
        "accountMarkRecentReceivedUnread",
        "accountMarkRecentReceivedUnsettled",
        "accountMarkRecentReceivedInProgress",
        "accountMarkRecentReceivedSettled",
        "accountMarkReceivedYearRead",
        "accountMarkReceivedYearUnread",
        "accountMarkReceivedYearUnsettled",
        "accountMarkReceivedYearInProgress",
        "accountMarkReceivedYearSettled",
        "messageItemsSelectedMarkRead",
        "messageItemsSelectedMarkUnread",
        "messageItemsSelectedMarkUnsettled",
        "messageItemsSelectedMarkInProgress",
        "messageItemsSelectedMarkSettled",
        "deleteSelectedDrafts",
        "showFilter",
        "showMessageFilter",
        "hideMessageFilter",
        "showDraftFilter",
        "hideDraftFilter",
        "updateActionActivation",
        "updateAccountSelection",
        "current",
        "previous",
        "updateMessageSelection",
        "QItemSelection",
        "selected",
        "deselected",
        "updateDraftSelection",
        "adaptToAccountSelection",
        "adaptToMessageSelection",
        "adaptToAttachmentSelection",
        "viewMenuBarContextMenu",
        "QPoint",
        "point",
        "viewToolBarContextMenu",
        "viewAccountToolBarContextMenu",
        "viewMessageToolBarContextMenu",
        "viewDetailToolBarContextMenu",
        "viewAttachmentToolBarContextMenu",
        "viewTagAssignmentInterface",
        "viewTagAssignmentDialogue",
        "viewTagAssignmentPopup",
        "toggleTimestampTracking",
        "enable",
        "viewAccountListContextMenu",
        "viewAccountTextInfoContextMenu",
        "viewAccountTextInfoAnchorClicked",
        "viewMessageListContextMenu",
        "viewAttachmentListContextMenu",
        "viewDraftListContextMenu",
        "messageItemClicked",
        "viewSelectedMessage",
        "openSelectedDraft",
        "watchSettingsChange",
        "offerDownloadForMessages",
        "AcntIdToMsgIdListHash",
        "acntMsgIds",
        "watchChangedLocallyRead",
        "AcntId",
        "acntId",
        "QList<MsgId>",
        "msgIds",
        "read",
        "watchChangedProcessState",
        "MessageDb::MessageProcessState",
        "state",
        "watchMessageInserted",
        "MsgId",
        "msgId",
        "direction",
        "watchMessageDataDeleted",
        "watchTagsUpdated",
        "Json::TagEntryList",
        "entries",
        "watchTagsDeleted",
        "Json::Int64StringList",
        "ids",
        "watchTagAssignmentChanged",
        "Json::TagAssignmentList",
        "assignments",
        "watchTagContConnected",
        "watchTagContDisconnected",
        "watchTagContReset",
        "watchIdentificationsInserted",
        "Json::MsgId2List",
        "watchIdentificationsDeleted",
        "watchDraftsInserted",
        "DraftEntryList",
        "watchDraftsUpdated",
        "watchDraftsDeleted",
        "watchDraftCountChanged",
        "DraftCounts",
        "draftCounts",
        "watchRmMsgEntriesDeleted",
        "QList<qint64>",
        "dmIds",
        "watchRmMsgEntryUpdated",
        "dmId",
        "locations",
        "watchPrefsModification",
        "valueType",
        "name",
        "QVariant",
        "value",
        "changeTheme",
        "refreshAccountList",
        "watchOpeningFailed",
        "fileName",
        "processSingleInstanceMessages",
        "msgType",
        "msgVal",
        "backgroundWorkersFinished",
        "collectDownloadMessageStatus",
        "deliveryTime",
        "errDesc",
        "processFlags",
        "recMgmtHierarchyId",
        "collectDownloadMessageListStatus",
        "AcntIdDb",
        "rAcntIdDb",
        "sAcntId",
        "add",
        "rt",
        "rn",
        "st",
        "sn",
        "collectDownloadedUnlistedMessages",
        "unlistedMsgIds",
        "collectImportZfoStatus",
        "resultDesc",
        "collectSendMessageStatus",
        "transactId",
        "dbIDRecipient",
        "recipientName",
        "isPDZ",
        "isVodz",
        "messageItemStoreSelection",
        "messageItemStoreSelectionOnModelChange",
        "messageItemRestoreSelectionOnModelChange",
        "messageItemRestoreSelectionAfterLayoutChange",
        "messageItemFromSearchSelection",
        "deliveryYear",
        "synchroniseAllShadowAccounts",
        "sendAttachmentsEmail",
        "filterMessages",
        "text",
        "filterDrafts",
        "onTableColumnResized",
        "oldSize",
        "newSize",
        "onTableColumnHeaderSectionClicked",
        "column",
        "updateProgressBar",
        "label",
        "updateStatusBarText",
        "clearProgressBar",
        "clearStatusBar",
        "setWindowsAfterInit",
        "msgSetSelectedMessageProcessState",
        "stateIndex",
        "showImportMessageResults",
        "userName",
        "errImportList",
        "totalMsgs",
        "importedMsgs",
        "dockMenuPopulate",
        "dockMenuActionTriggerred",
        "QAction*",
        "action",
        "showStatusTextWithTimeout",
        "qStr"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'haltRequested'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewAskCloseDlg'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closedAskCloseDlg'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'synchroniseAllAccounts'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewAddAccountDlg'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewRemoveAccountDlg'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewManageAccountsDlg'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewImportDatabaseDlg'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewDataBackupDlg'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Slot 'viewDataBackupDlg'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'viewDataRestoreDlg'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewProxySettingsDlg'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewRecordsManagementDlg'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getStoredMsgInfoFromRecordsManagement'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkUploadMessagesIntoRecordsManagement'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editTagContSettings'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewPreferencesDlg'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'synchroniseSelectedAccount'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'synchroniseSelectedAccountReceived'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'synchroniseSelectedAccountSent'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCreateMessageDlg'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCreateGovRequestDlg'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedRead'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedUnread'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedUnsettled'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedInProgress'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedSettled'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewChangePasswordDlg'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewManageSelectedAccountDlg'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveSelectedAccountUp'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveSelectedAccountDown'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewChangeDataDirectoryDlg'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewImportMessagesFromDatabaseDlg'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewImportMessagesFromZfosDlg'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewVacuumMessageDatabaseDlg'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSplitMessageDatabaseByYearsDlg'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadSelectedMessageAttachments'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewReplyMessageDlg'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewForwardMessageDlg'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCreateMessageFromMessageDlg'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSignatureDetailsDlg'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewAuthenticateMessageDlg'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openSelectedMessageExternally'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openAcceptanceInfoExternally'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSendSelectedMessageToRecordsManagementDlg'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedMessageZfos'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedAcceptanceInfoZfos'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedAcceptanceInfoPdfs'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedEnvelopePdfs'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedEnvelopeAttachments'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailWithZfos'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailWithAllAttachments'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailContentSelection'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSaveAllAttachmentsDlg'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSaveSelectedAttachmentsDlgs'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openSelectedAttachment'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'openSelectedAttachment'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'deleteSelectedMessages'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewFindDataBoxDlg'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewAuthenticateMessageFileDlg'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewMessageFromZfoDlg'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewExportCorrespondenceOverviewDlg'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewTimestampExpirationDlg'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewTimestampRenewalDlg'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeTimestampRenewalDlg'
        QtMocHelpers::SlotData<void(int)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'viewMessageSearchDlg'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeMessageSearchDlg'
        QtMocHelpers::SlotData<void(int)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'viewAvailableTagsDlg'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewConvertImportFromMobileAppDlg'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewConvertDataForMobileAppDlg'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'repairDatabaseFiles'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'repairDatabaseFile'
        QtMocHelpers::SlotData<void(const QUrl &)>(75, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 76, 77 },
        }}),
        // Slot 'toolBarSettingsToggled'
        QtMocHelpers::SlotData<void(bool)>(78, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 79 },
        }}),
        // Slot 'topPanelBarsToggled'
        QtMocHelpers::SlotData<void(bool)>(80, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 79 },
        }}),
        // Slot 'viewCustomiseToolBar'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCustomiseAccountBar'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCustomiseMessageBar'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCustomiseDetailBar'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewCustomiseAttachmentBar'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewLogDlg'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closedLogDlg'
        QtMocHelpers::SlotData<void(int)>(87, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'viewAllSettingsDlg'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewAboutAppDlg'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openHomepageUrl'
        QtMocHelpers::SlotData<void() const>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openHelpUrl'
        QtMocHelpers::SlotData<void() const>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkRecentReceivedRead'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkRecentReceivedUnread'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkRecentReceivedUnsettled'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkRecentReceivedInProgress'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkRecentReceivedSettled'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedYearRead'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedYearUnread'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedYearUnsettled'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedYearInProgress'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'accountMarkReceivedYearSettled'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemsSelectedMarkRead'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemsSelectedMarkUnread'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemsSelectedMarkUnsettled'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemsSelectedMarkInProgress'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemsSelectedMarkSettled'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteSelectedDrafts'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showFilter'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showMessageFilter'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideMessageFilter'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showDraftFilter'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideDraftFilter'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateActionActivation'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateAccountSelection'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(114, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 115 }, { 0x80000000 | 59, 116 },
        }}),
        // Slot 'updateAccountSelection'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(114, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 59, 115 },
        }}),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(117, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 118, 119 }, { 0x80000000 | 118, 120 },
        }}),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(117, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 118, 119 },
        }}),
        // Slot 'updateDraftSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(121, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 118, 119 }, { 0x80000000 | 118, 120 },
        }}),
        // Slot 'updateDraftSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(121, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 118, 119 },
        }}),
        // Slot 'adaptToAccountSelection'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'adaptToMessageSelection'
        QtMocHelpers::SlotData<void()>(123, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'adaptToAttachmentSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(124, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 118, 119 }, { 0x80000000 | 118, 120 },
        }}),
        // Slot 'adaptToAttachmentSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(124, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 118, 119 },
        }}),
        // Slot 'viewMenuBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(125, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewToolBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(128, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewAccountToolBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(129, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewMessageToolBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(130, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewDetailToolBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(131, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewAttachmentToolBarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(132, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewTagAssignmentInterface'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewTagAssignmentDialogue'
        QtMocHelpers::SlotData<void()>(134, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewTagAssignmentPopup'
        QtMocHelpers::SlotData<void()>(135, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleTimestampTracking'
        QtMocHelpers::SlotData<void(bool)>(136, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 137 },
        }}),
        // Slot 'viewAccountListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(138, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewAccountTextInfoContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(139, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewAccountTextInfoAnchorClicked'
        QtMocHelpers::SlotData<void(const QUrl &)>(140, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 76, 77 },
        }}),
        // Slot 'viewMessageListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(141, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewAttachmentListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(142, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'viewDraftListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(143, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 126, 127 },
        }}),
        // Slot 'messageItemClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(144, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 59, 60 },
        }}),
        // Slot 'viewSelectedMessage'
        QtMocHelpers::SlotData<void()>(145, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openSelectedDraft'
        QtMocHelpers::SlotData<void()>(146, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchSettingsChange'
        QtMocHelpers::SlotData<void()>(147, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'offerDownloadForMessages'
        QtMocHelpers::SlotData<void(const AcntIdToMsgIdListHash &)>(148, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 149, 150 },
        }}),
        // Slot 'watchChangedLocallyRead'
        QtMocHelpers::SlotData<void(const AcntId &, const QList<MsgId> &, bool)>(151, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { 0x80000000 | 154, 155 }, { QMetaType::Bool, 156 },
        }}),
        // Slot 'watchChangedProcessState'
        QtMocHelpers::SlotData<void(const AcntId &, const QList<MsgId> &, MessageDb::MessageProcessState)>(157, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { 0x80000000 | 154, 155 }, { 0x80000000 | 158, 159 },
        }}),
        // Slot 'watchMessageInserted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &, int)>(160, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { 0x80000000 | 161, 162 }, { QMetaType::Int, 163 },
        }}),
        // Slot 'watchMessageDataDeleted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &)>(164, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { 0x80000000 | 161, 162 },
        }}),
        // Slot 'watchTagsUpdated'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(165, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 166, 167 },
        }}),
        // Slot 'watchTagsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(168, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 169, 170 },
        }}),
        // Slot 'watchTagAssignmentChanged'
        QtMocHelpers::SlotData<void(const Json::TagAssignmentList &)>(171, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 172, 173 },
        }}),
        // Slot 'watchTagContConnected'
        QtMocHelpers::SlotData<void()>(174, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchTagContDisconnected'
        QtMocHelpers::SlotData<void()>(175, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchTagContReset'
        QtMocHelpers::SlotData<void()>(176, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchIdentificationsInserted'
        QtMocHelpers::SlotData<void(const Json::MsgId2List &, const AcntId &)>(177, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 178, 155 }, { 0x80000000 | 152, 153 },
        }}),
        // Slot 'watchIdentificationsDeleted'
        QtMocHelpers::SlotData<void(const Json::MsgId2List &)>(179, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 178, 155 },
        }}),
        // Slot 'watchDraftsInserted'
        QtMocHelpers::SlotData<void(const DraftEntryList &)>(180, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 181, 167 },
        }}),
        // Slot 'watchDraftsUpdated'
        QtMocHelpers::SlotData<void(const DraftEntryList &)>(182, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 181, 167 },
        }}),
        // Slot 'watchDraftsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(183, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 169, 170 },
        }}),
        // Slot 'watchDraftCountChanged'
        QtMocHelpers::SlotData<void(const DraftCounts &)>(184, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 185, 186 },
        }}),
        // Slot 'watchRmMsgEntriesDeleted'
        QtMocHelpers::SlotData<void(const QList<qint64> &)>(187, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 188, 189 },
        }}),
        // Slot 'watchRmMsgEntryUpdated'
        QtMocHelpers::SlotData<void(qint64, const QStringList &)>(190, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 191 }, { QMetaType::QStringList, 192 },
        }}),
        // Slot 'watchPrefsModification'
        QtMocHelpers::SlotData<void(int, const QString &, const QVariant &)>(193, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 194 }, { QMetaType::QString, 195 }, { 0x80000000 | 196, 197 },
        }}),
        // Slot 'changeTheme'
        QtMocHelpers::SlotData<void(int)>(198, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 60 },
        }}),
        // Slot 'refreshAccountList'
        QtMocHelpers::SlotData<void(const AcntId &)>(199, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 },
        }}),
        // Slot 'watchOpeningFailed'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &)>(200, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { QMetaType::QString, 201 },
        }}),
        // Slot 'processSingleInstanceMessages'
        QtMocHelpers::SlotData<void(int, const QString &)>(202, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 203 }, { QMetaType::QString, 204 },
        }}),
        // Slot 'backgroundWorkersFinished'
        QtMocHelpers::SlotData<void()>(205, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'collectDownloadMessageStatus'
        QtMocHelpers::SlotData<void(const AcntId &, qint64, const QDateTime &, int, const QString &, int, const QString &)>(206, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { QMetaType::LongLong, 162 }, { QMetaType::QDateTime, 207 }, { QMetaType::Int, 5 },
            { QMetaType::QString, 208 }, { QMetaType::Int, 209 }, { QMetaType::QString, 210 },
        }}),
        // Slot 'collectDownloadMessageListStatus'
        QtMocHelpers::SlotData<void(const AcntIdDb &, const AcntId &, int, int, const QString &, bool, int, int, int, int)>(211, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 212, 213 }, { 0x80000000 | 152, 214 }, { QMetaType::Int, 163 }, { QMetaType::Int, 5 },
            { QMetaType::QString, 208 }, { QMetaType::Bool, 215 }, { QMetaType::Int, 216 }, { QMetaType::Int, 217 },
            { QMetaType::Int, 218 }, { QMetaType::Int, 219 },
        }}),
        // Slot 'collectDownloadedUnlistedMessages'
        QtMocHelpers::SlotData<void(const AcntIdDb &, int, const QList<MsgId> &)>(220, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 212, 213 }, { QMetaType::Int, 163 }, { 0x80000000 | 154, 221 },
        }}),
        // Slot 'collectImportZfoStatus'
        QtMocHelpers::SlotData<void(const QString &, int, const QString &)>(222, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 201 }, { QMetaType::Int, 5 }, { QMetaType::QString, 223 },
        }}),
        // Slot 'collectSendMessageStatus'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, int, const QString &, const QString &, const QString &, bool, bool, qint64, int, const QString &)>(224, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { QMetaType::QString, 225 }, { QMetaType::Int, 5 }, { QMetaType::QString, 223 },
            { QMetaType::QString, 226 }, { QMetaType::QString, 227 }, { QMetaType::Bool, 228 }, { QMetaType::Bool, 229 },
            { QMetaType::LongLong, 191 }, { QMetaType::Int, 209 }, { QMetaType::QString, 210 },
        }}),
        // Slot 'messageItemStoreSelection'
        QtMocHelpers::SlotData<void(qint64)>(230, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 162 },
        }}),
        // Slot 'messageItemStoreSelectionOnModelChange'
        QtMocHelpers::SlotData<void()>(231, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemRestoreSelectionOnModelChange'
        QtMocHelpers::SlotData<void()>(232, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemRestoreSelectionAfterLayoutChange'
        QtMocHelpers::SlotData<void()>(233, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageItemFromSearchSelection'
        QtMocHelpers::SlotData<void(const AcntId &, qint64, const QString &, int)>(234, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 152, 153 }, { QMetaType::LongLong, 162 }, { QMetaType::QString, 235 }, { QMetaType::Int, 203 },
        }}),
        // Slot 'synchroniseAllShadowAccounts'
        QtMocHelpers::SlotData<void()>(236, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sendAttachmentsEmail'
        QtMocHelpers::SlotData<void()>(237, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterMessages'
        QtMocHelpers::SlotData<void(const QString &)>(238, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 239 },
        }}),
        // Slot 'filterDrafts'
        QtMocHelpers::SlotData<void(const QString &)>(240, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 239 },
        }}),
        // Slot 'onTableColumnResized'
        QtMocHelpers::SlotData<void(int, int, int)>(241, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 60 }, { QMetaType::Int, 242 }, { QMetaType::Int, 243 },
        }}),
        // Slot 'onTableColumnHeaderSectionClicked'
        QtMocHelpers::SlotData<void(int)>(244, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 245 },
        }}),
        // Slot 'updateProgressBar'
        QtMocHelpers::SlotData<void(const QString &, int)>(246, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 247 }, { QMetaType::Int, 197 },
        }}),
        // Slot 'updateStatusBarText'
        QtMocHelpers::SlotData<void(const QString &)>(248, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 239 },
        }}),
        // Slot 'clearProgressBar'
        QtMocHelpers::SlotData<void()>(249, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearStatusBar'
        QtMocHelpers::SlotData<void()>(250, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setWindowsAfterInit'
        QtMocHelpers::SlotData<void()>(251, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'msgSetSelectedMessageProcessState'
        QtMocHelpers::SlotData<void(int)>(252, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 253 },
        }}),
        // Slot 'showImportMessageResults'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, int, int)>(254, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 255 }, { QMetaType::QStringList, 256 }, { QMetaType::Int, 257 }, { QMetaType::Int, 258 },
        }}),
        // Slot 'dockMenuPopulate'
        QtMocHelpers::SlotData<void()>(259, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dockMenuActionTriggerred'
        QtMocHelpers::SlotData<void(QAction *)>(260, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 261, 262 },
        }}),
        // Slot 'showStatusTextWithTimeout'
        QtMocHelpers::SlotData<void(const QString &)>(263, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 264 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->haltRequested(); break;
        case 1: _t->viewAskCloseDlg(); break;
        case 2: _t->closedAskCloseDlg((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->synchroniseAllAccounts(); break;
        case 4: _t->viewAddAccountDlg(); break;
        case 5: _t->viewRemoveAccountDlg(); break;
        case 6: _t->viewManageAccountsDlg(); break;
        case 7: _t->viewImportDatabaseDlg(); break;
        case 8: _t->viewDataBackupDlg((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->viewDataBackupDlg(); break;
        case 10: _t->viewDataRestoreDlg(); break;
        case 11: _t->viewProxySettingsDlg(); break;
        case 12: _t->viewRecordsManagementDlg(); break;
        case 13: _t->getStoredMsgInfoFromRecordsManagement(); break;
        case 14: _t->checkUploadMessagesIntoRecordsManagement(); break;
        case 15: _t->editTagContSettings(); break;
        case 16: _t->viewPreferencesDlg(); break;
        case 17: _t->synchroniseSelectedAccount(); break;
        case 18: _t->synchroniseSelectedAccountReceived(); break;
        case 19: _t->synchroniseSelectedAccountSent(); break;
        case 20: _t->viewCreateMessageDlg(); break;
        case 21: _t->viewCreateGovRequestDlg(); break;
        case 22: _t->accountMarkReceivedRead(); break;
        case 23: _t->accountMarkReceivedUnread(); break;
        case 24: _t->accountMarkReceivedUnsettled(); break;
        case 25: _t->accountMarkReceivedInProgress(); break;
        case 26: _t->accountMarkReceivedSettled(); break;
        case 27: _t->viewChangePasswordDlg(); break;
        case 28: _t->viewManageSelectedAccountDlg(); break;
        case 29: _t->moveSelectedAccountUp(); break;
        case 30: _t->moveSelectedAccountDown(); break;
        case 31: _t->viewChangeDataDirectoryDlg(); break;
        case 32: _t->viewImportMessagesFromDatabaseDlg(); break;
        case 33: _t->viewImportMessagesFromZfosDlg(); break;
        case 34: _t->viewVacuumMessageDatabaseDlg(); break;
        case 35: _t->viewSplitMessageDatabaseByYearsDlg(); break;
        case 36: _t->downloadSelectedMessageAttachments(); break;
        case 37: _t->viewReplyMessageDlg(); break;
        case 38: _t->viewForwardMessageDlg(); break;
        case 39: _t->viewCreateMessageFromMessageDlg(); break;
        case 40: _t->viewSignatureDetailsDlg(); break;
        case 41: _t->viewAuthenticateMessageDlg(); break;
        case 42: _t->openSelectedMessageExternally(); break;
        case 43: _t->openAcceptanceInfoExternally(); break;
        case 44: _t->viewSendSelectedMessageToRecordsManagementDlg(); break;
        case 45: _t->exportSelectedMessageZfos(); break;
        case 46: _t->exportSelectedAcceptanceInfoZfos(); break;
        case 47: _t->exportSelectedAcceptanceInfoPdfs(); break;
        case 48: _t->exportSelectedEnvelopePdfs(); break;
        case 49: _t->exportSelectedEnvelopeAttachments(); break;
        case 50: _t->createEmailWithZfos(); break;
        case 51: _t->createEmailWithAllAttachments(); break;
        case 52: _t->createEmailContentSelection(); break;
        case 53: _t->viewSaveAllAttachmentsDlg(); break;
        case 54: _t->viewSaveSelectedAttachmentsDlgs(); break;
        case 55: _t->openSelectedAttachment((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 56: _t->openSelectedAttachment(); break;
        case 57: _t->deleteSelectedMessages(); break;
        case 58: _t->viewFindDataBoxDlg(); break;
        case 59: _t->viewAuthenticateMessageFileDlg(); break;
        case 60: _t->viewMessageFromZfoDlg(); break;
        case 61: _t->viewExportCorrespondenceOverviewDlg(); break;
        case 62: _t->viewTimestampExpirationDlg(); break;
        case 63: _t->viewTimestampRenewalDlg(); break;
        case 64: _t->closeTimestampRenewalDlg((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 65: _t->viewMessageSearchDlg(); break;
        case 66: _t->closeMessageSearchDlg((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 67: _t->viewAvailableTagsDlg(); break;
        case 68: _t->viewConvertImportFromMobileAppDlg(); break;
        case 69: _t->viewConvertDataForMobileAppDlg(); break;
        case 70: _t->repairDatabaseFiles(); break;
        case 71: _t->repairDatabaseFile((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 72: _t->toolBarSettingsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 73: _t->topPanelBarsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 74: _t->viewCustomiseToolBar(); break;
        case 75: _t->viewCustomiseAccountBar(); break;
        case 76: _t->viewCustomiseMessageBar(); break;
        case 77: _t->viewCustomiseDetailBar(); break;
        case 78: _t->viewCustomiseAttachmentBar(); break;
        case 79: _t->viewLogDlg(); break;
        case 80: _t->closedLogDlg((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 81: _t->viewAllSettingsDlg(); break;
        case 82: _t->viewAboutAppDlg(); break;
        case 83: _t->openHomepageUrl(); break;
        case 84: _t->openHelpUrl(); break;
        case 85: _t->accountMarkRecentReceivedRead(); break;
        case 86: _t->accountMarkRecentReceivedUnread(); break;
        case 87: _t->accountMarkRecentReceivedUnsettled(); break;
        case 88: _t->accountMarkRecentReceivedInProgress(); break;
        case 89: _t->accountMarkRecentReceivedSettled(); break;
        case 90: _t->accountMarkReceivedYearRead(); break;
        case 91: _t->accountMarkReceivedYearUnread(); break;
        case 92: _t->accountMarkReceivedYearUnsettled(); break;
        case 93: _t->accountMarkReceivedYearInProgress(); break;
        case 94: _t->accountMarkReceivedYearSettled(); break;
        case 95: _t->messageItemsSelectedMarkRead(); break;
        case 96: _t->messageItemsSelectedMarkUnread(); break;
        case 97: _t->messageItemsSelectedMarkUnsettled(); break;
        case 98: _t->messageItemsSelectedMarkInProgress(); break;
        case 99: _t->messageItemsSelectedMarkSettled(); break;
        case 100: _t->deleteSelectedDrafts(); break;
        case 101: _t->showFilter(); break;
        case 102: _t->showMessageFilter(); break;
        case 103: _t->hideMessageFilter(); break;
        case 104: _t->showDraftFilter(); break;
        case 105: _t->hideDraftFilter(); break;
        case 106: _t->updateActionActivation(); break;
        case 107: _t->updateAccountSelection((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 108: _t->updateAccountSelection((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 109: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 110: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 111: _t->updateDraftSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 112: _t->updateDraftSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 113: _t->adaptToAccountSelection(); break;
        case 114: _t->adaptToMessageSelection(); break;
        case 115: _t->adaptToAttachmentSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 116: _t->adaptToAttachmentSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 117: _t->viewMenuBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 118: _t->viewToolBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 119: _t->viewAccountToolBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 120: _t->viewMessageToolBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 121: _t->viewDetailToolBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 122: _t->viewAttachmentToolBarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 123: _t->viewTagAssignmentInterface(); break;
        case 124: _t->viewTagAssignmentDialogue(); break;
        case 125: _t->viewTagAssignmentPopup(); break;
        case 126: _t->toggleTimestampTracking((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 127: _t->viewAccountListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 128: _t->viewAccountTextInfoContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 129: _t->viewAccountTextInfoAnchorClicked((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 130: _t->viewMessageListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 131: _t->viewAttachmentListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 132: _t->viewDraftListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 133: _t->messageItemClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 134: _t->viewSelectedMessage(); break;
        case 135: _t->openSelectedDraft(); break;
        case 136: _t->watchSettingsChange(); break;
        case 137: _t->offerDownloadForMessages((*reinterpret_cast<std::add_pointer_t<AcntIdToMsgIdListHash>>(_a[1]))); break;
        case 138: _t->watchChangedLocallyRead((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 139: _t->watchChangedProcessState((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MessageDb::MessageProcessState>>(_a[3]))); break;
        case 140: _t->watchMessageInserted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 141: _t->watchMessageDataDeleted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2]))); break;
        case 142: _t->watchTagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 143: _t->watchTagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 144: _t->watchTagAssignmentChanged((*reinterpret_cast<std::add_pointer_t<Json::TagAssignmentList>>(_a[1]))); break;
        case 145: _t->watchTagContConnected(); break;
        case 146: _t->watchTagContDisconnected(); break;
        case 147: _t->watchTagContReset(); break;
        case 148: _t->watchIdentificationsInserted((*reinterpret_cast<std::add_pointer_t<Json::MsgId2List>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[2]))); break;
        case 149: _t->watchIdentificationsDeleted((*reinterpret_cast<std::add_pointer_t<Json::MsgId2List>>(_a[1]))); break;
        case 150: _t->watchDraftsInserted((*reinterpret_cast<std::add_pointer_t<DraftEntryList>>(_a[1]))); break;
        case 151: _t->watchDraftsUpdated((*reinterpret_cast<std::add_pointer_t<DraftEntryList>>(_a[1]))); break;
        case 152: _t->watchDraftsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 153: _t->watchDraftCountChanged((*reinterpret_cast<std::add_pointer_t<DraftCounts>>(_a[1]))); break;
        case 154: _t->watchRmMsgEntriesDeleted((*reinterpret_cast<std::add_pointer_t<QList<qint64>>>(_a[1]))); break;
        case 155: _t->watchRmMsgEntryUpdated((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 156: _t->watchPrefsModification((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3]))); break;
        case 157: _t->changeTheme((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 158: _t->refreshAccountList((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1]))); break;
        case 159: _t->watchOpeningFailed((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 160: _t->processSingleInstanceMessages((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 161: _t->backgroundWorkersFinished(); break;
        case 162: _t->collectDownloadMessageStatus((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7]))); break;
        case 163: _t->collectDownloadMessageListStatus((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10]))); break;
        case 164: _t->collectDownloadedUnlistedMessages((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[3]))); break;
        case 165: _t->collectImportZfoStatus((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 166: _t->collectSendMessageStatus((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[11]))); break;
        case 167: _t->messageItemStoreSelection((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 168: _t->messageItemStoreSelectionOnModelChange(); break;
        case 169: _t->messageItemRestoreSelectionOnModelChange(); break;
        case 170: _t->messageItemRestoreSelectionAfterLayoutChange(); break;
        case 171: _t->messageItemFromSearchSelection((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 172: _t->synchroniseAllShadowAccounts(); break;
        case 173: _t->sendAttachmentsEmail(); break;
        case 174: _t->filterMessages((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 175: _t->filterDrafts((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 176: _t->onTableColumnResized((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 177: _t->onTableColumnHeaderSectionClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 178: _t->updateProgressBar((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 179: _t->updateStatusBarText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 180: _t->clearProgressBar(); break;
        case 181: _t->clearStatusBar(); break;
        case 182: _t->setWindowsAfterInit(); break;
        case 183: _t->msgSetSelectedMessageProcessState((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 184: _t->showImportMessageResults((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 185: _t->dockMenuPopulate(); break;
        case 186: _t->dockMenuActionTriggerred((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 187: _t->showStatusTextWithTimeout((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 109:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 110:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 111:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 112:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 115:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 116:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 137:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdToMsgIdListHash >(); break;
            }
            break;
        case 138:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 139:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageDb::MessageProcessState >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 140:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 141:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 142:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 143:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::Int64StringList >(); break;
            }
            break;
        case 144:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagAssignmentList >(); break;
            }
            break;
        case 148:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::MsgId2List >(); break;
            }
            break;
        case 149:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::MsgId2List >(); break;
            }
            break;
        case 150:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftEntryList >(); break;
            }
            break;
        case 151:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftEntryList >(); break;
            }
            break;
        case 152:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::Int64StringList >(); break;
            }
            break;
        case 153:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< DraftCounts >(); break;
            }
            break;
        case 154:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<qint64> >(); break;
            }
            break;
        case 158:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 159:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 162:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 163:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            }
            break;
        case 164:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdDb >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 166:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 171:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 186:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 188)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 188;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 188)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 188;
    }
    return _id;
}
QT_WARNING_POP
