/********************************************************************************
** Form generated from reading UI file 'dlg_import_zfo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_IMPORT_ZFO_H
#define UI_DLG_IMPORT_ZFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgImportZFO
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *infoLabel;
    QFrame *line;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *allZFO;
    QRadioButton *messageZFO;
    QRadioButton *deliveryZFO;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *radioImportSelected;
    QRadioButton *radioImportAll;
    QHBoxLayout *horizontalLayout;
    QCheckBox *includeSubDir;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *checkOnServer;
    QSpacerItem *horizontalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgImportZFO)
    {
        if (DlgImportZFO->objectName().isEmpty())
            DlgImportZFO->setObjectName("DlgImportZFO");
        DlgImportZFO->resize(415, 460);
        DlgImportZFO->setMinimumSize(QSize(415, 395));
        verticalLayout_2 = new QVBoxLayout(DlgImportZFO);
        verticalLayout_2->setObjectName("verticalLayout_2");
        infoLabel = new QLabel(DlgImportZFO);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        infoLabel->setWordWrap(true);

        verticalLayout_2->addWidget(infoLabel);

        line = new QFrame(DlgImportZFO);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        groupBox_2 = new QGroupBox(DlgImportZFO);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_3 = new QVBoxLayout(groupBox_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        allZFO = new QRadioButton(groupBox_2);
        allZFO->setObjectName("allZFO");
        allZFO->setChecked(true);

        verticalLayout_3->addWidget(allZFO);

        messageZFO = new QRadioButton(groupBox_2);
        messageZFO->setObjectName("messageZFO");
        messageZFO->setChecked(false);

        verticalLayout_3->addWidget(messageZFO);

        deliveryZFO = new QRadioButton(groupBox_2);
        deliveryZFO->setObjectName("deliveryZFO");

        verticalLayout_3->addWidget(deliveryZFO);


        verticalLayout_2->addWidget(groupBox_2);

        groupBox = new QGroupBox(DlgImportZFO);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        radioImportSelected = new QRadioButton(groupBox);
        radioImportSelected->setObjectName("radioImportSelected");
        radioImportSelected->setChecked(true);

        verticalLayout->addWidget(radioImportSelected);

        radioImportAll = new QRadioButton(groupBox);
        radioImportAll->setObjectName("radioImportAll");
        radioImportAll->setChecked(false);

        verticalLayout->addWidget(radioImportAll);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        includeSubDir = new QCheckBox(groupBox);
        includeSubDir->setObjectName("includeSubDir");
        includeSubDir->setEnabled(false);
        includeSubDir->setChecked(false);

        horizontalLayout->addWidget(includeSubDir);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(groupBox);

        groupBox_3 = new QGroupBox(DlgImportZFO);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_5 = new QVBoxLayout(groupBox_3);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        checkOnServer = new QCheckBox(groupBox_3);
        checkOnServer->setObjectName("checkOnServer");

        horizontalLayout_2->addWidget(checkOnServer);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_5->addLayout(horizontalLayout_2);


        verticalLayout_2->addWidget(groupBox_3);

        buttonBox = new QDialogButtonBox(DlgImportZFO);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(allZFO, messageZFO);
        QWidget::setTabOrder(messageZFO, deliveryZFO);
        QWidget::setTabOrder(deliveryZFO, radioImportSelected);
        QWidget::setTabOrder(radioImportSelected, radioImportAll);
        QWidget::setTabOrder(radioImportAll, includeSubDir);
        QWidget::setTabOrder(includeSubDir, checkOnServer);

        retranslateUi(DlgImportZFO);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgImportZFO, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgImportZFO, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgImportZFO);
    } // setupUi

    void retranslateUi(QDialog *DlgImportZFO)
    {
        DlgImportZFO->setWindowTitle(QCoreApplication::translate("DlgImportZFO", "ZFO import", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgImportZFO", "n/a", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgImportZFO", "What do you want to import?", nullptr));
        allZFO->setText(QCoreApplication::translate("DlgImportZFO", "Message and acceptance info ZFO files", nullptr));
        messageZFO->setText(QCoreApplication::translate("DlgImportZFO", "Only message ZFO files", nullptr));
        deliveryZFO->setText(QCoreApplication::translate("DlgImportZFO", "Only acceptance info ZFO files", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgImportZFO", "How do you want to import?", nullptr));
        radioImportSelected->setText(QCoreApplication::translate("DlgImportZFO", "Import selected ZFO files", nullptr));
        radioImportAll->setText(QCoreApplication::translate("DlgImportZFO", "Import all ZFO files from selected directory", nullptr));
        includeSubDir->setText(QCoreApplication::translate("DlgImportZFO", "Include subdirectories to the import", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DlgImportZFO", "Import options", nullptr));
#if QT_CONFIG(tooltip)
        checkOnServer->setToolTip(QCoreApplication::translate("DlgImportZFO", "Sending imported data to ISDS server to check them\n"
"is significantly slower but safer.\n"
"By disabling this option you may introduce invalid data\n"
"into your local database.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkOnServer->setText(QCoreApplication::translate("DlgImportZFO", "Check imported ZFO files on server", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgImportZFO: public Ui_DlgImportZFO {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_IMPORT_ZFO_H
