/********************************************************************************
** Form generated from reading UI file 'dlg_send_message.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_SEND_MESSAGE_H
#define UI_DLG_SEND_MESSAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/views/attachment_table_view.h"
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgSendMessage
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoText;
    QLabel *warningText;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout_2;
    QLabel *accountLabel;
    QComboBox *fromComboBox;
    QLabel *databoxLabel;
    QLabel *databoxInfo;
    QLabel *subjectLabel;
    QLineEdit *subjectLine;
    QFrame *line_5;
    QSplitter *splitter;
    QWidget *widget_3;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_3;
    QLabel *recipientLabel;
    QPushButton *addRecipButton;
    QPushButton *removeRecipButton;
    QFrame *line;
    QPushButton *findRecipButton;
    QFrame *line_6;
    QPushButton *enterBoxIdButton;
    QSpacerItem *recipienthorizontalSpacer;
    LoweredTableView *recipTableView;
    QFormLayout *formLayout_6;
    QCheckBox *dmPublishOwnId;
    QCheckBox *dmAllowSubstDelivery;
    QWidget *widget_2;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_4;
    QLabel *attachmentLabel;
    QPushButton *addAttachButton;
    QPushButton *removeAttachButton;
    QFrame *line_2;
    QPushButton *openAttachButton;
    QFrame *line_3;
    QPushButton *addTextButton;
    QSpacerItem *attachhorizontalSpacer_3;
    AttachmentTableView *attachTableView;
    QLabel *attachmentSizeInfo;
    QCheckBox *immDownloadCheckBox;
    QCheckBox *immRecMgmtUploadCheckBox;
    QWidget *tab_1;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout;
    QLabel *dmSenderRefNumberLabel;
    QLineEdit *dmSenderRefNumber;
    QLabel *dmSenderIdentLabel;
    QLineEdit *dmSenderIdent;
    QLabel *dmRecipientRefNumberLabel;
    QLineEdit *dmRecipientRefNumber;
    QLabel *dmRecipientIdentLabel;
    QLineEdit *dmRecipientIdent;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_4;
    QFormLayout *formLayout_4;
    QLabel *label;
    QHBoxLayout *horizontalLayout_6;
    QLineEdit *dmLegalTitleLaw;
    QLabel *slashLabel;
    QLineEdit *dmLegalTitleYear;
    QLabel *paragraphSignLabel;
    QLineEdit *dmLegalTitleSect;
    QSpacerItem *horizontalSpacer_2;
    QLabel *paragraphLabel;
    QLabel *letterLabel;
    QHBoxLayout *horizontalLayout_7;
    QLineEdit *dmLegalTitlePar;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_8;
    QLineEdit *dmLegalTitlePoint;
    QSpacerItem *horizontalSpacer_4;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_6;
    QFormLayout *formLayout_3;
    QLabel *dmToHandsLabel;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *dmToHands;
    QLabel *dmPersonalDeliveryLabel;
    QCheckBox *dmPersonalDelivery;
    QSpacerItem *verticalSpacer;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_10;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout;
    QCheckBox *aiRobIdentCheckBox;
    QCheckBox *aiFullNameCheckBox;
    QCheckBox *aiBirthDateCheckBox;
    QCheckBox *aiBirthCityCheckBox;
    QCheckBox *aiBirthCountyCheckBox;
    QCheckBox *aiFullAddressCheckBox;
    QCheckBox *aiRuianCodeCheckBox;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_9;
    QLabel *infoTextLabel1;
    QLabel *infoTextLabel2;
    QFormLayout *textFormatFormLayout;
    QLabel *textFormatLabel;
    QComboBox *textFormatComboBox;
    QPlainTextEdit *textMessageEdit;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *previewButton;
    QPushButton *viewPdfButton;
    QPushButton *appendPdfButton;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer;
    QPushButton *sendButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *DlgSendMessage)
    {
        if (DlgSendMessage->objectName().isEmpty())
            DlgSendMessage->setObjectName("DlgSendMessage");
        DlgSendMessage->setWindowModality(Qt::NonModal);
        DlgSendMessage->resize(520, 630);
        QFont font;
        font.setBold(false);
        DlgSendMessage->setFont(font);
        verticalLayout = new QVBoxLayout(DlgSendMessage);
        verticalLayout->setObjectName("verticalLayout");
        infoText = new QLabel(DlgSendMessage);
        infoText->setObjectName("infoText");
        infoText->setFont(font);
        infoText->setLayoutDirection(Qt::LeftToRight);
        infoText->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(infoText);

        warningText = new QLabel(DlgSendMessage);
        warningText->setObjectName("warningText");
        warningText->setFont(font);
        warningText->setAlignment(Qt::AlignCenter);
        warningText->setWordWrap(true);

        verticalLayout->addWidget(warningText);

        tabWidget = new QTabWidget(DlgSendMessage);
        tabWidget->setObjectName("tabWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy);
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_2->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        accountLabel = new QLabel(tab);
        accountLabel->setObjectName("accountLabel");
        QFont font1;
        font1.setBold(true);
        accountLabel->setFont(font1);
        accountLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, accountLabel);

        fromComboBox = new QComboBox(tab);
        fromComboBox->setObjectName("fromComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(fromComboBox->sizePolicy().hasHeightForWidth());
        fromComboBox->setSizePolicy(sizePolicy1);
        fromComboBox->setFont(font1);
        fromComboBox->setFocusPolicy(Qt::StrongFocus);
        fromComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContentsOnFirstShow);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, fromComboBox);

        databoxLabel = new QLabel(tab);
        databoxLabel->setObjectName("databoxLabel");
        databoxLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, databoxLabel);

        databoxInfo = new QLabel(tab);
        databoxInfo->setObjectName("databoxInfo");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::FieldRole, databoxInfo);

        subjectLabel = new QLabel(tab);
        subjectLabel->setObjectName("subjectLabel");
        subjectLabel->setFont(font1);
        subjectLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::LabelRole, subjectLabel);

        subjectLine = new QLineEdit(tab);
        subjectLine->setObjectName("subjectLine");
        subjectLine->setFont(font);

        formLayout_2->setWidget(2, QFormLayout::ItemRole::FieldRole, subjectLine);


        verticalLayout_2->addLayout(formLayout_2);

        line_5 = new QFrame(tab);
        line_5->setObjectName("line_5");
        line_5->setFrameShape(QFrame::Shape::HLine);
        line_5->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_5);

        splitter = new QSplitter(tab);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        splitter->setChildrenCollapsible(false);
        widget_3 = new QWidget(splitter);
        widget_3->setObjectName("widget_3");
        verticalLayout_8 = new QVBoxLayout(widget_3);
        verticalLayout_8->setObjectName("verticalLayout_8");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        recipientLabel = new QLabel(widget_3);
        recipientLabel->setObjectName("recipientLabel");
        recipientLabel->setFont(font1);

        horizontalLayout_3->addWidget(recipientLabel);

        addRecipButton = new QPushButton(widget_3);
        addRecipButton->setObjectName("addRecipButton");

        horizontalLayout_3->addWidget(addRecipButton);

        removeRecipButton = new QPushButton(widget_3);
        removeRecipButton->setObjectName("removeRecipButton");
        removeRecipButton->setEnabled(false);

        horizontalLayout_3->addWidget(removeRecipButton);

        line = new QFrame(widget_3);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_3->addWidget(line);

        findRecipButton = new QPushButton(widget_3);
        findRecipButton->setObjectName("findRecipButton");
        findRecipButton->setEnabled(true);

        horizontalLayout_3->addWidget(findRecipButton);

        line_6 = new QFrame(widget_3);
        line_6->setObjectName("line_6");
        line_6->setFrameShape(QFrame::Shape::VLine);
        line_6->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_3->addWidget(line_6);

        enterBoxIdButton = new QPushButton(widget_3);
        enterBoxIdButton->setObjectName("enterBoxIdButton");

        horizontalLayout_3->addWidget(enterBoxIdButton);

        recipienthorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(recipienthorizontalSpacer);


        verticalLayout_8->addLayout(horizontalLayout_3);

        recipTableView = new LoweredTableView(widget_3);
        recipTableView->setObjectName("recipTableView");
        recipTableView->setShowGrid(false);
        recipTableView->setWordWrap(false);
        recipTableView->horizontalHeader()->setHighlightSections(false);
        recipTableView->horizontalHeader()->setStretchLastSection(true);
        recipTableView->verticalHeader()->setVisible(false);

        verticalLayout_8->addWidget(recipTableView);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName("formLayout_6");
        dmPublishOwnId = new QCheckBox(widget_3);
        dmPublishOwnId->setObjectName("dmPublishOwnId");
        dmPublishOwnId->setFont(font);

        formLayout_6->setWidget(0, QFormLayout::ItemRole::LabelRole, dmPublishOwnId);

        dmAllowSubstDelivery = new QCheckBox(widget_3);
        dmAllowSubstDelivery->setObjectName("dmAllowSubstDelivery");
        dmAllowSubstDelivery->setChecked(true);

        formLayout_6->setWidget(0, QFormLayout::ItemRole::FieldRole, dmAllowSubstDelivery);


        verticalLayout_8->addLayout(formLayout_6);

        splitter->addWidget(widget_3);
        widget_2 = new QWidget(splitter);
        widget_2->setObjectName("widget_2");
        verticalLayout_7 = new QVBoxLayout(widget_2);
        verticalLayout_7->setObjectName("verticalLayout_7");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        attachmentLabel = new QLabel(widget_2);
        attachmentLabel->setObjectName("attachmentLabel");
        attachmentLabel->setFont(font1);

        horizontalLayout_4->addWidget(attachmentLabel);

        addAttachButton = new QPushButton(widget_2);
        addAttachButton->setObjectName("addAttachButton");

        horizontalLayout_4->addWidget(addAttachButton);

        removeAttachButton = new QPushButton(widget_2);
        removeAttachButton->setObjectName("removeAttachButton");
        removeAttachButton->setEnabled(false);

        horizontalLayout_4->addWidget(removeAttachButton);

        line_2 = new QFrame(widget_2);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_4->addWidget(line_2);

        openAttachButton = new QPushButton(widget_2);
        openAttachButton->setObjectName("openAttachButton");
        openAttachButton->setEnabled(false);

        horizontalLayout_4->addWidget(openAttachButton);

        line_3 = new QFrame(widget_2);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::VLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_4->addWidget(line_3);

        addTextButton = new QPushButton(widget_2);
        addTextButton->setObjectName("addTextButton");

        horizontalLayout_4->addWidget(addTextButton);

        attachhorizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(attachhorizontalSpacer_3);


        verticalLayout_7->addLayout(horizontalLayout_4);

        attachTableView = new AttachmentTableView(widget_2);
        attachTableView->setObjectName("attachTableView");
        attachTableView->setShowGrid(false);
        attachTableView->setWordWrap(false);
        attachTableView->horizontalHeader()->setHighlightSections(false);
        attachTableView->horizontalHeader()->setStretchLastSection(true);
        attachTableView->verticalHeader()->setVisible(false);

        verticalLayout_7->addWidget(attachTableView);

        attachmentSizeInfo = new QLabel(widget_2);
        attachmentSizeInfo->setObjectName("attachmentSizeInfo");
        attachmentSizeInfo->setEnabled(false);
        attachmentSizeInfo->setFont(font);
        attachmentSizeInfo->setAlignment(Qt::AlignCenter);
        attachmentSizeInfo->setWordWrap(true);

        verticalLayout_7->addWidget(attachmentSizeInfo);

        immDownloadCheckBox = new QCheckBox(widget_2);
        immDownloadCheckBox->setObjectName("immDownloadCheckBox");

        verticalLayout_7->addWidget(immDownloadCheckBox);

        immRecMgmtUploadCheckBox = new QCheckBox(widget_2);
        immRecMgmtUploadCheckBox->setObjectName("immRecMgmtUploadCheckBox");

        verticalLayout_7->addWidget(immRecMgmtUploadCheckBox);

        splitter->addWidget(widget_2);

        verticalLayout_2->addWidget(splitter);

        tabWidget->addTab(tab, QString());
        tab_1 = new QWidget();
        tab_1->setObjectName("tab_1");
        verticalLayout_3 = new QVBoxLayout(tab_1);
        verticalLayout_3->setObjectName("verticalLayout_3");
        groupBox_2 = new QGroupBox(tab_1);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_5 = new QVBoxLayout(groupBox_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dmSenderRefNumberLabel = new QLabel(groupBox_2);
        dmSenderRefNumberLabel->setObjectName("dmSenderRefNumberLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, dmSenderRefNumberLabel);

        dmSenderRefNumber = new QLineEdit(groupBox_2);
        dmSenderRefNumber->setObjectName("dmSenderRefNumber");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, dmSenderRefNumber);

        dmSenderIdentLabel = new QLabel(groupBox_2);
        dmSenderIdentLabel->setObjectName("dmSenderIdentLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, dmSenderIdentLabel);

        dmSenderIdent = new QLineEdit(groupBox_2);
        dmSenderIdent->setObjectName("dmSenderIdent");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, dmSenderIdent);

        dmRecipientRefNumberLabel = new QLabel(groupBox_2);
        dmRecipientRefNumberLabel->setObjectName("dmRecipientRefNumberLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, dmRecipientRefNumberLabel);

        dmRecipientRefNumber = new QLineEdit(groupBox_2);
        dmRecipientRefNumber->setObjectName("dmRecipientRefNumber");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, dmRecipientRefNumber);

        dmRecipientIdentLabel = new QLabel(groupBox_2);
        dmRecipientIdentLabel->setObjectName("dmRecipientIdentLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, dmRecipientIdentLabel);

        dmRecipientIdent = new QLineEdit(groupBox_2);
        dmRecipientIdent->setObjectName("dmRecipientIdent");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, dmRecipientIdent);


        verticalLayout_5->addLayout(formLayout);


        verticalLayout_3->addWidget(groupBox_2);

        groupBox = new QGroupBox(tab_1);
        groupBox->setObjectName("groupBox");
        verticalLayout_4 = new QVBoxLayout(groupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName("formLayout_4");
        formLayout_4->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_4->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label = new QLabel(groupBox);
        label->setObjectName("label");

        formLayout_4->setWidget(1, QFormLayout::ItemRole::LabelRole, label);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(1);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        dmLegalTitleLaw = new QLineEdit(groupBox);
        dmLegalTitleLaw->setObjectName("dmLegalTitleLaw");
        dmLegalTitleLaw->setMaxLength(4);

        horizontalLayout_6->addWidget(dmLegalTitleLaw);

        slashLabel = new QLabel(groupBox);
        slashLabel->setObjectName("slashLabel");

        horizontalLayout_6->addWidget(slashLabel);

        dmLegalTitleYear = new QLineEdit(groupBox);
        dmLegalTitleYear->setObjectName("dmLegalTitleYear");
        dmLegalTitleYear->setMaxLength(4);

        horizontalLayout_6->addWidget(dmLegalTitleYear);

        paragraphSignLabel = new QLabel(groupBox);
        paragraphSignLabel->setObjectName("paragraphSignLabel");

        horizontalLayout_6->addWidget(paragraphSignLabel);

        dmLegalTitleSect = new QLineEdit(groupBox);
        dmLegalTitleSect->setObjectName("dmLegalTitleSect");
        dmLegalTitleSect->setMaxLength(4);

        horizontalLayout_6->addWidget(dmLegalTitleSect);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);


        formLayout_4->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_6);

        paragraphLabel = new QLabel(groupBox);
        paragraphLabel->setObjectName("paragraphLabel");

        formLayout_4->setWidget(2, QFormLayout::ItemRole::LabelRole, paragraphLabel);

        letterLabel = new QLabel(groupBox);
        letterLabel->setObjectName("letterLabel");

        formLayout_4->setWidget(3, QFormLayout::ItemRole::LabelRole, letterLabel);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        dmLegalTitlePar = new QLineEdit(groupBox);
        dmLegalTitlePar->setObjectName("dmLegalTitlePar");
        dmLegalTitlePar->setMaxLength(2);

        horizontalLayout_7->addWidget(dmLegalTitlePar);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_3);


        formLayout_4->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_7);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        dmLegalTitlePoint = new QLineEdit(groupBox);
        dmLegalTitlePoint->setObjectName("dmLegalTitlePoint");
        dmLegalTitlePoint->setMaxLength(2);

        horizontalLayout_8->addWidget(dmLegalTitlePoint);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_4);


        formLayout_4->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout_8);


        verticalLayout_4->addLayout(formLayout_4);


        verticalLayout_3->addWidget(groupBox);

        groupBox_3 = new QGroupBox(tab_1);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_6 = new QVBoxLayout(groupBox_3);
        verticalLayout_6->setObjectName("verticalLayout_6");
        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_3->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dmToHandsLabel = new QLabel(groupBox_3);
        dmToHandsLabel->setObjectName("dmToHandsLabel");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, dmToHandsLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        dmToHands = new QLineEdit(groupBox_3);
        dmToHands->setObjectName("dmToHands");

        horizontalLayout_2->addWidget(dmToHands);

        dmPersonalDeliveryLabel = new QLabel(groupBox_3);
        dmPersonalDeliveryLabel->setObjectName("dmPersonalDeliveryLabel");

        horizontalLayout_2->addWidget(dmPersonalDeliveryLabel);

        dmPersonalDelivery = new QCheckBox(groupBox_3);
        dmPersonalDelivery->setObjectName("dmPersonalDelivery");

        horizontalLayout_2->addWidget(dmPersonalDelivery);


        formLayout_3->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);


        verticalLayout_6->addLayout(formLayout_3);


        verticalLayout_3->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabWidget->addTab(tab_1, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_10 = new QVBoxLayout(tab_2);
        verticalLayout_10->setObjectName("verticalLayout_10");
        groupBox_4 = new QGroupBox(tab_2);
        groupBox_4->setObjectName("groupBox_4");
        gridLayout = new QGridLayout(groupBox_4);
        gridLayout->setObjectName("gridLayout");
        aiRobIdentCheckBox = new QCheckBox(groupBox_4);
        aiRobIdentCheckBox->setObjectName("aiRobIdentCheckBox");

        gridLayout->addWidget(aiRobIdentCheckBox, 3, 0, 1, 1);

        aiFullNameCheckBox = new QCheckBox(groupBox_4);
        aiFullNameCheckBox->setObjectName("aiFullNameCheckBox");
        aiFullNameCheckBox->setChecked(true);

        gridLayout->addWidget(aiFullNameCheckBox, 0, 0, 1, 1);

        aiBirthDateCheckBox = new QCheckBox(groupBox_4);
        aiBirthDateCheckBox->setObjectName("aiBirthDateCheckBox");

        gridLayout->addWidget(aiBirthDateCheckBox, 0, 1, 1, 1);

        aiBirthCityCheckBox = new QCheckBox(groupBox_4);
        aiBirthCityCheckBox->setObjectName("aiBirthCityCheckBox");

        gridLayout->addWidget(aiBirthCityCheckBox, 0, 2, 1, 1);

        aiBirthCountyCheckBox = new QCheckBox(groupBox_4);
        aiBirthCountyCheckBox->setObjectName("aiBirthCountyCheckBox");

        gridLayout->addWidget(aiBirthCountyCheckBox, 2, 0, 1, 1);

        aiFullAddressCheckBox = new QCheckBox(groupBox_4);
        aiFullAddressCheckBox->setObjectName("aiFullAddressCheckBox");

        gridLayout->addWidget(aiFullAddressCheckBox, 2, 2, 1, 1);

        aiRuianCodeCheckBox = new QCheckBox(groupBox_4);
        aiRuianCodeCheckBox->setObjectName("aiRuianCodeCheckBox");

        gridLayout->addWidget(aiRuianCodeCheckBox, 2, 1, 1, 1);


        verticalLayout_10->addWidget(groupBox_4);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_2);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_9 = new QVBoxLayout(tab_3);
        verticalLayout_9->setObjectName("verticalLayout_9");
        infoTextLabel1 = new QLabel(tab_3);
        infoTextLabel1->setObjectName("infoTextLabel1");
        infoTextLabel1->setAlignment(Qt::AlignCenter);
        infoTextLabel1->setWordWrap(false);

        verticalLayout_9->addWidget(infoTextLabel1);

        infoTextLabel2 = new QLabel(tab_3);
        infoTextLabel2->setObjectName("infoTextLabel2");
        infoTextLabel2->setAlignment(Qt::AlignCenter);

        verticalLayout_9->addWidget(infoTextLabel2);

        textFormatFormLayout = new QFormLayout();
        textFormatFormLayout->setObjectName("textFormatFormLayout");
        textFormatLabel = new QLabel(tab_3);
        textFormatLabel->setObjectName("textFormatLabel");

        textFormatFormLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, textFormatLabel);

        textFormatComboBox = new QComboBox(tab_3);
        textFormatComboBox->setObjectName("textFormatComboBox");

        textFormatFormLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, textFormatComboBox);


        verticalLayout_9->addLayout(textFormatFormLayout);

        textMessageEdit = new QPlainTextEdit(tab_3);
        textMessageEdit->setObjectName("textMessageEdit");

        verticalLayout_9->addWidget(textMessageEdit);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_5);

        previewButton = new QPushButton(tab_3);
        previewButton->setObjectName("previewButton");

        horizontalLayout->addWidget(previewButton);

        viewPdfButton = new QPushButton(tab_3);
        viewPdfButton->setObjectName("viewPdfButton");

        horizontalLayout->addWidget(viewPdfButton);

        appendPdfButton = new QPushButton(tab_3);
        appendPdfButton->setObjectName("appendPdfButton");

        horizontalLayout->addWidget(appendPdfButton);


        verticalLayout_9->addLayout(horizontalLayout);

        tabWidget->addTab(tab_3, QString());

        verticalLayout->addWidget(tabWidget);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);

        sendButton = new QPushButton(DlgSendMessage);
        sendButton->setObjectName("sendButton");
        sendButton->setEnabled(false);

        horizontalLayout_5->addWidget(sendButton);

        cancelButton = new QPushButton(DlgSendMessage);
        cancelButton->setObjectName("cancelButton");

        horizontalLayout_5->addWidget(cancelButton);


        verticalLayout->addLayout(horizontalLayout_5);

        QWidget::setTabOrder(subjectLine, addRecipButton);
        QWidget::setTabOrder(addRecipButton, addAttachButton);
        QWidget::setTabOrder(addAttachButton, sendButton);
        QWidget::setTabOrder(sendButton, cancelButton);
        QWidget::setTabOrder(cancelButton, tabWidget);
        QWidget::setTabOrder(tabWidget, fromComboBox);
        QWidget::setTabOrder(fromComboBox, removeRecipButton);
        QWidget::setTabOrder(removeRecipButton, findRecipButton);
        QWidget::setTabOrder(findRecipButton, enterBoxIdButton);
        QWidget::setTabOrder(enterBoxIdButton, recipTableView);
        QWidget::setTabOrder(recipTableView, removeAttachButton);
        QWidget::setTabOrder(removeAttachButton, openAttachButton);
        QWidget::setTabOrder(openAttachButton, addTextButton);
        QWidget::setTabOrder(addTextButton, attachTableView);
        QWidget::setTabOrder(attachTableView, immDownloadCheckBox);
        QWidget::setTabOrder(immDownloadCheckBox, immRecMgmtUploadCheckBox);
        QWidget::setTabOrder(immRecMgmtUploadCheckBox, dmSenderRefNumber);
        QWidget::setTabOrder(dmSenderRefNumber, dmSenderIdent);
        QWidget::setTabOrder(dmSenderIdent, dmRecipientRefNumber);
        QWidget::setTabOrder(dmRecipientRefNumber, dmRecipientIdent);
        QWidget::setTabOrder(dmRecipientIdent, dmLegalTitleLaw);
        QWidget::setTabOrder(dmLegalTitleLaw, dmLegalTitleYear);
        QWidget::setTabOrder(dmLegalTitleYear, dmLegalTitleSect);
        QWidget::setTabOrder(dmLegalTitleSect, dmLegalTitlePar);
        QWidget::setTabOrder(dmLegalTitlePar, dmLegalTitlePoint);
        QWidget::setTabOrder(dmLegalTitlePoint, dmToHands);
        QWidget::setTabOrder(dmToHands, dmPersonalDelivery);
        QWidget::setTabOrder(dmPersonalDelivery, dmPublishOwnId);
        QWidget::setTabOrder(dmPublishOwnId, dmAllowSubstDelivery);
        QWidget::setTabOrder(dmAllowSubstDelivery, aiFullNameCheckBox);
        QWidget::setTabOrder(aiFullNameCheckBox, aiBirthDateCheckBox);
        QWidget::setTabOrder(aiBirthDateCheckBox, aiBirthCityCheckBox);
        QWidget::setTabOrder(aiBirthCityCheckBox, aiBirthCountyCheckBox);
        QWidget::setTabOrder(aiBirthCountyCheckBox, aiRuianCodeCheckBox);
        QWidget::setTabOrder(aiRuianCodeCheckBox, aiFullAddressCheckBox);
        QWidget::setTabOrder(aiFullAddressCheckBox, aiRobIdentCheckBox);
        QWidget::setTabOrder(aiRobIdentCheckBox, textFormatComboBox);
        QWidget::setTabOrder(textFormatComboBox, textMessageEdit);
        QWidget::setTabOrder(textMessageEdit, previewButton);
        QWidget::setTabOrder(previewButton, viewPdfButton);
        QWidget::setTabOrder(viewPdfButton, appendPdfButton);

        retranslateUi(DlgSendMessage);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgSendMessage);
    } // setupUi

    void retranslateUi(QDialog *DlgSendMessage)
    {
        DlgSendMessage->setWindowTitle(QCoreApplication::translate("DlgSendMessage", "Create and send message", nullptr));
        infoText->setText(QCoreApplication::translate("DlgSendMessage", "Please fill in at least the <b>subject</b>, <b>one recipient</b> and <b>one attachment</b>:", nullptr));
        warningText->setText(QCoreApplication::translate("DlgSendMessage", "n/a", nullptr));
        accountLabel->setText(QCoreApplication::translate("DlgSendMessage", "Sender:", nullptr));
        databoxLabel->setText(QCoreApplication::translate("DlgSendMessage", "Data Box:", nullptr));
        databoxInfo->setText(QString());
        subjectLabel->setText(QCoreApplication::translate("DlgSendMessage", "Subject:", nullptr));
#if QT_CONFIG(tooltip)
        subjectLine->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter subject of the message", nullptr));
#endif // QT_CONFIG(tooltip)
        recipientLabel->setText(QCoreApplication::translate("DlgSendMessage", "Recipients:", nullptr));
#if QT_CONFIG(tooltip)
        addRecipButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Add recipient from contacts selected from existing messages", nullptr));
#endif // QT_CONFIG(tooltip)
        addRecipButton->setText(QCoreApplication::translate("DlgSendMessage", "Add", nullptr));
#if QT_CONFIG(tooltip)
        removeRecipButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Remove selected recipients from the list", nullptr));
#endif // QT_CONFIG(tooltip)
        removeRecipButton->setText(QCoreApplication::translate("DlgSendMessage", "Remove", nullptr));
#if QT_CONFIG(tooltip)
        findRecipButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Find and add a recipient from ISDS server", nullptr));
#endif // QT_CONFIG(tooltip)
        findRecipButton->setText(QCoreApplication::translate("DlgSendMessage", "Find", nullptr));
#if QT_CONFIG(tooltip)
        enterBoxIdButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Allows to enter a data box identifier manually", nullptr));
#endif // QT_CONFIG(tooltip)
        enterBoxIdButton->setText(QCoreApplication::translate("DlgSendMessage", "Enter Box ID", nullptr));
#if QT_CONFIG(tooltip)
        dmPublishOwnId->setToolTip(QCoreApplication::translate("DlgSendMessage", "Provide additional information about the sender of the message.", nullptr));
#endif // QT_CONFIG(tooltip)
        dmPublishOwnId->setText(QCoreApplication::translate("DlgSendMessage", "Include sender identification", nullptr));
#if QT_CONFIG(tooltip)
        dmAllowSubstDelivery->setToolTip(QCoreApplication::translate("DlgSendMessage", "Even if the recipient did not read this message,\n"
"the message is considered to be accepted after\n"
"(currently) 10 days. This is acceptance through fiction.", nullptr));
#endif // QT_CONFIG(tooltip)
        dmAllowSubstDelivery->setText(QCoreApplication::translate("DlgSendMessage", "Allow acceptance through fiction", nullptr));
        attachmentLabel->setText(QCoreApplication::translate("DlgSendMessage", "Attachments:", nullptr));
#if QT_CONFIG(tooltip)
        addAttachButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Add a new file to the attachments", nullptr));
#endif // QT_CONFIG(tooltip)
        addAttachButton->setText(QCoreApplication::translate("DlgSendMessage", "Add", nullptr));
#if QT_CONFIG(tooltip)
        removeAttachButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Remove the selected file from attachments", nullptr));
#endif // QT_CONFIG(tooltip)
        removeAttachButton->setText(QCoreApplication::translate("DlgSendMessage", "Remove", nullptr));
#if QT_CONFIG(tooltip)
        openAttachButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Open selected file in associated application", nullptr));
#endif // QT_CONFIG(tooltip)
        openAttachButton->setText(QCoreApplication::translate("DlgSendMessage", "Open", nullptr));
#if QT_CONFIG(tooltip)
        addTextButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Add a file containing a specified text to the attachments.", nullptr));
#endif // QT_CONFIG(tooltip)
        addTextButton->setText(QCoreApplication::translate("DlgSendMessage", "Add Text", nullptr));
        attachmentSizeInfo->setText(QCoreApplication::translate("DlgSendMessage", "Attachment size is larger than 50 MB. Message cannot be sent.", nullptr));
#if QT_CONFIG(tooltip)
        immDownloadCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "The complete sent message is going to be downloaded together with\n"
"all necessary digital signatures immediately after it has been successfully\n"
"accepted by the ISDS system.\n"
"\n"
"Downloading the content of the sent message won't cause the acceptance\n"
"of any delivered message.", nullptr));
#endif // QT_CONFIG(tooltip)
        immDownloadCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "Immediately download sent message content", nullptr));
#if QT_CONFIG(tooltip)
        immRecMgmtUploadCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "Before uploading the newly sent message into the records management service\n"
"the application needs to download the newly sent message from\n"
"the ISDS server. This is because the application needs to acquire all\n"
"necessary digital signatures.", nullptr));
#endif // QT_CONFIG(tooltip)
        immRecMgmtUploadCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "Immediately upload sent message into records management service", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("DlgSendMessage", "Basic", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgSendMessage", "Reference Numbers and File Marks", nullptr));
        dmSenderRefNumberLabel->setText(QCoreApplication::translate("DlgSendMessage", "Sender (our) ref. num.:", nullptr));
#if QT_CONFIG(tooltip)
        dmSenderRefNumber->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter reference number", nullptr));
#endif // QT_CONFIG(tooltip)
        dmSenderIdentLabel->setText(QCoreApplication::translate("DlgSendMessage", "Sender (our) file mark:", nullptr));
#if QT_CONFIG(tooltip)
        dmSenderIdent->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter file mark", nullptr));
#endif // QT_CONFIG(tooltip)
        dmRecipientRefNumberLabel->setText(QCoreApplication::translate("DlgSendMessage", "Recipient (your) ref. num.:", nullptr));
#if QT_CONFIG(tooltip)
        dmRecipientRefNumber->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter reference number", nullptr));
#endif // QT_CONFIG(tooltip)
        dmRecipientIdentLabel->setText(QCoreApplication::translate("DlgSendMessage", "Recipient (your) file mark:", nullptr));
#if QT_CONFIG(tooltip)
        dmRecipientIdent->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter file mark", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("DlgSendMessage", "Mandate", nullptr));
        label->setText(QCoreApplication::translate("DlgSendMessage", "Law:", nullptr));
#if QT_CONFIG(tooltip)
        dmLegalTitleLaw->setToolTip(QCoreApplication::translate("DlgSendMessage", "Number of law", nullptr));
#endif // QT_CONFIG(tooltip)
        slashLabel->setText(QCoreApplication::translate("DlgSendMessage", "/", nullptr));
#if QT_CONFIG(tooltip)
        dmLegalTitleYear->setToolTip(QCoreApplication::translate("DlgSendMessage", "Year of law", nullptr));
#endif // QT_CONFIG(tooltip)
        paragraphSignLabel->setText(QCoreApplication::translate("DlgSendMessage", "\302\247", nullptr));
#if QT_CONFIG(tooltip)
        dmLegalTitleSect->setToolTip(QCoreApplication::translate("DlgSendMessage", "Section of law", nullptr));
#endif // QT_CONFIG(tooltip)
        paragraphLabel->setText(QCoreApplication::translate("DlgSendMessage", "Paragraph:", nullptr));
        letterLabel->setText(QCoreApplication::translate("DlgSendMessage", "Point:", nullptr));
#if QT_CONFIG(tooltip)
        dmLegalTitlePar->setToolTip(QCoreApplication::translate("DlgSendMessage", "Paragraph of the corresponding section of the law", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        dmLegalTitlePoint->setToolTip(QCoreApplication::translate("DlgSendMessage", "Point of the paragraph", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_3->setTitle(QCoreApplication::translate("DlgSendMessage", "Delivery", nullptr));
        dmToHandsLabel->setText(QCoreApplication::translate("DlgSendMessage", "To hands:", nullptr));
#if QT_CONFIG(tooltip)
        dmToHands->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enter name of person", nullptr));
#endif // QT_CONFIG(tooltip)
        dmPersonalDeliveryLabel->setText(QCoreApplication::translate("DlgSendMessage", "Personal delivery:", nullptr));
#if QT_CONFIG(tooltip)
        dmPersonalDelivery->setToolTip(QCoreApplication::translate("DlgSendMessage", "Enable personal delivery", nullptr));
#endif // QT_CONFIG(tooltip)
        dmPersonalDelivery->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_1), QCoreApplication::translate("DlgSendMessage", "Optional", nullptr));
#if QT_CONFIG(tooltip)
        groupBox_4->setToolTip(QCoreApplication::translate("DlgSendMessage", "You may select which additional data you want to provide to the recipient.", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_4->setTitle(QCoreApplication::translate("DlgSendMessage", "Select author identification data to be provided to the recipient.", nullptr));
#if QT_CONFIG(tooltip)
        aiRobIdentCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "Provide the information whether the sender's personal data\n"
"are identified within the ROB and therefore being updated.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiRobIdentCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "ROB identification flag", nullptr));
#if QT_CONFIG(tooltip)
        aiFullNameCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "Full (personal) name of the sender.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiFullNameCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "full name", nullptr));
#if QT_CONFIG(tooltip)
        aiBirthDateCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "Date of birth of the sender.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiBirthDateCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "date of birth", nullptr));
#if QT_CONFIG(tooltip)
        aiBirthCityCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "City of birth of the sender.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiBirthCityCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "city of birth", nullptr));
#if QT_CONFIG(tooltip)
        aiBirthCountyCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "County where the city of birth lies in or\n"
"name of state if birth place lies outside of the Czech republic.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiBirthCountyCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "county of birth", nullptr));
#if QT_CONFIG(tooltip)
        aiFullAddressCheckBox->setToolTip(QCoreApplication::translate("DlgSendMessage", "Full residential address.", nullptr));
#endif // QT_CONFIG(tooltip)
        aiFullAddressCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "full address", nullptr));
        aiRuianCodeCheckBox->setText(QCoreApplication::translate("DlgSendMessage", "RUIAN address code", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("DlgSendMessage", "Author Info", nullptr));
        infoTextLabel1->setText(QCoreApplication::translate("DlgSendMessage", "Here you can create a short text message and add it to attachments as a PDF file.", nullptr));
        infoTextLabel2->setText(QCoreApplication::translate("DlgSendMessage", "Add the message to attachments after you have finished editing the text.", nullptr));
        textFormatLabel->setText(QCoreApplication::translate("DlgSendMessage", "Text format:", nullptr));
#if QT_CONFIG(tooltip)
        previewButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Shows a preview of the resulting document.", nullptr));
#endif // QT_CONFIG(tooltip)
        previewButton->setText(QCoreApplication::translate("DlgSendMessage", "Preview", nullptr));
#if QT_CONFIG(tooltip)
        viewPdfButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Generates a PDF and views it in a default application.", nullptr));
#endif // QT_CONFIG(tooltip)
        viewPdfButton->setText(QCoreApplication::translate("DlgSendMessage", "View as PDF", nullptr));
#if QT_CONFIG(tooltip)
        appendPdfButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Generates a PDF file and adds it to message attachments.", nullptr));
#endif // QT_CONFIG(tooltip)
        appendPdfButton->setText(QCoreApplication::translate("DlgSendMessage", "Append PDF", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("DlgSendMessage", "Text Message", nullptr));
#if QT_CONFIG(tooltip)
        sendButton->setToolTip(QCoreApplication::translate("DlgSendMessage", "Send message", nullptr));
#endif // QT_CONFIG(tooltip)
        sendButton->setText(QCoreApplication::translate("DlgSendMessage", "Send", nullptr));
        cancelButton->setText(QCoreApplication::translate("DlgSendMessage", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgSendMessage: public Ui_DlgSendMessage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_SEND_MESSAGE_H
