/********************************************************************************
** Form generated from reading UI file 'dlg_view_zfo.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_VIEW_ZFO_H
#define UI_DLG_VIEW_ZFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include "src/views/attachment_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgViewZfo
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QTextEdit *envelopeTextEdit;
    AttachmentTableView *attachmentTable;
    QHBoxLayout *horizontalLayout;
    QPushButton *signatureDetailsButton;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgViewZfo)
    {
        if (DlgViewZfo->objectName().isEmpty())
            DlgViewZfo->setObjectName("DlgViewZfo");
        DlgViewZfo->resize(433, 332);
        verticalLayout = new QVBoxLayout(DlgViewZfo);
        verticalLayout->setObjectName("verticalLayout");
        splitter = new QSplitter(DlgViewZfo);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Vertical);
        envelopeTextEdit = new QTextEdit(splitter);
        envelopeTextEdit->setObjectName("envelopeTextEdit");
        splitter->addWidget(envelopeTextEdit);
        attachmentTable = new AttachmentTableView(splitter);
        attachmentTable->setObjectName("attachmentTable");
        attachmentTable->setEnabled(false);
        attachmentTable->setSelectionBehavior(QAbstractItemView::SelectRows);
        attachmentTable->setShowGrid(false);
        attachmentTable->setWordWrap(false);
        splitter->addWidget(attachmentTable);
        attachmentTable->horizontalHeader()->setHighlightSections(false);
        attachmentTable->horizontalHeader()->setStretchLastSection(true);
        attachmentTable->verticalHeader()->setVisible(false);
        attachmentTable->verticalHeader()->setStretchLastSection(false);

        verticalLayout->addWidget(splitter);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        signatureDetailsButton = new QPushButton(DlgViewZfo);
        signatureDetailsButton->setObjectName("signatureDetailsButton");

        horizontalLayout->addWidget(signatureDetailsButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonBox = new QDialogButtonBox(DlgViewZfo);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);

        QWidget::setTabOrder(envelopeTextEdit, attachmentTable);
        QWidget::setTabOrder(attachmentTable, signatureDetailsButton);

        retranslateUi(DlgViewZfo);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgViewZfo, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgViewZfo, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgViewZfo);
    } // setupUi

    void retranslateUi(QDialog *DlgViewZfo)
    {
        DlgViewZfo->setWindowTitle(QCoreApplication::translate("DlgViewZfo", "View message from ZFO", nullptr));
        signatureDetailsButton->setText(QCoreApplication::translate("DlgViewZfo", "Signature details", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgViewZfo: public Ui_DlgViewZfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_VIEW_ZFO_H
