/****************************************************************************
** Meta object code from reading C++ file 'dlg_tag_settings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/gui/dlg_tag_settings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlg_tag_settings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DlgTagSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgTagSettings::qt_create_metaobjectdata<qt_meta_tag_ZN14DlgTagSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgTagSettings",
        "activateServiceButtons",
        "",
        "clientSelected",
        "selected",
        "selectCertFile",
        "checkClientConnection",
        "downloadProfileList",
        "changeProfile",
        "index",
        "handleLocalSelectionChange",
        "updateLocalTag",
        "sendSelectedTagsToServer",
        "handleServerSelectionChange",
        "updateServerTag",
        "watchConnected",
        "watchDisconnected",
        "watchLocalTagsInserted",
        "Json::TagEntryList",
        "entries",
        "watchLocalTagsUpdated",
        "watchLocalTagsDeleted",
        "Json::Int64StringList",
        "ids",
        "watchServerTagsInserted",
        "watchServerTagsUpdated",
        "watchServerTagsDeleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'activateServiceButtons'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clientSelected'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
        // Slot 'selectCertFile'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkClientConnection'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'downloadProfileList'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'changeProfile'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'handleLocalSelectionChange'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateLocalTag'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'sendSelectedTagsToServer'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleServerSelectionChange'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateServerTag'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchConnected'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchDisconnected'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchLocalTagsInserted'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'watchLocalTagsUpdated'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'watchLocalTagsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'watchServerTagsInserted'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'watchServerTagsUpdated'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'watchServerTagsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgTagSettings, qt_meta_tag_ZN14DlgTagSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgTagSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgTagSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgTagSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DlgTagSettingsE_t>.metaTypes,
    nullptr
} };

void DlgTagSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgTagSettings *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateServiceButtons(); break;
        case 1: _t->clientSelected((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->selectCertFile(); break;
        case 3: _t->checkClientConnection(); break;
        case 4: _t->downloadProfileList(); break;
        case 5: _t->changeProfile((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->handleLocalSelectionChange(); break;
        case 7: _t->updateLocalTag(); break;
        case 8: _t->sendSelectedTagsToServer(); break;
        case 9: _t->handleServerSelectionChange(); break;
        case 10: _t->updateServerTag(); break;
        case 11: _t->watchConnected(); break;
        case 12: _t->watchDisconnected(); break;
        case 13: _t->watchLocalTagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 14: _t->watchLocalTagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 15: _t->watchLocalTagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 16: _t->watchServerTagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 17: _t->watchServerTagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 18: _t->watchServerTagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::Int64StringList >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::Int64StringList >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DlgTagSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgTagSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgTagSettingsE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgTagSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
