/********************************************************************************
** Form generated from reading UI file 'dlg_tag_settings.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TAG_SETTINGS_H
#define UI_DLG_TAG_SETTINGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgTagSettings
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *storageSelectionGroupBox;
    QVBoxLayout *verticalLayout_24;
    QRadioButton *localTagDbRadioButton;
    QRadioButton *tagClientRadioButton;
    QGroupBox *clientGroupBox;
    QVBoxLayout *verticalLayout_25;
    QFormLayout *formLayout;
    QLabel *tagSrvrNameLabel;
    QLineEdit *tagSrvrNameLineEdit;
    QLabel *tagSrvrPortLabel;
    QLineEdit *tagSrvrPortLineEdit;
    QLabel *tagSrvrCertLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *tagSrvrCertLineEdit;
    QPushButton *tagSrvrCertButton;
    QFrame *line;
    QFormLayout *formLayout_2;
    QLabel *usernameLabel;
    QLineEdit *usernameLineEdit;
    QLabel *pwdLabel;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *pwdLineEdit;
    QHBoxLayout *horizontalLayout_2;
    QLabel *connectionStatusLabel;
    QSpacerItem *horizontalSpacer;
    QPushButton *checkConnectionButton;
    QFrame *line_2;
    QFormLayout *formLayout_3;
    QLabel *profileLabel;
    QHBoxLayout *horizontalLayout_5;
    QComboBox *profileComboBox;
    QPushButton *refreshProfilesPushButton;
    QLabel *profileErrorLabel;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QGroupBox *localGroupBox;
    QVBoxLayout *verticalLayout_5;
    QListView *localTagsView;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *updateLocalTagButton;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *asssignmentButtonLayout;
    QSpacerItem *verticalSpacer_2;
    QPushButton *toServerButton;
    QSpacerItem *verticalSpacer_3;
    QGroupBox *serverGroupBox;
    QVBoxLayout *verticalLayout_6;
    QListView *serverTagsView;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *updateServerTagButton;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgTagSettings)
    {
        if (DlgTagSettings->objectName().isEmpty())
            DlgTagSettings->setObjectName("DlgTagSettings");
        DlgTagSettings->resize(600, 480);
        verticalLayout = new QVBoxLayout(DlgTagSettings);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(DlgTagSettings);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        storageSelectionGroupBox = new QGroupBox(tab);
        storageSelectionGroupBox->setObjectName("storageSelectionGroupBox");
        QFont font;
        font.setBold(true);
        storageSelectionGroupBox->setFont(font);
        verticalLayout_24 = new QVBoxLayout(storageSelectionGroupBox);
        verticalLayout_24->setObjectName("verticalLayout_24");
        localTagDbRadioButton = new QRadioButton(storageSelectionGroupBox);
        localTagDbRadioButton->setObjectName("localTagDbRadioButton");
        QFont font1;
        font1.setBold(false);
        localTagDbRadioButton->setFont(font1);

        verticalLayout_24->addWidget(localTagDbRadioButton);

        tagClientRadioButton = new QRadioButton(storageSelectionGroupBox);
        tagClientRadioButton->setObjectName("tagClientRadioButton");
        tagClientRadioButton->setFont(font1);

        verticalLayout_24->addWidget(tagClientRadioButton);


        verticalLayout_2->addWidget(storageSelectionGroupBox);

        clientGroupBox = new QGroupBox(tab);
        clientGroupBox->setObjectName("clientGroupBox");
        clientGroupBox->setFont(font);
        verticalLayout_25 = new QVBoxLayout(clientGroupBox);
        verticalLayout_25->setObjectName("verticalLayout_25");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        tagSrvrNameLabel = new QLabel(clientGroupBox);
        tagSrvrNameLabel->setObjectName("tagSrvrNameLabel");
        tagSrvrNameLabel->setFont(font1);

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, tagSrvrNameLabel);

        tagSrvrNameLineEdit = new QLineEdit(clientGroupBox);
        tagSrvrNameLineEdit->setObjectName("tagSrvrNameLineEdit");
        tagSrvrNameLineEdit->setFont(font1);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, tagSrvrNameLineEdit);

        tagSrvrPortLabel = new QLabel(clientGroupBox);
        tagSrvrPortLabel->setObjectName("tagSrvrPortLabel");
        tagSrvrPortLabel->setFont(font1);

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, tagSrvrPortLabel);

        tagSrvrPortLineEdit = new QLineEdit(clientGroupBox);
        tagSrvrPortLineEdit->setObjectName("tagSrvrPortLineEdit");
        tagSrvrPortLineEdit->setFont(font1);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, tagSrvrPortLineEdit);

        tagSrvrCertLabel = new QLabel(clientGroupBox);
        tagSrvrCertLabel->setObjectName("tagSrvrCertLabel");
        tagSrvrCertLabel->setFont(font1);

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, tagSrvrCertLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        tagSrvrCertLineEdit = new QLineEdit(clientGroupBox);
        tagSrvrCertLineEdit->setObjectName("tagSrvrCertLineEdit");
        tagSrvrCertLineEdit->setFont(font1);

        horizontalLayout->addWidget(tagSrvrCertLineEdit);

        tagSrvrCertButton = new QPushButton(clientGroupBox);
        tagSrvrCertButton->setObjectName("tagSrvrCertButton");
        tagSrvrCertButton->setFont(font1);

        horizontalLayout->addWidget(tagSrvrCertButton);


        formLayout->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout);


        verticalLayout_25->addLayout(formLayout);

        line = new QFrame(clientGroupBox);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_25->addWidget(line);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        usernameLabel = new QLabel(clientGroupBox);
        usernameLabel->setObjectName("usernameLabel");
        usernameLabel->setFont(font1);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, usernameLabel);

        usernameLineEdit = new QLineEdit(clientGroupBox);
        usernameLineEdit->setObjectName("usernameLineEdit");
        usernameLineEdit->setFont(font1);

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, usernameLineEdit);

        pwdLabel = new QLabel(clientGroupBox);
        pwdLabel->setObjectName("pwdLabel");
        pwdLabel->setFont(font1);

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, pwdLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        pwdLineEdit = new QLineEdit(clientGroupBox);
        pwdLineEdit->setObjectName("pwdLineEdit");
        pwdLineEdit->setFont(font1);
        pwdLineEdit->setEchoMode(QLineEdit::Password);

        horizontalLayout_3->addWidget(pwdLineEdit);


        formLayout_2->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);


        verticalLayout_25->addLayout(formLayout_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        connectionStatusLabel = new QLabel(clientGroupBox);
        connectionStatusLabel->setObjectName("connectionStatusLabel");
        connectionStatusLabel->setFont(font1);

        horizontalLayout_2->addWidget(connectionStatusLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        checkConnectionButton = new QPushButton(clientGroupBox);
        checkConnectionButton->setObjectName("checkConnectionButton");
        checkConnectionButton->setFont(font1);

        horizontalLayout_2->addWidget(checkConnectionButton);


        verticalLayout_25->addLayout(horizontalLayout_2);

        line_2 = new QFrame(clientGroupBox);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_25->addWidget(line_2);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        profileLabel = new QLabel(clientGroupBox);
        profileLabel->setObjectName("profileLabel");
        profileLabel->setFont(font1);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, profileLabel);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        profileComboBox = new QComboBox(clientGroupBox);
        profileComboBox->setObjectName("profileComboBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(profileComboBox->sizePolicy().hasHeightForWidth());
        profileComboBox->setSizePolicy(sizePolicy);
        profileComboBox->setFont(font1);

        horizontalLayout_5->addWidget(profileComboBox);

        refreshProfilesPushButton = new QPushButton(clientGroupBox);
        refreshProfilesPushButton->setObjectName("refreshProfilesPushButton");
        refreshProfilesPushButton->setFont(font1);

        horizontalLayout_5->addWidget(refreshProfilesPushButton);


        formLayout_3->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_5);


        verticalLayout_25->addLayout(formLayout_3);

        profileErrorLabel = new QLabel(clientGroupBox);
        profileErrorLabel->setObjectName("profileErrorLabel");
        profileErrorLabel->setFont(font1);

        verticalLayout_25->addWidget(profileErrorLabel);


        verticalLayout_2->addWidget(clientGroupBox);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        localGroupBox = new QGroupBox(tab_2);
        localGroupBox->setObjectName("localGroupBox");
        verticalLayout_5 = new QVBoxLayout(localGroupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        localTagsView = new QListView(localGroupBox);
        localTagsView->setObjectName("localTagsView");

        verticalLayout_5->addWidget(localTagsView);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        updateLocalTagButton = new QPushButton(localGroupBox);
        updateLocalTagButton->setObjectName("updateLocalTagButton");

        horizontalLayout_6->addWidget(updateLocalTagButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_2);


        verticalLayout_5->addLayout(horizontalLayout_6);


        horizontalLayout_4->addWidget(localGroupBox);

        asssignmentButtonLayout = new QVBoxLayout();
        asssignmentButtonLayout->setObjectName("asssignmentButtonLayout");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        asssignmentButtonLayout->addItem(verticalSpacer_2);

        toServerButton = new QPushButton(tab_2);
        toServerButton->setObjectName("toServerButton");

        asssignmentButtonLayout->addWidget(toServerButton);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        asssignmentButtonLayout->addItem(verticalSpacer_3);


        horizontalLayout_4->addLayout(asssignmentButtonLayout);

        serverGroupBox = new QGroupBox(tab_2);
        serverGroupBox->setObjectName("serverGroupBox");
        verticalLayout_6 = new QVBoxLayout(serverGroupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        serverTagsView = new QListView(serverGroupBox);
        serverTagsView->setObjectName("serverTagsView");

        verticalLayout_6->addWidget(serverTagsView);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        updateServerTagButton = new QPushButton(serverGroupBox);
        updateServerTagButton->setObjectName("updateServerTagButton");

        horizontalLayout_7->addWidget(updateServerTagButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_3);


        verticalLayout_6->addLayout(horizontalLayout_7);


        horizontalLayout_4->addWidget(serverGroupBox);


        verticalLayout_3->addLayout(horizontalLayout_4);

        tabWidget->addTab(tab_2, QString());

        verticalLayout->addWidget(tabWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgTagSettings);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgTagSettings);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgTagSettings, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgTagSettings, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgTagSettings);
    } // setupUi

    void retranslateUi(QDialog *DlgTagSettings)
    {
        DlgTagSettings->setWindowTitle(QCoreApplication::translate("DlgTagSettings", "Tag Storage Settings", nullptr));
        storageSelectionGroupBox->setTitle(QCoreApplication::translate("DlgTagSettings", "Tag storage", nullptr));
#if QT_CONFIG(tooltip)
        localTagDbRadioButton->setToolTip(QCoreApplication::translate("DlgTagSettings", "Use local tag database stored on this computer.", nullptr));
#endif // QT_CONFIG(tooltip)
        localTagDbRadioButton->setText(QCoreApplication::translate("DlgTagSettings", "Use local tag database.", nullptr));
#if QT_CONFIG(tooltip)
        tagClientRadioButton->setToolTip(QCoreApplication::translate("DlgTagSettings", "Access tag data on a server.\n"
"These data can then be shared between multiple users.", nullptr));
#endif // QT_CONFIG(tooltip)
        tagClientRadioButton->setText(QCoreApplication::translate("DlgTagSettings", "Connect to tag server.", nullptr));
        clientGroupBox->setTitle(QCoreApplication::translate("DlgTagSettings", "Tag server", nullptr));
        tagSrvrNameLabel->setText(QCoreApplication::translate("DlgTagSettings", "Address or hostname:", nullptr));
#if QT_CONFIG(tooltip)
        tagSrvrNameLineEdit->setToolTip(QCoreApplication::translate("DlgTagSettings", "Enter the name of the tag server, e.g. 'servername.domain'.", nullptr));
#endif // QT_CONFIG(tooltip)
        tagSrvrPortLabel->setText(QCoreApplication::translate("DlgTagSettings", "Port number:", nullptr));
#if QT_CONFIG(tooltip)
        tagSrvrPortLineEdit->setToolTip(QCoreApplication::translate("DlgTagSettings", "Enter the port number.", nullptr));
#endif // QT_CONFIG(tooltip)
        tagSrvrCertLabel->setText(QCoreApplication::translate("DlgTagSettings", "Certificate file:", nullptr));
#if QT_CONFIG(tooltip)
        tagSrvrCertLineEdit->setToolTip(QCoreApplication::translate("DlgTagSettings", "Path to the CA (certification authority) certificate file\n"
"if the CA certificate isn't already stored in the CA certificate storage.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tagSrvrCertButton->setToolTip(QCoreApplication::translate("DlgTagSettings", "Select certificate file.", nullptr));
#endif // QT_CONFIG(tooltip)
        tagSrvrCertButton->setText(QCoreApplication::translate("DlgTagSettings", "Select", nullptr));
        usernameLabel->setText(QCoreApplication::translate("DlgTagSettings", "Username:", nullptr));
#if QT_CONFIG(tooltip)
        usernameLineEdit->setToolTip(QCoreApplication::translate("DlgTagSettings", "Enter the username to connect to the tag server.", nullptr));
#endif // QT_CONFIG(tooltip)
        pwdLabel->setText(QCoreApplication::translate("DlgTagSettings", "Password:", nullptr));
#if QT_CONFIG(tooltip)
        pwdLineEdit->setToolTip(QCoreApplication::translate("DlgTagSettings", "Provide a password to connect to the tag server.", nullptr));
#endif // QT_CONFIG(tooltip)
        connectionStatusLabel->setText(QCoreApplication::translate("DlgTagSettings", "n/a", nullptr));
#if QT_CONFIG(tooltip)
        checkConnectionButton->setToolTip(QCoreApplication::translate("DlgTagSettings", "Connect to tag server using the provided settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkConnectionButton->setText(QCoreApplication::translate("DlgTagSettings", "Connect", nullptr));
        profileLabel->setText(QCoreApplication::translate("DlgTagSettings", "Profile:", nullptr));
#if QT_CONFIG(tooltip)
        refreshProfilesPushButton->setToolTip(QCoreApplication::translate("DlgTagSettings", "Refresh the profile list.", nullptr));
#endif // QT_CONFIG(tooltip)
        refreshProfilesPushButton->setText(QCoreApplication::translate("DlgTagSettings", "Refresh", nullptr));
        profileErrorLabel->setText(QCoreApplication::translate("DlgTagSettings", "n/a", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("DlgTagSettings", "Connection", nullptr));
        localGroupBox->setTitle(QCoreApplication::translate("DlgTagSettings", "Local Database", nullptr));
        updateLocalTagButton->setText(QCoreApplication::translate("DlgTagSettings", "Update", nullptr));
        toServerButton->setText(QCoreApplication::translate("DlgTagSettings", "To Server", nullptr));
        serverGroupBox->setTitle(QCoreApplication::translate("DlgTagSettings", "Server", nullptr));
        updateServerTagButton->setText(QCoreApplication::translate("DlgTagSettings", "Update", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("DlgTagSettings", "Tags", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTagSettings: public Ui_DlgTagSettings {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TAG_SETTINGS_H
