/********************************************************************************
** Form generated from reading UI file 'dlg_change_directory.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CHANGE_DIRECTORY_H
#define UI_DLG_CHANGE_DIRECTORY_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgChangeDirectory
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *accoutLabel;
    QLabel *label1;
    QLineEdit *currentPathLineEdit;
    QLabel *label2;
    QHBoxLayout *horizontalLayout;
    QLabel *useDefaultLabel;
    QSpacerItem *horizontalSpacer;
    QPushButton *useDefaultButton;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *newPathLineEdit;
    QPushButton *chooseButton;
    QLabel *labelWarning;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *moveDataRadioButton;
    QRadioButton *copyDataRadioButton;
    QRadioButton *startDataRadioButtonEraseSrc;
    QRadioButton *startDataRadioButtonLeaveSrc;
    QRadioButton *useDataRadioButtonEraseSrc;
    QRadioButton *useDataRadioButtonLeaveSrc;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgChangeDirectory)
    {
        if (DlgChangeDirectory->objectName().isEmpty())
            DlgChangeDirectory->setObjectName("DlgChangeDirectory");
        DlgChangeDirectory->resize(500, 410);
        verticalLayout = new QVBoxLayout(DlgChangeDirectory);
        verticalLayout->setObjectName("verticalLayout");
        accoutLabel = new QLabel(DlgChangeDirectory);
        accoutLabel->setObjectName("accoutLabel");
        QFont font;
        font.setItalic(true);
        accoutLabel->setFont(font);

        verticalLayout->addWidget(accoutLabel);

        label1 = new QLabel(DlgChangeDirectory);
        label1->setObjectName("label1");

        verticalLayout->addWidget(label1);

        currentPathLineEdit = new QLineEdit(DlgChangeDirectory);
        currentPathLineEdit->setObjectName("currentPathLineEdit");
        currentPathLineEdit->setReadOnly(true);

        verticalLayout->addWidget(currentPathLineEdit);

        label2 = new QLabel(DlgChangeDirectory);
        label2->setObjectName("label2");
        QFont font1;
        font1.setBold(true);
        label2->setFont(font1);

        verticalLayout->addWidget(label2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        useDefaultLabel = new QLabel(DlgChangeDirectory);
        useDefaultLabel->setObjectName("useDefaultLabel");
        useDefaultLabel->setWordWrap(true);

        horizontalLayout->addWidget(useDefaultLabel);

        horizontalSpacer = new QSpacerItem(40, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        useDefaultButton = new QPushButton(DlgChangeDirectory);
        useDefaultButton->setObjectName("useDefaultButton");

        horizontalLayout->addWidget(useDefaultButton);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        newPathLineEdit = new QLineEdit(DlgChangeDirectory);
        newPathLineEdit->setObjectName("newPathLineEdit");
        newPathLineEdit->setReadOnly(true);

        horizontalLayout_2->addWidget(newPathLineEdit);

        chooseButton = new QPushButton(DlgChangeDirectory);
        chooseButton->setObjectName("chooseButton");

        horizontalLayout_2->addWidget(chooseButton);


        verticalLayout->addLayout(horizontalLayout_2);

        labelWarning = new QLabel(DlgChangeDirectory);
        labelWarning->setObjectName("labelWarning");
        labelWarning->setFont(font1);
        labelWarning->setAlignment(Qt::AlignCenter);
        labelWarning->setWordWrap(true);

        verticalLayout->addWidget(labelWarning);

        groupBox = new QGroupBox(DlgChangeDirectory);
        groupBox->setObjectName("groupBox");
        groupBox->setFont(font1);
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        moveDataRadioButton = new QRadioButton(groupBox);
        moveDataRadioButton->setObjectName("moveDataRadioButton");
        QFont font2;
        font2.setBold(false);
        moveDataRadioButton->setFont(font2);
        moveDataRadioButton->setChecked(true);

        verticalLayout_2->addWidget(moveDataRadioButton);

        copyDataRadioButton = new QRadioButton(groupBox);
        copyDataRadioButton->setObjectName("copyDataRadioButton");
        copyDataRadioButton->setFont(font2);

        verticalLayout_2->addWidget(copyDataRadioButton);

        startDataRadioButtonEraseSrc = new QRadioButton(groupBox);
        startDataRadioButtonEraseSrc->setObjectName("startDataRadioButtonEraseSrc");
        startDataRadioButtonEraseSrc->setFont(font2);

        verticalLayout_2->addWidget(startDataRadioButtonEraseSrc);

        startDataRadioButtonLeaveSrc = new QRadioButton(groupBox);
        startDataRadioButtonLeaveSrc->setObjectName("startDataRadioButtonLeaveSrc");
        startDataRadioButtonLeaveSrc->setFont(font2);

        verticalLayout_2->addWidget(startDataRadioButtonLeaveSrc);

        useDataRadioButtonEraseSrc = new QRadioButton(groupBox);
        useDataRadioButtonEraseSrc->setObjectName("useDataRadioButtonEraseSrc");
        useDataRadioButtonEraseSrc->setFont(font2);

        verticalLayout_2->addWidget(useDataRadioButtonEraseSrc);

        useDataRadioButtonLeaveSrc = new QRadioButton(groupBox);
        useDataRadioButtonLeaveSrc->setObjectName("useDataRadioButtonLeaveSrc");
        useDataRadioButtonLeaveSrc->setFont(font2);

        verticalLayout_2->addWidget(useDataRadioButtonLeaveSrc);


        verticalLayout->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgChangeDirectory);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(chooseButton, useDefaultButton);
        QWidget::setTabOrder(useDefaultButton, moveDataRadioButton);
        QWidget::setTabOrder(moveDataRadioButton, copyDataRadioButton);
        QWidget::setTabOrder(copyDataRadioButton, startDataRadioButtonEraseSrc);
        QWidget::setTabOrder(startDataRadioButtonEraseSrc, startDataRadioButtonLeaveSrc);
        QWidget::setTabOrder(startDataRadioButtonLeaveSrc, useDataRadioButtonEraseSrc);
        QWidget::setTabOrder(useDataRadioButtonEraseSrc, useDataRadioButtonLeaveSrc);
        QWidget::setTabOrder(useDataRadioButtonLeaveSrc, currentPathLineEdit);
        QWidget::setTabOrder(currentPathLineEdit, newPathLineEdit);

        retranslateUi(DlgChangeDirectory);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgChangeDirectory, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgChangeDirectory, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgChangeDirectory);
    } // setupUi

    void retranslateUi(QDialog *DlgChangeDirectory)
    {
        DlgChangeDirectory->setWindowTitle(QCoreApplication::translate("DlgChangeDirectory", "Change Data Directory for Current Data Box", nullptr));
        accoutLabel->setText(QCoreApplication::translate("DlgChangeDirectory", "n/a", nullptr));
        label1->setText(QCoreApplication::translate("DlgChangeDirectory", "Current Data Directory", nullptr));
        label2->setText(QCoreApplication::translate("DlgChangeDirectory", "New Data Directory", nullptr));
        useDefaultLabel->setText(QCoreApplication::translate("DlgChangeDirectory", "The current location doesn't match the default location.", nullptr));
#if QT_CONFIG(tooltip)
        useDefaultButton->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "Select the default directory as the target.", nullptr));
#endif // QT_CONFIG(tooltip)
        useDefaultButton->setText(QCoreApplication::translate("DlgChangeDirectory", "Use Default", nullptr));
        newPathLineEdit->setPlaceholderText(QCoreApplication::translate("DlgChangeDirectory", "Select a new directory where data should be stored.", nullptr));
#if QT_CONFIG(tooltip)
        chooseButton->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "Select target directory.", nullptr));
#endif // QT_CONFIG(tooltip)
        chooseButton->setText(QCoreApplication::translate("DlgChangeDirectory", "Choose", nullptr));
        labelWarning->setText(QCoreApplication::translate("DlgChangeDirectory", "n/a", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgChangeDirectory", "Action", nullptr));
#if QT_CONFIG(tooltip)
        moveDataRadioButton->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "The operation will move local data from this data box to the new location.", nullptr));
#endif // QT_CONFIG(tooltip)
        moveDataRadioButton->setText(QCoreApplication::translate("DlgChangeDirectory", "Move data to the new directory", nullptr));
#if QT_CONFIG(tooltip)
        copyDataRadioButton->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "The operation will copy local data from this data box to the new location.", nullptr));
#endif // QT_CONFIG(tooltip)
        copyDataRadioButton->setText(QCoreApplication::translate("DlgChangeDirectory", "Copy data to the new directory (leave data in original directory)", nullptr));
#if QT_CONFIG(tooltip)
        startDataRadioButtonEraseSrc->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "The operation will erase currently used local data from this data box.\n"
"The operation will erase any corresponding data in the new location if there are any.\n"
"A new empty storage for local data for this data box will be opened in the new location.", nullptr));
#endif // QT_CONFIG(tooltip)
        startDataRadioButtonEraseSrc->setText(QCoreApplication::translate("DlgChangeDirectory", "Start afresh in the new directory and erase data in original directory", nullptr));
#if QT_CONFIG(tooltip)
        startDataRadioButtonLeaveSrc->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "The operation will erase any corresponding data in the new location if there are any.\n"
"A new empty storage for local data for this data box will be opened in the new location.\n"
"Currently used data will remain in the old location.", nullptr));
#endif // QT_CONFIG(tooltip)
        startDataRadioButtonLeaveSrc->setText(QCoreApplication::translate("DlgChangeDirectory", "Start afresh in the new directory (leave data in original directory)", nullptr));
#if QT_CONFIG(tooltip)
        useDataRadioButtonEraseSrc->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "The operation will erase currently used local data from this data box.\n"
"An existing storage for local data for this data box will be opened in the new location.\n"
"Data found in the new location will be used.", nullptr));
#endif // QT_CONFIG(tooltip)
        useDataRadioButtonEraseSrc->setText(QCoreApplication::translate("DlgChangeDirectory", "Use exising in the new directory and erase data in original directory", nullptr));
#if QT_CONFIG(tooltip)
        useDataRadioButtonLeaveSrc->setToolTip(QCoreApplication::translate("DlgChangeDirectory", "An existing storage for local data for this data box will be opened in the new location.\n"
"Data found in the new location will be used.\n"
"Currently used data will remain in the old location.", nullptr));
#endif // QT_CONFIG(tooltip)
        useDataRadioButtonLeaveSrc->setText(QCoreApplication::translate("DlgChangeDirectory", "Use exising in the new directory (leave data in original directory)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgChangeDirectory: public Ui_DlgChangeDirectory {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CHANGE_DIRECTORY_H
