/********************************************************************************
** Form generated from reading UI file 'dlg_pin_input.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_PIN_INPUT_H
#define UI_DLG_PIN_INPUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgPinInput
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *logoLabel;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QLabel *versionLabel;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer_3;
    QLineEdit *pinLine;
    QSpacerItem *verticalSpacer_4;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgPinInput)
    {
        if (DlgPinInput->objectName().isEmpty())
            DlgPinInput->setObjectName("DlgPinInput");
        DlgPinInput->resize(300, 200);
        verticalLayout = new QVBoxLayout(DlgPinInput);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        logoLabel = new QLabel(DlgPinInput);
        logoLabel->setObjectName("logoLabel");

        horizontalLayout_3->addWidget(logoLabel);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        versionLabel = new QLabel(DlgPinInput);
        versionLabel->setObjectName("versionLabel");

        horizontalLayout_2->addWidget(versionLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        pinLine = new QLineEdit(DlgPinInput);
        pinLine->setObjectName("pinLine");

        verticalLayout_3->addWidget(pinLine);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_4);


        horizontalLayout->addLayout(verticalLayout_3);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(DlgPinInput);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgPinInput);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgPinInput, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgPinInput, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgPinInput);
    } // setupUi

    void retranslateUi(QDialog *DlgPinInput)
    {
        DlgPinInput->setWindowTitle(QCoreApplication::translate("DlgPinInput", "Enter PIN", nullptr));
        logoLabel->setText(QCoreApplication::translate("DlgPinInput", "n/a", nullptr));
        versionLabel->setText(QCoreApplication::translate("DlgPinInput", "n/a", nullptr));
#if QT_CONFIG(tooltip)
        pinLine->setToolTip(QCoreApplication::translate("DlgPinInput", "Enter the PIN value.", nullptr));
#endif // QT_CONFIG(tooltip)
        pinLine->setPlaceholderText(QCoreApplication::translate("DlgPinInput", "Enter PIN code", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPinInput: public Ui_DlgPinInput {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_PIN_INPUT_H
