/********************************************************************************
** Form generated from reading UI file 'mw_db_repair.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MW_DB_REPAIR_H
#define UI_MW_DB_REPAIR_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_MWDbRepair
{
public:
    QAction *actionQuit;
    QAction *actionLoadDatabaseFile;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *dbFileLabel;
    QLineEdit *dbFilePath;
    QHBoxLayout *horizontalLayout;
    QLineEdit *workDirPath;
    QPushButton *workDirButton;
    QLabel *workDirLabel;
    QFrame *line;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *reassembleDbCheckBox;
    QSpacerItem *horizontalSpacer;
    QPushButton *runRecoveryButton;
    QGroupBox *recoverdGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label;
    QLineEdit *recoveredDbPath;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *copyOriginalDbCheckbox;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *replaceCorruptButton;
    QTextEdit *repairInfo;
    QMenuBar *menubar;
    QMenu *menuFile;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *MWDbRepair)
    {
        if (MWDbRepair->objectName().isEmpty())
            MWDbRepair->setObjectName("MWDbRepair");
        MWDbRepair->resize(800, 600);
        actionQuit = new QAction(MWDbRepair);
        actionQuit->setObjectName("actionQuit");
        actionQuit->setMenuRole(QAction::QuitRole);
        actionLoadDatabaseFile = new QAction(MWDbRepair);
        actionLoadDatabaseFile->setObjectName("actionLoadDatabaseFile");
        actionLoadDatabaseFile->setMenuRole(QAction::NoRole);
        centralwidget = new QWidget(MWDbRepair);
        centralwidget->setObjectName("centralwidget");
        verticalLayout = new QVBoxLayout(centralwidget);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dbFileLabel = new QLabel(centralwidget);
        dbFileLabel->setObjectName("dbFileLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, dbFileLabel);

        dbFilePath = new QLineEdit(centralwidget);
        dbFilePath->setObjectName("dbFilePath");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, dbFilePath);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        workDirPath = new QLineEdit(centralwidget);
        workDirPath->setObjectName("workDirPath");

        horizontalLayout->addWidget(workDirPath);

        workDirButton = new QPushButton(centralwidget);
        workDirButton->setObjectName("workDirButton");

        horizontalLayout->addWidget(workDirButton);


        formLayout->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        workDirLabel = new QLabel(centralwidget);
        workDirLabel->setObjectName("workDirLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, workDirLabel);


        verticalLayout->addLayout(formLayout);

        line = new QFrame(centralwidget);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        reassembleDbCheckBox = new QCheckBox(centralwidget);
        reassembleDbCheckBox->setObjectName("reassembleDbCheckBox");

        horizontalLayout_2->addWidget(reassembleDbCheckBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        runRecoveryButton = new QPushButton(centralwidget);
        runRecoveryButton->setObjectName("runRecoveryButton");

        horizontalLayout_2->addWidget(runRecoveryButton);


        verticalLayout->addLayout(horizontalLayout_2);

        recoverdGroupBox = new QGroupBox(centralwidget);
        recoverdGroupBox->setObjectName("recoverdGroupBox");
        verticalLayout_2 = new QVBoxLayout(recoverdGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label = new QLabel(recoverdGroupBox);
        label->setObjectName("label");

        horizontalLayout_3->addWidget(label);

        recoveredDbPath = new QLineEdit(recoverdGroupBox);
        recoveredDbPath->setObjectName("recoveredDbPath");

        horizontalLayout_3->addWidget(recoveredDbPath);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        copyOriginalDbCheckbox = new QCheckBox(recoverdGroupBox);
        copyOriginalDbCheckbox->setObjectName("copyOriginalDbCheckbox");

        horizontalLayout_4->addWidget(copyOriginalDbCheckbox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        replaceCorruptButton = new QPushButton(recoverdGroupBox);
        replaceCorruptButton->setObjectName("replaceCorruptButton");

        horizontalLayout_4->addWidget(replaceCorruptButton);


        verticalLayout_2->addLayout(horizontalLayout_4);


        verticalLayout->addWidget(recoverdGroupBox);

        repairInfo = new QTextEdit(centralwidget);
        repairInfo->setObjectName("repairInfo");

        verticalLayout->addWidget(repairInfo);

        MWDbRepair->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MWDbRepair);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 800, 19));
        menuFile = new QMenu(menubar);
        menuFile->setObjectName("menuFile");
        MWDbRepair->setMenuBar(menubar);
        statusbar = new QStatusBar(MWDbRepair);
        statusbar->setObjectName("statusbar");
        MWDbRepair->setStatusBar(statusbar);

        menubar->addAction(menuFile->menuAction());
        menuFile->addAction(actionLoadDatabaseFile);
        menuFile->addSeparator();
        menuFile->addAction(actionQuit);

        retranslateUi(MWDbRepair);
        QObject::connect(actionQuit, &QAction::triggered, MWDbRepair, qOverload<>(&QMainWindow::close));

        QMetaObject::connectSlotsByName(MWDbRepair);
    } // setupUi

    void retranslateUi(QMainWindow *MWDbRepair)
    {
        MWDbRepair->setWindowTitle(QCoreApplication::translate("MWDbRepair", "Database Repair", nullptr));
        actionQuit->setText(QCoreApplication::translate("MWDbRepair", "Quit", nullptr));
#if QT_CONFIG(tooltip)
        actionQuit->setToolTip(QCoreApplication::translate("MWDbRepair", "Quit the application.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionQuit->setShortcut(QCoreApplication::translate("MWDbRepair", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        actionLoadDatabaseFile->setText(QCoreApplication::translate("MWDbRepair", "Load message database file.", nullptr));
        dbFileLabel->setText(QCoreApplication::translate("MWDbRepair", "Source (corrupted) database file:", nullptr));
        dbFilePath->setPlaceholderText(QCoreApplication::translate("MWDbRepair", "Select source message database file.", nullptr));
#if QT_CONFIG(tooltip)
        workDirPath->setToolTip(QCoreApplication::translate("MWDbRepair", "Extracted data are stored in the work directory.", nullptr));
#endif // QT_CONFIG(tooltip)
        workDirPath->setPlaceholderText(QCoreApplication::translate("MWDbRepair", "Select work directory.", nullptr));
#if QT_CONFIG(tooltip)
        workDirButton->setToolTip(QCoreApplication::translate("MWDbRepair", "Select work directory.", nullptr));
#endif // QT_CONFIG(tooltip)
        workDirButton->setText(QCoreApplication::translate("MWDbRepair", "Choose Directory", nullptr));
        workDirLabel->setText(QCoreApplication::translate("MWDbRepair", "Work directory:", nullptr));
        reassembleDbCheckBox->setText(QCoreApplication::translate("MWDbRepair", "Try reassembling new database from recovered data.", nullptr));
#if QT_CONFIG(tooltip)
        runRecoveryButton->setToolTip(QCoreApplication::translate("MWDbRepair", "Start the data recovery.\n"
"Scans the source database for readable data.\n"
"Extracts message and delivery info data and stores them as separate files.", nullptr));
#endif // QT_CONFIG(tooltip)
        runRecoveryButton->setText(QCoreApplication::translate("MWDbRepair", "Run Recovery", nullptr));
        recoverdGroupBox->setTitle(QCoreApplication::translate("MWDbRepair", "Recovered Database", nullptr));
        label->setText(QCoreApplication::translate("MWDbRepair", "Recovered database file:", nullptr));
        copyOriginalDbCheckbox->setText(QCoreApplication::translate("MWDbRepair", "Create a copy of the source (corrupted) database file before replacing it.", nullptr));
#if QT_CONFIG(tooltip)
        replaceCorruptButton->setToolTip(QCoreApplication::translate("MWDbRepair", "Overwrites the source (corrupted) database file with the restored database.", nullptr));
#endif // QT_CONFIG(tooltip)
        replaceCorruptButton->setText(QCoreApplication::translate("MWDbRepair", "Replace Corrupted", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MWDbRepair", "File", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MWDbRepair: public Ui_MWDbRepair {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MW_DB_REPAIR_H
