/****************************************************************************
** Meta object code from reading C++ file 'dlg_msg_search.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/gui/dlg_msg_search.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlg_msg_search.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12DlgMsgSearchE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgMsgSearch::qt_create_metaobjectdata<qt_meta_tag_ZN12DlgMsgSearchE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgMsgSearch",
        "focusSelectedMsg",
        "",
        "AcntId",
        "acntId",
        "dmId",
        "year",
        "msgType",
        "checkInputFields",
        "fromDateEnable",
        "enable",
        "toDateEnable",
        "reflectDateChange",
        "searchMessages",
        "filterFound",
        "text",
        "updateMessageSelection",
        "QItemSelection",
        "selected",
        "deselected",
        "messageItemDoubleClicked",
        "QModelIndex",
        "index",
        "viewMessageListContextMenu",
        "QPoint",
        "point",
        "offerDownloadForMessages",
        "AcntIdToMsgIdListHash",
        "acntMsgIds",
        "watchMessageInserted",
        "MsgId",
        "msgId",
        "direction",
        "watchMessageDataDeleted",
        "downloadSelectedMessageAttachments",
        "exportSelectedMessageZfos",
        "exportSelectedAcceptanceInfoZfos",
        "exportSelectedAcceptanceInfoPdfs",
        "exportSelectedEnvelopePdfs",
        "exportSelectedEnvelopeAttachments",
        "createEmailWithZfos",
        "createEmailWithAllAttachments",
        "createEmailContentSelection"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusSelectedMsg'
        QtMocHelpers::SignalData<void(AcntId, qint64, QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::LongLong, 5 }, { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'checkInputFields'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fromDateEnable'
        QtMocHelpers::SlotData<void(bool)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'toDateEnable'
        QtMocHelpers::SlotData<void(bool)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'reflectDateChange'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchMessages'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterFound'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { 0x80000000 | 17, 19 },
        }}),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(16, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'messageItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'viewMessageListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'offerDownloadForMessages'
        QtMocHelpers::SlotData<void(const AcntIdToMsgIdListHash &)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'watchMessageInserted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &, int)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 30, 31 }, { QMetaType::Int, 32 },
        }}),
        // Slot 'watchMessageDataDeleted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 30, 31 },
        }}),
        // Slot 'downloadSelectedMessageAttachments'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedMessageZfos'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedAcceptanceInfoZfos'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedAcceptanceInfoPdfs'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedEnvelopePdfs'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportSelectedEnvelopeAttachments'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailWithZfos'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailWithAllAttachments'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'createEmailContentSelection'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgMsgSearch, qt_meta_tag_ZN12DlgMsgSearchE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgMsgSearch::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgMsgSearchE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgMsgSearchE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12DlgMsgSearchE_t>.metaTypes,
    nullptr
} };

void DlgMsgSearch::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgMsgSearch *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusSelectedMsg((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->checkInputFields(); break;
        case 2: _t->fromDateEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->toDateEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->reflectDateChange(); break;
        case 5: _t->searchMessages(); break;
        case 6: _t->filterFound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 8: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 9: _t->messageItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 10: _t->viewMessageListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 11: _t->offerDownloadForMessages((*reinterpret_cast<std::add_pointer_t<AcntIdToMsgIdListHash>>(_a[1]))); break;
        case 12: _t->watchMessageInserted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 13: _t->watchMessageDataDeleted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2]))); break;
        case 14: _t->downloadSelectedMessageAttachments(); break;
        case 15: _t->exportSelectedMessageZfos(); break;
        case 16: _t->exportSelectedAcceptanceInfoZfos(); break;
        case 17: _t->exportSelectedAcceptanceInfoPdfs(); break;
        case 18: _t->exportSelectedEnvelopePdfs(); break;
        case 19: _t->exportSelectedEnvelopeAttachments(); break;
        case 20: _t->createEmailWithZfos(); break;
        case 21: _t->createEmailWithAllAttachments(); break;
        case 22: _t->createEmailContentSelection(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntIdToMsgIdListHash >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgMsgSearch::*)(AcntId , qint64 , QString , int )>(_a, &DlgMsgSearch::focusSelectedMsg, 0))
            return;
    }
}

const QMetaObject *DlgMsgSearch::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgMsgSearch::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12DlgMsgSearchE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgMsgSearch::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void DlgMsgSearch::focusSelectedMsg(AcntId _t1, qint64 _t2, QString _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
