/****************************************************************************
** Meta object code from reading C++ file 'message_db.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/io/message_db.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'message_db.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9MessageDbE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageDb::qt_create_metaobjectdata<qt_meta_tag_ZN9MessageDbE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageDb",
        "changedLocallyRead",
        "",
        "QList<MsgId>",
        "msgIds",
        "read",
        "changedProcessState",
        "MessageDb::MessageProcessState",
        "state",
        "messageInserted",
        "MsgId",
        "msgId",
        "direction",
        "messageDataDeleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changedLocallyRead'
        QtMocHelpers::SignalData<void(const QList<MsgId> &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
        // Signal 'changedProcessState'
        QtMocHelpers::SignalData<void(const QList<MsgId> &, MessageDb::MessageProcessState)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'messageInserted'
        QtMocHelpers::SignalData<void(const MsgId &, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'messageDataDeleted'
        QtMocHelpers::SignalData<void(const MsgId &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageDb, qt_meta_tag_ZN9MessageDbE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageDb::staticMetaObject = { {
    QMetaObject::SuperData::link<DelayedAccessSQLiteDb::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9MessageDbE_t>.metaTypes,
    nullptr
} };

void MessageDb::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageDb *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changedLocallyRead((*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->changedProcessState((*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageDb::MessageProcessState>>(_a[2]))); break;
        case 2: _t->messageInserted((*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->messageDataDeleted((*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageDb::MessageProcessState >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const QList<MsgId> & , bool )>(_a, &MessageDb::changedLocallyRead, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const QList<MsgId> & , MessageDb::MessageProcessState )>(_a, &MessageDb::changedProcessState, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const MsgId & , int )>(_a, &MessageDb::messageInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDb::*)(const MsgId & )>(_a, &MessageDb::messageDataDeleted, 3))
            return;
    }
}

const QMetaObject *MessageDb::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageDb::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MessageDbE_t>.strings))
        return static_cast<void*>(this);
    return DelayedAccessSQLiteDb::qt_metacast(_clname);
}

int MessageDb::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DelayedAccessSQLiteDb::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void MessageDb::changedLocallyRead(const QList<MsgId> & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MessageDb::changedProcessState(const QList<MsgId> & _t1, MessageDb::MessageProcessState _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MessageDb::messageInserted(const MsgId & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void MessageDb::messageDataDeleted(const MsgId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
