/****************************************************************************
** Meta object code from reading C++ file 'dlg_send_message.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/gui/dlg_send_message.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlg_send_message.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14DlgSendMessageE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgSendMessage::qt_create_metaobjectdata<qt_meta_tag_ZN14DlgSendMessageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgSendMessage",
        "done",
        "",
        "resultCode",
        "dmPublishOwnIdStateChanged",
        "state",
        "immDownloadStateChanged",
        "immRecMgmtUploadStateChanged",
        "checkInputFields",
        "addRecipientFromLocalContact",
        "addRecipientFromISDSSearch",
        "addRecipientManually",
        "recipientSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "deleteRecipientEntries",
        "showOptionalFormElements",
        "addAttachmentFile",
        "attachmentSelectionChanged",
        "deleteSelectedAttachmentFiles",
        "openSelectedAttachment",
        "QModelIndex",
        "index",
        "selectTabTextMessage",
        "setAccountInfo",
        "fromComboIdx",
        "sendMessage",
        "collectSendMessageStatus",
        "AcntId",
        "acntId",
        "transactId",
        "result",
        "resultDesc",
        "dbIDRecipient",
        "recipientName",
        "isPDZ",
        "isVodz",
        "dmId",
        "processFlags",
        "recMgmtHierarchyId",
        "pasteAttachmentsFromClipboard",
        "pasteAttachmentsViaFilter",
        "dlgPtr",
        "appendPdf",
        "watchAppendedPdfModification",
        "preview",
        "previewGenerate",
        "QPrinter*",
        "printer",
        "viewPdf",
        "textMessageChanged",
        "watchUploadProgress",
        "uploadTotal",
        "uploadCurrent",
        "watchUploadProgressFinished",
        "QVariant",
        "resultVal"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'done'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Slot 'dmPublishOwnIdStateChanged'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'immDownloadStateChanged'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'immRecMgmtUploadStateChanged'
        QtMocHelpers::SlotData<void(int)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'checkInputFields'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addRecipientFromLocalContact'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addRecipientFromISDSSearch'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addRecipientManually'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'recipientSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'deleteRecipientEntries'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showOptionalFormElements'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addAttachmentFile'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'attachmentSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 13, 15 },
        }}),
        // Slot 'deleteSelectedAttachmentFiles'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openSelectedAttachment'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'openSelectedAttachment'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'selectTabTextMessage'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setAccountInfo'
        QtMocHelpers::SlotData<void(int)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 26 },
        }}),
        // Slot 'sendMessage'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'collectSendMessageStatus'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, int, const QString &, const QString &, const QString &, bool, bool, qint64, int, const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 }, { QMetaType::Int, 32 }, { QMetaType::QString, 33 },
            { QMetaType::QString, 34 }, { QMetaType::QString, 35 }, { QMetaType::Bool, 36 }, { QMetaType::Bool, 37 },
            { QMetaType::LongLong, 38 }, { QMetaType::Int, 39 }, { QMetaType::QString, 40 },
        }}),
        // Slot 'pasteAttachmentsFromClipboard'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pasteAttachmentsViaFilter'
        QtMocHelpers::SlotData<void(QObject *)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 43 },
        }}),
        // Slot 'appendPdf'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchAppendedPdfModification'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'preview'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'previewGenerate'
        QtMocHelpers::SlotData<void(QPrinter *)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 },
        }}),
        // Slot 'viewPdf'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'textMessageChanged'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchUploadProgress'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, qint64, qint64)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 }, { QMetaType::LongLong, 53 }, { QMetaType::LongLong, 54 },
        }}),
        // Slot 'watchUploadProgressFinished'
        QtMocHelpers::SlotData<void(const AcntId &, const QString &, int, const QString &, const QVariant &)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { QMetaType::QString, 31 }, { QMetaType::Int, 32 }, { QMetaType::QString, 33 },
            { 0x80000000 | 56, 57 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgSendMessage, qt_meta_tag_ZN14DlgSendMessageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgSendMessage::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgSendMessageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgSendMessageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14DlgSendMessageE_t>.metaTypes,
    nullptr
} };

void DlgSendMessage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgSendMessage *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->done((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->dmPublishOwnIdStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->immDownloadStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->immRecMgmtUploadStateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->checkInputFields(); break;
        case 5: _t->addRecipientFromLocalContact(); break;
        case 6: _t->addRecipientFromISDSSearch(); break;
        case 7: _t->addRecipientManually(); break;
        case 8: _t->recipientSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 9: _t->deleteRecipientEntries(); break;
        case 10: _t->showOptionalFormElements(); break;
        case 11: _t->addAttachmentFile(); break;
        case 12: _t->attachmentSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 13: _t->deleteSelectedAttachmentFiles(); break;
        case 14: _t->openSelectedAttachment((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 15: _t->openSelectedAttachment(); break;
        case 16: _t->selectTabTextMessage(); break;
        case 17: _t->setAccountInfo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->sendMessage(); break;
        case 19: _t->collectSendMessageStatus((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[11]))); break;
        case 20: _t->pasteAttachmentsFromClipboard(); break;
        case 21: _t->pasteAttachmentsViaFilter((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 22: _t->appendPdf(); break;
        case 23: _t->watchAppendedPdfModification(); break;
        case 24: _t->preview(); break;
        case 25: _t->previewGenerate((*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[1]))); break;
        case 26: _t->viewPdf(); break;
        case 27: _t->textMessageChanged(); break;
        case 28: _t->watchUploadProgress((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4]))); break;
        case 29: _t->watchUploadProgressFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        }
    }
}

const QMetaObject *DlgSendMessage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgSendMessage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14DlgSendMessageE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgSendMessage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    return _id;
}
QT_WARNING_POP
