/********************************************************************************
** Form generated from reading UI file 'dlg_licence.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_LICENCE_H
#define UI_DLG_LICENCE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgLicence
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *textEdit;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgLicence)
    {
        if (DlgLicence->objectName().isEmpty())
            DlgLicence->setObjectName("DlgLicence");
        DlgLicence->resize(480, 420);
        DlgLicence->setMinimumSize(QSize(480, 420));
        verticalLayout = new QVBoxLayout(DlgLicence);
        verticalLayout->setObjectName("verticalLayout");
        textEdit = new QTextEdit(DlgLicence);
        textEdit->setObjectName("textEdit");
        textEdit->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout->addWidget(textEdit);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonBox = new QDialogButtonBox(DlgLicence);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(DlgLicence);

        QMetaObject::connectSlotsByName(DlgLicence);
    } // setupUi

    void retranslateUi(QDialog *DlgLicence)
    {
        DlgLicence->setWindowTitle(QCoreApplication::translate("DlgLicence", "Licence", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgLicence: public Ui_DlgLicence {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_LICENCE_H
