/********************************************************************************
** Form generated from reading UI file 'dlg_correspondence_overview.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CORRESPONDENCE_OVERVIEW_H
#define UI_DLG_CORRESPONDENCE_OVERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCalendarWidget>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgCorrespondenceOverview
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *accountLabel;
    QLabel *accountName;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_5;
    QLabel *outputFormatLabel;
    QComboBox *outputFormatComboBox;
    QSpacerItem *horizontalSpacer_4;
    QLabel *infoLabel;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout;
    QLabel *fromDateLabel;
    QCalendarWidget *fromCalendarWidget;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_2;
    QLabel *toDateLabel;
    QCalendarWidget *toCalendarWidget;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *messageType;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *sentCheckBox;
    QLabel *sentCntLabel;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *receivedCheckBox;
    QLabel *receivedCntLabel;
    QSpacerItem *horizontalSpacer_3;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_8;
    QCheckBox *addTagsCheckBox;
    QCheckBox *colourTagsCheckBox;
    QGroupBox *ExportFiles;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_6;
    QCheckBox *exportZfoCheckBox;
    QCheckBox *exportMessageEnvelopePDFCheckBox;
    QHBoxLayout *horizontalLayout_7;
    QCheckBox *exportDeliveryZfoCheckBox;
    QCheckBox *exportDeliveryPDFCheckBox;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgCorrespondenceOverview)
    {
        if (DlgCorrespondenceOverview->objectName().isEmpty())
            DlgCorrespondenceOverview->setObjectName("DlgCorrespondenceOverview");
        DlgCorrespondenceOverview->resize(462, 550);
        verticalLayout_3 = new QVBoxLayout(DlgCorrespondenceOverview);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        accountLabel = new QLabel(DlgCorrespondenceOverview);
        accountLabel->setObjectName("accountLabel");
        QFont font;
        font.setBold(true);
        accountLabel->setFont(font);

        horizontalLayout->addWidget(accountLabel);

        accountName = new QLabel(DlgCorrespondenceOverview);
        accountName->setObjectName("accountName");
        QFont font1;
        font1.setBold(true);
        font1.setItalic(true);
        accountName->setFont(font1);

        horizontalLayout->addWidget(accountName);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        outputFormatLabel = new QLabel(DlgCorrespondenceOverview);
        outputFormatLabel->setObjectName("outputFormatLabel");

        horizontalLayout_5->addWidget(outputFormatLabel);

        outputFormatComboBox = new QComboBox(DlgCorrespondenceOverview);
        outputFormatComboBox->setObjectName("outputFormatComboBox");

        horizontalLayout_5->addWidget(outputFormatComboBox);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);


        verticalLayout_3->addLayout(horizontalLayout_5);

        infoLabel = new QLabel(DlgCorrespondenceOverview);
        infoLabel->setObjectName("infoLabel");

        verticalLayout_3->addWidget(infoLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        fromDateLabel = new QLabel(DlgCorrespondenceOverview);
        fromDateLabel->setObjectName("fromDateLabel");

        verticalLayout->addWidget(fromDateLabel);

        fromCalendarWidget = new QCalendarWidget(DlgCorrespondenceOverview);
        fromCalendarWidget->setObjectName("fromCalendarWidget");

        verticalLayout->addWidget(fromCalendarWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        toDateLabel = new QLabel(DlgCorrespondenceOverview);
        toDateLabel->setObjectName("toDateLabel");

        verticalLayout_2->addWidget(toDateLabel);

        toCalendarWidget = new QCalendarWidget(DlgCorrespondenceOverview);
        toCalendarWidget->setObjectName("toCalendarWidget");

        verticalLayout_2->addWidget(toCalendarWidget);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);


        horizontalLayout_2->addLayout(verticalLayout_2);


        verticalLayout_3->addLayout(horizontalLayout_2);

        messageType = new QGroupBox(DlgCorrespondenceOverview);
        messageType->setObjectName("messageType");
        verticalLayout_4 = new QVBoxLayout(messageType);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        sentCheckBox = new QCheckBox(messageType);
        sentCheckBox->setObjectName("sentCheckBox");
        sentCheckBox->setChecked(true);

        horizontalLayout_3->addWidget(sentCheckBox);

        sentCntLabel = new QLabel(messageType);
        sentCntLabel->setObjectName("sentCntLabel");

        horizontalLayout_3->addWidget(sentCntLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        receivedCheckBox = new QCheckBox(messageType);
        receivedCheckBox->setObjectName("receivedCheckBox");
        receivedCheckBox->setChecked(true);

        horizontalLayout_4->addWidget(receivedCheckBox);

        receivedCntLabel = new QLabel(messageType);
        receivedCntLabel->setObjectName("receivedCntLabel");

        horizontalLayout_4->addWidget(receivedCntLabel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        verticalLayout_4->addLayout(horizontalLayout_4);


        verticalLayout_3->addWidget(messageType);

        groupBox = new QGroupBox(DlgCorrespondenceOverview);
        groupBox->setObjectName("groupBox");
        verticalLayout_6 = new QVBoxLayout(groupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        addTagsCheckBox = new QCheckBox(groupBox);
        addTagsCheckBox->setObjectName("addTagsCheckBox");

        horizontalLayout_8->addWidget(addTagsCheckBox);

        colourTagsCheckBox = new QCheckBox(groupBox);
        colourTagsCheckBox->setObjectName("colourTagsCheckBox");

        horizontalLayout_8->addWidget(colourTagsCheckBox);


        verticalLayout_6->addLayout(horizontalLayout_8);


        verticalLayout_3->addWidget(groupBox);

        ExportFiles = new QGroupBox(DlgCorrespondenceOverview);
        ExportFiles->setObjectName("ExportFiles");
        verticalLayout_5 = new QVBoxLayout(ExportFiles);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        exportZfoCheckBox = new QCheckBox(ExportFiles);
        exportZfoCheckBox->setObjectName("exportZfoCheckBox");

        horizontalLayout_6->addWidget(exportZfoCheckBox);

        exportMessageEnvelopePDFCheckBox = new QCheckBox(ExportFiles);
        exportMessageEnvelopePDFCheckBox->setObjectName("exportMessageEnvelopePDFCheckBox");

        horizontalLayout_6->addWidget(exportMessageEnvelopePDFCheckBox);


        verticalLayout_5->addLayout(horizontalLayout_6);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        exportDeliveryZfoCheckBox = new QCheckBox(ExportFiles);
        exportDeliveryZfoCheckBox->setObjectName("exportDeliveryZfoCheckBox");

        horizontalLayout_7->addWidget(exportDeliveryZfoCheckBox);

        exportDeliveryPDFCheckBox = new QCheckBox(ExportFiles);
        exportDeliveryPDFCheckBox->setObjectName("exportDeliveryPDFCheckBox");

        horizontalLayout_7->addWidget(exportDeliveryPDFCheckBox);


        verticalLayout_5->addLayout(horizontalLayout_7);


        verticalLayout_3->addWidget(ExportFiles);

        buttonBox = new QDialogButtonBox(DlgCorrespondenceOverview);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setEnabled(true);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(buttonBox);

        QWidget::setTabOrder(outputFormatComboBox, fromCalendarWidget);
        QWidget::setTabOrder(fromCalendarWidget, toCalendarWidget);
        QWidget::setTabOrder(toCalendarWidget, sentCheckBox);
        QWidget::setTabOrder(sentCheckBox, receivedCheckBox);
        QWidget::setTabOrder(receivedCheckBox, addTagsCheckBox);
        QWidget::setTabOrder(addTagsCheckBox, colourTagsCheckBox);
        QWidget::setTabOrder(colourTagsCheckBox, exportZfoCheckBox);
        QWidget::setTabOrder(exportZfoCheckBox, exportMessageEnvelopePDFCheckBox);
        QWidget::setTabOrder(exportMessageEnvelopePDFCheckBox, exportDeliveryZfoCheckBox);
        QWidget::setTabOrder(exportDeliveryZfoCheckBox, exportDeliveryPDFCheckBox);

        retranslateUi(DlgCorrespondenceOverview);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgCorrespondenceOverview, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgCorrespondenceOverview, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgCorrespondenceOverview);
    } // setupUi

    void retranslateUi(QDialog *DlgCorrespondenceOverview)
    {
        DlgCorrespondenceOverview->setWindowTitle(QCoreApplication::translate("DlgCorrespondenceOverview", "Correspondence overview", nullptr));
        accountLabel->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Data box:", nullptr));
        accountName->setText(QString());
        outputFormatLabel->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Output format:", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Select the dates and types of messages to export.", nullptr));
        fromDateLabel->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "From date:", nullptr));
        toDateLabel->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "To date:", nullptr));
        messageType->setTitle(QCoreApplication::translate("DlgCorrespondenceOverview", "Message type:", nullptr));
        sentCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Sent", nullptr));
        sentCntLabel->setText(QString());
        receivedCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Received", nullptr));
        receivedCntLabel->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("DlgCorrespondenceOverview", "HTML overview:", nullptr));
        addTagsCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Add tags", nullptr));
        colourTagsCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "Colour tags", nullptr));
        ExportFiles->setTitle(QCoreApplication::translate("DlgCorrespondenceOverview", "Export selected messages also as:", nullptr));
        exportZfoCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "ZFO message files", nullptr));
        exportMessageEnvelopePDFCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "PDF envelope files", nullptr));
        exportDeliveryZfoCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "ZFO acceptance info files", nullptr));
        exportDeliveryPDFCheckBox->setText(QCoreApplication::translate("DlgCorrespondenceOverview", "PDF acceptance info files", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgCorrespondenceOverview: public Ui_DlgCorrespondenceOverview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CORRESPONDENCE_OVERVIEW_H
