/********************************************************************************
** Form generated from reading UI file 'dlg_message_check.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_MESSAGE_CHECK_H
#define UI_DLG_MESSAGE_CHECK_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgMessageCheck
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *taskLabel;
    QProgressBar *singleProgressBar;
    QLabel *singleResultLabel;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgMessageCheck)
    {
        if (DlgMessageCheck->objectName().isEmpty())
            DlgMessageCheck->setObjectName("DlgMessageCheck");
        DlgMessageCheck->resize(400, 120);
        verticalLayout = new QVBoxLayout(DlgMessageCheck);
        verticalLayout->setObjectName("verticalLayout");
        taskLabel = new QLabel(DlgMessageCheck);
        taskLabel->setObjectName("taskLabel");

        verticalLayout->addWidget(taskLabel);

        singleProgressBar = new QProgressBar(DlgMessageCheck);
        singleProgressBar->setObjectName("singleProgressBar");
        singleProgressBar->setValue(24);

        verticalLayout->addWidget(singleProgressBar);

        singleResultLabel = new QLabel(DlgMessageCheck);
        singleResultLabel->setObjectName("singleResultLabel");
        singleResultLabel->setWordWrap(true);

        verticalLayout->addWidget(singleResultLabel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(DlgMessageCheck);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgMessageCheck);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgMessageCheck, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgMessageCheck, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgMessageCheck);
    } // setupUi

    void retranslateUi(QDialog *DlgMessageCheck)
    {
        taskLabel->setText(QCoreApplication::translate("DlgMessageCheck", "n/a", nullptr));
        singleResultLabel->setText(QCoreApplication::translate("DlgMessageCheck", "n/a", nullptr));
        (void)DlgMessageCheck;
    } // retranslateUi

};

namespace Ui {
    class DlgMessageCheck: public Ui_DlgMessageCheck {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_MESSAGE_CHECK_H
