/********************************************************************************
** Form generated from reading UI file 'dlg_about.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_ABOUT_H
#define UI_DLG_ABOUT_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgAbout
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab_0_about;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout1;
    QLabel *logo;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout_4;
    QLabel *labelTitle;
    QLabel *labelDescription;
    QLabel *labelVersion;
    QHBoxLayout *horizontalLayout_5;
    QLabel *labelVersionCheck;
    QPushButton *downloadButton;
    QLabel *labelDownload;
    QPushButton *updateButton;
    QSpacerItem *horizontalSpacer_3;
    QLabel *labelDonate;
    QSpacerItem *verticalSpacer;
    QWidget *tab_1_development;
    QVBoxLayout *verticalLayout_5;
    QLabel *labelDevelopment;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *showNewsButton;
    QFrame *line;
    QLabel *labelLibs;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2_licence;
    QVBoxLayout *verticalLayout_6;
    QLabel *labelDisclaimer;
    QTextEdit *textEditLicence;
    QWidget *tab_3_donate;
    QVBoxLayout *verticalLayout_7;
    QLabel *donateHtmlText;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *donateButton;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout2;
    QLabel *homepage;
    QLabel *userguide;
    QLabel *faq;
    QLabel *bestpractice;
    QFrame *line_2;
    QLabel *labelCopy;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgAbout)
    {
        if (DlgAbout->objectName().isEmpty())
            DlgAbout->setObjectName("DlgAbout");
        DlgAbout->resize(480, 440);
        DlgAbout->setMinimumSize(QSize(620, 530));
        verticalLayout = new QVBoxLayout(DlgAbout);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(DlgAbout);
        tabWidget->setObjectName("tabWidget");
        tab_0_about = new QWidget();
        tab_0_about->setObjectName("tab_0_about");
        verticalLayout_3 = new QVBoxLayout(tab_0_about);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout1 = new QHBoxLayout();
        horizontalLayout1->setSpacing(30);
        horizontalLayout1->setObjectName("horizontalLayout1");
        logo = new QLabel(tab_0_about);
        logo->setObjectName("logo");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(logo->sizePolicy().hasHeightForWidth());
        logo->setSizePolicy(sizePolicy);
        logo->setPixmap(QPixmap(QString::fromUtf8(":/icons/png_app_logo/128x128/datovka.png")));
        logo->setScaledContents(false);

        horizontalLayout1->addWidget(logo);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        labelTitle = new QLabel(tab_0_about);
        labelTitle->setObjectName("labelTitle");
        QFont font;
        font.setPointSize(22);
        font.setBold(true);
        labelTitle->setFont(font);
        labelTitle->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        verticalLayout_4->addWidget(labelTitle);

        labelDescription = new QLabel(tab_0_about);
        labelDescription->setObjectName("labelDescription");
        labelDescription->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        verticalLayout_4->addWidget(labelDescription);


        verticalLayout_2->addLayout(verticalLayout_4);

        labelVersion = new QLabel(tab_0_about);
        labelVersion->setObjectName("labelVersion");
        QFont font1;
        font1.setBold(true);
        labelVersion->setFont(font1);
        labelVersion->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        verticalLayout_2->addWidget(labelVersion);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        labelVersionCheck = new QLabel(tab_0_about);
        labelVersionCheck->setObjectName("labelVersionCheck");

        horizontalLayout_5->addWidget(labelVersionCheck);

        downloadButton = new QPushButton(tab_0_about);
        downloadButton->setObjectName("downloadButton");

        horizontalLayout_5->addWidget(downloadButton);

        labelDownload = new QLabel(tab_0_about);
        labelDownload->setObjectName("labelDownload");

        horizontalLayout_5->addWidget(labelDownload);

        updateButton = new QPushButton(tab_0_about);
        updateButton->setObjectName("updateButton");
        updateButton->setFont(font1);

        horizontalLayout_5->addWidget(updateButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);


        verticalLayout_2->addLayout(horizontalLayout_5);


        horizontalLayout1->addLayout(verticalLayout_2);


        verticalLayout_3->addLayout(horizontalLayout1);

        labelDonate = new QLabel(tab_0_about);
        labelDonate->setObjectName("labelDonate");
        labelDonate->setEnabled(true);
        labelDonate->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        labelDonate->setWordWrap(true);
        labelDonate->setOpenExternalLinks(true);
        labelDonate->setTextInteractionFlags(Qt::TextBrowserInteraction);

        verticalLayout_3->addWidget(labelDonate);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabWidget->addTab(tab_0_about, QString());
        tab_1_development = new QWidget();
        tab_1_development->setObjectName("tab_1_development");
        verticalLayout_5 = new QVBoxLayout(tab_1_development);
        verticalLayout_5->setObjectName("verticalLayout_5");
        labelDevelopment = new QLabel(tab_1_development);
        labelDevelopment->setObjectName("labelDevelopment");
        labelDevelopment->setTextFormat(Qt::RichText);
        labelDevelopment->setAlignment(Qt::AlignCenter);
        labelDevelopment->setWordWrap(true);
        labelDevelopment->setOpenExternalLinks(true);

        verticalLayout_5->addWidget(labelDevelopment);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(-1, 10, -1, -1);
        showNewsButton = new QPushButton(tab_1_development);
        showNewsButton->setObjectName("showNewsButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(showNewsButton->sizePolicy().hasHeightForWidth());
        showNewsButton->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(showNewsButton);


        verticalLayout_5->addLayout(horizontalLayout_3);

        line = new QFrame(tab_1_development);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line);

        labelLibs = new QLabel(tab_1_development);
        labelLibs->setObjectName("labelLibs");
        labelLibs->setEnabled(true);
        labelLibs->setMinimumSize(QSize(0, 0));
        labelLibs->setAlignment(Qt::AlignCenter);
        labelLibs->setWordWrap(true);
        labelLibs->setTextInteractionFlags(Qt::TextBrowserInteraction);

        verticalLayout_5->addWidget(labelLibs);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_2);

        tabWidget->addTab(tab_1_development, QString());
        tab_2_licence = new QWidget();
        tab_2_licence->setObjectName("tab_2_licence");
        verticalLayout_6 = new QVBoxLayout(tab_2_licence);
        verticalLayout_6->setObjectName("verticalLayout_6");
        labelDisclaimer = new QLabel(tab_2_licence);
        labelDisclaimer->setObjectName("labelDisclaimer");
        labelDisclaimer->setAlignment(Qt::AlignCenter);
        labelDisclaimer->setWordWrap(true);

        verticalLayout_6->addWidget(labelDisclaimer);

        textEditLicence = new QTextEdit(tab_2_licence);
        textEditLicence->setObjectName("textEditLicence");
        textEditLicence->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_6->addWidget(textEditLicence);

        tabWidget->addTab(tab_2_licence, QString());
        tab_3_donate = new QWidget();
        tab_3_donate->setObjectName("tab_3_donate");
        verticalLayout_7 = new QVBoxLayout(tab_3_donate);
        verticalLayout_7->setObjectName("verticalLayout_7");
        donateHtmlText = new QLabel(tab_3_donate);
        donateHtmlText->setObjectName("donateHtmlText");
        donateHtmlText->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        donateHtmlText->setWordWrap(true);

        verticalLayout_7->addWidget(donateHtmlText);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(-1, 10, -1, -1);
        donateButton = new QPushButton(tab_3_donate);
        donateButton->setObjectName("donateButton");
        sizePolicy1.setHeightForWidth(donateButton->sizePolicy().hasHeightForWidth());
        donateButton->setSizePolicy(sizePolicy1);
        donateButton->setFont(font1);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/3party/tick_32.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        donateButton->setIcon(icon);
        donateButton->setIconSize(QSize(80, 30));

        horizontalLayout_2->addWidget(donateButton);


        verticalLayout_7->addLayout(horizontalLayout_2);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_3);

        tabWidget->addTab(tab_3_donate, QString());

        verticalLayout->addWidget(tabWidget);

        horizontalLayout2 = new QHBoxLayout();
        horizontalLayout2->setObjectName("horizontalLayout2");
        homepage = new QLabel(DlgAbout);
        homepage->setObjectName("homepage");
        homepage->setTextFormat(Qt::RichText);
        homepage->setAlignment(Qt::AlignCenter);
        homepage->setOpenExternalLinks(true);
        homepage->setTextInteractionFlags(Qt::TextBrowserInteraction);

        horizontalLayout2->addWidget(homepage);

        userguide = new QLabel(DlgAbout);
        userguide->setObjectName("userguide");
        userguide->setTextFormat(Qt::RichText);
        userguide->setAlignment(Qt::AlignCenter);
        userguide->setOpenExternalLinks(true);
        userguide->setTextInteractionFlags(Qt::TextBrowserInteraction);

        horizontalLayout2->addWidget(userguide);

        faq = new QLabel(DlgAbout);
        faq->setObjectName("faq");
        faq->setTextFormat(Qt::RichText);
        faq->setAlignment(Qt::AlignCenter);
        faq->setOpenExternalLinks(true);
        faq->setTextInteractionFlags(Qt::TextBrowserInteraction);

        horizontalLayout2->addWidget(faq);

        bestpractice = new QLabel(DlgAbout);
        bestpractice->setObjectName("bestpractice");
        bestpractice->setTextFormat(Qt::RichText);
        bestpractice->setAlignment(Qt::AlignCenter);
        bestpractice->setOpenExternalLinks(true);
        bestpractice->setTextInteractionFlags(Qt::TextBrowserInteraction);

        horizontalLayout2->addWidget(bestpractice);


        verticalLayout->addLayout(horizontalLayout2);

        line_2 = new QFrame(DlgAbout);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        labelCopy = new QLabel(DlgAbout);
        labelCopy->setObjectName("labelCopy");
        QFont font2;
        font2.setKerning(false);
        labelCopy->setFont(font2);
        labelCopy->setTextFormat(Qt::RichText);
        labelCopy->setAlignment(Qt::AlignCenter);
        labelCopy->setOpenExternalLinks(true);
        labelCopy->setTextInteractionFlags(Qt::TextBrowserInteraction);

        verticalLayout->addWidget(labelCopy);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonBox = new QDialogButtonBox(DlgAbout);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(DlgAbout);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgAbout);
    } // setupUi

    void retranslateUi(QDialog *DlgAbout)
    {
        DlgAbout->setWindowTitle(QCoreApplication::translate("DlgAbout", "About Datovka", nullptr));
        logo->setText(QString());
        labelTitle->setText(QCoreApplication::translate("DlgAbout", "Datovka", nullptr));
        labelDescription->setText(QCoreApplication::translate("DlgAbout", "Description", nullptr));
        labelVersion->setText(QCoreApplication::translate("DlgAbout", "Version", nullptr));
        labelVersionCheck->setText(QCoreApplication::translate("DlgAbout", "Checking...", nullptr));
        downloadButton->setText(QCoreApplication::translate("DlgAbout", "Download", nullptr));
        labelDownload->setText(QCoreApplication::translate("DlgAbout", "Downloading...", nullptr));
        updateButton->setText(QCoreApplication::translate("DlgAbout", "Update", nullptr));
        labelDonate->setText(QCoreApplication::translate("DlgAbout", "Donate", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_0_about), QCoreApplication::translate("DlgAbout", "About", nullptr));
        labelDevelopment->setText(QCoreApplication::translate("DlgAbout", "n/a", nullptr));
        showNewsButton->setText(QCoreApplication::translate("DlgAbout", "Show News", nullptr));
        labelLibs->setText(QCoreApplication::translate("DlgAbout", "n/a", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_1_development), QCoreApplication::translate("DlgAbout", "Development", nullptr));
        labelDisclaimer->setText(QCoreApplication::translate("DlgAbout", "n/a", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2_licence), QCoreApplication::translate("DlgAbout", "Licence", nullptr));
        donateHtmlText->setText(QCoreApplication::translate("DlgAbout", "n/a", nullptr));
        donateButton->setText(QCoreApplication::translate("DlgAbout", "Donate Now", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3_donate), QCoreApplication::translate("DlgAbout", "Support Us", nullptr));
        homepage->setText(QCoreApplication::translate("DlgAbout", "Home Page", nullptr));
        userguide->setText(QCoreApplication::translate("DlgAbout", "User Guide", nullptr));
        faq->setText(QCoreApplication::translate("DlgAbout", "FAQ", nullptr));
        bestpractice->setText(QCoreApplication::translate("DlgAbout", "Best Practice", nullptr));
        labelCopy->setText(QCoreApplication::translate("DlgAbout", "Copyright \302\251", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgAbout: public Ui_DlgAbout {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_ABOUT_H
