/********************************************************************************
** Form generated from reading UI file 'dlg_tool_bar.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TOOL_BAR_H
#define UI_DLG_TOOL_BAR_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgToolBar
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_4;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    QLabel *categoryLabel;
    QComboBox *categoryComboBox;
    QGroupBox *availableGroup;
    QVBoxLayout *verticalLayout_2;
    QListView *availableOperationsView;
    QLabel *descriptionLabel;
    QPlainTextEdit *descriptionText;
    QVBoxLayout *assignmentButonGroup;
    QSpacerItem *verticalSpacer;
    QPushButton *assignButton;
    QPushButton *removeButton;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_7;
    QRadioButton *iconOnlyRadio;
    QRadioButton *textBesideIconRadio;
    QRadioButton *textUnderIconRadio;
    QGroupBox *assignedGroup;
    QVBoxLayout *verticalLayout_3;
    QListView *assignedOperationsView;
    QLabel *customiseLabel;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *insertSeparatorButton;
    QSpacerItem *horizontalSpacer;
    QPushButton *defaultsButton;
    QVBoxLayout *verticalLayout_5;
    QSpacerItem *verticalSpacer_3;
    QPushButton *upButton;
    QPushButton *downButton;
    QSpacerItem *verticalSpacer_4;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgToolBar)
    {
        if (DlgToolBar->objectName().isEmpty())
            DlgToolBar->setObjectName("DlgToolBar");
        DlgToolBar->resize(800, 480);
        verticalLayout = new QVBoxLayout(DlgToolBar);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        filterLabel = new QLabel(DlgToolBar);
        filterLabel->setObjectName("filterLabel");

        verticalLayout_4->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgToolBar);
        filterLine->setObjectName("filterLine");

        verticalLayout_4->addWidget(filterLine);

        categoryLabel = new QLabel(DlgToolBar);
        categoryLabel->setObjectName("categoryLabel");

        verticalLayout_4->addWidget(categoryLabel);

        categoryComboBox = new QComboBox(DlgToolBar);
        categoryComboBox->setObjectName("categoryComboBox");

        verticalLayout_4->addWidget(categoryComboBox);

        availableGroup = new QGroupBox(DlgToolBar);
        availableGroup->setObjectName("availableGroup");
        verticalLayout_2 = new QVBoxLayout(availableGroup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        availableOperationsView = new QListView(availableGroup);
        availableOperationsView->setObjectName("availableOperationsView");

        verticalLayout_2->addWidget(availableOperationsView);


        verticalLayout_4->addWidget(availableGroup);

        descriptionLabel = new QLabel(DlgToolBar);
        descriptionLabel->setObjectName("descriptionLabel");

        verticalLayout_4->addWidget(descriptionLabel);

        descriptionText = new QPlainTextEdit(DlgToolBar);
        descriptionText->setObjectName("descriptionText");

        verticalLayout_4->addWidget(descriptionText);


        horizontalLayout->addLayout(verticalLayout_4);

        assignmentButonGroup = new QVBoxLayout();
        assignmentButonGroup->setObjectName("assignmentButonGroup");
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        assignmentButonGroup->addItem(verticalSpacer);

        assignButton = new QPushButton(DlgToolBar);
        assignButton->setObjectName("assignButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/png_app_logo/16x16/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        assignButton->setIcon(icon);

        assignmentButonGroup->addWidget(assignButton);

        removeButton = new QPushButton(DlgToolBar);
        removeButton->setObjectName("removeButton");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/png/png_app_logo/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        removeButton->setIcon(icon1);

        assignmentButonGroup->addWidget(removeButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        assignmentButonGroup->addItem(verticalSpacer_2);


        horizontalLayout->addLayout(assignmentButonGroup);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName("verticalLayout_6");
        groupBox = new QGroupBox(DlgToolBar);
        groupBox->setObjectName("groupBox");
        verticalLayout_7 = new QVBoxLayout(groupBox);
        verticalLayout_7->setObjectName("verticalLayout_7");
        iconOnlyRadio = new QRadioButton(groupBox);
        iconOnlyRadio->setObjectName("iconOnlyRadio");

        verticalLayout_7->addWidget(iconOnlyRadio);

        textBesideIconRadio = new QRadioButton(groupBox);
        textBesideIconRadio->setObjectName("textBesideIconRadio");

        verticalLayout_7->addWidget(textBesideIconRadio);

        textUnderIconRadio = new QRadioButton(groupBox);
        textUnderIconRadio->setObjectName("textUnderIconRadio");

        verticalLayout_7->addWidget(textUnderIconRadio);


        verticalLayout_6->addWidget(groupBox);

        assignedGroup = new QGroupBox(DlgToolBar);
        assignedGroup->setObjectName("assignedGroup");
        verticalLayout_3 = new QVBoxLayout(assignedGroup);
        verticalLayout_3->setObjectName("verticalLayout_3");
        assignedOperationsView = new QListView(assignedGroup);
        assignedOperationsView->setObjectName("assignedOperationsView");

        verticalLayout_3->addWidget(assignedOperationsView);

        customiseLabel = new QLabel(assignedGroup);
        customiseLabel->setObjectName("customiseLabel");

        verticalLayout_3->addWidget(customiseLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        insertSeparatorButton = new QPushButton(assignedGroup);
        insertSeparatorButton->setObjectName("insertSeparatorButton");

        horizontalLayout_2->addWidget(insertSeparatorButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        defaultsButton = new QPushButton(assignedGroup);
        defaultsButton->setObjectName("defaultsButton");

        horizontalLayout_2->addWidget(defaultsButton);


        verticalLayout_3->addLayout(horizontalLayout_2);


        verticalLayout_6->addWidget(assignedGroup);


        horizontalLayout->addLayout(verticalLayout_6);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_3);

        upButton = new QPushButton(DlgToolBar);
        upButton->setObjectName("upButton");
        upButton->setIcon(icon);

        verticalLayout_5->addWidget(upButton);

        downButton = new QPushButton(DlgToolBar);
        downButton->setObjectName("downButton");
        downButton->setIcon(icon);

        verticalLayout_5->addWidget(downButton);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_4);


        horizontalLayout->addLayout(verticalLayout_5);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(DlgToolBar);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(filterLine, categoryComboBox);
        QWidget::setTabOrder(categoryComboBox, availableOperationsView);
        QWidget::setTabOrder(availableOperationsView, assignButton);
        QWidget::setTabOrder(assignButton, descriptionText);
        QWidget::setTabOrder(descriptionText, iconOnlyRadio);
        QWidget::setTabOrder(iconOnlyRadio, textBesideIconRadio);
        QWidget::setTabOrder(textBesideIconRadio, textUnderIconRadio);
        QWidget::setTabOrder(textUnderIconRadio, assignedOperationsView);
        QWidget::setTabOrder(assignedOperationsView, removeButton);
        QWidget::setTabOrder(removeButton, upButton);
        QWidget::setTabOrder(upButton, downButton);
        QWidget::setTabOrder(downButton, insertSeparatorButton);
        QWidget::setTabOrder(insertSeparatorButton, defaultsButton);

        retranslateUi(DlgToolBar);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgToolBar, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgToolBar, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgToolBar);
    } // setupUi

    void retranslateUi(QDialog *DlgToolBar)
    {
        DlgToolBar->setWindowTitle(QCoreApplication::translate("DlgToolBar", "Tool Bar", nullptr));
#if QT_CONFIG(tooltip)
        DlgToolBar->setToolTip(QCoreApplication::translate("DlgToolBar", "Assign selected available actions to tool bar.", nullptr));
#endif // QT_CONFIG(tooltip)
        filterLabel->setText(QCoreApplication::translate("DlgToolBar", "Filter", nullptr));
        categoryLabel->setText(QCoreApplication::translate("DlgToolBar", "Category", nullptr));
        availableGroup->setTitle(QCoreApplication::translate("DlgToolBar", "Available Operations", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("DlgToolBar", "Description", nullptr));
        assignButton->setText(QString());
#if QT_CONFIG(tooltip)
        removeButton->setToolTip(QCoreApplication::translate("DlgToolBar", "Cancel selected assignment.", nullptr));
#endif // QT_CONFIG(tooltip)
        removeButton->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("DlgToolBar", "Style of Buttons", nullptr));
#if QT_CONFIG(tooltip)
        iconOnlyRadio->setToolTip(QCoreApplication::translate("DlgToolBar", "Only icons will be displayed in the toolbar.", nullptr));
#endif // QT_CONFIG(tooltip)
        iconOnlyRadio->setText(QCoreApplication::translate("DlgToolBar", "Only display the icon", nullptr));
#if QT_CONFIG(tooltip)
        textBesideIconRadio->setToolTip(QCoreApplication::translate("DlgToolBar", "Icons and text beside them will be displayed in the toolbar.", nullptr));
#endif // QT_CONFIG(tooltip)
        textBesideIconRadio->setText(QCoreApplication::translate("DlgToolBar", "The text appears beside the icon", nullptr));
#if QT_CONFIG(tooltip)
        textUnderIconRadio->setToolTip(QCoreApplication::translate("DlgToolBar", "Icons and text under them will be displayed in the toolbar.", nullptr));
#endif // QT_CONFIG(tooltip)
        textUnderIconRadio->setText(QCoreApplication::translate("DlgToolBar", "The text appears under the icon", nullptr));
        assignedGroup->setTitle(QCoreApplication::translate("DlgToolBar", "Assigned Operations", nullptr));
        customiseLabel->setText(QCoreApplication::translate("DlgToolBar", "Customise", nullptr));
        insertSeparatorButton->setText(QCoreApplication::translate("DlgToolBar", "Insert Separator", nullptr));
        defaultsButton->setText(QCoreApplication::translate("DlgToolBar", "Defaults", nullptr));
        upButton->setText(QString());
        downButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DlgToolBar: public Ui_DlgToolBar {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TOOL_BAR_H
