/****************************************************************************
** Meta object code from reading C++ file 'account_container.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/datovka_shared/settings/account_container.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'account_container.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13AcntContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto AcntContainer::qt_create_metaobjectdata<qt_meta_tag_ZN13AcntContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AcntContainer",
        "accountAboutToBeAdded",
        "",
        "AcntId",
        "acntId",
        "index",
        "accountAdded",
        "accountAboutToBeRemoved",
        "accountRemoved",
        "accountsAboutToBeMoved",
        "srcFrst",
        "srcLst",
        "dst",
        "accountsMoved",
        "accountUsernameChanged",
        "oldId",
        "newId",
        "accountDataChanged",
        "contentAboutToBeReset",
        "contentReset",
        "contentChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accountAboutToBeAdded'
        QtMocHelpers::SignalData<void(const AcntId &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'accountAdded'
        QtMocHelpers::SignalData<void(const AcntId &, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'accountAboutToBeRemoved'
        QtMocHelpers::SignalData<void(const AcntId &, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'accountRemoved'
        QtMocHelpers::SignalData<void(const AcntId &, int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'accountsAboutToBeMoved'
        QtMocHelpers::SignalData<void(int, int, int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'accountsMoved'
        QtMocHelpers::SignalData<void(int, int, int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'accountUsernameChanged'
        QtMocHelpers::SignalData<void(const AcntId &, const AcntId &, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 15 }, { 0x80000000 | 3, 16 }, { QMetaType::Int, 5 },
        }}),
        // Signal 'accountDataChanged'
        QtMocHelpers::SignalData<void(const AcntId &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'contentAboutToBeReset'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contentReset'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contentChanged'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AcntContainer, qt_meta_tag_ZN13AcntContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AcntContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AcntContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AcntContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13AcntContainerE_t>.metaTypes,
    nullptr
} };

void AcntContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AcntContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accountAboutToBeAdded((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->accountAdded((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->accountAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->accountRemoved((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 4: _t->accountsAboutToBeMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->accountsMoved((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 6: _t->accountUsernameChanged((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 7: _t->accountDataChanged((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1]))); break;
        case 8: _t->contentAboutToBeReset(); break;
        case 9: _t->contentReset(); break;
        case 10: _t->contentChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & , int )>(_a, &AcntContainer::accountAboutToBeAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & , int )>(_a, &AcntContainer::accountAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & , int )>(_a, &AcntContainer::accountAboutToBeRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & , int )>(_a, &AcntContainer::accountRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(int , int , int )>(_a, &AcntContainer::accountsAboutToBeMoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(int , int , int )>(_a, &AcntContainer::accountsMoved, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & , const AcntId & , int )>(_a, &AcntContainer::accountUsernameChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)(const AcntId & )>(_a, &AcntContainer::accountDataChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)()>(_a, &AcntContainer::contentAboutToBeReset, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)()>(_a, &AcntContainer::contentReset, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (AcntContainer::*)()>(_a, &AcntContainer::contentChanged, 10))
            return;
    }
}

const QMetaObject *AcntContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AcntContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13AcntContainerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AcntContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void AcntContainer::accountAboutToBeAdded(const AcntId & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void AcntContainer::accountAdded(const AcntId & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void AcntContainer::accountAboutToBeRemoved(const AcntId & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void AcntContainer::accountRemoved(const AcntId & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void AcntContainer::accountsAboutToBeMoved(int _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3);
}

// SIGNAL 5
void AcntContainer::accountsMoved(int _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void AcntContainer::accountUsernameChanged(const AcntId & _t1, const AcntId & _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void AcntContainer::accountDataChanged(const AcntId & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void AcntContainer::contentAboutToBeReset()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void AcntContainer::contentReset()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void AcntContainer::contentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
