/********************************************************************************
** Form generated from reading UI file 'dlg_user_ext2.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_USER_EXT2_H
#define UI_DLG_USER_EXT2_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgUserExt2
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *userGroupBox;
    QFormLayout *formLayout;
    QLabel *namesLabel;
    QLineEdit *namesLine;
    QLabel *surnameLabel;
    QLineEdit *surnameLine;
    QLabel *birthDateLabel;
    QDateEdit *birthDateEdit;
    QGroupBox *addressGroupBox;
    QFormLayout *formLayout_2;
    QLabel *streetLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *streetLine;
    QLabel *noInStreetLabel;
    QLineEdit *noInStreetLine;
    QLabel *districtLabel;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *districtLine;
    QLabel *conscriptNoLabel;
    QLineEdit *conscriptNoLine;
    QLabel *cityLabel;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *cityLine;
    QLabel *zipCodeLabel;
    QLineEdit *zipCodeLine;
    QLabel *stateLabel;
    QHBoxLayout *horizontalLayout_5;
    QComboBox *stateComboBox;
    QLineEdit *stateLine;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer;
    QLabel *evidenceNoLabel;
    QLineEdit *evidenceNoLine;
    QGroupBox *caAddressGroupBox;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *useResidenceRadioButton;
    QRadioButton *useFirmRadioButton;
    QRadioButton *useCaRadioButton;
    QWidget *caAddressWidget;
    QFormLayout *formLayout_5;
    QLabel *caStreetLabel;
    QLineEdit *caStreetLine;
    QLabel *caCityLabel;
    QHBoxLayout *horizontalLayout_4;
    QLineEdit *caCityLine;
    QLabel *caZipCodeLabel;
    QLineEdit *caZipCodeLine;
    QLabel *caStateLabel;
    QHBoxLayout *horizontalLayout_6;
    QComboBox *caStateComboBox;
    QLineEdit *caStateLine;
    QGroupBox *permissionsGroupBox;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout_4;
    QLabel *userTypeLabel;
    QComboBox *userTypeComboBox;
    QGridLayout *gridLayout;
    QCheckBox *readNonPersonalCheckBox;
    QCheckBox *readAllCheckBox;
    QCheckBox *createDmCheckBox;
    QCheckBox *viewInfoCheckBox;
    QCheckBox *searchDbCheckBox;
    QCheckBox *erasevaultCheckBox;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgUserExt2)
    {
        if (DlgUserExt2->objectName().isEmpty())
            DlgUserExt2->setObjectName("DlgUserExt2");
        DlgUserExt2->resize(550, 730);
        verticalLayout = new QVBoxLayout(DlgUserExt2);
        verticalLayout->setObjectName("verticalLayout");
        userGroupBox = new QGroupBox(DlgUserExt2);
        userGroupBox->setObjectName("userGroupBox");
        formLayout = new QFormLayout(userGroupBox);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        namesLabel = new QLabel(userGroupBox);
        namesLabel->setObjectName("namesLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, namesLabel);

        namesLine = new QLineEdit(userGroupBox);
        namesLine->setObjectName("namesLine");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, namesLine);

        surnameLabel = new QLabel(userGroupBox);
        surnameLabel->setObjectName("surnameLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, surnameLabel);

        surnameLine = new QLineEdit(userGroupBox);
        surnameLine->setObjectName("surnameLine");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, surnameLine);

        birthDateLabel = new QLabel(userGroupBox);
        birthDateLabel->setObjectName("birthDateLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, birthDateLabel);

        birthDateEdit = new QDateEdit(userGroupBox);
        birthDateEdit->setObjectName("birthDateEdit");
        birthDateEdit->setDisplayFormat(QString::fromUtf8("dd.MM.yyyy"));
        birthDateEdit->setCalendarPopup(true);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, birthDateEdit);


        verticalLayout->addWidget(userGroupBox);

        addressGroupBox = new QGroupBox(DlgUserExt2);
        addressGroupBox->setObjectName("addressGroupBox");
        formLayout_2 = new QFormLayout(addressGroupBox);
        formLayout_2->setObjectName("formLayout_2");
        formLayout_2->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_2->setLabelAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        streetLabel = new QLabel(addressGroupBox);
        streetLabel->setObjectName("streetLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, streetLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        streetLine = new QLineEdit(addressGroupBox);
        streetLine->setObjectName("streetLine");

        horizontalLayout->addWidget(streetLine);

        noInStreetLabel = new QLabel(addressGroupBox);
        noInStreetLabel->setObjectName("noInStreetLabel");

        horizontalLayout->addWidget(noInStreetLabel);

        noInStreetLine = new QLineEdit(addressGroupBox);
        noInStreetLine->setObjectName("noInStreetLine");
        noInStreetLine->setMaxLength(8);

        horizontalLayout->addWidget(noInStreetLine);


        formLayout_2->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        districtLabel = new QLabel(addressGroupBox);
        districtLabel->setObjectName("districtLabel");

        formLayout_2->setWidget(1, QFormLayout::ItemRole::LabelRole, districtLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        districtLine = new QLineEdit(addressGroupBox);
        districtLine->setObjectName("districtLine");

        horizontalLayout_2->addWidget(districtLine);

        conscriptNoLabel = new QLabel(addressGroupBox);
        conscriptNoLabel->setObjectName("conscriptNoLabel");

        horizontalLayout_2->addWidget(conscriptNoLabel);

        conscriptNoLine = new QLineEdit(addressGroupBox);
        conscriptNoLine->setObjectName("conscriptNoLine");
        conscriptNoLine->setMaxLength(12);

        horizontalLayout_2->addWidget(conscriptNoLine);


        formLayout_2->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        cityLabel = new QLabel(addressGroupBox);
        cityLabel->setObjectName("cityLabel");

        formLayout_2->setWidget(3, QFormLayout::ItemRole::LabelRole, cityLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        cityLine = new QLineEdit(addressGroupBox);
        cityLine->setObjectName("cityLine");

        horizontalLayout_3->addWidget(cityLine);

        zipCodeLabel = new QLabel(addressGroupBox);
        zipCodeLabel->setObjectName("zipCodeLabel");

        horizontalLayout_3->addWidget(zipCodeLabel);

        zipCodeLine = new QLineEdit(addressGroupBox);
        zipCodeLine->setObjectName("zipCodeLine");
        zipCodeLine->setMaxLength(16);

        horizontalLayout_3->addWidget(zipCodeLine);


        formLayout_2->setLayout(3, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);

        stateLabel = new QLabel(addressGroupBox);
        stateLabel->setObjectName("stateLabel");

        formLayout_2->setWidget(4, QFormLayout::ItemRole::LabelRole, stateLabel);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        stateComboBox = new QComboBox(addressGroupBox);
        stateComboBox->setObjectName("stateComboBox");

        horizontalLayout_5->addWidget(stateComboBox);

        stateLine = new QLineEdit(addressGroupBox);
        stateLine->setObjectName("stateLine");

        horizontalLayout_5->addWidget(stateLine);


        formLayout_2->setLayout(4, QFormLayout::ItemRole::FieldRole, horizontalLayout_5);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer);

        evidenceNoLabel = new QLabel(addressGroupBox);
        evidenceNoLabel->setObjectName("evidenceNoLabel");

        horizontalLayout_7->addWidget(evidenceNoLabel);

        evidenceNoLine = new QLineEdit(addressGroupBox);
        evidenceNoLine->setObjectName("evidenceNoLine");

        horizontalLayout_7->addWidget(evidenceNoLine);


        formLayout_2->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_7);


        verticalLayout->addWidget(addressGroupBox);

        caAddressGroupBox = new QGroupBox(DlgUserExt2);
        caAddressGroupBox->setObjectName("caAddressGroupBox");
        verticalLayout_3 = new QVBoxLayout(caAddressGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        useResidenceRadioButton = new QRadioButton(caAddressGroupBox);
        useResidenceRadioButton->setObjectName("useResidenceRadioButton");

        verticalLayout_4->addWidget(useResidenceRadioButton);

        useFirmRadioButton = new QRadioButton(caAddressGroupBox);
        useFirmRadioButton->setObjectName("useFirmRadioButton");

        verticalLayout_4->addWidget(useFirmRadioButton);

        useCaRadioButton = new QRadioButton(caAddressGroupBox);
        useCaRadioButton->setObjectName("useCaRadioButton");

        verticalLayout_4->addWidget(useCaRadioButton);


        verticalLayout_3->addLayout(verticalLayout_4);

        caAddressWidget = new QWidget(caAddressGroupBox);
        caAddressWidget->setObjectName("caAddressWidget");
        formLayout_5 = new QFormLayout(caAddressWidget);
        formLayout_5->setObjectName("formLayout_5");
        formLayout_5->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_5->setLabelAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        caStreetLabel = new QLabel(caAddressWidget);
        caStreetLabel->setObjectName("caStreetLabel");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::LabelRole, caStreetLabel);

        caStreetLine = new QLineEdit(caAddressWidget);
        caStreetLine->setObjectName("caStreetLine");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::FieldRole, caStreetLine);

        caCityLabel = new QLabel(caAddressWidget);
        caCityLabel->setObjectName("caCityLabel");

        formLayout_5->setWidget(1, QFormLayout::ItemRole::LabelRole, caCityLabel);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        caCityLine = new QLineEdit(caAddressWidget);
        caCityLine->setObjectName("caCityLine");

        horizontalLayout_4->addWidget(caCityLine);

        caZipCodeLabel = new QLabel(caAddressWidget);
        caZipCodeLabel->setObjectName("caZipCodeLabel");

        horizontalLayout_4->addWidget(caZipCodeLabel);

        caZipCodeLine = new QLineEdit(caAddressWidget);
        caZipCodeLine->setObjectName("caZipCodeLine");
        caZipCodeLine->setMaxLength(16);

        horizontalLayout_4->addWidget(caZipCodeLine);


        formLayout_5->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_4);

        caStateLabel = new QLabel(caAddressWidget);
        caStateLabel->setObjectName("caStateLabel");

        formLayout_5->setWidget(2, QFormLayout::ItemRole::LabelRole, caStateLabel);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        caStateComboBox = new QComboBox(caAddressWidget);
        caStateComboBox->setObjectName("caStateComboBox");

        horizontalLayout_6->addWidget(caStateComboBox);

        caStateLine = new QLineEdit(caAddressWidget);
        caStateLine->setObjectName("caStateLine");

        horizontalLayout_6->addWidget(caStateLine);


        formLayout_5->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_6);


        verticalLayout_3->addWidget(caAddressWidget);


        verticalLayout->addWidget(caAddressGroupBox);

        permissionsGroupBox = new QGroupBox(DlgUserExt2);
        permissionsGroupBox->setObjectName("permissionsGroupBox");
        verticalLayout_5 = new QVBoxLayout(permissionsGroupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName("formLayout_4");
        formLayout_4->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_4->setLabelAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        userTypeLabel = new QLabel(permissionsGroupBox);
        userTypeLabel->setObjectName("userTypeLabel");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::LabelRole, userTypeLabel);

        userTypeComboBox = new QComboBox(permissionsGroupBox);
        userTypeComboBox->setObjectName("userTypeComboBox");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::FieldRole, userTypeComboBox);


        verticalLayout_5->addLayout(formLayout_4);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        readNonPersonalCheckBox = new QCheckBox(permissionsGroupBox);
        readNonPersonalCheckBox->setObjectName("readNonPersonalCheckBox");

        gridLayout->addWidget(readNonPersonalCheckBox, 0, 0, 1, 1);

        readAllCheckBox = new QCheckBox(permissionsGroupBox);
        readAllCheckBox->setObjectName("readAllCheckBox");

        gridLayout->addWidget(readAllCheckBox, 1, 0, 1, 1);

        createDmCheckBox = new QCheckBox(permissionsGroupBox);
        createDmCheckBox->setObjectName("createDmCheckBox");

        gridLayout->addWidget(createDmCheckBox, 2, 0, 1, 1);

        viewInfoCheckBox = new QCheckBox(permissionsGroupBox);
        viewInfoCheckBox->setObjectName("viewInfoCheckBox");

        gridLayout->addWidget(viewInfoCheckBox, 0, 1, 1, 1);

        searchDbCheckBox = new QCheckBox(permissionsGroupBox);
        searchDbCheckBox->setObjectName("searchDbCheckBox");

        gridLayout->addWidget(searchDbCheckBox, 1, 1, 1, 1);

        erasevaultCheckBox = new QCheckBox(permissionsGroupBox);
        erasevaultCheckBox->setObjectName("erasevaultCheckBox");

        gridLayout->addWidget(erasevaultCheckBox, 2, 1, 1, 1);


        verticalLayout_5->addLayout(gridLayout);


        verticalLayout->addWidget(permissionsGroupBox);

        buttonBox = new QDialogButtonBox(DlgUserExt2);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgUserExt2);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgUserExt2, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgUserExt2, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgUserExt2);
    } // setupUi

    void retranslateUi(QDialog *DlgUserExt2)
    {
        DlgUserExt2->setWindowTitle(QCoreApplication::translate("DlgUserExt2", "User Information", nullptr));
        userGroupBox->setTitle(QCoreApplication::translate("DlgUserExt2", "User", nullptr));
#if QT_CONFIG(tooltip)
        namesLabel->setToolTip(QCoreApplication::translate("DlgUserExt2", "Name or names of the person.", nullptr));
#endif // QT_CONFIG(tooltip)
        namesLabel->setText(QCoreApplication::translate("DlgUserExt2", "Names:", nullptr));
#if QT_CONFIG(tooltip)
        surnameLabel->setToolTip(QCoreApplication::translate("DlgUserExt2", "Surname of the person.", nullptr));
#endif // QT_CONFIG(tooltip)
        surnameLabel->setText(QCoreApplication::translate("DlgUserExt2", "Surname:", nullptr));
#if QT_CONFIG(tooltip)
        birthDateLabel->setToolTip(QCoreApplication::translate("DlgUserExt2", "Person birth date. (dd.mm.yyyy)", nullptr));
#endif // QT_CONFIG(tooltip)
        birthDateLabel->setText(QCoreApplication::translate("DlgUserExt2", "Birth date:", nullptr));
        addressGroupBox->setTitle(QCoreApplication::translate("DlgUserExt2", "Address of residence", nullptr));
        streetLabel->setText(QCoreApplication::translate("DlgUserExt2", "Street:", nullptr));
        noInStreetLabel->setText(QCoreApplication::translate("DlgUserExt2", "No. in street:", nullptr));
        districtLabel->setText(QCoreApplication::translate("DlgUserExt2", "District:", nullptr));
        conscriptNoLabel->setText(QCoreApplication::translate("DlgUserExt2", "Conscription no.:", nullptr));
#if QT_CONFIG(tooltip)
        cityLabel->setToolTip(QCoreApplication::translate("DlgUserExt2", "Name of the municipality.", nullptr));
#endif // QT_CONFIG(tooltip)
        cityLabel->setText(QCoreApplication::translate("DlgUserExt2", "City:", nullptr));
        zipCodeLabel->setText(QCoreApplication::translate("DlgUserExt2", "Postal code:", nullptr));
        stateLabel->setText(QCoreApplication::translate("DlgUserExt2", "State:", nullptr));
        evidenceNoLabel->setText(QCoreApplication::translate("DlgUserExt2", "Evidence no.:", nullptr));
        caAddressGroupBox->setTitle(QCoreApplication::translate("DlgUserExt2", "Address where the login credetials should be delivered", nullptr));
        useResidenceRadioButton->setText(QCoreApplication::translate("DlgUserExt2", "Use the above mentioned residence address.", nullptr));
        useFirmRadioButton->setText(QCoreApplication::translate("DlgUserExt2", "Use the company's headquarter address.", nullptr));
        useCaRadioButton->setText(QCoreApplication::translate("DlgUserExt2", "Use different address.", nullptr));
        caStreetLabel->setText(QCoreApplication::translate("DlgUserExt2", "Street and number:", nullptr));
        caCityLabel->setText(QCoreApplication::translate("DlgUserExt2", "City:", nullptr));
        caZipCodeLabel->setText(QCoreApplication::translate("DlgUserExt2", "Postal code:", nullptr));
        caStateLabel->setText(QCoreApplication::translate("DlgUserExt2", "State:", nullptr));
        permissionsGroupBox->setTitle(QCoreApplication::translate("DlgUserExt2", "Access rights", nullptr));
        userTypeLabel->setText(QCoreApplication::translate("DlgUserExt2", "Access type", nullptr));
        readNonPersonalCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "Read received messages", nullptr));
        readAllCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "Read received messages into own hands", nullptr));
        createDmCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "Send messages and read sent messages", nullptr));
        viewInfoCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "View lists, history and delivery info", nullptr));
        searchDbCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "Search for data boxes", nullptr));
        erasevaultCheckBox->setText(QCoreApplication::translate("DlgUserExt2", "Erase messages in vault", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgUserExt2: public Ui_DlgUserExt2 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_USER_EXT2_H
