/********************************************************************************
** Form generated from reading UI file 'dlg_create_account.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CREATE_ACCOUNT_H
#define UI_DLG_CREATE_ACCOUNT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgCreateAccount
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QWidget *layoutWidget;
    QVBoxLayout *verticalLayout_2;
    QTreeView *treeView;
    QToolButton *treeViewButton;
    QStackedWidget *verticalStackedWidget;
    QWidget *verticalStackedWidgetPage1;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *descriptionLabel;
    QSpacerItem *verticalSpacer;
    QWidget *verticalStackedWidgetPage2;
    QVBoxLayout *verticalLayout_4;
    QLabel *infoLabel;
    QFrame *line;
    QFormLayout *formLayout;
    QLabel *acntNameLabel;
    QLineEdit *acntNameLine;
    QFrame *Line_0;
    QLabel *usernameLabel;
    QLineEdit *usernameLine;
    QSpacerItem *horizontalSpacer;
    QCheckBox *testAcntCheckBox;
    QLabel *loginMethodLabel;
    QComboBox *loginMethodComboBox;
    QFrame *line_1;
    QLabel *pwdLabel;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *pwdLine;
    QPushButton *showHidePwdButton;
    QProgressBar *viewPwdProgress;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *rememberPwdCheckBox;
    QLabel *certLabel;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *addCertButton;
    QSpacerItem *horizontalSpacer_3;
    QLabel *mepLabel;
    QLineEdit *mepLine;
    QFrame *line_2;
    QCheckBox *syncAllCheckBox;
    QSpacerItem *verticalSpacer_2;
    QWidget *verticalStackedWidgetPage3;
    QVBoxLayout *verticalLayout_5;
    QLabel *accountInfoLabel;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_6;
    QPushButton *downloadAccountInfoButton;
    QTextEdit *accountTextInfo;
    QWidget *verticalStackedWidgetPage4;
    QVBoxLayout *verticalLayout_6;
    QLabel *userListInfoLabel;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_4;
    QPushButton *downloadUserListButton;
    QSplitter *splitter_2;
    LoweredTableView *userTableView;
    QTextEdit *userTextInfo;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_5;
    QToolButton *userListButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgCreateAccount)
    {
        if (DlgCreateAccount->objectName().isEmpty())
            DlgCreateAccount->setObjectName("DlgCreateAccount");
        DlgCreateAccount->resize(620, 430);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgCreateAccount->sizePolicy().hasHeightForWidth());
        DlgCreateAccount->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(DlgCreateAccount);
        verticalLayout->setObjectName("verticalLayout");
        splitter = new QSplitter(DlgCreateAccount);
        splitter->setObjectName("splitter");
        splitter->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(splitter);
        layoutWidget->setObjectName("layoutWidget");
        verticalLayout_2 = new QVBoxLayout(layoutWidget);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        treeView = new QTreeView(layoutWidget);
        treeView->setObjectName("treeView");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(treeView->sizePolicy().hasHeightForWidth());
        treeView->setSizePolicy(sizePolicy1);
        treeView->setMinimumSize(QSize(220, 0));
        treeView->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        treeView->header()->setMinimumSectionSize(100);

        verticalLayout_2->addWidget(treeView);

        treeViewButton = new QToolButton(layoutWidget);
        treeViewButton->setObjectName("treeViewButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(treeViewButton->sizePolicy().hasHeightForWidth());
        treeViewButton->setSizePolicy(sizePolicy2);

        verticalLayout_2->addWidget(treeViewButton);

        splitter->addWidget(layoutWidget);
        verticalStackedWidget = new QStackedWidget(splitter);
        verticalStackedWidget->setObjectName("verticalStackedWidget");
        verticalStackedWidgetPage1 = new QWidget();
        verticalStackedWidgetPage1->setObjectName("verticalStackedWidgetPage1");
        verticalLayout_3 = new QVBoxLayout(verticalStackedWidgetPage1);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        descriptionLabel = new QLabel(verticalStackedWidgetPage1);
        descriptionLabel->setObjectName("descriptionLabel");
        descriptionLabel->setWordWrap(true);

        horizontalLayout->addWidget(descriptionLabel);


        verticalLayout_3->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        verticalStackedWidget->addWidget(verticalStackedWidgetPage1);
        verticalStackedWidgetPage2 = new QWidget();
        verticalStackedWidgetPage2->setObjectName("verticalStackedWidgetPage2");
        verticalLayout_4 = new QVBoxLayout(verticalStackedWidgetPage2);
        verticalLayout_4->setObjectName("verticalLayout_4");
        infoLabel = new QLabel(verticalStackedWidgetPage2);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_4->addWidget(infoLabel);

        line = new QFrame(verticalStackedWidgetPage2);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_4->addWidget(line);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        acntNameLabel = new QLabel(verticalStackedWidgetPage2);
        acntNameLabel->setObjectName("acntNameLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, acntNameLabel);

        acntNameLine = new QLineEdit(verticalStackedWidgetPage2);
        acntNameLine->setObjectName("acntNameLine");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, acntNameLine);

        Line_0 = new QFrame(verticalStackedWidgetPage2);
        Line_0->setObjectName("Line_0");
        Line_0->setFrameShape(QFrame::Shape::HLine);
        Line_0->setFrameShadow(QFrame::Shadow::Sunken);

        formLayout->setWidget(1, QFormLayout::ItemRole::SpanningRole, Line_0);

        usernameLabel = new QLabel(verticalStackedWidgetPage2);
        usernameLabel->setObjectName("usernameLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, usernameLabel);

        usernameLine = new QLineEdit(verticalStackedWidgetPage2);
        usernameLine->setObjectName("usernameLine");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, usernameLine);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        formLayout->setItem(3, QFormLayout::ItemRole::LabelRole, horizontalSpacer);

        testAcntCheckBox = new QCheckBox(verticalStackedWidgetPage2);
        testAcntCheckBox->setObjectName("testAcntCheckBox");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, testAcntCheckBox);

        loginMethodLabel = new QLabel(verticalStackedWidgetPage2);
        loginMethodLabel->setObjectName("loginMethodLabel");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, loginMethodLabel);

        loginMethodComboBox = new QComboBox(verticalStackedWidgetPage2);
        loginMethodComboBox->setObjectName("loginMethodComboBox");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, loginMethodComboBox);

        line_1 = new QFrame(verticalStackedWidgetPage2);
        line_1->setObjectName("line_1");
        line_1->setFrameShape(QFrame::Shape::HLine);
        line_1->setFrameShadow(QFrame::Shadow::Sunken);

        formLayout->setWidget(5, QFormLayout::ItemRole::SpanningRole, line_1);

        pwdLabel = new QLabel(verticalStackedWidgetPage2);
        pwdLabel->setObjectName("pwdLabel");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, pwdLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        pwdLine = new QLineEdit(verticalStackedWidgetPage2);
        pwdLine->setObjectName("pwdLine");
        pwdLine->setEchoMode(QLineEdit::Password);

        horizontalLayout_3->addWidget(pwdLine);

        showHidePwdButton = new QPushButton(verticalStackedWidgetPage2);
        showHidePwdButton->setObjectName("showHidePwdButton");

        horizontalLayout_3->addWidget(showHidePwdButton);


        formLayout->setLayout(6, QFormLayout::ItemRole::FieldRole, horizontalLayout_3);

        viewPwdProgress = new QProgressBar(verticalStackedWidgetPage2);
        viewPwdProgress->setObjectName("viewPwdProgress");

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, viewPwdProgress);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        formLayout->setItem(8, QFormLayout::ItemRole::LabelRole, horizontalSpacer_2);

        rememberPwdCheckBox = new QCheckBox(verticalStackedWidgetPage2);
        rememberPwdCheckBox->setObjectName("rememberPwdCheckBox");

        formLayout->setWidget(8, QFormLayout::ItemRole::FieldRole, rememberPwdCheckBox);

        certLabel = new QLabel(verticalStackedWidgetPage2);
        certLabel->setObjectName("certLabel");

        formLayout->setWidget(9, QFormLayout::ItemRole::LabelRole, certLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        addCertButton = new QPushButton(verticalStackedWidgetPage2);
        addCertButton->setObjectName("addCertButton");

        horizontalLayout_2->addWidget(addCertButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);


        formLayout->setLayout(9, QFormLayout::ItemRole::FieldRole, horizontalLayout_2);

        mepLabel = new QLabel(verticalStackedWidgetPage2);
        mepLabel->setObjectName("mepLabel");

        formLayout->setWidget(10, QFormLayout::ItemRole::LabelRole, mepLabel);

        mepLine = new QLineEdit(verticalStackedWidgetPage2);
        mepLine->setObjectName("mepLine");

        formLayout->setWidget(10, QFormLayout::ItemRole::FieldRole, mepLine);

        line_2 = new QFrame(verticalStackedWidgetPage2);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        formLayout->setWidget(11, QFormLayout::ItemRole::SpanningRole, line_2);


        verticalLayout_4->addLayout(formLayout);

        syncAllCheckBox = new QCheckBox(verticalStackedWidgetPage2);
        syncAllCheckBox->setObjectName("syncAllCheckBox");
        syncAllCheckBox->setChecked(true);

        verticalLayout_4->addWidget(syncAllCheckBox);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        verticalStackedWidget->addWidget(verticalStackedWidgetPage2);
        verticalStackedWidgetPage3 = new QWidget();
        verticalStackedWidgetPage3->setObjectName("verticalStackedWidgetPage3");
        verticalLayout_5 = new QVBoxLayout(verticalStackedWidgetPage3);
        verticalLayout_5->setObjectName("verticalLayout_5");
        accountInfoLabel = new QLabel(verticalStackedWidgetPage3);
        accountInfoLabel->setObjectName("accountInfoLabel");
        accountInfoLabel->setWordWrap(true);

        verticalLayout_5->addWidget(accountInfoLabel);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_6);

        downloadAccountInfoButton = new QPushButton(verticalStackedWidgetPage3);
        downloadAccountInfoButton->setObjectName("downloadAccountInfoButton");

        horizontalLayout_4->addWidget(downloadAccountInfoButton);


        verticalLayout_5->addLayout(horizontalLayout_4);

        accountTextInfo = new QTextEdit(verticalStackedWidgetPage3);
        accountTextInfo->setObjectName("accountTextInfo");

        verticalLayout_5->addWidget(accountTextInfo);

        verticalStackedWidget->addWidget(verticalStackedWidgetPage3);
        verticalStackedWidgetPage4 = new QWidget();
        verticalStackedWidgetPage4->setObjectName("verticalStackedWidgetPage4");
        verticalLayout_6 = new QVBoxLayout(verticalStackedWidgetPage4);
        verticalLayout_6->setObjectName("verticalLayout_6");
        userListInfoLabel = new QLabel(verticalStackedWidgetPage4);
        userListInfoLabel->setObjectName("userListInfoLabel");
        userListInfoLabel->setWordWrap(true);

        verticalLayout_6->addWidget(userListInfoLabel);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);

        downloadUserListButton = new QPushButton(verticalStackedWidgetPage4);
        downloadUserListButton->setObjectName("downloadUserListButton");

        horizontalLayout_5->addWidget(downloadUserListButton);


        verticalLayout_6->addLayout(horizontalLayout_5);

        splitter_2 = new QSplitter(verticalStackedWidgetPage4);
        splitter_2->setObjectName("splitter_2");
        splitter_2->setOrientation(Qt::Vertical);
        userTableView = new LoweredTableView(splitter_2);
        userTableView->setObjectName("userTableView");
        userTableView->setShowGrid(false);
        userTableView->setWordWrap(false);
        splitter_2->addWidget(userTableView);
        userTableView->horizontalHeader()->setHighlightSections(false);
        userTableView->horizontalHeader()->setStretchLastSection(true);
        userTableView->verticalHeader()->setVisible(false);
        userTableView->verticalHeader()->setHighlightSections(false);
        userTextInfo = new QTextEdit(splitter_2);
        userTextInfo->setObjectName("userTextInfo");
        splitter_2->addWidget(userTextInfo);

        verticalLayout_6->addWidget(splitter_2);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_5);

        userListButton = new QToolButton(verticalStackedWidgetPage4);
        userListButton->setObjectName("userListButton");

        horizontalLayout_6->addWidget(userListButton);


        verticalLayout_6->addLayout(horizontalLayout_6);

        verticalStackedWidget->addWidget(verticalStackedWidgetPage4);
        splitter->addWidget(verticalStackedWidget);

        verticalLayout->addWidget(splitter);

        buttonBox = new QDialogButtonBox(DlgCreateAccount);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(acntNameLine, usernameLine);
        QWidget::setTabOrder(usernameLine, testAcntCheckBox);
        QWidget::setTabOrder(testAcntCheckBox, loginMethodComboBox);
        QWidget::setTabOrder(loginMethodComboBox, pwdLine);
        QWidget::setTabOrder(pwdLine, showHidePwdButton);
        QWidget::setTabOrder(showHidePwdButton, rememberPwdCheckBox);
        QWidget::setTabOrder(rememberPwdCheckBox, addCertButton);
        QWidget::setTabOrder(addCertButton, syncAllCheckBox);

        retranslateUi(DlgCreateAccount);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgCreateAccount, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgCreateAccount, qOverload<>(&QDialog::reject));

        verticalStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgCreateAccount);
    } // setupUi

    void retranslateUi(QDialog *DlgCreateAccount)
    {
        DlgCreateAccount->setWindowTitle(QCoreApplication::translate("DlgCreateAccount", "Create a new data box", nullptr));
        treeViewButton->setText(QCoreApplication::translate("DlgCreateAccount", "Data-Box Actions", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("DlgCreateAccount", "n/a", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Please enter credentials for your data box.", nullptr));
#if QT_CONFIG(tooltip)
        acntNameLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "The databox title is a user-specified name used for the identification\n"
"of the databox in the application (e.g. 'My Personal Data Box', 'Firm Box',\n"
"etc.). The chosen name serves only for your convenience.\n"
"The entry must be filled in.", nullptr));
#endif // QT_CONFIG(tooltip)
        acntNameLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Data-box title:", nullptr));
#if QT_CONFIG(tooltip)
        acntNameLine->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Enter custom data-box name.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        usernameLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "The username must consist of at least 6 characters without spaces (only\n"
"combinations of lower-case letters and digits are allowed).\n"
"The entry must be filled in.\n"
"\n"
"Notification: The username is not a data-box ID.", nullptr));
#endif // QT_CONFIG(tooltip)
        usernameLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Username:", nullptr));
#if QT_CONFIG(tooltip)
        usernameLine->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Enter the login name for this data box.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        testAcntCheckBox->setToolTip(QCoreApplication::translate("DlgCreateAccount", "If the credentials you entered are for a test data box,\n"
"select this option. For normal data box\n"
"(created at Czech Point) leave this unchecked.", nullptr));
#endif // QT_CONFIG(tooltip)
        testAcntCheckBox->setText(QCoreApplication::translate("DlgCreateAccount", "This is a test data box", nullptr));
#if QT_CONFIG(tooltip)
        loginMethodLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Select the login method which you use to access the data box in the ISDS.\n"
"Note: NIA login methods such as bank ID, mojeID or eCitizen aren't\n"
"supported because the ISDS system doesn't provide such\n"
"functionality for third-party applications.", nullptr));
#endif // QT_CONFIG(tooltip)
        loginMethodLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Login method:", nullptr));
#if QT_CONFIG(tooltip)
        loginMethodComboBox->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Select authorisation method for logging in to the data box.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pwdLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "The password must be valid and non-expired. To check whether you've entered\n"
"the password correctly you may use the button 'View' in the field on the right.\n"
"\n"
"Notification: If it is your very first attempt to log into the data box then you must\n"
"log into the ISDS web portal first where you will need to change your password.", nullptr));
#endif // QT_CONFIG(tooltip)
        pwdLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Password:", nullptr));
#if QT_CONFIG(tooltip)
        pwdLine->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Enter your password.", nullptr));
#endif // QT_CONFIG(tooltip)
        showHidePwdButton->setText(QCoreApplication::translate("DlgCreateAccount", "View", nullptr));
#if QT_CONFIG(tooltip)
        rememberPwdCheckBox->setToolTip(QCoreApplication::translate("DlgCreateAccount", "If no PIN is configured then the password will be saved\n"
"in a readable format in the configuration file which is\n"
"usually located in your home directory. Do not use this\n"
"possibility if you are concerned about the security\n"
"of your account.\n"
"\n"
"If you configure a PIN (master password) then the password\n"
"is going to be stored in an encrypted form into the configuration\n"
"file. If you forget the PIN then there is no easy way how to obtain\n"
"the stored password in decrypted form.", nullptr));
#endif // QT_CONFIG(tooltip)
        rememberPwdCheckBox->setText(QCoreApplication::translate("DlgCreateAccount", "Remember password", nullptr));
#if QT_CONFIG(tooltip)
        certLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "The certificate file is needed for authentication purposes. The supplied file\n"
"must contain a certificate and its corresponding private key. Only PEM and PFX file\n"
"formats are accepted.", nullptr));
#endif // QT_CONFIG(tooltip)
        certLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Certificate file:", nullptr));
#if QT_CONFIG(tooltip)
        addCertButton->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Select a certificate.", nullptr));
#endif // QT_CONFIG(tooltip)
        addCertButton->setText(QCoreApplication::translate("DlgCreateAccount", "Select File", nullptr));
#if QT_CONFIG(tooltip)
        mepLabel->setToolTip(QCoreApplication::translate("DlgCreateAccount", "The communication code is a string which can be generated in the ISDS web portal.", nullptr));
#endif // QT_CONFIG(tooltip)
        mepLabel->setText(QCoreApplication::translate("DlgCreateAccount", "Code:", nullptr));
#if QT_CONFIG(tooltip)
        mepLine->setToolTip(QCoreApplication::translate("DlgCreateAccount", "Enter the communication code.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        syncAllCheckBox->setToolTip(QCoreApplication::translate("DlgCreateAccount", "If selected, the data box will be included into\n"
"the synchronisation process of all data boxes.", nullptr));
#endif // QT_CONFIG(tooltip)
        syncAllCheckBox->setText(QCoreApplication::translate("DlgCreateAccount", "Synchronise this data box together with all data boxes", nullptr));
        accountInfoLabel->setText(QCoreApplication::translate("DlgCreateAccount", "n/a", nullptr));
        downloadAccountInfoButton->setText(QCoreApplication::translate("DlgCreateAccount", "Update Information", nullptr));
        userListInfoLabel->setText(QCoreApplication::translate("DlgCreateAccount", "n/a", nullptr));
        downloadUserListButton->setText(QCoreApplication::translate("DlgCreateAccount", "Update Information", nullptr));
        userListButton->setText(QCoreApplication::translate("DlgCreateAccount", "User Actions", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgCreateAccount: public Ui_DlgCreateAccount {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CREATE_ACCOUNT_H
