/********************************************************************************
** Form generated from reading UI file 'dlg_pin_setup.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_PIN_SETUP_H
#define UI_DLG_PIN_SETUP_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgPinSetup
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *pinLabel;
    QFormLayout *formLayout;
    QLabel *currentPinLabel;
    QLineEdit *currentPinLine;
    QLabel *newPinLabel;
    QLineEdit *newPinLine;
    QLabel *newPinLabel2;
    QLineEdit *newPinLine2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgPinSetup)
    {
        if (DlgPinSetup->objectName().isEmpty())
            DlgPinSetup->setObjectName("DlgPinSetup");
        DlgPinSetup->resize(300, 240);
        verticalLayout = new QVBoxLayout(DlgPinSetup);
        verticalLayout->setObjectName("verticalLayout");
        pinLabel = new QLabel(DlgPinSetup);
        pinLabel->setObjectName("pinLabel");
        QFont font;
        font.setItalic(true);
        pinLabel->setFont(font);
        pinLabel->setWordWrap(true);

        verticalLayout->addWidget(pinLabel);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        currentPinLabel = new QLabel(DlgPinSetup);
        currentPinLabel->setObjectName("currentPinLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, currentPinLabel);

        currentPinLine = new QLineEdit(DlgPinSetup);
        currentPinLine->setObjectName("currentPinLine");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, currentPinLine);

        newPinLabel = new QLabel(DlgPinSetup);
        newPinLabel->setObjectName("newPinLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, newPinLabel);

        newPinLine = new QLineEdit(DlgPinSetup);
        newPinLine->setObjectName("newPinLine");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, newPinLine);

        newPinLabel2 = new QLabel(DlgPinSetup);
        newPinLabel2->setObjectName("newPinLabel2");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, newPinLabel2);

        newPinLine2 = new QLineEdit(DlgPinSetup);
        newPinLine2->setObjectName("newPinLine2");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, newPinLine2);


        verticalLayout->addLayout(formLayout);

        buttonBox = new QDialogButtonBox(DlgPinSetup);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(currentPinLine, newPinLine);
        QWidget::setTabOrder(newPinLine, newPinLine2);

        retranslateUi(DlgPinSetup);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgPinSetup, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgPinSetup, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgPinSetup);
    } // setupUi

    void retranslateUi(QDialog *DlgPinSetup)
    {
        DlgPinSetup->setWindowTitle(QCoreApplication::translate("DlgPinSetup", "PIN Setup", nullptr));
        pinLabel->setText(QCoreApplication::translate("DlgPinSetup", "The PIN (master password) may consist of any character. It isn't restricted to digits only. Easy passwords provide less protection.", nullptr));
        currentPinLabel->setText(QCoreApplication::translate("DlgPinSetup", "Current PIN", nullptr));
        newPinLabel->setText(QCoreApplication::translate("DlgPinSetup", "New PIN", nullptr));
        newPinLabel2->setText(QCoreApplication::translate("DlgPinSetup", "Repeat new PIN", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPinSetup: public Ui_DlgPinSetup {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_PIN_SETUP_H
