/****************************************************************************
** Meta object code from reading C++ file 'tag_container.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/io/tag_container.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tag_container.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12TagContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto TagContainer::qt_create_metaobjectdata<qt_meta_tag_ZN12TagContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagContainer",
        "connected",
        "",
        "disconnected",
        "reset",
        "tagsInserted",
        "Json::TagEntryList",
        "entries",
        "tagsUpdated",
        "tagsDeleted",
        "Json::Int64StringList",
        "ids",
        "tagAssignmentChanged",
        "Json::TagAssignmentList",
        "assignments",
        "watchConnected",
        "watchDisconnected",
        "watchReset",
        "watchTagsInserted",
        "watchTagsUpdated",
        "watchTagsDeleted",
        "watchTagAssignmentChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connected'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disconnected'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reset'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tagsInserted'
        QtMocHelpers::SignalData<void(const Json::TagEntryList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'tagsUpdated'
        QtMocHelpers::SignalData<void(const Json::TagEntryList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'tagsDeleted'
        QtMocHelpers::SignalData<void(const Json::Int64StringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'tagAssignmentChanged'
        QtMocHelpers::SignalData<void(const Json::TagAssignmentList &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'watchConnected'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchDisconnected'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchReset'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchTagsInserted'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'watchTagsUpdated'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'watchTagsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'watchTagAssignmentChanged'
        QtMocHelpers::SlotData<void(const Json::TagAssignmentList &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagContainer, qt_meta_tag_ZN12TagContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TagContainerE_t>.metaTypes,
    nullptr
} };

void TagContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->reset(); break;
        case 3: _t->tagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 4: _t->tagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 5: _t->tagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 6: _t->tagAssignmentChanged((*reinterpret_cast<std::add_pointer_t<Json::TagAssignmentList>>(_a[1]))); break;
        case 7: _t->watchConnected(); break;
        case 8: _t->watchDisconnected(); break;
        case 9: _t->watchReset(); break;
        case 10: _t->watchTagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 11: _t->watchTagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 12: _t->watchTagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 13: _t->watchTagAssignmentChanged((*reinterpret_cast<std::add_pointer_t<Json::TagAssignmentList>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)()>(_a, &TagContainer::connected, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)()>(_a, &TagContainer::disconnected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)()>(_a, &TagContainer::reset, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)(const Json::TagEntryList & )>(_a, &TagContainer::tagsInserted, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)(const Json::TagEntryList & )>(_a, &TagContainer::tagsUpdated, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)(const Json::Int64StringList & )>(_a, &TagContainer::tagsDeleted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagContainer::*)(const Json::TagAssignmentList & )>(_a, &TagContainer::tagAssignmentChanged, 6))
            return;
    }
}

const QMetaObject *TagContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagContainerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void TagContainer::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TagContainer::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TagContainer::reset()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TagContainer::tagsInserted(const Json::TagEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TagContainer::tagsUpdated(const Json::TagEntryList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TagContainer::tagsDeleted(const Json::Int64StringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TagContainer::tagAssignmentChanged(const Json::TagAssignmentList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
