/********************************************************************************
** Form generated from reading UI file 'dlg_download_messages.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_DOWNLOAD_MESSAGES_H
#define UI_DLG_DOWNLOAD_MESSAGES_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgDownloadMessages
{
public:
    QVBoxLayout *verticalLayout;
    LoweredTableView *downloadView;
    QProgressBar *downloadProgress;
    QLabel *statusLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgDownloadMessages)
    {
        if (DlgDownloadMessages->objectName().isEmpty())
            DlgDownloadMessages->setObjectName("DlgDownloadMessages");
        DlgDownloadMessages->resize(400, 300);
        verticalLayout = new QVBoxLayout(DlgDownloadMessages);
        verticalLayout->setObjectName("verticalLayout");
        downloadView = new LoweredTableView(DlgDownloadMessages);
        downloadView->setObjectName("downloadView");
        downloadView->setShowGrid(false);
        downloadView->setWordWrap(false);
        downloadView->horizontalHeader()->setHighlightSections(false);
        downloadView->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        downloadView->horizontalHeader()->setStretchLastSection(true);
        downloadView->verticalHeader()->setVisible(false);
        downloadView->verticalHeader()->setHighlightSections(false);

        verticalLayout->addWidget(downloadView);

        downloadProgress = new QProgressBar(DlgDownloadMessages);
        downloadProgress->setObjectName("downloadProgress");
        downloadProgress->setValue(24);

        verticalLayout->addWidget(downloadProgress);

        statusLabel = new QLabel(DlgDownloadMessages);
        statusLabel->setObjectName("statusLabel");

        verticalLayout->addWidget(statusLabel);

        buttonBox = new QDialogButtonBox(DlgDownloadMessages);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgDownloadMessages);

        QMetaObject::connectSlotsByName(DlgDownloadMessages);
    } // setupUi

    void retranslateUi(QDialog *DlgDownloadMessages)
    {
        DlgDownloadMessages->setWindowTitle(QCoreApplication::translate("DlgDownloadMessages", "Download Messages", nullptr));
        statusLabel->setText(QCoreApplication::translate("DlgDownloadMessages", "n/a", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgDownloadMessages: public Ui_DlgDownloadMessages {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_DOWNLOAD_MESSAGES_H
