/********************************************************************************
** Form generated from reading UI file 'dlg_convert_import_from_mobile_app.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CONVERT_IMPORT_FROM_MOBILE_APP_H
#define UI_DLG_CONVERT_IMPORT_FROM_MOBILE_APP_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgConvertImportFromMobileApp
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoText;
    QGroupBox *zipPackageGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *openZipButton;
    QLabel *labelZipFileName;
    QSpacerItem *horizontalSpacer;
    QLabel *zipPackageInfoText;
    QGroupBox *acntSelectionGroupBox;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *allCheckBox;
    QSpacerItem *horizontalSpacer_4;
    QLabel *label;
    QLineEdit *accountFilterLine;
    LoweredTableView *accountTableView;
    QGroupBox *importGroupBox;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QPushButton *importButton;
    QLabel *importInfoLabel;
    QSpacerItem *horizontalSpacer_3;
    QProgressBar *progressBar;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgConvertImportFromMobileApp)
    {
        if (DlgConvertImportFromMobileApp->objectName().isEmpty())
            DlgConvertImportFromMobileApp->setObjectName("DlgConvertImportFromMobileApp");
        DlgConvertImportFromMobileApp->resize(700, 620);
        verticalLayout = new QVBoxLayout(DlgConvertImportFromMobileApp);
        verticalLayout->setObjectName("verticalLayout");
        infoText = new QLabel(DlgConvertImportFromMobileApp);
        infoText->setObjectName("infoText");
        infoText->setAlignment(Qt::AlignCenter);
        infoText->setWordWrap(true);

        verticalLayout->addWidget(infoText);

        zipPackageGroupBox = new QGroupBox(DlgConvertImportFromMobileApp);
        zipPackageGroupBox->setObjectName("zipPackageGroupBox");
        verticalLayout_2 = new QVBoxLayout(zipPackageGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        openZipButton = new QPushButton(zipPackageGroupBox);
        openZipButton->setObjectName("openZipButton");

        horizontalLayout_2->addWidget(openZipButton);

        labelZipFileName = new QLabel(zipPackageGroupBox);
        labelZipFileName->setObjectName("labelZipFileName");

        horizontalLayout_2->addWidget(labelZipFileName);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout_2);

        zipPackageInfoText = new QLabel(zipPackageGroupBox);
        zipPackageInfoText->setObjectName("zipPackageInfoText");
        zipPackageInfoText->setWordWrap(true);

        verticalLayout_2->addWidget(zipPackageInfoText);


        verticalLayout->addWidget(zipPackageGroupBox);

        acntSelectionGroupBox = new QGroupBox(DlgConvertImportFromMobileApp);
        acntSelectionGroupBox->setObjectName("acntSelectionGroupBox");
        verticalLayout_5 = new QVBoxLayout(acntSelectionGroupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        allCheckBox = new QCheckBox(acntSelectionGroupBox);
        allCheckBox->setObjectName("allCheckBox");

        horizontalLayout_4->addWidget(allCheckBox);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        label = new QLabel(acntSelectionGroupBox);
        label->setObjectName("label");

        horizontalLayout_4->addWidget(label);

        accountFilterLine = new QLineEdit(acntSelectionGroupBox);
        accountFilterLine->setObjectName("accountFilterLine");

        horizontalLayout_4->addWidget(accountFilterLine);


        verticalLayout_5->addLayout(horizontalLayout_4);

        accountTableView = new LoweredTableView(acntSelectionGroupBox);
        accountTableView->setObjectName("accountTableView");
        accountTableView->setShowGrid(false);
        accountTableView->setWordWrap(false);
        accountTableView->horizontalHeader()->setHighlightSections(false);
        accountTableView->horizontalHeader()->setStretchLastSection(true);
        accountTableView->verticalHeader()->setVisible(false);
        accountTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout_5->addWidget(accountTableView);


        verticalLayout->addWidget(acntSelectionGroupBox);

        importGroupBox = new QGroupBox(DlgConvertImportFromMobileApp);
        importGroupBox->setObjectName("importGroupBox");
        verticalLayout_3 = new QVBoxLayout(importGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        importButton = new QPushButton(importGroupBox);
        importButton->setObjectName("importButton");

        horizontalLayout->addWidget(importButton);

        importInfoLabel = new QLabel(importGroupBox);
        importInfoLabel->setObjectName("importInfoLabel");

        horizontalLayout->addWidget(importInfoLabel);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        verticalLayout_3->addLayout(horizontalLayout);

        progressBar = new QProgressBar(importGroupBox);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(0);

        verticalLayout_3->addWidget(progressBar);


        verticalLayout->addWidget(importGroupBox);

        buttonBox = new QDialogButtonBox(DlgConvertImportFromMobileApp);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgConvertImportFromMobileApp);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgConvertImportFromMobileApp, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgConvertImportFromMobileApp, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgConvertImportFromMobileApp);
    } // setupUi

    void retranslateUi(QDialog *DlgConvertImportFromMobileApp)
    {
        DlgConvertImportFromMobileApp->setWindowTitle(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Mobile Data Import Tool", nullptr));
        infoText->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "ZIP archives containing transfer data from the mobile application can be read and their content can be imported into the desktop application. The original ZIP archive isn't modified.", nullptr));
        zipPackageGroupBox->setTitle(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Load transfer ZIP archive from mobile application", nullptr));
        openZipButton->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Choose ZIP", nullptr));
        labelZipFileName->setText(QString());
        zipPackageInfoText->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "n/a", nullptr));
        acntSelectionGroupBox->setTitle(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Select data boxes to be imported", nullptr));
        allCheckBox->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Select all", nullptr));
        label->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Filter:", nullptr));
        importGroupBox->setTitle(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Import data of selected data boxes", nullptr));
        importButton->setText(QCoreApplication::translate("DlgConvertImportFromMobileApp", "Import", nullptr));
        importInfoLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class DlgConvertImportFromMobileApp: public Ui_DlgConvertImportFromMobileApp {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CONVERT_IMPORT_FROM_MOBILE_APP_H
