/********************************************************************************
** Form generated from reading UI file 'dlg_contacts.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CONTACTS_H
#define UI_DLG_CONTACTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgContacts
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    LoweredTableView *contactTableView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgContacts)
    {
        if (DlgContacts->objectName().isEmpty())
            DlgContacts->setObjectName("DlgContacts");
        DlgContacts->resize(560, 300);
        verticalLayout = new QVBoxLayout(DlgContacts);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(DlgContacts);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        filterLabel = new QLabel(DlgContacts);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgContacts);
        filterLine->setObjectName("filterLine");

        horizontalLayout->addWidget(filterLine);


        verticalLayout->addLayout(horizontalLayout);

        contactTableView = new LoweredTableView(DlgContacts);
        contactTableView->setObjectName("contactTableView");
        contactTableView->setShowGrid(false);
        contactTableView->setWordWrap(false);
        contactTableView->horizontalHeader()->setHighlightSections(false);
        contactTableView->horizontalHeader()->setStretchLastSection(true);
        contactTableView->verticalHeader()->setVisible(false);
        contactTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout->addWidget(contactTableView);

        buttonBox = new QDialogButtonBox(DlgContacts);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(contactTableView, filterLine);

        retranslateUi(DlgContacts);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgContacts, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgContacts, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgContacts);
    } // setupUi

    void retranslateUi(QDialog *DlgContacts)
    {
        DlgContacts->setWindowTitle(QCoreApplication::translate("DlgContacts", "Add recipient", nullptr));
        label->setText(QCoreApplication::translate("DlgContacts", "Select recipients from the list of current contacts (collected from existing messages):", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgContacts", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgContacts: public Ui_DlgContacts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CONTACTS_H
