/****************************************************************************
** Meta object code from reading C++ file 'message_db_set.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/io/message_db_set.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'message_db_set.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12MessageDbSetE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageDbSet::qt_create_metaobjectdata<qt_meta_tag_ZN12MessageDbSetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageDbSet",
        "madeAvailable",
        "",
        "primaryKey",
        "testing",
        "opened",
        "openingFailed",
        "fileName",
        "changedLocallyRead",
        "QList<MsgId>",
        "msgIds",
        "read",
        "changedProcessState",
        "MessageDb::MessageProcessState",
        "state",
        "messageInserted",
        "MsgId",
        "msgId",
        "direction",
        "messageDataDeleted",
        "watchOpened",
        "watchOpeningFailed",
        "watchChangedLocallyRead",
        "watchChangedProcessState",
        "watchMessageInserted",
        "watchMessageDataDeleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'madeAvailable'
        QtMocHelpers::SignalData<void(const QString &, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'opened'
        QtMocHelpers::SignalData<void(const QString &, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 },
        }}),
        // Signal 'openingFailed'
        QtMocHelpers::SignalData<void(const QString &, bool, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'changedLocallyRead'
        QtMocHelpers::SignalData<void(const QString &, bool, const QList<MsgId> &, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'changedProcessState'
        QtMocHelpers::SignalData<void(const QString &, bool, const QList<MsgId> &, MessageDb::MessageProcessState)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'messageInserted'
        QtMocHelpers::SignalData<void(const QString &, bool, const MsgId &, int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { 0x80000000 | 16, 17 }, { QMetaType::Int, 18 },
        }}),
        // Signal 'messageDataDeleted'
        QtMocHelpers::SignalData<void(const QString &, bool, const MsgId &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Bool, 4 }, { 0x80000000 | 16, 17 },
        }}),
        // Slot 'watchOpened'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'watchOpeningFailed'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'watchChangedLocallyRead'
        QtMocHelpers::SlotData<void(const QList<MsgId> &, bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'watchChangedProcessState'
        QtMocHelpers::SlotData<void(const QList<MsgId> &, MessageDb::MessageProcessState)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 13, 14 },
        }}),
        // Slot 'watchMessageInserted'
        QtMocHelpers::SlotData<void(const MsgId &, int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'watchMessageDataDeleted'
        QtMocHelpers::SlotData<void(const MsgId &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageDbSet, qt_meta_tag_ZN12MessageDbSetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageDbSet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageDbSetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageDbSetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MessageDbSetE_t>.metaTypes,
    nullptr
} };

void MessageDbSet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageDbSet *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->madeAvailable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->opened((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 2: _t->openingFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: _t->changedLocallyRead((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 4: _t->changedProcessState((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<MessageDb::MessageProcessState>>(_a[4]))); break;
        case 5: _t->messageInserted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 6: _t->messageDataDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3]))); break;
        case 7: _t->watchOpened((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->watchOpeningFailed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->watchChangedLocallyRead((*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->watchChangedProcessState((*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MessageDb::MessageProcessState>>(_a[2]))); break;
        case 11: _t->watchMessageInserted((*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->watchMessageDataDeleted((*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageDb::MessageProcessState >(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MessageDb::MessageProcessState >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool )>(_a, &MessageDbSet::madeAvailable, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool )>(_a, &MessageDbSet::opened, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool , const QString & )>(_a, &MessageDbSet::openingFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool , const QList<MsgId> & , bool )>(_a, &MessageDbSet::changedLocallyRead, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool , const QList<MsgId> & , MessageDb::MessageProcessState )>(_a, &MessageDbSet::changedProcessState, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool , const MsgId & , int )>(_a, &MessageDbSet::messageInserted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageDbSet::*)(const QString & , bool , const MsgId & )>(_a, &MessageDbSet::messageDataDeleted, 6))
            return;
    }
}

const QMetaObject *MessageDbSet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageDbSet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MessageDbSetE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageDbSet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void MessageDbSet::madeAvailable(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MessageDbSet::opened(const QString & _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MessageDbSet::openingFailed(const QString & _t1, bool _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void MessageDbSet::changedLocallyRead(const QString & _t1, bool _t2, const QList<MsgId> & _t3, bool _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void MessageDbSet::changedProcessState(const QString & _t1, bool _t2, const QList<MsgId> & _t3, MessageDb::MessageProcessState _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void MessageDbSet::messageInserted(const QString & _t1, bool _t2, const MsgId & _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 6
void MessageDbSet::messageDataDeleted(const QString & _t1, bool _t2, const MsgId & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
