/********************************************************************************
** Form generated from reading UI file 'dlg_tags.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TAGS_H
#define UI_DLG_TAGS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgTags
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QGroupBox *availableGroup;
    QVBoxLayout *verticalLayout_5;
    QListView *availableTagsView;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *addTagButton;
    QPushButton *deleteTagButton;
    QPushButton *updateTagButton;
    QSpacerItem *horizontalSpacer_2;
    QVBoxLayout *asssignmentButtonGroup;
    QSpacerItem *verticalSpacer_2;
    QPushButton *assignButton;
    QPushButton *removeButton;
    QSpacerItem *verticalSpacer;
    QGroupBox *assignedGroup;
    QVBoxLayout *verticalLayout_3;
    QListView *assignedTagsView;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *removeAllButton;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgTags)
    {
        if (DlgTags->objectName().isEmpty())
            DlgTags->setObjectName("DlgTags");
        DlgTags->resize(740, 320);
        verticalLayout_2 = new QVBoxLayout(DlgTags);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        availableGroup = new QGroupBox(DlgTags);
        availableGroup->setObjectName("availableGroup");
        verticalLayout_5 = new QVBoxLayout(availableGroup);
        verticalLayout_5->setObjectName("verticalLayout_5");
        availableTagsView = new QListView(availableGroup);
        availableTagsView->setObjectName("availableTagsView");

        verticalLayout_5->addWidget(availableTagsView);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        addTagButton = new QPushButton(availableGroup);
        addTagButton->setObjectName("addTagButton");

        horizontalLayout_2->addWidget(addTagButton);

        deleteTagButton = new QPushButton(availableGroup);
        deleteTagButton->setObjectName("deleteTagButton");

        horizontalLayout_2->addWidget(deleteTagButton);

        updateTagButton = new QPushButton(availableGroup);
        updateTagButton->setObjectName("updateTagButton");

        horizontalLayout_2->addWidget(updateTagButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_5->addLayout(horizontalLayout_2);


        horizontalLayout->addWidget(availableGroup);

        asssignmentButtonGroup = new QVBoxLayout();
        asssignmentButtonGroup->setObjectName("asssignmentButtonGroup");
        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        asssignmentButtonGroup->addItem(verticalSpacer_2);

        assignButton = new QPushButton(DlgTags);
        assignButton->setObjectName("assignButton");

        asssignmentButtonGroup->addWidget(assignButton);

        removeButton = new QPushButton(DlgTags);
        removeButton->setObjectName("removeButton");

        asssignmentButtonGroup->addWidget(removeButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        asssignmentButtonGroup->addItem(verticalSpacer);


        horizontalLayout->addLayout(asssignmentButtonGroup);

        assignedGroup = new QGroupBox(DlgTags);
        assignedGroup->setObjectName("assignedGroup");
        verticalLayout_3 = new QVBoxLayout(assignedGroup);
        verticalLayout_3->setObjectName("verticalLayout_3");
        assignedTagsView = new QListView(assignedGroup);
        assignedTagsView->setObjectName("assignedTagsView");

        verticalLayout_3->addWidget(assignedTagsView);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        removeAllButton = new QPushButton(assignedGroup);
        removeAllButton->setObjectName("removeAllButton");

        horizontalLayout_3->addWidget(removeAllButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_3);


        horizontalLayout->addWidget(assignedGroup);


        verticalLayout_2->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(DlgTags);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(availableTagsView, addTagButton);
        QWidget::setTabOrder(addTagButton, deleteTagButton);
        QWidget::setTabOrder(deleteTagButton, updateTagButton);
        QWidget::setTabOrder(updateTagButton, assignButton);
        QWidget::setTabOrder(assignButton, assignedTagsView);
        QWidget::setTabOrder(assignedTagsView, removeButton);
        QWidget::setTabOrder(removeButton, removeAllButton);

        retranslateUi(DlgTags);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgTags, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgTags, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgTags);
    } // setupUi

    void retranslateUi(QDialog *DlgTags)
    {
        DlgTags->setWindowTitle(QCoreApplication::translate("DlgTags", "Tag manager", nullptr));
        availableGroup->setTitle(QCoreApplication::translate("DlgTags", "Available Tags", nullptr));
        addTagButton->setText(QCoreApplication::translate("DlgTags", "Add", nullptr));
        deleteTagButton->setText(QCoreApplication::translate("DlgTags", "Delete", nullptr));
        updateTagButton->setText(QCoreApplication::translate("DlgTags", "Update", nullptr));
#if QT_CONFIG(tooltip)
        assignButton->setToolTip(QCoreApplication::translate("DlgTags", "Assign selected available tags.", nullptr));
#endif // QT_CONFIG(tooltip)
        assignButton->setText(QCoreApplication::translate("DlgTags", "Assign", nullptr));
#if QT_CONFIG(tooltip)
        removeButton->setToolTip(QCoreApplication::translate("DlgTags", "Cancel selected assignment.", nullptr));
#endif // QT_CONFIG(tooltip)
        removeButton->setText(QCoreApplication::translate("DlgTags", "Remove", nullptr));
        assignedGroup->setTitle(QCoreApplication::translate("DlgTags", "Assigned Tags", nullptr));
        removeAllButton->setText(QCoreApplication::translate("DlgTags", "Remove all", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTags: public Ui_DlgTags {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TAGS_H
