/****************************************************************************
** Meta object code from reading C++ file 'types.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/datovka_shared/isds/types.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'types.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4Isds4TypeE_t {};
} // unnamed namespace

template <> constexpr inline auto Isds::Type::qt_create_metaobjectdata<qt_meta_tag_ZN4Isds4TypeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Isds::Type",
        "Privileges",
        "Privilege",
        "PRIVIL_NONE",
        "PRIVIL_READ_NON_PERSONAL",
        "PRIVIL_READ_ALL",
        "PRIVIL_CREATE_DM",
        "PRIVIL_VIEW_INFO",
        "PRIVIL_SEARCH_DB",
        "PRIVIL_OWNER_ADM",
        "PRIVIL_READ_VAULT",
        "PRIVIL_ERASE_VAULT",
        "DmFiltStates",
        "DmFiltState",
        "MFS_POSTED",
        "MFS_STAMPED",
        "MFS_INFECTED",
        "MFS_DELIVERED",
        "MFS_ACCEPTED_FICT",
        "MFS_ACCEPTED",
        "MFS_READ",
        "MFS_UNDELIVERABLE",
        "MFS_REMOVED",
        "MFS_IN_VAULT",
        "MFS_ANY"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'Privileges'
        QtMocHelpers::EnumData<Privileges>(1, 2, QMC::EnumIsFlag).add({
            {    3, Privilege::PRIVIL_NONE },
            {    4, Privilege::PRIVIL_READ_NON_PERSONAL },
            {    5, Privilege::PRIVIL_READ_ALL },
            {    6, Privilege::PRIVIL_CREATE_DM },
            {    7, Privilege::PRIVIL_VIEW_INFO },
            {    8, Privilege::PRIVIL_SEARCH_DB },
            {    9, Privilege::PRIVIL_OWNER_ADM },
            {   10, Privilege::PRIVIL_READ_VAULT },
            {   11, Privilege::PRIVIL_ERASE_VAULT },
        }),
        // flag 'DmFiltStates'
        QtMocHelpers::EnumData<DmFiltStates>(12, 13, QMC::EnumIsFlag).add({
            {   14, DmFiltState::MFS_POSTED },
            {   15, DmFiltState::MFS_STAMPED },
            {   16, DmFiltState::MFS_INFECTED },
            {   17, DmFiltState::MFS_DELIVERED },
            {   18, DmFiltState::MFS_ACCEPTED_FICT },
            {   19, DmFiltState::MFS_ACCEPTED },
            {   20, DmFiltState::MFS_READ },
            {   21, DmFiltState::MFS_UNDELIVERABLE },
            {   22, DmFiltState::MFS_REMOVED },
            {   23, DmFiltState::MFS_IN_VAULT },
            {   24, DmFiltState::MFS_ANY },
        }),
    };
    return QtMocHelpers::metaObjectData<Type, qt_meta_tag_ZN4Isds4TypeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Isds::Type::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Isds4TypeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Isds4TypeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4Isds4TypeE_t>.metaTypes,
    nullptr
} };

void Isds::Type::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Type *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Isds::Type::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Isds::Type::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Isds4TypeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Isds::Type::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
