/********************************************************************************
** Form generated from reading UI file 'dlg_ds_search.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_DS_SEARCH_H
#define UI_DLG_DS_SEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgDsSearch
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelAccount;
    QLabel *accountInfo;
    QSpacerItem *horizontalSpacer;
    QFrame *line_3;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *useFulltextCheckBox;
    QLabel *labelSearchDescr;
    QLabel *labelSearchInfo;
    QFormLayout *formLayout;
    QLabel *dataBoxTypeLabel;
    QComboBox *dataBoxTypeCBox;
    QLabel *fulltextTargetLabel;
    QComboBox *fulltextTargetCBox;
    QFrame *line;
    QLabel *textLineLabel;
    QLineEdit *textLineEdit;
    QLabel *iDLineLabel;
    QLineEdit *iDLineEdit;
    QLabel *iCLineLabel;
    QLineEdit *iCLineEdit;
    QLabel *nameLineLabel;
    QLineEdit *nameLineEdit;
    QLabel *pscLineLabel;
    QLineEdit *pscLineEdit;
    QPushButton *searchPushButton;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout;
    QLabel *searchResultText;
    QSpacerItem *horizontalSpacer_2;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    LoweredTableView *contactTableView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgDsSearch)
    {
        if (DlgDsSearch->objectName().isEmpty())
            DlgDsSearch->setObjectName("DlgDsSearch");
        DlgDsSearch->resize(560, 600);
        verticalLayout = new QVBoxLayout(DlgDsSearch);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelAccount = new QLabel(DlgDsSearch);
        labelAccount->setObjectName("labelAccount");

        horizontalLayout_2->addWidget(labelAccount);

        accountInfo = new QLabel(DlgDsSearch);
        accountInfo->setObjectName("accountInfo");

        horizontalLayout_2->addWidget(accountInfo);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        line_3 = new QFrame(DlgDsSearch);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_3);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        useFulltextCheckBox = new QCheckBox(DlgDsSearch);
        useFulltextCheckBox->setObjectName("useFulltextCheckBox");

        verticalLayout_2->addWidget(useFulltextCheckBox);

        labelSearchDescr = new QLabel(DlgDsSearch);
        labelSearchDescr->setObjectName("labelSearchDescr");

        verticalLayout_2->addWidget(labelSearchDescr);

        labelSearchInfo = new QLabel(DlgDsSearch);
        labelSearchInfo->setObjectName("labelSearchInfo");
        labelSearchInfo->setEnabled(false);
        QFont font;
        font.setItalic(true);
        labelSearchInfo->setFont(font);
        labelSearchInfo->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(labelSearchInfo);


        verticalLayout->addLayout(verticalLayout_2);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        dataBoxTypeLabel = new QLabel(DlgDsSearch);
        dataBoxTypeLabel->setObjectName("dataBoxTypeLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, dataBoxTypeLabel);

        dataBoxTypeCBox = new QComboBox(DlgDsSearch);
        dataBoxTypeCBox->setObjectName("dataBoxTypeCBox");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, dataBoxTypeCBox);

        fulltextTargetLabel = new QLabel(DlgDsSearch);
        fulltextTargetLabel->setObjectName("fulltextTargetLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, fulltextTargetLabel);

        fulltextTargetCBox = new QComboBox(DlgDsSearch);
        fulltextTargetCBox->setObjectName("fulltextTargetCBox");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, fulltextTargetCBox);

        line = new QFrame(DlgDsSearch);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        formLayout->setWidget(2, QFormLayout::ItemRole::SpanningRole, line);

        textLineLabel = new QLabel(DlgDsSearch);
        textLineLabel->setObjectName("textLineLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, textLineLabel);

        textLineEdit = new QLineEdit(DlgDsSearch);
        textLineEdit->setObjectName("textLineEdit");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, textLineEdit);

        iDLineLabel = new QLabel(DlgDsSearch);
        iDLineLabel->setObjectName("iDLineLabel");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, iDLineLabel);

        iDLineEdit = new QLineEdit(DlgDsSearch);
        iDLineEdit->setObjectName("iDLineEdit");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, iDLineEdit);

        iCLineLabel = new QLabel(DlgDsSearch);
        iCLineLabel->setObjectName("iCLineLabel");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, iCLineLabel);

        iCLineEdit = new QLineEdit(DlgDsSearch);
        iCLineEdit->setObjectName("iCLineEdit");

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, iCLineEdit);

        nameLineLabel = new QLabel(DlgDsSearch);
        nameLineLabel->setObjectName("nameLineLabel");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, nameLineLabel);

        nameLineEdit = new QLineEdit(DlgDsSearch);
        nameLineEdit->setObjectName("nameLineEdit");

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, nameLineEdit);

        pscLineLabel = new QLabel(DlgDsSearch);
        pscLineLabel->setObjectName("pscLineLabel");

        formLayout->setWidget(7, QFormLayout::ItemRole::LabelRole, pscLineLabel);

        pscLineEdit = new QLineEdit(DlgDsSearch);
        pscLineEdit->setObjectName("pscLineEdit");

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, pscLineEdit);


        verticalLayout->addLayout(formLayout);

        searchPushButton = new QPushButton(DlgDsSearch);
        searchPushButton->setObjectName("searchPushButton");
        searchPushButton->setEnabled(false);

        verticalLayout->addWidget(searchPushButton);

        line_2 = new QFrame(DlgDsSearch);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        searchResultText = new QLabel(DlgDsSearch);
        searchResultText->setObjectName("searchResultText");

        horizontalLayout->addWidget(searchResultText);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        filterLabel = new QLabel(DlgDsSearch);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgDsSearch);
        filterLine->setObjectName("filterLine");

        horizontalLayout->addWidget(filterLine);


        verticalLayout->addLayout(horizontalLayout);

        contactTableView = new LoweredTableView(DlgDsSearch);
        contactTableView->setObjectName("contactTableView");
        contactTableView->setShowGrid(false);
        contactTableView->setWordWrap(false);
        contactTableView->horizontalHeader()->setHighlightSections(false);
        contactTableView->horizontalHeader()->setStretchLastSection(true);
        contactTableView->verticalHeader()->setVisible(false);
        contactTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout->addWidget(contactTableView);

        buttonBox = new QDialogButtonBox(DlgDsSearch);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(useFulltextCheckBox, dataBoxTypeCBox);
        QWidget::setTabOrder(dataBoxTypeCBox, fulltextTargetCBox);
        QWidget::setTabOrder(fulltextTargetCBox, textLineEdit);
        QWidget::setTabOrder(textLineEdit, iDLineEdit);
        QWidget::setTabOrder(iDLineEdit, iCLineEdit);
        QWidget::setTabOrder(iCLineEdit, nameLineEdit);
        QWidget::setTabOrder(nameLineEdit, pscLineEdit);
        QWidget::setTabOrder(pscLineEdit, searchPushButton);
        QWidget::setTabOrder(searchPushButton, contactTableView);
        QWidget::setTabOrder(contactTableView, filterLine);

        retranslateUi(DlgDsSearch);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgDsSearch, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgDsSearch, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgDsSearch);
    } // setupUi

    void retranslateUi(QDialog *DlgDsSearch)
    {
        DlgDsSearch->setWindowTitle(QCoreApplication::translate("DlgDsSearch", "Search recipient", nullptr));
        labelAccount->setText(QCoreApplication::translate("DlgDsSearch", "Current data box:", nullptr));
        accountInfo->setText(QCoreApplication::translate("DlgDsSearch", "n/a", nullptr));
        useFulltextCheckBox->setText(QCoreApplication::translate("DlgDsSearch", "Use full-text search similar to the ISDS client portal.", nullptr));
        labelSearchDescr->setText(QCoreApplication::translate("DlgDsSearch", "n/a", nullptr));
#if QT_CONFIG(tooltip)
        labelSearchInfo->setToolTip(QCoreApplication::translate("DlgDsSearch", "Your data box is not of type OVM (i.e. non-OVM). Sending of post data messages\n"
"from your data box is activated. This means that you can only search for data boxes\n"
"of the type OVM and data boxes that have the the receiving of post data messages\n"
"activated. Because of this limitation the results of your current search may not\n"
"contain all otherwise matching databoxes.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelSearchInfo->setText(QCoreApplication::translate("DlgDsSearch", "Note: Your search results will be limited. See tooltip for more information.", nullptr));
        dataBoxTypeLabel->setText(QCoreApplication::translate("DlgDsSearch", "Databox type:", nullptr));
        fulltextTargetLabel->setText(QCoreApplication::translate("DlgDsSearch", "Search in fileds:", nullptr));
        textLineLabel->setText(QCoreApplication::translate("DlgDsSearch", "Text:", nullptr));
        iDLineLabel->setText(QCoreApplication::translate("DlgDsSearch", "Box ID:", nullptr));
        iCLineLabel->setText(QCoreApplication::translate("DlgDsSearch", "Identification number (I\304\214O):", nullptr));
        nameLineLabel->setText(QCoreApplication::translate("DlgDsSearch", "Name:", nullptr));
        pscLineLabel->setText(QCoreApplication::translate("DlgDsSearch", "Postal code:", nullptr));
        searchPushButton->setText(QCoreApplication::translate("DlgDsSearch", "Search", nullptr));
        searchResultText->setText(QCoreApplication::translate("DlgDsSearch", "n/a", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgDsSearch", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgDsSearch: public Ui_DlgDsSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_DS_SEARCH_H
