/********************************************************************************
** Form generated from reading UI file 'dlg_change_pwd.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CHANGE_PWD_H
#define UI_DLG_CHANGE_PWD_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgChangePwd
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoLabel;
    QFormLayout *formLayout;
    QLabel *label_2;
    QLineEdit *accountLine;
    QLabel *label_3;
    QLineEdit *currentPwdLine;
    QLabel *label_4;
    QLineEdit *newPwdLine;
    QLabel *label_5;
    QLineEdit *newPwdLine2;
    QLabel *label_6;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *togglePwdVisibilityButton;
    QPushButton *generateButton;
    QLabel *label;
    QLineEdit *userNameLine;
    QLabel *infoPwdLabel;
    QFrame *line;
    QHBoxLayout *horizontalLayout_2;
    QLabel *otpLabel;
    QLineEdit *secCodeLine;
    QPushButton *smsPushButton;
    QFrame *line_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgChangePwd)
    {
        if (DlgChangePwd->objectName().isEmpty())
            DlgChangePwd->setObjectName("DlgChangePwd");
        DlgChangePwd->resize(351, 386);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DlgChangePwd->sizePolicy().hasHeightForWidth());
        DlgChangePwd->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(DlgChangePwd);
        verticalLayout->setObjectName("verticalLayout");
        infoLabel = new QLabel(DlgChangePwd);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignCenter);
        infoLabel->setWordWrap(true);

        verticalLayout->addWidget(infoLabel);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        label_2 = new QLabel(DlgChangePwd);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        accountLine = new QLineEdit(DlgChangePwd);
        accountLine->setObjectName("accountLine");
        accountLine->setEnabled(false);
        accountLine->setReadOnly(true);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, accountLine);

        label_3 = new QLabel(DlgChangePwd);
        label_3->setObjectName("label_3");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_3);

        currentPwdLine = new QLineEdit(DlgChangePwd);
        currentPwdLine->setObjectName("currentPwdLine");
        currentPwdLine->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, currentPwdLine);

        label_4 = new QLabel(DlgChangePwd);
        label_4->setObjectName("label_4");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_4);

        newPwdLine = new QLineEdit(DlgChangePwd);
        newPwdLine->setObjectName("newPwdLine");
        newPwdLine->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, newPwdLine);

        label_5 = new QLabel(DlgChangePwd);
        label_5->setObjectName("label_5");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_5);

        newPwdLine2 = new QLineEdit(DlgChangePwd);
        newPwdLine2->setObjectName("newPwdLine2");
        newPwdLine2->setEchoMode(QLineEdit::Password);

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, newPwdLine2);

        label_6 = new QLabel(DlgChangePwd);
        label_6->setObjectName("label_6");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, label_6);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        togglePwdVisibilityButton = new QPushButton(DlgChangePwd);
        togglePwdVisibilityButton->setObjectName("togglePwdVisibilityButton");

        horizontalLayout->addWidget(togglePwdVisibilityButton);

        generateButton = new QPushButton(DlgChangePwd);
        generateButton->setObjectName("generateButton");

        horizontalLayout->addWidget(generateButton);


        formLayout->setLayout(5, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        label = new QLabel(DlgChangePwd);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        userNameLine = new QLineEdit(DlgChangePwd);
        userNameLine->setObjectName("userNameLine");
        userNameLine->setEnabled(false);
        userNameLine->setReadOnly(true);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, userNameLine);


        verticalLayout->addLayout(formLayout);

        infoPwdLabel = new QLabel(DlgChangePwd);
        infoPwdLabel->setObjectName("infoPwdLabel");
        QFont font;
        font.setItalic(true);
        infoPwdLabel->setFont(font);

        verticalLayout->addWidget(infoPwdLabel);

        line = new QFrame(DlgChangePwd);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        otpLabel = new QLabel(DlgChangePwd);
        otpLabel->setObjectName("otpLabel");

        horizontalLayout_2->addWidget(otpLabel);

        secCodeLine = new QLineEdit(DlgChangePwd);
        secCodeLine->setObjectName("secCodeLine");

        horizontalLayout_2->addWidget(secCodeLine);

        smsPushButton = new QPushButton(DlgChangePwd);
        smsPushButton->setObjectName("smsPushButton");

        horizontalLayout_2->addWidget(smsPushButton);


        verticalLayout->addLayout(horizontalLayout_2);

        line_2 = new QFrame(DlgChangePwd);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        buttonBox = new QDialogButtonBox(DlgChangePwd);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(currentPwdLine, newPwdLine);
        QWidget::setTabOrder(newPwdLine, newPwdLine2);
        QWidget::setTabOrder(newPwdLine2, togglePwdVisibilityButton);
        QWidget::setTabOrder(togglePwdVisibilityButton, generateButton);
        QWidget::setTabOrder(generateButton, smsPushButton);
        QWidget::setTabOrder(smsPushButton, secCodeLine);
        QWidget::setTabOrder(secCodeLine, userNameLine);
        QWidget::setTabOrder(userNameLine, accountLine);

        retranslateUi(DlgChangePwd);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgChangePwd, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgChangePwd, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgChangePwd);
    } // setupUi

    void retranslateUi(QDialog *DlgChangePwd)
    {
        DlgChangePwd->setWindowTitle(QCoreApplication::translate("DlgChangePwd", "Change password", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgChangePwd", "This changes the password on the ISDS server. Please enter your current and the new password below:", nullptr));
        label_2->setText(QCoreApplication::translate("DlgChangePwd", "Data-box ID:", nullptr));
        label_3->setText(QCoreApplication::translate("DlgChangePwd", "Current password:", nullptr));
#if QT_CONFIG(tooltip)
        label_4->setToolTip(QCoreApplication::translate("DlgChangePwd", "The password must be at least 8 characters long and\n"
"must contain at least one digit and one capital letter.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("DlgChangePwd", "New password:", nullptr));
#if QT_CONFIG(tooltip)
        newPwdLine->setToolTip(QCoreApplication::translate("DlgChangePwd", "The password must be at least 8 characters long and\n"
"must contain at least one digit and one capital letter.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        label_5->setToolTip(QCoreApplication::translate("DlgChangePwd", "Repeat the password. The password must be\n"
"at least 8 characters long and must contain\n"
"at least one digit and one capital letter.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_5->setText(QCoreApplication::translate("DlgChangePwd", "Repeat new password:", nullptr));
#if QT_CONFIG(tooltip)
        newPwdLine2->setToolTip(QCoreApplication::translate("DlgChangePwd", "Repeat the password. The password must be\n"
"at least 8 characters long and must contain\n"
"at least one digit and one capital letter.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_6->setText(QString());
        togglePwdVisibilityButton->setText(QCoreApplication::translate("DlgChangePwd", "Show", nullptr));
        generateButton->setText(QCoreApplication::translate("DlgChangePwd", "Generate", nullptr));
        label->setText(QCoreApplication::translate("DlgChangePwd", "Username:", nullptr));
        infoPwdLabel->setText(QCoreApplication::translate("DlgChangePwd", "Note: Remember your new password.", nullptr));
        otpLabel->setText(QCoreApplication::translate("DlgChangePwd", "Enter SMS code:", nullptr));
#if QT_CONFIG(tooltip)
        secCodeLine->setToolTip(QCoreApplication::translate("DlgChangePwd", "Enter SMS or security code", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        smsPushButton->setToolTip(QCoreApplication::translate("DlgChangePwd", "Send SMS security code.", nullptr));
#endif // QT_CONFIG(tooltip)
        smsPushButton->setText(QCoreApplication::translate("DlgChangePwd", "Send SMS code", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgChangePwd: public Ui_DlgChangePwd {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CHANGE_PWD_H
