/********************************************************************************
** Form generated from reading UI file 'dlg_convert_export_for_mobile_app.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_CONVERT_EXPORT_FOR_MOBILE_APP_H
#define UI_DLG_CONVERT_EXPORT_FOR_MOBILE_APP_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgConvertExportForMobileApp
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoLabel;
    QFrame *line;
    QGroupBox *whereGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *dirLabel;
    QLineEdit *directoryLine;
    QPushButton *chooseDirButton;
    QLabel *availableSpaceLabel;
    QGroupBox *whatGroupBox;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *allCheckBox;
    LoweredTableView *accountView;
    QLabel *requiredSpaceLabel;
    QLabel *warningLabel;
    QProgressBar *progressBar;
    QHBoxLayout *horizontalLayout_02;
    QDialogButtonBox *buttonBox;
    QPushButton *convertButton;

    void setupUi(QDialog *DlgConvertExportForMobileApp)
    {
        if (DlgConvertExportForMobileApp->objectName().isEmpty())
            DlgConvertExportForMobileApp->setObjectName("DlgConvertExportForMobileApp");
        DlgConvertExportForMobileApp->resize(415, 560);
        verticalLayout = new QVBoxLayout(DlgConvertExportForMobileApp);
        verticalLayout->setObjectName("verticalLayout");
        infoLabel = new QLabel(DlgConvertExportForMobileApp);
        infoLabel->setObjectName("infoLabel");
        infoLabel->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        infoLabel->setWordWrap(true);

        verticalLayout->addWidget(infoLabel);

        line = new QFrame(DlgConvertExportForMobileApp);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        whereGroupBox = new QGroupBox(DlgConvertExportForMobileApp);
        whereGroupBox->setObjectName("whereGroupBox");
        verticalLayout_2 = new QVBoxLayout(whereGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        dirLabel = new QLabel(whereGroupBox);
        dirLabel->setObjectName("dirLabel");

        horizontalLayout->addWidget(dirLabel);

        directoryLine = new QLineEdit(whereGroupBox);
        directoryLine->setObjectName("directoryLine");

        horizontalLayout->addWidget(directoryLine);

        chooseDirButton = new QPushButton(whereGroupBox);
        chooseDirButton->setObjectName("chooseDirButton");

        horizontalLayout->addWidget(chooseDirButton);


        verticalLayout_2->addLayout(horizontalLayout);

        availableSpaceLabel = new QLabel(whereGroupBox);
        availableSpaceLabel->setObjectName("availableSpaceLabel");

        verticalLayout_2->addWidget(availableSpaceLabel);


        verticalLayout->addWidget(whereGroupBox);

        whatGroupBox = new QGroupBox(DlgConvertExportForMobileApp);
        whatGroupBox->setObjectName("whatGroupBox");
        verticalLayout_3 = new QVBoxLayout(whatGroupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        allCheckBox = new QCheckBox(whatGroupBox);
        allCheckBox->setObjectName("allCheckBox");

        verticalLayout_3->addWidget(allCheckBox);

        accountView = new LoweredTableView(whatGroupBox);
        accountView->setObjectName("accountView");
        accountView->setShowGrid(false);
        accountView->setWordWrap(false);
        accountView->horizontalHeader()->setHighlightSections(false);
        accountView->horizontalHeader()->setStretchLastSection(true);
        accountView->verticalHeader()->setVisible(false);
        accountView->verticalHeader()->setHighlightSections(false);

        verticalLayout_3->addWidget(accountView);

        requiredSpaceLabel = new QLabel(whatGroupBox);
        requiredSpaceLabel->setObjectName("requiredSpaceLabel");

        verticalLayout_3->addWidget(requiredSpaceLabel);


        verticalLayout->addWidget(whatGroupBox);

        warningLabel = new QLabel(DlgConvertExportForMobileApp);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(warningLabel);

        progressBar = new QProgressBar(DlgConvertExportForMobileApp);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(0);

        verticalLayout->addWidget(progressBar);

        horizontalLayout_02 = new QHBoxLayout();
        horizontalLayout_02->setObjectName("horizontalLayout_02");
        buttonBox = new QDialogButtonBox(DlgConvertExportForMobileApp);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        horizontalLayout_02->addWidget(buttonBox);

        convertButton = new QPushButton(DlgConvertExportForMobileApp);
        convertButton->setObjectName("convertButton");

        horizontalLayout_02->addWidget(convertButton);


        verticalLayout->addLayout(horizontalLayout_02);


        retranslateUi(DlgConvertExportForMobileApp);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgConvertExportForMobileApp, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgConvertExportForMobileApp);
    } // setupUi

    void retranslateUi(QDialog *DlgConvertExportForMobileApp)
    {
        DlgConvertExportForMobileApp->setWindowTitle(QCoreApplication::translate("DlgConvertExportForMobileApp", "Transfer Data", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "n/a", nullptr));
        whereGroupBox->setTitle(QCoreApplication::translate("DlgConvertExportForMobileApp", "Where do you want to export your data?", nullptr));
        dirLabel->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "Directory", nullptr));
        chooseDirButton->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "Choose", nullptr));
        availableSpaceLabel->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "n/a", nullptr));
        whatGroupBox->setTitle(QCoreApplication::translate("DlgConvertExportForMobileApp", "What do you want to export?", nullptr));
        allCheckBox->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "Select all", nullptr));
        requiredSpaceLabel->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "n/a", nullptr));
        warningLabel->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "n/a", nullptr));
        convertButton->setText(QCoreApplication::translate("DlgConvertExportForMobileApp", "Convert", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgConvertExportForMobileApp: public Ui_DlgConvertExportForMobileApp {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_CONVERT_EXPORT_FOR_MOBILE_APP_H
