/********************************************************************************
** Form generated from reading UI file 'dlg_import_zfo_result.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_IMPORT_ZFO_RESULT_H
#define UI_DLG_IMPORT_ZFO_RESULT_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgImportZFOResult
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout_1;
    QLabel *label_1;
    QLabel *zfoTotal;
    QFrame *line;
    QFormLayout *formLayout_2;
    QLabel *labelDone;
    QLabel *zfoInserted;
    QLabel *succListHeader;
    QTextEdit *successList;
    QFrame *line_3;
    QFormLayout *formLayout;
    QLabel *labelExist;
    QLabel *zfoExist;
    QLabel *existListHeader;
    QTextEdit *existList;
    QFrame *line_2;
    QFormLayout *formLayout_3;
    QLabel *labelError;
    QLabel *zfoErrors;
    QLabel *errListHeader;
    QTextEdit *errorList;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgImportZFOResult)
    {
        if (DlgImportZFOResult->objectName().isEmpty())
            DlgImportZFOResult->setObjectName("DlgImportZFOResult");
        DlgImportZFOResult->resize(400, 451);
        verticalLayout = new QVBoxLayout(DlgImportZFOResult);
        verticalLayout->setObjectName("verticalLayout");
        formLayout_1 = new QFormLayout();
        formLayout_1->setObjectName("formLayout_1");
        label_1 = new QLabel(DlgImportZFOResult);
        label_1->setObjectName("label_1");

        formLayout_1->setWidget(0, QFormLayout::ItemRole::LabelRole, label_1);

        zfoTotal = new QLabel(DlgImportZFOResult);
        zfoTotal->setObjectName("zfoTotal");
        QFont font;
        font.setBold(true);
        zfoTotal->setFont(font);
        zfoTotal->setText(QString::fromUtf8("0"));

        formLayout_1->setWidget(0, QFormLayout::ItemRole::FieldRole, zfoTotal);


        verticalLayout->addLayout(formLayout_1);

        line = new QFrame(DlgImportZFOResult);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        labelDone = new QLabel(DlgImportZFOResult);
        labelDone->setObjectName("labelDone");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, labelDone);

        zfoInserted = new QLabel(DlgImportZFOResult);
        zfoInserted->setObjectName("zfoInserted");
        zfoInserted->setFont(font);
        zfoInserted->setText(QString::fromUtf8("0"));

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, zfoInserted);


        verticalLayout->addLayout(formLayout_2);

        succListHeader = new QLabel(DlgImportZFOResult);
        succListHeader->setObjectName("succListHeader");

        verticalLayout->addWidget(succListHeader);

        successList = new QTextEdit(DlgImportZFOResult);
        successList->setObjectName("successList");
        successList->setReadOnly(true);

        verticalLayout->addWidget(successList);

        line_3 = new QFrame(DlgImportZFOResult);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_3);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        labelExist = new QLabel(DlgImportZFOResult);
        labelExist->setObjectName("labelExist");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, labelExist);

        zfoExist = new QLabel(DlgImportZFOResult);
        zfoExist->setObjectName("zfoExist");
        zfoExist->setFont(font);
        zfoExist->setText(QString::fromUtf8("0"));

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, zfoExist);


        verticalLayout->addLayout(formLayout);

        existListHeader = new QLabel(DlgImportZFOResult);
        existListHeader->setObjectName("existListHeader");

        verticalLayout->addWidget(existListHeader);

        existList = new QTextEdit(DlgImportZFOResult);
        existList->setObjectName("existList");

        verticalLayout->addWidget(existList);

        line_2 = new QFrame(DlgImportZFOResult);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        labelError = new QLabel(DlgImportZFOResult);
        labelError->setObjectName("labelError");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, labelError);

        zfoErrors = new QLabel(DlgImportZFOResult);
        zfoErrors->setObjectName("zfoErrors");
        zfoErrors->setFont(font);
        zfoErrors->setText(QString::fromUtf8("0"));

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, zfoErrors);


        verticalLayout->addLayout(formLayout_3);

        errListHeader = new QLabel(DlgImportZFOResult);
        errListHeader->setObjectName("errListHeader");

        verticalLayout->addWidget(errListHeader);

        errorList = new QTextEdit(DlgImportZFOResult);
        errorList->setObjectName("errorList");
        errorList->setReadOnly(true);

        verticalLayout->addWidget(errorList);

        buttonBox = new QDialogButtonBox(DlgImportZFOResult);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgImportZFOResult);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgImportZFOResult, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgImportZFOResult, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgImportZFOResult);
    } // setupUi

    void retranslateUi(QDialog *DlgImportZFOResult)
    {
        DlgImportZFOResult->setWindowTitle(QCoreApplication::translate("DlgImportZFOResult", "Import ZFO result", nullptr));
        label_1->setText(QCoreApplication::translate("DlgImportZFOResult", "Number of ZFO files for import to databases:", nullptr));
        labelDone->setText(QCoreApplication::translate("DlgImportZFOResult", "Number of newly imported files:", nullptr));
        succListHeader->setText(QCoreApplication::translate("DlgImportZFOResult", "Details:", nullptr));
        labelExist->setText(QCoreApplication::translate("DlgImportZFOResult", "Number of existing files:", nullptr));
        existListHeader->setText(QCoreApplication::translate("DlgImportZFOResult", "Details:", nullptr));
        labelError->setText(QCoreApplication::translate("DlgImportZFOResult", "Number of unsuccessfully imported files:", nullptr));
        errListHeader->setText(QCoreApplication::translate("DlgImportZFOResult", "Details:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgImportZFOResult: public Ui_DlgImportZFOResult {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_IMPORT_ZFO_RESULT_H
