/********************************************************************************
** Form generated from reading UI file 'datovka.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DATOVKA_H
#define UI_DATOVKA_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/views/attachment_table_view.h"
#include "src/views/lowered_table_view.h"
#include "src/views/lowered_tree_view.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionSynchroniseAllAccounts;
    QAction *actionAddAccount;
    QAction *actionRemoveAccount;
    QAction *actionProxySettings;
    QAction *actionPreferences;
    QAction *actionQuit;
    QAction *actionImportAccountFromDatabase;
    QAction *actionSynchroniseAccount;
    QAction *actionSynchroniseAccountReceived;
    QAction *actionSynchroniseAccountSent;
    QAction *actionSendMessage;
    QAction *actionChangePassword;
    QAction *actionAccountProperties;
    QAction *actionMoveAccountUp;
    QAction *actionMoveAccountDown;
    QAction *actionChangeDataDirectory;
    QAction *actionAboutApplication;
    QAction *actionHelp;
    QAction *actionFindDataBox;
    QAction *actionAuthenticateMessageFile;
    QAction *actionViewMessageFile;
    QAction *actionExportCorrespondenceOverview;
    QAction *actionDownloadMessage;
    QAction *actionReply;
    QAction *actionSignatureDetail;
    QAction *actionAuthenticateMessage;
    QAction *actionDeleteMessage;
    QAction *actionExportMessageZfo;
    QAction *actionOpenMessageExternal;
    QAction *actionOpenAcceptanceInfoExternal;
    QAction *actionExportAcceptanceInfoZfo;
    QAction *actionExportAcceptanceInfoPdf;
    QAction *actionExportEnvelopePdf;
    QAction *actionExportEnvelopePdfAndAttachments;
    QAction *actionOpenAttachment;
    QAction *actionSaveSelectedAttachments;
    QAction *actionSaveAllAttachments;
    QAction *actionImportZfoIntoDatabase;
    QAction *actionUseAsTemplate;
    QAction *actionMessageSearch;
    QAction *actionCheckMessageTimestampExpiration;
    QAction *actionHomepage;
    QAction *actionImportMessagesFromDatabase;
    QAction *actionSplitDatabaseYears;
    QAction *actionEmailContentSelection;
    QAction *actionEmail_selected_attachments;
    QAction *actionEditAvailableTags;
    QAction *actionVacuumMessageDatabase;
    QAction *actionForward;
    QAction *actionRecordsManagementSettings;
    QAction *actionSendToRecordsManagement;
    QAction *actionUpdateRecordsManagementInformation;
    QAction *actionViewLog;
    QAction *actionSendEGovRequest;
    QAction *actionBackUp;
    QAction *actionRestore;
    QAction *actionViewAllSettings;
    QAction *actionAccountSettings;
    QAction *actionMarkAllReceivedRead;
    QAction *actionMarkAllReceivedUnread;
    QAction *actionMarkAllReceivedUnsettled;
    QAction *actionMarkAllReceivedInProgress;
    QAction *actionMarkAllReceivedSettled;
    QAction *actionEmailZfos;
    QAction *actionEmailAllAttachments;
    QAction *actionCheckUploadMessagesRecordsManagement;
    QAction *actionConvertImportFromMobileApp;
    QAction *actionConvertExportForMobileApp;
    QAction *actionTagStorageSettings;
    QAction *actionCustomiseToolBar;
    QAction *actionCustomiseAttachmentBar;
    QAction *actionCustomiseDetailBar;
    QAction *actionCustomiseMessageBar;
    QAction *actionCustomiseAccountBar;
    QAction *actionFilterMessages;
    QAction *actionToolBar;
    QAction *actionTopPanelBars;
    QAction *actionEditTagAssignment;
    QAction *actionRepairMessageDatabaseFiles;
    QAction *actionTrackTimestampExpiration;
    QAction *actionTimeStampRenewal;
    QAction *actionDeleteDraft;
    QAction *actionEditDraft;
    QWidget *centralWidgetWindows;
    QHBoxLayout *horizontalLayout_4;
    QSplitter *hSplitterAccount;
    QWidget *verticalLayoutWidget;
    QVBoxLayout *accountsVerticalLayout;
    LoweredTreeView *accountList;
    QStackedWidget *messageStackedWidget;
    QWidget *accountInfoPage;
    QHBoxLayout *horizontalLayout_3;
    QTextBrowser *accountTextInfo;
    QWidget *messagePage;
    QHBoxLayout *horizontalLayout_2;
    QSplitter *vSplitterMessage;
    QWidget *verticalLayoutWidget_2;
    QVBoxLayout *messagesVerticalLayout;
    LoweredTableView *messageList;
    QWidget *messagesFilterHorizontalLayout;
    QHBoxLayout *messagesFilterHorizontalLayout_2;
    QLineEdit *messageFilterLine;
    QPushButton *messageFilterCloseButton;
    QSplitter *hSplitterMessageInfo;
    QWidget *layoutWidget;
    QVBoxLayout *detailVerticalLayout;
    QTextEdit *messageInfo;
    QWidget *layoutWidget1;
    QVBoxLayout *attachmentsVerticalLayout;
    AttachmentTableView *attachmentList;
    QWidget *draftPage;
    QVBoxLayout *verticalLayout;
    QVBoxLayout *draftsVerticalLayout;
    LoweredTableView *draftList;
    QWidget *draftsFilterHorizontalLayout;
    QHBoxLayout *draftsFilterHorizontalLayout_2;
    QLineEdit *draftFilterLine;
    QPushButton *draftFilterCloseButton;
    QMenuBar *menubar;
    QMenu *menuFile;
    QMenu *menuRecordsManagement;
    QMenu *menuDatabox;
    QMenu *menuMarkAllReceivedAs;
    QMenu *menuSynchronisation;
    QMenu *menuMessage;
    QMenu *menuEmailWith;
    QMenu *menuTools;
    QMenu *menuToolBars;
    QMenu *menuHelp;
    QToolBar *toolBar;
    QStatusBar *statusBar;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(800, 600);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MainWindow->sizePolicy().hasHeightForWidth());
        MainWindow->setSizePolicy(sizePolicy);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/png_app_logo/128x128/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        MainWindow->setWindowIcon(icon);
        MainWindow->setLayoutDirection(Qt::LeftToRight);
        MainWindow->setLocale(QLocale(QLocale::English, QLocale::UnitedKingdom));
        actionSynchroniseAllAccounts = new QAction(MainWindow);
        actionSynchroniseAllAccounts->setObjectName("actionSynchroniseAllAccounts");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/png_app_logo/16x16/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionSynchroniseAllAccounts->setIcon(icon1);
        actionSynchroniseAllAccounts->setMenuRole(QAction::NoRole);
        actionAddAccount = new QAction(MainWindow);
        actionAddAccount->setObjectName("actionAddAccount");
        actionAddAccount->setIcon(icon1);
        actionAddAccount->setMenuRole(QAction::NoRole);
        actionRemoveAccount = new QAction(MainWindow);
        actionRemoveAccount->setObjectName("actionRemoveAccount");
        actionRemoveAccount->setIcon(icon1);
        actionRemoveAccount->setMenuRole(QAction::NoRole);
        actionProxySettings = new QAction(MainWindow);
        actionProxySettings->setObjectName("actionProxySettings");
        actionProxySettings->setEnabled(true);
        actionProxySettings->setIcon(icon1);
        actionProxySettings->setMenuRole(QAction::NoRole);
        actionPreferences = new QAction(MainWindow);
        actionPreferences->setObjectName("actionPreferences");
        actionPreferences->setIcon(icon1);
        actionPreferences->setMenuRole(QAction::PreferencesRole);
        actionQuit = new QAction(MainWindow);
        actionQuit->setObjectName("actionQuit");
        actionQuit->setIcon(icon1);
        actionQuit->setMenuRole(QAction::QuitRole);
        actionImportAccountFromDatabase = new QAction(MainWindow);
        actionImportAccountFromDatabase->setObjectName("actionImportAccountFromDatabase");
        actionImportAccountFromDatabase->setIcon(icon1);
        actionImportAccountFromDatabase->setMenuRole(QAction::NoRole);
        actionSynchroniseAccount = new QAction(MainWindow);
        actionSynchroniseAccount->setObjectName("actionSynchroniseAccount");
        actionSynchroniseAccount->setIcon(icon1);
        actionSynchroniseAccount->setMenuRole(QAction::NoRole);
        actionSynchroniseAccountReceived = new QAction(MainWindow);
        actionSynchroniseAccountReceived->setObjectName("actionSynchroniseAccountReceived");
        actionSynchroniseAccountReceived->setIcon(icon1);
        actionSynchroniseAccountReceived->setMenuRole(QAction::NoRole);
        actionSynchroniseAccountSent = new QAction(MainWindow);
        actionSynchroniseAccountSent->setObjectName("actionSynchroniseAccountSent");
        actionSynchroniseAccountSent->setIcon(icon1);
        actionSynchroniseAccountSent->setMenuRole(QAction::NoRole);
        actionSendMessage = new QAction(MainWindow);
        actionSendMessage->setObjectName("actionSendMessage");
        actionSendMessage->setIcon(icon1);
        actionSendMessage->setMenuRole(QAction::NoRole);
        actionChangePassword = new QAction(MainWindow);
        actionChangePassword->setObjectName("actionChangePassword");
        actionChangePassword->setIcon(icon1);
        actionChangePassword->setMenuRole(QAction::NoRole);
        actionAccountProperties = new QAction(MainWindow);
        actionAccountProperties->setObjectName("actionAccountProperties");
        actionAccountProperties->setIcon(icon1);
        actionAccountProperties->setMenuRole(QAction::NoRole);
        actionMoveAccountUp = new QAction(MainWindow);
        actionMoveAccountUp->setObjectName("actionMoveAccountUp");
        actionMoveAccountUp->setIcon(icon1);
        actionMoveAccountUp->setMenuRole(QAction::NoRole);
        actionMoveAccountDown = new QAction(MainWindow);
        actionMoveAccountDown->setObjectName("actionMoveAccountDown");
        actionMoveAccountDown->setIcon(icon1);
        actionMoveAccountDown->setMenuRole(QAction::NoRole);
        actionChangeDataDirectory = new QAction(MainWindow);
        actionChangeDataDirectory->setObjectName("actionChangeDataDirectory");
        actionChangeDataDirectory->setIcon(icon1);
        actionChangeDataDirectory->setMenuRole(QAction::NoRole);
        actionAboutApplication = new QAction(MainWindow);
        actionAboutApplication->setObjectName("actionAboutApplication");
        actionAboutApplication->setIcon(icon1);
        actionAboutApplication->setMenuRole(QAction::AboutRole);
        actionHelp = new QAction(MainWindow);
        actionHelp->setObjectName("actionHelp");
        actionHelp->setEnabled(true);
        actionHelp->setIcon(icon1);
        actionHelp->setMenuRole(QAction::NoRole);
        actionFindDataBox = new QAction(MainWindow);
        actionFindDataBox->setObjectName("actionFindDataBox");
        actionFindDataBox->setIcon(icon1);
        actionFindDataBox->setMenuRole(QAction::NoRole);
        actionAuthenticateMessageFile = new QAction(MainWindow);
        actionAuthenticateMessageFile->setObjectName("actionAuthenticateMessageFile");
        actionAuthenticateMessageFile->setIcon(icon1);
        actionAuthenticateMessageFile->setMenuRole(QAction::NoRole);
        actionViewMessageFile = new QAction(MainWindow);
        actionViewMessageFile->setObjectName("actionViewMessageFile");
        actionViewMessageFile->setIcon(icon1);
        actionViewMessageFile->setMenuRole(QAction::NoRole);
        actionExportCorrespondenceOverview = new QAction(MainWindow);
        actionExportCorrespondenceOverview->setObjectName("actionExportCorrespondenceOverview");
        actionExportCorrespondenceOverview->setIcon(icon1);
        actionExportCorrespondenceOverview->setMenuRole(QAction::NoRole);
        actionDownloadMessage = new QAction(MainWindow);
        actionDownloadMessage->setObjectName("actionDownloadMessage");
        actionDownloadMessage->setIcon(icon1);
        actionDownloadMessage->setMenuRole(QAction::NoRole);
        actionReply = new QAction(MainWindow);
        actionReply->setObjectName("actionReply");
        actionReply->setIcon(icon1);
        actionReply->setMenuRole(QAction::NoRole);
        actionSignatureDetail = new QAction(MainWindow);
        actionSignatureDetail->setObjectName("actionSignatureDetail");
        actionSignatureDetail->setIcon(icon1);
        actionSignatureDetail->setMenuRole(QAction::NoRole);
        actionAuthenticateMessage = new QAction(MainWindow);
        actionAuthenticateMessage->setObjectName("actionAuthenticateMessage");
        actionAuthenticateMessage->setIcon(icon1);
        actionAuthenticateMessage->setMenuRole(QAction::NoRole);
        actionDeleteMessage = new QAction(MainWindow);
        actionDeleteMessage->setObjectName("actionDeleteMessage");
        actionDeleteMessage->setIcon(icon1);
        actionDeleteMessage->setMenuRole(QAction::NoRole);
        actionExportMessageZfo = new QAction(MainWindow);
        actionExportMessageZfo->setObjectName("actionExportMessageZfo");
        actionExportMessageZfo->setMenuRole(QAction::NoRole);
        actionOpenMessageExternal = new QAction(MainWindow);
        actionOpenMessageExternal->setObjectName("actionOpenMessageExternal");
        actionOpenMessageExternal->setMenuRole(QAction::NoRole);
        actionOpenAcceptanceInfoExternal = new QAction(MainWindow);
        actionOpenAcceptanceInfoExternal->setObjectName("actionOpenAcceptanceInfoExternal");
        actionOpenAcceptanceInfoExternal->setMenuRole(QAction::NoRole);
        actionExportAcceptanceInfoZfo = new QAction(MainWindow);
        actionExportAcceptanceInfoZfo->setObjectName("actionExportAcceptanceInfoZfo");
        actionExportAcceptanceInfoZfo->setMenuRole(QAction::NoRole);
        actionExportAcceptanceInfoPdf = new QAction(MainWindow);
        actionExportAcceptanceInfoPdf->setObjectName("actionExportAcceptanceInfoPdf");
        actionExportAcceptanceInfoPdf->setMenuRole(QAction::NoRole);
        actionExportEnvelopePdf = new QAction(MainWindow);
        actionExportEnvelopePdf->setObjectName("actionExportEnvelopePdf");
        actionExportEnvelopePdf->setMenuRole(QAction::NoRole);
        actionExportEnvelopePdfAndAttachments = new QAction(MainWindow);
        actionExportEnvelopePdfAndAttachments->setObjectName("actionExportEnvelopePdfAndAttachments");
        actionExportEnvelopePdfAndAttachments->setMenuRole(QAction::NoRole);
        actionOpenAttachment = new QAction(MainWindow);
        actionOpenAttachment->setObjectName("actionOpenAttachment");
        actionOpenAttachment->setIcon(icon1);
        actionOpenAttachment->setMenuRole(QAction::NoRole);
        actionSaveSelectedAttachments = new QAction(MainWindow);
        actionSaveSelectedAttachments->setObjectName("actionSaveSelectedAttachments");
        actionSaveSelectedAttachments->setIcon(icon1);
        actionSaveSelectedAttachments->setMenuRole(QAction::NoRole);
        actionSaveAllAttachments = new QAction(MainWindow);
        actionSaveAllAttachments->setObjectName("actionSaveAllAttachments");
        actionSaveAllAttachments->setIcon(icon1);
        actionSaveAllAttachments->setMenuRole(QAction::NoRole);
        actionImportZfoIntoDatabase = new QAction(MainWindow);
        actionImportZfoIntoDatabase->setObjectName("actionImportZfoIntoDatabase");
        actionImportZfoIntoDatabase->setIcon(icon1);
        actionImportZfoIntoDatabase->setMenuRole(QAction::NoRole);
        actionUseAsTemplate = new QAction(MainWindow);
        actionUseAsTemplate->setObjectName("actionUseAsTemplate");
        actionUseAsTemplate->setIcon(icon1);
        actionUseAsTemplate->setMenuRole(QAction::NoRole);
        actionMessageSearch = new QAction(MainWindow);
        actionMessageSearch->setObjectName("actionMessageSearch");
        actionMessageSearch->setIcon(icon1);
        actionMessageSearch->setMenuRole(QAction::NoRole);
        actionCheckMessageTimestampExpiration = new QAction(MainWindow);
        actionCheckMessageTimestampExpiration->setObjectName("actionCheckMessageTimestampExpiration");
        actionCheckMessageTimestampExpiration->setIcon(icon1);
        actionCheckMessageTimestampExpiration->setMenuRole(QAction::NoRole);
        actionHomepage = new QAction(MainWindow);
        actionHomepage->setObjectName("actionHomepage");
        actionHomepage->setIcon(icon1);
        actionHomepage->setMenuRole(QAction::NoRole);
        actionImportMessagesFromDatabase = new QAction(MainWindow);
        actionImportMessagesFromDatabase->setObjectName("actionImportMessagesFromDatabase");
        actionImportMessagesFromDatabase->setIcon(icon1);
        actionImportMessagesFromDatabase->setMenuRole(QAction::NoRole);
        actionSplitDatabaseYears = new QAction(MainWindow);
        actionSplitDatabaseYears->setObjectName("actionSplitDatabaseYears");
        actionSplitDatabaseYears->setIcon(icon1);
        actionSplitDatabaseYears->setMenuRole(QAction::NoRole);
        actionEmailContentSelection = new QAction(MainWindow);
        actionEmailContentSelection->setObjectName("actionEmailContentSelection");
        actionEmailContentSelection->setMenuRole(QAction::NoRole);
        actionEmail_selected_attachments = new QAction(MainWindow);
        actionEmail_selected_attachments->setObjectName("actionEmail_selected_attachments");
        actionEmail_selected_attachments->setMenuRole(QAction::NoRole);
        actionEditAvailableTags = new QAction(MainWindow);
        actionEditAvailableTags->setObjectName("actionEditAvailableTags");
        actionEditAvailableTags->setIcon(icon1);
        actionEditAvailableTags->setMenuRole(QAction::NoRole);
        actionVacuumMessageDatabase = new QAction(MainWindow);
        actionVacuumMessageDatabase->setObjectName("actionVacuumMessageDatabase");
        actionVacuumMessageDatabase->setIcon(icon1);
        actionVacuumMessageDatabase->setMenuRole(QAction::NoRole);
        actionForward = new QAction(MainWindow);
        actionForward->setObjectName("actionForward");
        actionForward->setIcon(icon1);
        actionForward->setMenuRole(QAction::NoRole);
        actionRecordsManagementSettings = new QAction(MainWindow);
        actionRecordsManagementSettings->setObjectName("actionRecordsManagementSettings");
        actionRecordsManagementSettings->setIcon(icon1);
        actionRecordsManagementSettings->setMenuRole(QAction::NoRole);
        actionSendToRecordsManagement = new QAction(MainWindow);
        actionSendToRecordsManagement->setObjectName("actionSendToRecordsManagement");
        actionSendToRecordsManagement->setIcon(icon1);
        actionSendToRecordsManagement->setMenuRole(QAction::NoRole);
        actionUpdateRecordsManagementInformation = new QAction(MainWindow);
        actionUpdateRecordsManagementInformation->setObjectName("actionUpdateRecordsManagementInformation");
        actionUpdateRecordsManagementInformation->setIcon(icon1);
        actionUpdateRecordsManagementInformation->setMenuRole(QAction::NoRole);
        actionViewLog = new QAction(MainWindow);
        actionViewLog->setObjectName("actionViewLog");
        actionViewLog->setMenuRole(QAction::NoRole);
        actionSendEGovRequest = new QAction(MainWindow);
        actionSendEGovRequest->setObjectName("actionSendEGovRequest");
        actionSendEGovRequest->setIcon(icon1);
        actionSendEGovRequest->setMenuRole(QAction::NoRole);
        actionBackUp = new QAction(MainWindow);
        actionBackUp->setObjectName("actionBackUp");
        actionBackUp->setIcon(icon1);
        actionBackUp->setMenuRole(QAction::NoRole);
        actionRestore = new QAction(MainWindow);
        actionRestore->setObjectName("actionRestore");
        actionRestore->setIcon(icon1);
        actionRestore->setMenuRole(QAction::NoRole);
        actionViewAllSettings = new QAction(MainWindow);
        actionViewAllSettings->setObjectName("actionViewAllSettings");
        actionViewAllSettings->setMenuRole(QAction::NoRole);
        actionAccountSettings = new QAction(MainWindow);
        actionAccountSettings->setObjectName("actionAccountSettings");
        actionAccountSettings->setIcon(icon1);
        actionAccountSettings->setMenuRole(QAction::NoRole);
        actionMarkAllReceivedRead = new QAction(MainWindow);
        actionMarkAllReceivedRead->setObjectName("actionMarkAllReceivedRead");
        actionMarkAllReceivedRead->setIcon(icon1);
        actionMarkAllReceivedRead->setMenuRole(QAction::NoRole);
        actionMarkAllReceivedUnread = new QAction(MainWindow);
        actionMarkAllReceivedUnread->setObjectName("actionMarkAllReceivedUnread");
        actionMarkAllReceivedUnread->setIcon(icon1);
        actionMarkAllReceivedUnread->setMenuRole(QAction::NoRole);
        actionMarkAllReceivedUnsettled = new QAction(MainWindow);
        actionMarkAllReceivedUnsettled->setObjectName("actionMarkAllReceivedUnsettled");
        actionMarkAllReceivedUnsettled->setIcon(icon1);
        actionMarkAllReceivedUnsettled->setMenuRole(QAction::NoRole);
        actionMarkAllReceivedInProgress = new QAction(MainWindow);
        actionMarkAllReceivedInProgress->setObjectName("actionMarkAllReceivedInProgress");
        actionMarkAllReceivedInProgress->setIcon(icon1);
        actionMarkAllReceivedInProgress->setMenuRole(QAction::NoRole);
        actionMarkAllReceivedSettled = new QAction(MainWindow);
        actionMarkAllReceivedSettled->setObjectName("actionMarkAllReceivedSettled");
        actionMarkAllReceivedSettled->setIcon(icon1);
        actionMarkAllReceivedSettled->setMenuRole(QAction::NoRole);
        actionEmailZfos = new QAction(MainWindow);
        actionEmailZfos->setObjectName("actionEmailZfos");
        actionEmailZfos->setMenuRole(QAction::NoRole);
        actionEmailAllAttachments = new QAction(MainWindow);
        actionEmailAllAttachments->setObjectName("actionEmailAllAttachments");
        actionEmailAllAttachments->setMenuRole(QAction::NoRole);
        actionCheckUploadMessagesRecordsManagement = new QAction(MainWindow);
        actionCheckUploadMessagesRecordsManagement->setObjectName("actionCheckUploadMessagesRecordsManagement");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/png_app_logo/64x64/datovka.png"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        actionCheckUploadMessagesRecordsManagement->setIcon(icon2);
        actionCheckUploadMessagesRecordsManagement->setMenuRole(QAction::NoRole);
        actionConvertImportFromMobileApp = new QAction(MainWindow);
        actionConvertImportFromMobileApp->setObjectName("actionConvertImportFromMobileApp");
        actionConvertImportFromMobileApp->setIcon(icon1);
        actionConvertImportFromMobileApp->setMenuRole(QAction::NoRole);
        actionConvertExportForMobileApp = new QAction(MainWindow);
        actionConvertExportForMobileApp->setObjectName("actionConvertExportForMobileApp");
        actionConvertExportForMobileApp->setIcon(icon1);
        actionConvertExportForMobileApp->setMenuRole(QAction::NoRole);
        actionTagStorageSettings = new QAction(MainWindow);
        actionTagStorageSettings->setObjectName("actionTagStorageSettings");
        actionTagStorageSettings->setIcon(icon1);
        actionTagStorageSettings->setMenuRole(QAction::NoRole);
        actionCustomiseToolBar = new QAction(MainWindow);
        actionCustomiseToolBar->setObjectName("actionCustomiseToolBar");
        actionCustomiseToolBar->setMenuRole(QAction::NoRole);
        actionCustomiseAttachmentBar = new QAction(MainWindow);
        actionCustomiseAttachmentBar->setObjectName("actionCustomiseAttachmentBar");
        actionCustomiseAttachmentBar->setMenuRole(QAction::NoRole);
        actionCustomiseDetailBar = new QAction(MainWindow);
        actionCustomiseDetailBar->setObjectName("actionCustomiseDetailBar");
        actionCustomiseDetailBar->setMenuRole(QAction::NoRole);
        actionCustomiseMessageBar = new QAction(MainWindow);
        actionCustomiseMessageBar->setObjectName("actionCustomiseMessageBar");
        actionCustomiseMessageBar->setMenuRole(QAction::NoRole);
        actionCustomiseAccountBar = new QAction(MainWindow);
        actionCustomiseAccountBar->setObjectName("actionCustomiseAccountBar");
        actionCustomiseAccountBar->setMenuRole(QAction::NoRole);
        actionFilterMessages = new QAction(MainWindow);
        actionFilterMessages->setObjectName("actionFilterMessages");
        actionFilterMessages->setIcon(icon1);
        actionFilterMessages->setMenuRole(QAction::NoRole);
        actionToolBar = new QAction(MainWindow);
        actionToolBar->setObjectName("actionToolBar");
        actionToolBar->setCheckable(true);
        actionToolBar->setMenuRole(QAction::NoRole);
        actionTopPanelBars = new QAction(MainWindow);
        actionTopPanelBars->setObjectName("actionTopPanelBars");
        actionTopPanelBars->setCheckable(true);
        actionTopPanelBars->setMenuRole(QAction::NoRole);
        actionEditTagAssignment = new QAction(MainWindow);
        actionEditTagAssignment->setObjectName("actionEditTagAssignment");
        actionEditTagAssignment->setIcon(icon1);
        actionEditTagAssignment->setMenuRole(QAction::NoRole);
        actionRepairMessageDatabaseFiles = new QAction(MainWindow);
        actionRepairMessageDatabaseFiles->setObjectName("actionRepairMessageDatabaseFiles");
        actionRepairMessageDatabaseFiles->setMenuRole(QAction::NoRole);
        actionTrackTimestampExpiration = new QAction(MainWindow);
        actionTrackTimestampExpiration->setObjectName("actionTrackTimestampExpiration");
        actionTrackTimestampExpiration->setCheckable(true);
        actionTrackTimestampExpiration->setMenuRole(QAction::NoRole);
        actionTimeStampRenewal = new QAction(MainWindow);
        actionTimeStampRenewal->setObjectName("actionTimeStampRenewal");
        actionTimeStampRenewal->setIcon(icon1);
        actionTimeStampRenewal->setMenuRole(QAction::NoRole);
        actionDeleteDraft = new QAction(MainWindow);
        actionDeleteDraft->setObjectName("actionDeleteDraft");
        actionDeleteDraft->setMenuRole(QAction::NoRole);
        actionEditDraft = new QAction(MainWindow);
        actionEditDraft->setObjectName("actionEditDraft");
        actionEditDraft->setMenuRole(QAction::NoRole);
        centralWidgetWindows = new QWidget(MainWindow);
        centralWidgetWindows->setObjectName("centralWidgetWindows");
        horizontalLayout_4 = new QHBoxLayout(centralWidgetWindows);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        hSplitterAccount = new QSplitter(centralWidgetWindows);
        hSplitterAccount->setObjectName("hSplitterAccount");
        hSplitterAccount->setMouseTracking(false);
        hSplitterAccount->setOrientation(Qt::Horizontal);
        verticalLayoutWidget = new QWidget(hSplitterAccount);
        verticalLayoutWidget->setObjectName("verticalLayoutWidget");
        accountsVerticalLayout = new QVBoxLayout(verticalLayoutWidget);
        accountsVerticalLayout->setObjectName("accountsVerticalLayout");
        accountsVerticalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        accountsVerticalLayout->setContentsMargins(0, 0, 0, 0);
        accountList = new LoweredTreeView(verticalLayoutWidget);
        accountList->setObjectName("accountList");
        accountList->setEnabled(true);
        sizePolicy.setHeightForWidth(accountList->sizePolicy().hasHeightForWidth());
        accountList->setSizePolicy(sizePolicy);
        accountList->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        accountList->header()->setMinimumSectionSize(100);

        accountsVerticalLayout->addWidget(accountList);

        hSplitterAccount->addWidget(verticalLayoutWidget);
        messageStackedWidget = new QStackedWidget(hSplitterAccount);
        messageStackedWidget->setObjectName("messageStackedWidget");
        accountInfoPage = new QWidget();
        accountInfoPage->setObjectName("accountInfoPage");
        horizontalLayout_3 = new QHBoxLayout(accountInfoPage);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        accountTextInfo = new QTextBrowser(accountInfoPage);
        accountTextInfo->setObjectName("accountTextInfo");

        horizontalLayout_3->addWidget(accountTextInfo);

        messageStackedWidget->addWidget(accountInfoPage);
        messagePage = new QWidget();
        messagePage->setObjectName("messagePage");
        horizontalLayout_2 = new QHBoxLayout(messagePage);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        vSplitterMessage = new QSplitter(messagePage);
        vSplitterMessage->setObjectName("vSplitterMessage");
        vSplitterMessage->setOrientation(Qt::Vertical);
        verticalLayoutWidget_2 = new QWidget(vSplitterMessage);
        verticalLayoutWidget_2->setObjectName("verticalLayoutWidget_2");
        messagesVerticalLayout = new QVBoxLayout(verticalLayoutWidget_2);
        messagesVerticalLayout->setObjectName("messagesVerticalLayout");
        messagesVerticalLayout->setContentsMargins(0, 0, 0, 0);
        messageList = new LoweredTableView(verticalLayoutWidget_2);
        messageList->setObjectName("messageList");
        messageList->setShowGrid(false);
        messageList->setWordWrap(false);
        messageList->horizontalHeader()->setHighlightSections(false);
        messageList->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        messageList->horizontalHeader()->setStretchLastSection(true);
        messageList->verticalHeader()->setVisible(false);
        messageList->verticalHeader()->setHighlightSections(false);

        messagesVerticalLayout->addWidget(messageList);

        messagesFilterHorizontalLayout = new QWidget(verticalLayoutWidget_2);
        messagesFilterHorizontalLayout->setObjectName("messagesFilterHorizontalLayout");
        messagesFilterHorizontalLayout_2 = new QHBoxLayout(messagesFilterHorizontalLayout);
        messagesFilterHorizontalLayout_2->setObjectName("messagesFilterHorizontalLayout_2");
        messageFilterLine = new QLineEdit(messagesFilterHorizontalLayout);
        messageFilterLine->setObjectName("messageFilterLine");
        messageFilterLine->setClearButtonEnabled(true);

        messagesFilterHorizontalLayout_2->addWidget(messageFilterLine);

        messageFilterCloseButton = new QPushButton(messagesFilterHorizontalLayout);
        messageFilterCloseButton->setObjectName("messageFilterCloseButton");

        messagesFilterHorizontalLayout_2->addWidget(messageFilterCloseButton);


        messagesVerticalLayout->addWidget(messagesFilterHorizontalLayout);

        vSplitterMessage->addWidget(verticalLayoutWidget_2);
        hSplitterMessageInfo = new QSplitter(vSplitterMessage);
        hSplitterMessageInfo->setObjectName("hSplitterMessageInfo");
        hSplitterMessageInfo->setOrientation(Qt::Horizontal);
        layoutWidget = new QWidget(hSplitterMessageInfo);
        layoutWidget->setObjectName("layoutWidget");
        detailVerticalLayout = new QVBoxLayout(layoutWidget);
        detailVerticalLayout->setObjectName("detailVerticalLayout");
        detailVerticalLayout->setContentsMargins(0, 0, 0, 0);
        messageInfo = new QTextEdit(layoutWidget);
        messageInfo->setObjectName("messageInfo");
        messageInfo->setReadOnly(true);

        detailVerticalLayout->addWidget(messageInfo);

        hSplitterMessageInfo->addWidget(layoutWidget);
        layoutWidget1 = new QWidget(hSplitterMessageInfo);
        layoutWidget1->setObjectName("layoutWidget1");
        attachmentsVerticalLayout = new QVBoxLayout(layoutWidget1);
        attachmentsVerticalLayout->setObjectName("attachmentsVerticalLayout");
        attachmentsVerticalLayout->setContentsMargins(0, 0, 0, 0);
        attachmentList = new AttachmentTableView(layoutWidget1);
        attachmentList->setObjectName("attachmentList");
        attachmentList->setShowGrid(false);
        attachmentList->setWordWrap(false);
        attachmentList->horizontalHeader()->setHighlightSections(false);
        attachmentList->horizontalHeader()->setStretchLastSection(true);
        attachmentList->verticalHeader()->setVisible(false);
        attachmentList->verticalHeader()->setHighlightSections(false);

        attachmentsVerticalLayout->addWidget(attachmentList);

        hSplitterMessageInfo->addWidget(layoutWidget1);
        vSplitterMessage->addWidget(hSplitterMessageInfo);

        horizontalLayout_2->addWidget(vSplitterMessage);

        messageStackedWidget->addWidget(messagePage);
        draftPage = new QWidget();
        draftPage->setObjectName("draftPage");
        verticalLayout = new QVBoxLayout(draftPage);
        verticalLayout->setObjectName("verticalLayout");
        draftsVerticalLayout = new QVBoxLayout();
        draftsVerticalLayout->setObjectName("draftsVerticalLayout");
        draftList = new LoweredTableView(draftPage);
        draftList->setObjectName("draftList");
        draftList->setShowGrid(false);
        draftList->setWordWrap(false);
        draftList->horizontalHeader()->setHighlightSections(false);
        draftList->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        draftList->horizontalHeader()->setStretchLastSection(true);
        draftList->verticalHeader()->setVisible(false);
        draftList->verticalHeader()->setHighlightSections(false);

        draftsVerticalLayout->addWidget(draftList);

        draftsFilterHorizontalLayout = new QWidget(draftPage);
        draftsFilterHorizontalLayout->setObjectName("draftsFilterHorizontalLayout");
        draftsFilterHorizontalLayout_2 = new QHBoxLayout(draftsFilterHorizontalLayout);
        draftsFilterHorizontalLayout_2->setObjectName("draftsFilterHorizontalLayout_2");
        draftFilterLine = new QLineEdit(draftsFilterHorizontalLayout);
        draftFilterLine->setObjectName("draftFilterLine");
        draftFilterLine->setClearButtonEnabled(true);

        draftsFilterHorizontalLayout_2->addWidget(draftFilterLine);

        draftFilterCloseButton = new QPushButton(draftsFilterHorizontalLayout);
        draftFilterCloseButton->setObjectName("draftFilterCloseButton");

        draftsFilterHorizontalLayout_2->addWidget(draftFilterCloseButton);


        draftsVerticalLayout->addWidget(draftsFilterHorizontalLayout);


        verticalLayout->addLayout(draftsVerticalLayout);

        messageStackedWidget->addWidget(draftPage);
        hSplitterAccount->addWidget(messageStackedWidget);

        horizontalLayout_4->addWidget(hSplitterAccount);

        MainWindow->setCentralWidget(centralWidgetWindows);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 800, 27));
        menuFile = new QMenu(menubar);
        menuFile->setObjectName("menuFile");
        menuRecordsManagement = new QMenu(menuFile);
        menuRecordsManagement->setObjectName("menuRecordsManagement");
        menuDatabox = new QMenu(menubar);
        menuDatabox->setObjectName("menuDatabox");
        menuDatabox->setEnabled(true);
        menuMarkAllReceivedAs = new QMenu(menuDatabox);
        menuMarkAllReceivedAs->setObjectName("menuMarkAllReceivedAs");
        menuSynchronisation = new QMenu(menuDatabox);
        menuSynchronisation->setObjectName("menuSynchronisation");
        menuMessage = new QMenu(menubar);
        menuMessage->setObjectName("menuMessage");
        menuMessage->setEnabled(true);
        menuEmailWith = new QMenu(menuMessage);
        menuEmailWith->setObjectName("menuEmailWith");
        menuTools = new QMenu(menubar);
        menuTools->setObjectName("menuTools");
        menuToolBars = new QMenu(menuTools);
        menuToolBars->setObjectName("menuToolBars");
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName("menuHelp");
        menuHelp->setLayoutDirection(Qt::LeftToRight);
        MainWindow->setMenuBar(menubar);
        toolBar = new QToolBar(MainWindow);
        toolBar->setObjectName("toolBar");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(toolBar->sizePolicy().hasHeightForWidth());
        toolBar->setSizePolicy(sizePolicy1);
        toolBar->setIconSize(QSize(32, 32));
        toolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        MainWindow->addToolBar(Qt::ToolBarArea::TopToolBarArea, toolBar);
        statusBar = new QStatusBar(MainWindow);
        statusBar->setObjectName("statusBar");
        MainWindow->setStatusBar(statusBar);

        menubar->addAction(menuFile->menuAction());
        menubar->addAction(menuDatabox->menuAction());
        menubar->addAction(menuMessage->menuAction());
        menubar->addAction(menuTools->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menuFile->addAction(actionSynchroniseAllAccounts);
        menuFile->addSeparator();
        menuFile->addAction(actionAddAccount);
        menuFile->addAction(actionRemoveAccount);
        menuFile->addAction(actionAccountSettings);
        menuFile->addSeparator();
        menuFile->addAction(actionImportAccountFromDatabase);
        menuFile->addAction(actionBackUp);
        menuFile->addAction(actionRestore);
        menuFile->addSeparator();
        menuFile->addAction(menuRecordsManagement->menuAction());
        menuFile->addSeparator();
        menuFile->addAction(actionProxySettings);
        menuFile->addSeparator();
        menuFile->addAction(actionTagStorageSettings);
        menuFile->addSeparator();
        menuFile->addAction(actionPreferences);
        menuFile->addSeparator();
        menuFile->addAction(actionQuit);
        menuRecordsManagement->addAction(actionRecordsManagementSettings);
        menuRecordsManagement->addAction(actionUpdateRecordsManagementInformation);
        menuRecordsManagement->addAction(actionCheckUploadMessagesRecordsManagement);
        menuDatabox->addAction(actionSynchroniseAccount);
        menuDatabox->addAction(menuSynchronisation->menuAction());
        menuDatabox->addAction(actionSendMessage);
        menuDatabox->addAction(actionSendEGovRequest);
        menuDatabox->addSeparator();
        menuDatabox->addAction(menuMarkAllReceivedAs->menuAction());
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionChangePassword);
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionAccountProperties);
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionMoveAccountUp);
        menuDatabox->addAction(actionMoveAccountDown);
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionChangeDataDirectory);
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionImportMessagesFromDatabase);
        menuDatabox->addAction(actionImportZfoIntoDatabase);
        menuDatabox->addSeparator();
        menuDatabox->addAction(actionVacuumMessageDatabase);
        menuDatabox->addAction(actionSplitDatabaseYears);
        menuMarkAllReceivedAs->addAction(actionMarkAllReceivedRead);
        menuMarkAllReceivedAs->addAction(actionMarkAllReceivedUnread);
        menuMarkAllReceivedAs->addSeparator();
        menuMarkAllReceivedAs->addAction(actionMarkAllReceivedUnsettled);
        menuMarkAllReceivedAs->addAction(actionMarkAllReceivedInProgress);
        menuMarkAllReceivedAs->addAction(actionMarkAllReceivedSettled);
        menuSynchronisation->addAction(actionSynchroniseAccountReceived);
        menuSynchronisation->addAction(actionSynchroniseAccountSent);
        menuMessage->addAction(actionDownloadMessage);
        menuMessage->addAction(actionReply);
        menuMessage->addAction(actionForward);
        menuMessage->addAction(actionUseAsTemplate);
        menuMessage->addSeparator();
        menuMessage->addAction(actionSignatureDetail);
        menuMessage->addAction(actionAuthenticateMessage);
        menuMessage->addSeparator();
        menuMessage->addAction(actionOpenMessageExternal);
        menuMessage->addAction(actionOpenAcceptanceInfoExternal);
        menuMessage->addSeparator();
        menuMessage->addAction(actionSendToRecordsManagement);
        menuMessage->addSeparator();
        menuMessage->addAction(actionExportMessageZfo);
        menuMessage->addAction(actionExportAcceptanceInfoZfo);
        menuMessage->addAction(actionExportAcceptanceInfoPdf);
        menuMessage->addAction(actionExportEnvelopePdf);
        menuMessage->addAction(actionExportEnvelopePdfAndAttachments);
        menuMessage->addSeparator();
        menuMessage->addAction(menuEmailWith->menuAction());
        menuMessage->addSeparator();
        menuMessage->addAction(actionSaveAllAttachments);
        menuMessage->addAction(actionSaveSelectedAttachments);
        menuMessage->addAction(actionOpenAttachment);
        menuMessage->addSeparator();
        menuMessage->addAction(actionEditTagAssignment);
        menuMessage->addSeparator();
        menuMessage->addAction(actionTrackTimestampExpiration);
        menuMessage->addSeparator();
        menuMessage->addAction(actionDeleteMessage);
        menuEmailWith->addAction(actionEmailZfos);
        menuEmailWith->addAction(actionEmailAllAttachments);
        menuEmailWith->addAction(actionEmailContentSelection);
        menuTools->addAction(actionFindDataBox);
        menuTools->addSeparator();
        menuTools->addAction(actionAuthenticateMessageFile);
        menuTools->addAction(actionViewMessageFile);
        menuTools->addAction(actionExportCorrespondenceOverview);
        menuTools->addAction(actionCheckMessageTimestampExpiration);
        menuTools->addAction(actionTimeStampRenewal);
        menuTools->addSeparator();
        menuTools->addAction(actionMessageSearch);
        menuTools->addSeparator();
        menuTools->addAction(actionEditAvailableTags);
        menuTools->addSeparator();
        menuTools->addAction(actionConvertImportFromMobileApp);
        menuTools->addAction(actionConvertExportForMobileApp);
        menuTools->addSeparator();
        menuTools->addAction(actionRepairMessageDatabaseFiles);
        menuTools->addSeparator();
        menuTools->addAction(menuToolBars->menuAction());
        menuTools->addAction(actionViewLog);
        menuTools->addAction(actionViewAllSettings);
        menuToolBars->addAction(actionToolBar);
        menuToolBars->addAction(actionTopPanelBars);
        menuToolBars->addAction(actionCustomiseToolBar);
        menuToolBars->addAction(actionCustomiseAccountBar);
        menuToolBars->addAction(actionCustomiseMessageBar);
        menuToolBars->addAction(actionCustomiseDetailBar);
        menuToolBars->addAction(actionCustomiseAttachmentBar);
        menuHelp->addAction(actionAboutApplication);
        menuHelp->addAction(actionHomepage);
        menuHelp->addSeparator();
        menuHelp->addAction(actionHelp);

        retranslateUi(MainWindow);
        QObject::connect(actionQuit, &QAction::triggered, MainWindow, qOverload<>(&QMainWindow::close));

        messageStackedWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "Datovka", nullptr));
        actionSynchroniseAllAccounts->setText(QCoreApplication::translate("MainWindow", "Synchronise All Data Boxes", nullptr));
#if QT_CONFIG(tooltip)
        actionSynchroniseAllAccounts->setToolTip(QCoreApplication::translate("MainWindow", "Synchronise all of those data boxes which have been set\n"
"to be synchronised when this action is triggered.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSynchroniseAllAccounts->setShortcut(QCoreApplication::translate("MainWindow", "Shift+F5", nullptr));
#endif // QT_CONFIG(shortcut)
        actionAddAccount->setText(QCoreApplication::translate("MainWindow", "Add a New Data Box", nullptr));
#if QT_CONFIG(tooltip)
        actionAddAccount->setToolTip(QCoreApplication::translate("MainWindow", "Add a new data box.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRemoveAccount->setText(QCoreApplication::translate("MainWindow", "Remove Data Box", nullptr));
#if QT_CONFIG(tooltip)
        actionRemoveAccount->setToolTip(QCoreApplication::translate("MainWindow", "Remove selected data box.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionProxySettings->setText(QCoreApplication::translate("MainWindow", "Proxy Settings", nullptr));
#if QT_CONFIG(tooltip)
        actionProxySettings->setToolTip(QCoreApplication::translate("MainWindow", "View proxy settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionPreferences->setText(QCoreApplication::translate("MainWindow", "Preferences", nullptr));
#if QT_CONFIG(tooltip)
        actionPreferences->setToolTip(QCoreApplication::translate("MainWindow", "View application prefererences.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionQuit->setText(QCoreApplication::translate("MainWindow", "Quit", nullptr));
#if QT_CONFIG(tooltip)
        actionQuit->setToolTip(QCoreApplication::translate("MainWindow", "Quit the application.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionQuit->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
        actionImportAccountFromDatabase->setText(QCoreApplication::translate("MainWindow", "Create Data Box from Database", nullptr));
#if QT_CONFIG(tooltip)
        actionImportAccountFromDatabase->setToolTip(QCoreApplication::translate("MainWindow", "Create a data box from supplied message database.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSynchroniseAccount->setText(QCoreApplication::translate("MainWindow", "Synchronise Data Box", nullptr));
#if QT_CONFIG(tooltip)
        actionSynchroniseAccount->setToolTip(QCoreApplication::translate("MainWindow", "Synchronise the received and sent message lists of the selected data box.\n"
"The synchronisation causes the acceptance of delivered messages.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSynchroniseAccount->setShortcut(QCoreApplication::translate("MainWindow", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSynchroniseAccountReceived->setText(QCoreApplication::translate("MainWindow", "Synchronise Data Box - Received Messages Only", nullptr));
#if QT_CONFIG(tooltip)
        actionSynchroniseAccountReceived->setToolTip(QCoreApplication::translate("MainWindow", "Synchronise only the received message list of the selected data box.\n"
"The synchronisation causes the acceptance of delivered messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSynchroniseAccountSent->setText(QCoreApplication::translate("MainWindow", "Synchronise Data Box - Sent Messages Only", nullptr));
#if QT_CONFIG(tooltip)
        actionSynchroniseAccountSent->setToolTip(QCoreApplication::translate("MainWindow", "Synchronise only the sent message list of the selected data box.\n"
"The synchronisation doesn't cause the acceptance of delivered messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSendMessage->setText(QCoreApplication::translate("MainWindow", "Create a Message", nullptr));
#if QT_CONFIG(tooltip)
        actionSendMessage->setToolTip(QCoreApplication::translate("MainWindow", "Create and send a new data message.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSendMessage->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        actionChangePassword->setText(QCoreApplication::translate("MainWindow", "Change Password", nullptr));
#if QT_CONFIG(tooltip)
        actionChangePassword->setToolTip(QCoreApplication::translate("MainWindow", "Sets a new password to the selected data box.\n"
"The password is going to be modified on the ISDS server.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAccountProperties->setText(QCoreApplication::translate("MainWindow", "Data-Box Properties", nullptr));
#if QT_CONFIG(tooltip)
        actionAccountProperties->setToolTip(QCoreApplication::translate("MainWindow", "Manage data-box properties.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMoveAccountUp->setText(QCoreApplication::translate("MainWindow", "Move Data Box Up", nullptr));
#if QT_CONFIG(tooltip)
        actionMoveAccountUp->setToolTip(QCoreApplication::translate("MainWindow", "Move selectet data box one position up.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMoveAccountDown->setText(QCoreApplication::translate("MainWindow", "Move Data Box Down", nullptr));
#if QT_CONFIG(tooltip)
        actionMoveAccountDown->setToolTip(QCoreApplication::translate("MainWindow", "Move selectet data box one position down.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionChangeDataDirectory->setText(QCoreApplication::translate("MainWindow", "Change Data Directory", nullptr));
#if QT_CONFIG(tooltip)
        actionChangeDataDirectory->setToolTip(QCoreApplication::translate("MainWindow", "Change the directory where data are being stored for the selected data box.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAboutApplication->setText(QCoreApplication::translate("MainWindow", "About the Application", nullptr));
#if QT_CONFIG(tooltip)
        actionAboutApplication->setToolTip(QCoreApplication::translate("MainWindow", "Show information about the application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionHelp->setText(QCoreApplication::translate("MainWindow", "User manual", nullptr));
#if QT_CONFIG(tooltip)
        actionHelp->setToolTip(QCoreApplication::translate("MainWindow", "Open the user manual webpage.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionFindDataBox->setText(QCoreApplication::translate("MainWindow", "Find Data Box", nullptr));
#if QT_CONFIG(tooltip)
        actionFindDataBox->setToolTip(QCoreApplication::translate("MainWindow", "Search for data box on the ISDS server.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAuthenticateMessageFile->setText(QCoreApplication::translate("MainWindow", "Authenticate Message from ZFO File", nullptr));
#if QT_CONFIG(tooltip)
        actionAuthenticateMessageFile->setToolTip(QCoreApplication::translate("MainWindow", "Authenticate the message file on the ISDS server.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionViewMessageFile->setText(QCoreApplication::translate("MainWindow", "View Message from ZFO File", nullptr));
#if QT_CONFIG(tooltip)
        actionViewMessageFile->setToolTip(QCoreApplication::translate("MainWindow", "View content of a ZFO file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportCorrespondenceOverview->setText(QCoreApplication::translate("MainWindow", "Export Correspondence Overview", nullptr));
#if QT_CONFIG(tooltip)
        actionExportCorrespondenceOverview->setToolTip(QCoreApplication::translate("MainWindow", "Create a correspondence overview.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDownloadMessage->setText(QCoreApplication::translate("MainWindow", "Download Signed Message", nullptr));
#if QT_CONFIG(tooltip)
        actionDownloadMessage->setToolTip(QCoreApplication::translate("MainWindow", "Download the complete message, i.e. including attachments, and verify its signature.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionReply->setText(QCoreApplication::translate("MainWindow", "Reply to Message", nullptr));
#if QT_CONFIG(tooltip)
        actionReply->setToolTip(QCoreApplication::translate("MainWindow", "Reply to the selected message.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionReply->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSignatureDetail->setText(QCoreApplication::translate("MainWindow", "Signature Detail", nullptr));
#if QT_CONFIG(tooltip)
        actionSignatureDetail->setToolTip(QCoreApplication::translate("MainWindow", "Display details about the digital signature of the selected message.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAuthenticateMessage->setText(QCoreApplication::translate("MainWindow", "Authenticate Message", nullptr));
#if QT_CONFIG(tooltip)
        actionAuthenticateMessage->setToolTip(QCoreApplication::translate("MainWindow", "Verify the selected message hash on the ISDS server.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDeleteMessage->setText(QCoreApplication::translate("MainWindow", "Delete Message", nullptr));
#if QT_CONFIG(tooltip)
        actionDeleteMessage->setToolTip(QCoreApplication::translate("MainWindow", "Delete message from local database and/or from ISDS server.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportMessageZfo->setText(QCoreApplication::translate("MainWindow", "Export Message as ZFO", nullptr));
#if QT_CONFIG(tooltip)
        actionExportMessageZfo->setToolTip(QCoreApplication::translate("MainWindow", "Export the selected message as a ZFO file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionOpenMessageExternal->setText(QCoreApplication::translate("MainWindow", "Open Message Externally", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenMessageExternal->setToolTip(QCoreApplication::translate("MainWindow", "Pass the selected message to an external application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionOpenAcceptanceInfoExternal->setText(QCoreApplication::translate("MainWindow", "Open Acceptance Info Externally", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenAcceptanceInfoExternal->setToolTip(QCoreApplication::translate("MainWindow", "Pass the acceptance information of the selected message to an external application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportAcceptanceInfoZfo->setText(QCoreApplication::translate("MainWindow", "Export Acceptance Info as ZFO", nullptr));
#if QT_CONFIG(tooltip)
        actionExportAcceptanceInfoZfo->setToolTip(QCoreApplication::translate("MainWindow", "Export the acceptance information of the selected message as a ZFO file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportAcceptanceInfoPdf->setText(QCoreApplication::translate("MainWindow", "Export Acceptance Info as PDF", nullptr));
#if QT_CONFIG(tooltip)
        actionExportAcceptanceInfoPdf->setToolTip(QCoreApplication::translate("MainWindow", "Export the content of the acceptance information\n"
"of the selected message into a PDF file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportEnvelopePdf->setText(QCoreApplication::translate("MainWindow", "Export Message Envelope as PDF", nullptr));
#if QT_CONFIG(tooltip)
        actionExportEnvelopePdf->setToolTip(QCoreApplication::translate("MainWindow", "Export content of the envelope\n"
"of the selected message as a PDF file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportEnvelopePdfAndAttachments->setText(QCoreApplication::translate("MainWindow", "Export Envelope PDF with Attachments", nullptr));
#if QT_CONFIG(tooltip)
        actionExportEnvelopePdfAndAttachments->setToolTip(QCoreApplication::translate("MainWindow", "Export the content of the envelope\n"
"of the selected message to a PDF file.\n"
"Also export message attachments.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionOpenAttachment->setText(QCoreApplication::translate("MainWindow", "Open Attachment", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenAttachment->setToolTip(QCoreApplication::translate("MainWindow", "Open attachment in an associated application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveSelectedAttachments->setText(QCoreApplication::translate("MainWindow", "Save Attachment", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveSelectedAttachments->setToolTip(QCoreApplication::translate("MainWindow", "Save selected attachments to files.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSaveAllAttachments->setText(QCoreApplication::translate("MainWindow", "Save All Attachments", nullptr));
#if QT_CONFIG(tooltip)
        actionSaveAllAttachments->setToolTip(QCoreApplication::translate("MainWindow", "Save all attachments of the selected message.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionImportZfoIntoDatabase->setText(QCoreApplication::translate("MainWindow", "Import Messages from ZFO Files", nullptr));
#if QT_CONFIG(tooltip)
        actionImportZfoIntoDatabase->setToolTip(QCoreApplication::translate("MainWindow", "Import a message from a ZFO file into the database.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionUseAsTemplate->setText(QCoreApplication::translate("MainWindow", "Use Message as Template", nullptr));
#if QT_CONFIG(tooltip)
        actionUseAsTemplate->setToolTip(QCoreApplication::translate("MainWindow", "Use the selected message as a template.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMessageSearch->setText(QCoreApplication::translate("MainWindow", "Search Message", nullptr));
#if QT_CONFIG(tooltip)
        actionMessageSearch->setToolTip(QCoreApplication::translate("MainWindow", "Search in message envelopes.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionMessageSearch->setShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+F", nullptr));
#endif // QT_CONFIG(shortcut)
        actionCheckMessageTimestampExpiration->setText(QCoreApplication::translate("MainWindow", "Time Stamp Expiration Check", nullptr));
#if QT_CONFIG(tooltip)
        actionCheckMessageTimestampExpiration->setToolTip(QCoreApplication::translate("MainWindow", "Check whether the message time stamp is not expired or expiring.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionHomepage->setText(QCoreApplication::translate("MainWindow", "Homepage", nullptr));
#if QT_CONFIG(tooltip)
        actionHomepage->setToolTip(QCoreApplication::translate("MainWindow", "Open the application homepage.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionImportMessagesFromDatabase->setText(QCoreApplication::translate("MainWindow", "Import Messages from Database", nullptr));
#if QT_CONFIG(tooltip)
        actionImportMessagesFromDatabase->setToolTip(QCoreApplication::translate("MainWindow", "Import messages into database from an external database file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSplitDatabaseYears->setText(QCoreApplication::translate("MainWindow", "Split Database According to Years", nullptr));
#if QT_CONFIG(tooltip)
        actionSplitDatabaseYears->setToolTip(QCoreApplication::translate("MainWindow", "Messages are going to be stored into separate database files according to years.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEmailContentSelection->setText(QCoreApplication::translate("MainWindow", "Selected Content", nullptr));
#if QT_CONFIG(tooltip)
        actionEmailContentSelection->setToolTip(QCoreApplication::translate("MainWindow", "Creates an e-mail containing selected content.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEmail_selected_attachments->setText(QCoreApplication::translate("MainWindow", "E-mail with selected attachments", nullptr));
#if QT_CONFIG(tooltip)
        actionEmail_selected_attachments->setToolTip(QCoreApplication::translate("MainWindow", "Creates an e-mail containing selected attachments", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEditAvailableTags->setText(QCoreApplication::translate("MainWindow", "Edit Available Tags", nullptr));
#if QT_CONFIG(tooltip)
        actionEditAvailableTags->setToolTip(QCoreApplication::translate("MainWindow", "Manage all available tags.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionVacuumMessageDatabase->setText(QCoreApplication::translate("MainWindow", "Vacuum the Message Database", nullptr));
#if QT_CONFIG(tooltip)
        actionVacuumMessageDatabase->setToolTip(QCoreApplication::translate("MainWindow", "It may reduce the database file size and optimise the access speed.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionForward->setText(QCoreApplication::translate("MainWindow", "Forward Message", nullptr));
#if QT_CONFIG(tooltip)
        actionForward->setToolTip(QCoreApplication::translate("MainWindow", "Forward the message.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRecordsManagementSettings->setText(QCoreApplication::translate("MainWindow", "Records Management Settings", nullptr));
#if QT_CONFIG(tooltip)
        actionRecordsManagementSettings->setToolTip(QCoreApplication::translate("MainWindow", "View records management settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSendToRecordsManagement->setText(QCoreApplication::translate("MainWindow", "Send to Records Management", nullptr));
#if QT_CONFIG(tooltip)
        actionSendToRecordsManagement->setToolTip(QCoreApplication::translate("MainWindow", "Send message to records management service.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionUpdateRecordsManagementInformation->setText(QCoreApplication::translate("MainWindow", "Update Records Management Information", nullptr));
#if QT_CONFIG(tooltip)
        actionUpdateRecordsManagementInformation->setToolTip(QCoreApplication::translate("MainWindow", "Update records management information about the data messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionViewLog->setText(QCoreApplication::translate("MainWindow", "View Log", nullptr));
#if QT_CONFIG(tooltip)
        actionViewLog->setToolTip(QCoreApplication::translate("MainWindow", "View application log content.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSendEGovRequest->setText(QCoreApplication::translate("MainWindow", "Send E-Gov Request", nullptr));
#if QT_CONFIG(tooltip)
        actionSendEGovRequest->setToolTip(QCoreApplication::translate("MainWindow", "Create and send an e-gov request.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionBackUp->setText(QCoreApplication::translate("MainWindow", "Back Up Data-Box Data", nullptr));
#if QT_CONFIG(tooltip)
        actionBackUp->setToolTip(QCoreApplication::translate("MainWindow", "Back up data-box data.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRestore->setText(QCoreApplication::translate("MainWindow", "Restore Data-Box Data", nullptr));
#if QT_CONFIG(tooltip)
        actionRestore->setToolTip(QCoreApplication::translate("MainWindow", "Restore data box data.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionViewAllSettings->setText(QCoreApplication::translate("MainWindow", "View All Settings", nullptr));
#if QT_CONFIG(tooltip)
        actionViewAllSettings->setToolTip(QCoreApplication::translate("MainWindow", "View internal application settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAccountSettings->setText(QCoreApplication::translate("MainWindow", "Data-Box Settings", nullptr));
#if QT_CONFIG(tooltip)
        actionAccountSettings->setToolTip(QCoreApplication::translate("MainWindow", "View all data-box settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMarkAllReceivedRead->setText(QCoreApplication::translate("MainWindow", "Read", nullptr));
#if QT_CONFIG(tooltip)
        actionMarkAllReceivedRead->setToolTip(QCoreApplication::translate("MainWindow", "Mark all received messages as read.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMarkAllReceivedUnread->setText(QCoreApplication::translate("MainWindow", "Unread", nullptr));
#if QT_CONFIG(tooltip)
        actionMarkAllReceivedUnread->setToolTip(QCoreApplication::translate("MainWindow", "Mark all received messages as unread.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMarkAllReceivedUnsettled->setText(QCoreApplication::translate("MainWindow", "Unsettled", nullptr));
#if QT_CONFIG(tooltip)
        actionMarkAllReceivedUnsettled->setToolTip(QCoreApplication::translate("MainWindow", "Mark all received messages as unsettled.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMarkAllReceivedInProgress->setText(QCoreApplication::translate("MainWindow", "In Progress", nullptr));
#if QT_CONFIG(tooltip)
        actionMarkAllReceivedInProgress->setToolTip(QCoreApplication::translate("MainWindow", "Mark all received messages as in progress.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMarkAllReceivedSettled->setText(QCoreApplication::translate("MainWindow", "Settled", nullptr));
#if QT_CONFIG(tooltip)
        actionMarkAllReceivedSettled->setToolTip(QCoreApplication::translate("MainWindow", "Mark all received messages as settled.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEmailZfos->setText(QCoreApplication::translate("MainWindow", "Message ZFOs", nullptr));
#if QT_CONFIG(tooltip)
        actionEmailZfos->setToolTip(QCoreApplication::translate("MainWindow", "Creates an e-mail containing ZFOs of selected messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEmailAllAttachments->setText(QCoreApplication::translate("MainWindow", "All Attachments", nullptr));
#if QT_CONFIG(tooltip)
        actionEmailAllAttachments->setToolTip(QCoreApplication::translate("MainWindow", "Creates an e-mail containing all attachments of selected messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCheckUploadMessagesRecordsManagement->setText(QCoreApplication::translate("MainWindow", "Check Uploaded Messages In Records Management", nullptr));
#if QT_CONFIG(tooltip)
        actionCheckUploadMessagesRecordsManagement->setToolTip(QCoreApplication::translate("MainWindow", "Check messages for presence in the records management", nullptr));
#endif // QT_CONFIG(tooltip)
        actionConvertImportFromMobileApp->setText(QCoreApplication::translate("MainWindow", "Import Data from Mobile App", nullptr));
#if QT_CONFIG(tooltip)
        actionConvertImportFromMobileApp->setToolTip(QCoreApplication::translate("MainWindow", "Loads data from mobile application transfer ZIP archive.\n"
"These data can then be imported into the desktop application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionConvertExportForMobileApp->setText(QCoreApplication::translate("MainWindow", "Convert and Export Data for Mobile App", nullptr));
#if QT_CONFIG(tooltip)
        actionConvertExportForMobileApp->setToolTip(QCoreApplication::translate("MainWindow", "Prepares data and creates ZIP archive for the mobile application.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionTagStorageSettings->setText(QCoreApplication::translate("MainWindow", "Tag Storage Settings", nullptr));
#if QT_CONFIG(tooltip)
        actionTagStorageSettings->setToolTip(QCoreApplication::translate("MainWindow", "Set where to store tag data.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCustomiseToolBar->setText(QCoreApplication::translate("MainWindow", "Customise Tool Bar", nullptr));
        actionCustomiseAttachmentBar->setText(QCoreApplication::translate("MainWindow", "Customise Attachment Bar", nullptr));
        actionCustomiseDetailBar->setText(QCoreApplication::translate("MainWindow", "Customise Message Detail Bar", nullptr));
        actionCustomiseMessageBar->setText(QCoreApplication::translate("MainWindow", "Customise Message Bar", nullptr));
        actionCustomiseAccountBar->setText(QCoreApplication::translate("MainWindow", "Customise Data-Box Bar", nullptr));
        actionFilterMessages->setText(QCoreApplication::translate("MainWindow", "Filter Messages", nullptr));
#if QT_CONFIG(tooltip)
        actionFilterMessages->setToolTip(QCoreApplication::translate("MainWindow", "Filter message list.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionToolBar->setText(QCoreApplication::translate("MainWindow", "Tool Bar", nullptr));
        actionTopPanelBars->setText(QCoreApplication::translate("MainWindow", "Top Panel Bars", nullptr));
        actionEditTagAssignment->setText(QCoreApplication::translate("MainWindow", "Edit Tag Assignment", nullptr));
#if QT_CONFIG(tooltip)
        actionEditTagAssignment->setToolTip(QCoreApplication::translate("MainWindow", "Manage assignment of tags to messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRepairMessageDatabaseFiles->setText(QCoreApplication::translate("MainWindow", "Repair Message Database Files", nullptr));
#if QT_CONFIG(tooltip)
        actionRepairMessageDatabaseFiles->setToolTip(QCoreApplication::translate("MainWindow", "Launches a repair repair tool in order to attempt\n"
"to recover data from message database files.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionTrackTimestampExpiration->setText(QCoreApplication::translate("MainWindow", "Track Timestamp Expiration", nullptr));
#if QT_CONFIG(tooltip)
        actionTrackTimestampExpiration->setToolTip(QCoreApplication::translate("MainWindow", "Select whether to track the timestamp expiration of the message ZFO.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionTimeStampRenewal->setText(QCoreApplication::translate("MainWindow", "Time Stamp Renewal", nullptr));
#if QT_CONFIG(tooltip)
        actionTimeStampRenewal->setToolTip(QCoreApplication::translate("MainWindow", "Manage messages with tracked timestamps.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionDeleteDraft->setText(QCoreApplication::translate("MainWindow", "Delete Draft", nullptr));
#if QT_CONFIG(tooltip)
        actionDeleteDraft->setToolTip(QCoreApplication::translate("MainWindow", "Delete selected drafts.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionEditDraft->setText(QCoreApplication::translate("MainWindow", "Edit Draft", nullptr));
#if QT_CONFIG(tooltip)
        actionEditDraft->setToolTip(QCoreApplication::translate("MainWindow", "Edit or use a message draft.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        messageFilterLine->setToolTip(QCoreApplication::translate("MainWindow", "Enter sought expression.", nullptr));
#endif // QT_CONFIG(tooltip)
        messageFilterLine->setPlaceholderText(QCoreApplication::translate("MainWindow", "Type to filter.", nullptr));
#if QT_CONFIG(tooltip)
        messageFilterCloseButton->setToolTip(QCoreApplication::translate("MainWindow", "Hide filter field.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        draftFilterLine->setToolTip(QCoreApplication::translate("MainWindow", "Enter sought expression.", nullptr));
#endif // QT_CONFIG(tooltip)
        draftFilterLine->setPlaceholderText(QCoreApplication::translate("MainWindow", "Type to filter.", nullptr));
#if QT_CONFIG(tooltip)
        draftFilterCloseButton->setToolTip(QCoreApplication::translate("MainWindow", "Hide filter field.", nullptr));
#endif // QT_CONFIG(tooltip)
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "File", nullptr));
        menuRecordsManagement->setTitle(QCoreApplication::translate("MainWindow", "Records Management", nullptr));
        menuDatabox->setTitle(QCoreApplication::translate("MainWindow", "Data Box", nullptr));
        menuMarkAllReceivedAs->setTitle(QCoreApplication::translate("MainWindow", "Mark All Received as", nullptr));
        menuSynchronisation->setTitle(QCoreApplication::translate("MainWindow", "Synchronisation", nullptr));
        menuMessage->setTitle(QCoreApplication::translate("MainWindow", "Message", nullptr));
        menuEmailWith->setTitle(QCoreApplication::translate("MainWindow", "E-mail with", nullptr));
        menuTools->setTitle(QCoreApplication::translate("MainWindow", "Tools", nullptr));
        menuToolBars->setTitle(QCoreApplication::translate("MainWindow", "Tool Bars", nullptr));
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "Help", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("MainWindow", "toolBar", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DATOVKA_H
