/****************************************************************************
** Meta object code from reading C++ file 'dlg_records_management_check_msgs.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/records_management/gui/dlg_records_management_check_msgs.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dlg_records_management_check_msgs.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t {};
} // unnamed namespace

template <> constexpr inline auto DlgRecordsManagementCheckMsgs::qt_create_metaobjectdata<qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DlgRecordsManagementCheckMsgs",
        "focusSelectedMsg",
        "",
        "AcntId",
        "acntId",
        "dmId",
        "year",
        "msgType",
        "applyAllSelection",
        "state",
        "filterAccount",
        "text",
        "filterMessage",
        "modifiedAccountSelection",
        "reflectCalendarChange",
        "enableCanlendars",
        "modifiedMessageDownloadStatus",
        "messageListLengthChanged",
        "updateMessageSelection",
        "QItemSelection",
        "selected",
        "deselected",
        "messageItemDoubleClicked",
        "QModelIndex",
        "index",
        "viewMessageListContextMenu",
        "QPoint",
        "point",
        "watchMessageInserted",
        "MsgId",
        "msgId",
        "direction",
        "watchMessageDataDeleted",
        "watchRmMsgEntryUpdated",
        "locations",
        "downloadSelectedMessageAttachments",
        "viewSendSelectedMessageToRecordsManagementDlg",
        "checkUploadedMsgs",
        "uploadMsgsToRm",
        "exportMsgs",
        "abortUpload"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'focusSelectedMsg'
        QtMocHelpers::SignalData<void(AcntId, qint64, QString, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::LongLong, 5 }, { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
        }}),
        // Slot 'applyAllSelection'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'filterAccount'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'filterMessage'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'modifiedAccountSelection'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reflectCalendarChange'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'enableCanlendars'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'modifiedMessageDownloadStatus'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'messageListLengthChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 19, 21 },
        }}),
        // Slot 'updateMessageSelection'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(18, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'messageItemDoubleClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Slot 'viewMessageListContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'watchMessageInserted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &, int)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 29, 30 }, { QMetaType::Int, 31 },
        }}),
        // Slot 'watchMessageDataDeleted'
        QtMocHelpers::SlotData<void(const AcntId &, const MsgId &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'watchRmMsgEntryUpdated'
        QtMocHelpers::SlotData<void(qint64, const QStringList &)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::LongLong, 5 }, { QMetaType::QStringList, 34 },
        }}),
        // Slot 'downloadSelectedMessageAttachments'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'viewSendSelectedMessageToRecordsManagementDlg'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'checkUploadedMsgs'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'uploadMsgsToRm'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'exportMsgs'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'abortUpload'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DlgRecordsManagementCheckMsgs, qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DlgRecordsManagementCheckMsgs::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>.metaTypes,
    nullptr
} };

void DlgRecordsManagementCheckMsgs::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DlgRecordsManagementCheckMsgs *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->focusSelectedMsg((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 1: _t->applyAllSelection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->filterAccount((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->filterMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->modifiedAccountSelection(); break;
        case 5: _t->reflectCalendarChange(); break;
        case 6: _t->enableCanlendars((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->modifiedMessageDownloadStatus(); break;
        case 8: _t->messageListLengthChanged(); break;
        case 9: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 10: _t->updateMessageSelection((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 11: _t->messageItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->viewMessageListContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 13: _t->watchMessageInserted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 14: _t->watchMessageDataDeleted((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[2]))); break;
        case 15: _t->watchRmMsgEntryUpdated((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 16: _t->downloadSelectedMessageAttachments(); break;
        case 17: _t->viewSendSelectedMessageToRecordsManagementDlg(); break;
        case 18: _t->checkUploadedMsgs(); break;
        case 19: _t->uploadMsgsToRm(); break;
        case 20: _t->exportMsgs(); break;
        case 21: _t->abortUpload(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< AcntId >(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DlgRecordsManagementCheckMsgs::*)(AcntId , qint64 , QString , int )>(_a, &DlgRecordsManagementCheckMsgs::focusSelectedMsg, 0))
            return;
    }
}

const QMetaObject *DlgRecordsManagementCheckMsgs::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DlgRecordsManagementCheckMsgs::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29DlgRecordsManagementCheckMsgsE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int DlgRecordsManagementCheckMsgs::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void DlgRecordsManagementCheckMsgs::focusSelectedMsg(AcntId _t1, qint64 _t2, QString _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
