/********************************************************************************
** Form generated from reading UI file 'dlg_account_from_db.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_ACCOUNT_FROM_DB_H
#define UI_DLG_ACCOUNT_FROM_DB_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgCreateAccountFromDb
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *info;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *directory;
    QRadioButton *files;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgCreateAccountFromDb)
    {
        if (DlgCreateAccountFromDb->objectName().isEmpty())
            DlgCreateAccountFromDb->setObjectName("DlgCreateAccountFromDb");
        DlgCreateAccountFromDb->resize(412, 306);
        verticalLayout = new QVBoxLayout(DlgCreateAccountFromDb);
        verticalLayout->setObjectName("verticalLayout");
        info = new QTextEdit(DlgCreateAccountFromDb);
        info->setObjectName("info");
        info->setReadOnly(true);

        verticalLayout->addWidget(info);

        groupBox = new QGroupBox(DlgCreateAccountFromDb);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        directory = new QRadioButton(groupBox);
        directory->setObjectName("directory");
        directory->setChecked(true);

        verticalLayout_2->addWidget(directory);

        files = new QRadioButton(groupBox);
        files->setObjectName("files");

        verticalLayout_2->addWidget(files);


        verticalLayout->addWidget(groupBox);

        buttonBox = new QDialogButtonBox(DlgCreateAccountFromDb);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(info, directory);
        QWidget::setTabOrder(directory, files);

        retranslateUi(DlgCreateAccountFromDb);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgCreateAccountFromDb, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgCreateAccountFromDb, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgCreateAccountFromDb);
    } // setupUi

    void retranslateUi(QDialog *DlgCreateAccountFromDb)
    {
        DlgCreateAccountFromDb->setWindowTitle(QCoreApplication::translate("DlgCreateAccountFromDb", "Create data box from database file", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgCreateAccountFromDb", "What do you want to create the data box(es) from?", nullptr));
        directory->setText(QCoreApplication::translate("DlgCreateAccountFromDb", "Database files from directory", nullptr));
        files->setText(QCoreApplication::translate("DlgCreateAccountFromDb", "Selected database files", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgCreateAccountFromDb: public Ui_DlgCreateAccountFromDb {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_ACCOUNT_FROM_DB_H
