/*
 * Copyright (C) 2014-2026 CZ.NIC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations including
 * the two.
 */

#include<QStringBuilder>

#include "src/datovka_shared/app_version_info.h"

#define textLineNL(text) \
	(QString("‣ ") % (text) % QLatin1String("<br>"))

QString AppVersionInfo::releaseNewsText(void)
{
	QString content;

	content.append(textLineNL(tr("Added a default global option whether to immediately download signed message content after it has been sent.")));
	content.append(textLineNL(tr("Applying time stamp expiration notification interval from global settings in time stamp tracking and restamping. The default notification interval is set to 30 days.")));
	content.append(textLineNL(tr("Added options into settings whether to automatically start timestamp tracking of freshly downloaded messages.")));

	return content;
}
