/********************************************************************************
** Form generated from reading UI file 'dlg_restore.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_RESTORE_H
#define UI_DLG_RESTORE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgRestore
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoLabel;
    QFrame *line;
    QGroupBox *whereGroupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QLabel *jsonLabel;
    QLineEdit *jsonLine;
    QPushButton *chooseDirButton;
    QLabel *backupTimeLabel;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_3;
    LoweredTableView *accountView;
    QFormLayout *formLayout;
    QCheckBox *tagsCheckBox;
    QLabel *tagsActionLabel;
    QLabel *warningLabel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgRestore)
    {
        if (DlgRestore->objectName().isEmpty())
            DlgRestore->setObjectName("DlgRestore");
        DlgRestore->resize(415, 460);
        verticalLayout = new QVBoxLayout(DlgRestore);
        verticalLayout->setObjectName("verticalLayout");
        infoLabel = new QLabel(DlgRestore);
        infoLabel->setObjectName("infoLabel");

        verticalLayout->addWidget(infoLabel);

        line = new QFrame(DlgRestore);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        whereGroupBox = new QGroupBox(DlgRestore);
        whereGroupBox->setObjectName("whereGroupBox");
        verticalLayout_2 = new QVBoxLayout(whereGroupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        jsonLabel = new QLabel(whereGroupBox);
        jsonLabel->setObjectName("jsonLabel");

        horizontalLayout->addWidget(jsonLabel);

        jsonLine = new QLineEdit(whereGroupBox);
        jsonLine->setObjectName("jsonLine");

        horizontalLayout->addWidget(jsonLine);

        chooseDirButton = new QPushButton(whereGroupBox);
        chooseDirButton->setObjectName("chooseDirButton");

        horizontalLayout->addWidget(chooseDirButton);


        verticalLayout_2->addLayout(horizontalLayout);

        backupTimeLabel = new QLabel(whereGroupBox);
        backupTimeLabel->setObjectName("backupTimeLabel");

        verticalLayout_2->addWidget(backupTimeLabel);


        verticalLayout->addWidget(whereGroupBox);

        groupBox = new QGroupBox(DlgRestore);
        groupBox->setObjectName("groupBox");
        verticalLayout_3 = new QVBoxLayout(groupBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        accountView = new LoweredTableView(groupBox);
        accountView->setObjectName("accountView");
        accountView->setShowGrid(false);
        accountView->setWordWrap(false);
        accountView->horizontalHeader()->setHighlightSections(false);
        accountView->horizontalHeader()->setStretchLastSection(true);
        accountView->verticalHeader()->setVisible(false);
        accountView->verticalHeader()->setHighlightSections(false);

        verticalLayout_3->addWidget(accountView);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        tagsCheckBox = new QCheckBox(groupBox);
        tagsCheckBox->setObjectName("tagsCheckBox");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, tagsCheckBox);

        tagsActionLabel = new QLabel(groupBox);
        tagsActionLabel->setObjectName("tagsActionLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, tagsActionLabel);


        verticalLayout_3->addLayout(formLayout);


        verticalLayout->addWidget(groupBox);

        warningLabel = new QLabel(DlgRestore);
        warningLabel->setObjectName("warningLabel");
        warningLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(warningLabel);

        buttonBox = new QDialogButtonBox(DlgRestore);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgRestore);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgRestore, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgRestore);
    } // setupUi

    void retranslateUi(QDialog *DlgRestore)
    {
        DlgRestore->setWindowTitle(QCoreApplication::translate("DlgRestore", "Restore Data", nullptr));
        infoLabel->setText(QCoreApplication::translate("DlgRestore", "n/a", nullptr));
        whereGroupBox->setTitle(QCoreApplication::translate("DlgRestore", "Where do you want to load the back-up from?", nullptr));
        jsonLabel->setText(QCoreApplication::translate("DlgRestore", "JSON file", nullptr));
        chooseDirButton->setText(QCoreApplication::translate("DlgRestore", "Choose", nullptr));
        backupTimeLabel->setText(QCoreApplication::translate("DlgRestore", "n/a", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgRestore", "What data do you want to restore?", nullptr));
        tagsCheckBox->setText(QCoreApplication::translate("DlgRestore", "Tag database", nullptr));
        tagsActionLabel->setText(QCoreApplication::translate("DlgRestore", "n/a", nullptr));
        warningLabel->setText(QCoreApplication::translate("DlgRestore", "n/a", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgRestore: public Ui_DlgRestore {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_RESTORE_H
