/****************************************************************************
** Meta object code from reading C++ file 'tag_assignment_model.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/models/tag_assignment_model.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tag_assignment_model.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TagAssignmentModelE_t {};
} // unnamed namespace

template <> constexpr inline auto TagAssignmentModel::qt_create_metaobjectdata<qt_meta_tag_ZN18TagAssignmentModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagAssignmentModel",
        "watchTagsInserted",
        "",
        "Json::TagEntryList",
        "entries",
        "watchTagsUpdated",
        "watchTagsDeleted",
        "Json::Int64StringList",
        "ids",
        "watchTagAssignmentChanged",
        "Json::TagAssignmentList",
        "assignments",
        "watchTagContConnected",
        "watchTagContDisconnected",
        "watchTagContReset",
        "UserRoles",
        "ROLE_PROXYSORT",
        "Columns",
        "COL_TAG_NAME",
        "MAX_COLNUM"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'watchTagsInserted'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'watchTagsUpdated'
        QtMocHelpers::SlotData<void(const Json::TagEntryList &)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'watchTagsDeleted'
        QtMocHelpers::SlotData<void(const Json::Int64StringList &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'watchTagAssignmentChanged'
        QtMocHelpers::SlotData<void(const Json::TagAssignmentList &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'watchTagContConnected'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchTagContDisconnected'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'watchTagContReset'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'UserRoles'
        QtMocHelpers::EnumData<enum UserRoles>(15, 15, QMC::EnumFlags{}).add({
            {   16, UserRoles::ROLE_PROXYSORT },
        }),
        // enum 'Columns'
        QtMocHelpers::EnumData<enum Columns>(17, 17, QMC::EnumFlags{}).add({
            {   18, Columns::COL_TAG_NAME },
            {   19, Columns::MAX_COLNUM },
        }),
    };
    return QtMocHelpers::metaObjectData<TagAssignmentModel, qt_meta_tag_ZN18TagAssignmentModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagAssignmentModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagAssignmentModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagAssignmentModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18TagAssignmentModelE_t>.metaTypes,
    nullptr
} };

void TagAssignmentModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagAssignmentModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->watchTagsInserted((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 1: _t->watchTagsUpdated((*reinterpret_cast<std::add_pointer_t<Json::TagEntryList>>(_a[1]))); break;
        case 2: _t->watchTagsDeleted((*reinterpret_cast<std::add_pointer_t<Json::Int64StringList>>(_a[1]))); break;
        case 3: _t->watchTagAssignmentChanged((*reinterpret_cast<std::add_pointer_t<Json::TagAssignmentList>>(_a[1]))); break;
        case 4: _t->watchTagContConnected(); break;
        case 5: _t->watchTagContDisconnected(); break;
        case 6: _t->watchTagContReset(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagEntryList >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::Int64StringList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Json::TagAssignmentList >(); break;
            }
            break;
        }
    }
}

const QMetaObject *TagAssignmentModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagAssignmentModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TagAssignmentModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int TagAssignmentModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}
QT_WARNING_POP
