/****************************************************************************
** Meta object code from reading C++ file 'message_emitter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/worker/message_emitter.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'message_emitter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24MessageProcessingEmitterE_t {};
} // unnamed namespace

template <> constexpr inline auto MessageProcessingEmitter::qt_create_metaobjectdata<qt_meta_tag_ZN24MessageProcessingEmitterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MessageProcessingEmitter",
        "downloadMessageFinished",
        "",
        "AcntId",
        "acntId",
        "msgId",
        "deliveryTime",
        "result",
        "errDesc",
        "processFlags",
        "recMgmtHierarchyId",
        "downloadMessageListFinished",
        "AcntIdDb",
        "rAcntIdDb",
        "sAcntId",
        "direction",
        "add",
        "rt",
        "rn",
        "st",
        "sn",
        "foundDownloadedUnlistedMessages",
        "QList<MsgId>",
        "unlistedMsgIds",
        "importZfoFinished",
        "fileName",
        "resultDesc",
        "importMessageFinished",
        "userName",
        "errImportList",
        "totalMsgs",
        "importedMsgs",
        "progressChange",
        "label",
        "value",
        "sendMessageFinished",
        "transactId",
        "dbIDRecipient",
        "recipientName",
        "isPDZ",
        "isVodz",
        "dmId",
        "statusBarChange",
        "text",
        "recordsManagementStoredMessagesFinished",
        "taskId",
        "downloadProgress",
        "MsgId",
        "downloadTotal",
        "downloadCurrent",
        "downloadProgressFinished",
        "uploadProgress",
        "uploadTotal",
        "uploadCurrent",
        "uploadProgressFinished",
        "QVariant",
        "resultVal"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'downloadMessageFinished'
        QtMocHelpers::SignalData<void(const AcntId &, qint64, const QDateTime &, int, const QString &, int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::LongLong, 5 }, { QMetaType::QDateTime, 6 }, { QMetaType::Int, 7 },
            { QMetaType::QString, 8 }, { QMetaType::Int, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'downloadMessageListFinished'
        QtMocHelpers::SignalData<void(const AcntIdDb &, const AcntId &, int, int, const QString &, bool, int, int, int, int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 3, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 7 },
            { QMetaType::QString, 8 }, { QMetaType::Bool, 16 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
            { QMetaType::Int, 19 }, { QMetaType::Int, 20 },
        }}),
        // Signal 'foundDownloadedUnlistedMessages'
        QtMocHelpers::SignalData<void(const AcntIdDb &, int, const QList<MsgId> &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { QMetaType::Int, 15 }, { 0x80000000 | 22, 23 },
        }}),
        // Signal 'importZfoFinished'
        QtMocHelpers::SignalData<void(const QString &, int, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::Int, 7 }, { QMetaType::QString, 26 },
        }}),
        // Signal 'importMessageFinished'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &, int, int)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 28 }, { QMetaType::QStringList, 29 }, { QMetaType::Int, 30 }, { QMetaType::Int, 31 },
        }}),
        // Signal 'progressChange'
        QtMocHelpers::SignalData<void(const QString &, int)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 }, { QMetaType::Int, 34 },
        }}),
        // Signal 'sendMessageFinished'
        QtMocHelpers::SignalData<void(const AcntId &, const QString &, int, const QString &, const QString &, const QString &, bool, bool, qint64, int, const QString &)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 36 }, { QMetaType::Int, 7 }, { QMetaType::QString, 26 },
            { QMetaType::QString, 37 }, { QMetaType::QString, 38 }, { QMetaType::Bool, 39 }, { QMetaType::Bool, 40 },
            { QMetaType::LongLong, 41 }, { QMetaType::Int, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'statusBarChange'
        QtMocHelpers::SignalData<void(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Signal 'recordsManagementStoredMessagesFinished'
        QtMocHelpers::SignalData<void(const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 45 },
        }}),
        // Signal 'downloadProgress'
        QtMocHelpers::SignalData<void(const AcntId &, const QString &, const MsgId &, qint64, qint64)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 36 }, { 0x80000000 | 47, 5 }, { QMetaType::LongLong, 48 },
            { QMetaType::LongLong, 49 },
        }}),
        // Signal 'downloadProgressFinished'
        QtMocHelpers::SignalData<void(const AcntId &, const QString &, const MsgId &, int, const QString &)>(50, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 36 }, { 0x80000000 | 47, 5 }, { QMetaType::Int, 7 },
            { QMetaType::QString, 26 },
        }}),
        // Signal 'uploadProgress'
        QtMocHelpers::SignalData<void(const AcntId &, const QString &, qint64, qint64)>(51, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 36 }, { QMetaType::LongLong, 52 }, { QMetaType::LongLong, 53 },
        }}),
        // Signal 'uploadProgressFinished'
        QtMocHelpers::SignalData<void(const AcntId &, const QString &, int, const QString &, const QVariant &)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 36 }, { QMetaType::Int, 7 }, { QMetaType::QString, 26 },
            { 0x80000000 | 55, 56 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MessageProcessingEmitter, qt_meta_tag_ZN24MessageProcessingEmitterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MessageProcessingEmitter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MessageProcessingEmitterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MessageProcessingEmitterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24MessageProcessingEmitterE_t>.metaTypes,
    nullptr
} };

void MessageProcessingEmitter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MessageProcessingEmitter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->downloadMessageFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7]))); break;
        case 1: _t->downloadMessageListFinished((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10]))); break;
        case 2: _t->foundDownloadedUnlistedMessages((*reinterpret_cast<std::add_pointer_t<AcntIdDb>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<MsgId>>>(_a[3]))); break;
        case 3: _t->importZfoFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 4: _t->importMessageFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 5: _t->progressChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->sendMessageFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[11]))); break;
        case 7: _t->statusBarChange((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->recordsManagementStoredMessagesFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->downloadProgress((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[5]))); break;
        case 10: _t->downloadProgressFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<MsgId>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 11: _t->uploadProgress((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qint64>>(_a[4]))); break;
        case 12: _t->uploadProgressFinished((*reinterpret_cast<std::add_pointer_t<AcntId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[5]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<MsgId> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MsgId >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , qint64 , const QDateTime & , int , const QString & , int , const QString & )>(_a, &MessageProcessingEmitter::downloadMessageFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntIdDb & , const AcntId & , int , int , const QString & , bool , int , int , int , int )>(_a, &MessageProcessingEmitter::downloadMessageListFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntIdDb & , int , const QList<MsgId> & )>(_a, &MessageProcessingEmitter::foundDownloadedUnlistedMessages, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const QString & , int , const QString & )>(_a, &MessageProcessingEmitter::importZfoFinished, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const QString & , const QStringList & , int , int )>(_a, &MessageProcessingEmitter::importMessageFinished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const QString & , int )>(_a, &MessageProcessingEmitter::progressChange, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , const QString & , int , const QString & , const QString & , const QString & , bool , bool , qint64 , int , const QString & )>(_a, &MessageProcessingEmitter::sendMessageFinished, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const QString & )>(_a, &MessageProcessingEmitter::statusBarChange, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const QString & )>(_a, &MessageProcessingEmitter::recordsManagementStoredMessagesFinished, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , const QString & , const MsgId & , qint64 , qint64 )>(_a, &MessageProcessingEmitter::downloadProgress, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , const QString & , const MsgId & , int , const QString & )>(_a, &MessageProcessingEmitter::downloadProgressFinished, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , const QString & , qint64 , qint64 )>(_a, &MessageProcessingEmitter::uploadProgress, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (MessageProcessingEmitter::*)(const AcntId & , const QString & , int , const QString & , const QVariant & )>(_a, &MessageProcessingEmitter::uploadProgressFinished, 12))
            return;
    }
}

const QMetaObject *MessageProcessingEmitter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MessageProcessingEmitter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MessageProcessingEmitterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MessageProcessingEmitter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void MessageProcessingEmitter::downloadMessageFinished(const AcntId & _t1, qint64 _t2, const QDateTime & _t3, int _t4, const QString & _t5, int _t6, const QString & _t7)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7);
}

// SIGNAL 1
void MessageProcessingEmitter::downloadMessageListFinished(const AcntIdDb & _t1, const AcntId & _t2, int _t3, int _t4, const QString & _t5, bool _t6, int _t7, int _t8, int _t9, int _t10)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10);
}

// SIGNAL 2
void MessageProcessingEmitter::foundDownloadedUnlistedMessages(const AcntIdDb & _t1, int _t2, const QList<MsgId> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void MessageProcessingEmitter::importZfoFinished(const QString & _t1, int _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void MessageProcessingEmitter::importMessageFinished(const QString & _t1, const QStringList & _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void MessageProcessingEmitter::progressChange(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void MessageProcessingEmitter::sendMessageFinished(const AcntId & _t1, const QString & _t2, int _t3, const QString & _t4, const QString & _t5, const QString & _t6, bool _t7, bool _t8, qint64 _t9, int _t10, const QString & _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 7
void MessageProcessingEmitter::statusBarChange(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void MessageProcessingEmitter::recordsManagementStoredMessagesFinished(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MessageProcessingEmitter::downloadProgress(const AcntId & _t1, const QString & _t2, const MsgId & _t3, qint64 _t4, qint64 _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 10
void MessageProcessingEmitter::downloadProgressFinished(const AcntId & _t1, const QString & _t2, const MsgId & _t3, int _t4, const QString & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 11
void MessageProcessingEmitter::uploadProgress(const AcntId & _t1, const QString & _t2, qint64 _t3, qint64 _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 12
void MessageProcessingEmitter::uploadProgressFinished(const AcntId & _t1, const QString & _t2, int _t3, const QString & _t4, const QVariant & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3, _t4, _t5);
}
QT_WARNING_POP
