/********************************************************************************
** Form generated from reading UI file 'dlg_records_management_progress.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_RECORDS_MANAGEMENT_PROGRESS_H
#define UI_DLG_RECORDS_MANAGEMENT_PROGRESS_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DlgRecordsManagementProgress
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout_2;
    QLabel *pixmapLabel;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_3;
    QLabel *taskLabel;
    QProgressBar *taskProgress;
    QSpacerItem *verticalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgRecordsManagementProgress)
    {
        if (DlgRecordsManagementProgress->objectName().isEmpty())
            DlgRecordsManagementProgress->setObjectName("DlgRecordsManagementProgress");
        DlgRecordsManagementProgress->resize(400, 120);
        verticalLayout = new QVBoxLayout(DlgRecordsManagementProgress);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        pixmapLabel = new QLabel(DlgRecordsManagementProgress);
        pixmapLabel->setObjectName("pixmapLabel");

        verticalLayout_2->addWidget(pixmapLabel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName("verticalLayout_3");
        taskLabel = new QLabel(DlgRecordsManagementProgress);
        taskLabel->setObjectName("taskLabel");

        verticalLayout_3->addWidget(taskLabel);

        taskProgress = new QProgressBar(DlgRecordsManagementProgress);
        taskProgress->setObjectName("taskProgress");
        taskProgress->setValue(24);

        verticalLayout_3->addWidget(taskProgress);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);


        horizontalLayout->addLayout(verticalLayout_3);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(DlgRecordsManagementProgress);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgRecordsManagementProgress);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgRecordsManagementProgress, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgRecordsManagementProgress, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgRecordsManagementProgress);
    } // setupUi

    void retranslateUi(QDialog *DlgRecordsManagementProgress)
    {
        pixmapLabel->setText(QString());
        taskLabel->setText(QString());
        (void)DlgRecordsManagementProgress;
    } // retranslateUi

};

namespace Ui {
    class DlgRecordsManagementProgress: public Ui_DlgRecordsManagementProgress {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_RECORDS_MANAGEMENT_PROGRESS_H
