/********************************************************************************
** Form generated from reading UI file 'dlg_timestamp_renewal.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_TIMESTAMP_RENEWAL_H
#define UI_DLG_TIMESTAMP_RENEWAL_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgTimestampRenewal
{
public:
    QAction *actionTrackedTrackTimestampExpiration;
    QAction *actionTrackedSelectedRestamp;
    QAction *actionAllStoredTrackTimestampExpiration;
    QAction *actionAllStoredSelectedRestamp;
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab_0_tracked;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_01;
    QSpacerItem *horizontalSpacer_01;
    QPushButton *restampSelectedTrackedPushButton;
    LoweredTableView *trackedMessagesView;
    QWidget *trackedMessagesFilterHorizontalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *trackedMessagesFilterLine;
    QPushButton *closeTrackedFilterButton;
    QWidget *tab_1_all_stored;
    QVBoxLayout *verticalLayout_5;
    QSplitter *hSplitterAllStored;
    QGroupBox *accountGroupBox;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_02;
    QCheckBox *allCheckBox;
    QSpacerItem *horizontalSpacer_02;
    QPushButton *allStoredScanPushButton;
    QPushButton *allStoredScanInterruptPushButton;
    LoweredTableView *accountView;
    QGroupBox *allStoredGroupBox;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_03;
    QCheckBox *onlyRestampableCheckBox;
    QSpacerItem *horizontalSpacer_03;
    QPushButton *restampSelectedAllStoredPushButton;
    QPushButton *restampListedAllStoredPushButton;
    LoweredTableView *allMessagesView;
    QWidget *allMessagesFilterHorizontalLayout;
    QHBoxLayout *horizontalLayout;
    QLineEdit *allMessagesFilterLine;
    QPushButton *closeAllFilterButton;
    QWidget *tab_2_external;
    QVBoxLayout *verticalLayout_3;
    QFormLayout *formLayout_2;
    QLabel *crntAcntLabel;
    QComboBox *accountComboBox;
    QFrame *line;
    QGridLayout *gridLayout;
    QLineEdit *srcZfoLineEdit;
    QLabel *srcZfoLabel;
    QLineEdit *tgtZfoLineEdit;
    QPushButton *chooseSrcPushButton;
    QPushButton *chooseTgtPushButton;
    QLabel *tgtZfoLabel;
    QPushButton *viewSrcPushButton;
    QPushButton *restampExternalPushButton;
    QProgressBar *singleProgressBar;
    QLabel *singleResultLabel;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgTimestampRenewal)
    {
        if (DlgTimestampRenewal->objectName().isEmpty())
            DlgTimestampRenewal->setObjectName("DlgTimestampRenewal");
        DlgTimestampRenewal->resize(440, 440);
        actionTrackedTrackTimestampExpiration = new QAction(DlgTimestampRenewal);
        actionTrackedTrackTimestampExpiration->setObjectName("actionTrackedTrackTimestampExpiration");
        actionTrackedTrackTimestampExpiration->setCheckable(true);
        actionTrackedTrackTimestampExpiration->setMenuRole(QAction::NoRole);
        actionTrackedSelectedRestamp = new QAction(DlgTimestampRenewal);
        actionTrackedSelectedRestamp->setObjectName("actionTrackedSelectedRestamp");
        actionTrackedSelectedRestamp->setMenuRole(QAction::NoRole);
        actionAllStoredTrackTimestampExpiration = new QAction(DlgTimestampRenewal);
        actionAllStoredTrackTimestampExpiration->setObjectName("actionAllStoredTrackTimestampExpiration");
        actionAllStoredTrackTimestampExpiration->setCheckable(true);
        actionAllStoredTrackTimestampExpiration->setMenuRole(QAction::NoRole);
        actionAllStoredSelectedRestamp = new QAction(DlgTimestampRenewal);
        actionAllStoredSelectedRestamp->setObjectName("actionAllStoredSelectedRestamp");
        actionAllStoredSelectedRestamp->setMenuRole(QAction::NoRole);
        verticalLayout = new QVBoxLayout(DlgTimestampRenewal);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(DlgTimestampRenewal);
        tabWidget->setObjectName("tabWidget");
        tab_0_tracked = new QWidget();
        tab_0_tracked->setObjectName("tab_0_tracked");
        verticalLayout_2 = new QVBoxLayout(tab_0_tracked);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_01 = new QHBoxLayout();
        horizontalLayout_01->setObjectName("horizontalLayout_01");
        horizontalSpacer_01 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_01->addItem(horizontalSpacer_01);

        restampSelectedTrackedPushButton = new QPushButton(tab_0_tracked);
        restampSelectedTrackedPushButton->setObjectName("restampSelectedTrackedPushButton");

        horizontalLayout_01->addWidget(restampSelectedTrackedPushButton);


        verticalLayout_2->addLayout(horizontalLayout_01);

        trackedMessagesView = new LoweredTableView(tab_0_tracked);
        trackedMessagesView->setObjectName("trackedMessagesView");
        trackedMessagesView->setShowGrid(false);
        trackedMessagesView->setWordWrap(false);
        trackedMessagesView->horizontalHeader()->setHighlightSections(false);
        trackedMessagesView->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        trackedMessagesView->horizontalHeader()->setStretchLastSection(true);
        trackedMessagesView->verticalHeader()->setVisible(false);
        trackedMessagesView->verticalHeader()->setHighlightSections(false);

        verticalLayout_2->addWidget(trackedMessagesView);

        trackedMessagesFilterHorizontalLayout = new QWidget(tab_0_tracked);
        trackedMessagesFilterHorizontalLayout->setObjectName("trackedMessagesFilterHorizontalLayout");
        horizontalLayout_2 = new QHBoxLayout(trackedMessagesFilterHorizontalLayout);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        trackedMessagesFilterLine = new QLineEdit(trackedMessagesFilterHorizontalLayout);
        trackedMessagesFilterLine->setObjectName("trackedMessagesFilterLine");
        trackedMessagesFilterLine->setClearButtonEnabled(true);

        horizontalLayout_2->addWidget(trackedMessagesFilterLine);

        closeTrackedFilterButton = new QPushButton(trackedMessagesFilterHorizontalLayout);
        closeTrackedFilterButton->setObjectName("closeTrackedFilterButton");

        horizontalLayout_2->addWidget(closeTrackedFilterButton);


        verticalLayout_2->addWidget(trackedMessagesFilterHorizontalLayout);

        tabWidget->addTab(tab_0_tracked, QString());
        tab_1_all_stored = new QWidget();
        tab_1_all_stored->setObjectName("tab_1_all_stored");
        verticalLayout_5 = new QVBoxLayout(tab_1_all_stored);
        verticalLayout_5->setObjectName("verticalLayout_5");
        hSplitterAllStored = new QSplitter(tab_1_all_stored);
        hSplitterAllStored->setObjectName("hSplitterAllStored");
        hSplitterAllStored->setOrientation(Qt::Vertical);
        accountGroupBox = new QGroupBox(hSplitterAllStored);
        accountGroupBox->setObjectName("accountGroupBox");
        verticalLayout_4 = new QVBoxLayout(accountGroupBox);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_02 = new QHBoxLayout();
        horizontalLayout_02->setObjectName("horizontalLayout_02");
        allCheckBox = new QCheckBox(accountGroupBox);
        allCheckBox->setObjectName("allCheckBox");

        horizontalLayout_02->addWidget(allCheckBox);

        horizontalSpacer_02 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_02->addItem(horizontalSpacer_02);

        allStoredScanPushButton = new QPushButton(accountGroupBox);
        allStoredScanPushButton->setObjectName("allStoredScanPushButton");

        horizontalLayout_02->addWidget(allStoredScanPushButton);

        allStoredScanInterruptPushButton = new QPushButton(accountGroupBox);
        allStoredScanInterruptPushButton->setObjectName("allStoredScanInterruptPushButton");

        horizontalLayout_02->addWidget(allStoredScanInterruptPushButton);


        verticalLayout_4->addLayout(horizontalLayout_02);

        accountView = new LoweredTableView(accountGroupBox);
        accountView->setObjectName("accountView");
        accountView->setShowGrid(false);
        accountView->setWordWrap(false);
        accountView->horizontalHeader()->setHighlightSections(false);
        accountView->horizontalHeader()->setStretchLastSection(true);
        accountView->verticalHeader()->setVisible(false);
        accountView->verticalHeader()->setHighlightSections(false);

        verticalLayout_4->addWidget(accountView);

        hSplitterAllStored->addWidget(accountGroupBox);
        allStoredGroupBox = new QGroupBox(hSplitterAllStored);
        allStoredGroupBox->setObjectName("allStoredGroupBox");
        verticalLayout_6 = new QVBoxLayout(allStoredGroupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_03 = new QHBoxLayout();
        horizontalLayout_03->setObjectName("horizontalLayout_03");
        onlyRestampableCheckBox = new QCheckBox(allStoredGroupBox);
        onlyRestampableCheckBox->setObjectName("onlyRestampableCheckBox");

        horizontalLayout_03->addWidget(onlyRestampableCheckBox);

        horizontalSpacer_03 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_03->addItem(horizontalSpacer_03);

        restampSelectedAllStoredPushButton = new QPushButton(allStoredGroupBox);
        restampSelectedAllStoredPushButton->setObjectName("restampSelectedAllStoredPushButton");

        horizontalLayout_03->addWidget(restampSelectedAllStoredPushButton);

        restampListedAllStoredPushButton = new QPushButton(allStoredGroupBox);
        restampListedAllStoredPushButton->setObjectName("restampListedAllStoredPushButton");

        horizontalLayout_03->addWidget(restampListedAllStoredPushButton);


        verticalLayout_6->addLayout(horizontalLayout_03);

        allMessagesView = new LoweredTableView(allStoredGroupBox);
        allMessagesView->setObjectName("allMessagesView");
        allMessagesView->setShowGrid(false);
        allMessagesView->setWordWrap(false);
        allMessagesView->horizontalHeader()->setHighlightSections(false);
        allMessagesView->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        allMessagesView->horizontalHeader()->setStretchLastSection(true);
        allMessagesView->verticalHeader()->setVisible(false);
        allMessagesView->verticalHeader()->setHighlightSections(false);

        verticalLayout_6->addWidget(allMessagesView);

        allMessagesFilterHorizontalLayout = new QWidget(allStoredGroupBox);
        allMessagesFilterHorizontalLayout->setObjectName("allMessagesFilterHorizontalLayout");
        horizontalLayout = new QHBoxLayout(allMessagesFilterHorizontalLayout);
        horizontalLayout->setObjectName("horizontalLayout");
        allMessagesFilterLine = new QLineEdit(allMessagesFilterHorizontalLayout);
        allMessagesFilterLine->setObjectName("allMessagesFilterLine");
        allMessagesFilterLine->setClearButtonEnabled(true);

        horizontalLayout->addWidget(allMessagesFilterLine);

        closeAllFilterButton = new QPushButton(allMessagesFilterHorizontalLayout);
        closeAllFilterButton->setObjectName("closeAllFilterButton");

        horizontalLayout->addWidget(closeAllFilterButton);


        verticalLayout_6->addWidget(allMessagesFilterHorizontalLayout);

        hSplitterAllStored->addWidget(allStoredGroupBox);

        verticalLayout_5->addWidget(hSplitterAllStored);

        tabWidget->addTab(tab_1_all_stored, QString());
        tab_2_external = new QWidget();
        tab_2_external->setObjectName("tab_2_external");
        verticalLayout_3 = new QVBoxLayout(tab_2_external);
        verticalLayout_3->setObjectName("verticalLayout_3");
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        crntAcntLabel = new QLabel(tab_2_external);
        crntAcntLabel->setObjectName("crntAcntLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, crntAcntLabel);

        accountComboBox = new QComboBox(tab_2_external);
        accountComboBox->setObjectName("accountComboBox");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, accountComboBox);


        verticalLayout_3->addLayout(formLayout_2);

        line = new QFrame(tab_2_external);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        srcZfoLineEdit = new QLineEdit(tab_2_external);
        srcZfoLineEdit->setObjectName("srcZfoLineEdit");

        gridLayout->addWidget(srcZfoLineEdit, 1, 1, 1, 1);

        srcZfoLabel = new QLabel(tab_2_external);
        srcZfoLabel->setObjectName("srcZfoLabel");

        gridLayout->addWidget(srcZfoLabel, 1, 0, 1, 1);

        tgtZfoLineEdit = new QLineEdit(tab_2_external);
        tgtZfoLineEdit->setObjectName("tgtZfoLineEdit");

        gridLayout->addWidget(tgtZfoLineEdit, 3, 1, 1, 1);

        chooseSrcPushButton = new QPushButton(tab_2_external);
        chooseSrcPushButton->setObjectName("chooseSrcPushButton");

        gridLayout->addWidget(chooseSrcPushButton, 1, 2, 1, 1);

        chooseTgtPushButton = new QPushButton(tab_2_external);
        chooseTgtPushButton->setObjectName("chooseTgtPushButton");

        gridLayout->addWidget(chooseTgtPushButton, 3, 2, 1, 1);

        tgtZfoLabel = new QLabel(tab_2_external);
        tgtZfoLabel->setObjectName("tgtZfoLabel");

        gridLayout->addWidget(tgtZfoLabel, 3, 0, 1, 1);

        viewSrcPushButton = new QPushButton(tab_2_external);
        viewSrcPushButton->setObjectName("viewSrcPushButton");

        gridLayout->addWidget(viewSrcPushButton, 2, 2, 1, 1);

        restampExternalPushButton = new QPushButton(tab_2_external);
        restampExternalPushButton->setObjectName("restampExternalPushButton");

        gridLayout->addWidget(restampExternalPushButton, 4, 2, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        singleProgressBar = new QProgressBar(tab_2_external);
        singleProgressBar->setObjectName("singleProgressBar");
        singleProgressBar->setValue(24);

        verticalLayout_3->addWidget(singleProgressBar);

        singleResultLabel = new QLabel(tab_2_external);
        singleResultLabel->setObjectName("singleResultLabel");
        singleResultLabel->setAlignment(Qt::AlignCenter);
        singleResultLabel->setWordWrap(true);

        verticalLayout_3->addWidget(singleResultLabel);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);

        tabWidget->addTab(tab_2_external, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(DlgTimestampRenewal);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DlgTimestampRenewal);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgTimestampRenewal, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgTimestampRenewal, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgTimestampRenewal);
    } // setupUi

    void retranslateUi(QDialog *DlgTimestampRenewal)
    {
        DlgTimestampRenewal->setWindowTitle(QCoreApplication::translate("DlgTimestampRenewal", "Time Stamp Renewal", nullptr));
        actionTrackedTrackTimestampExpiration->setText(QCoreApplication::translate("DlgTimestampRenewal", "Track Timestamp Expiration", nullptr));
#if QT_CONFIG(tooltip)
        actionTrackedTrackTimestampExpiration->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Select whether to track the timestamp expiration of the message ZFO.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionTrackedSelectedRestamp->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp", nullptr));
#if QT_CONFIG(tooltip)
        actionTrackedSelectedRestamp->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Renew the message's timestamp.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAllStoredTrackTimestampExpiration->setText(QCoreApplication::translate("DlgTimestampRenewal", "Track Timestamp Expiration", nullptr));
#if QT_CONFIG(tooltip)
        actionAllStoredTrackTimestampExpiration->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Select whether to track the timestamp expiration of the message ZFO.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionAllStoredSelectedRestamp->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp", nullptr));
#if QT_CONFIG(tooltip)
        actionAllStoredSelectedRestamp->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Renew the message's timestamp.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        restampSelectedTrackedPushButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Renew selected message's timestamp.", nullptr));
#endif // QT_CONFIG(tooltip)
        restampSelectedTrackedPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp", nullptr));
#if QT_CONFIG(tooltip)
        trackedMessagesFilterLine->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Enter sought expression.", nullptr));
#endif // QT_CONFIG(tooltip)
        trackedMessagesFilterLine->setPlaceholderText(QCoreApplication::translate("DlgTimestampRenewal", "Type to filter.", nullptr));
#if QT_CONFIG(tooltip)
        closeTrackedFilterButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Hide filter field.", nullptr));
#endif // QT_CONFIG(tooltip)
        closeTrackedFilterButton->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_0_tracked), QCoreApplication::translate("DlgTimestampRenewal", "Tracked", nullptr));
        accountGroupBox->setTitle(QCoreApplication::translate("DlgTimestampRenewal", "Which data boxes do you want to scan?", nullptr));
#if QT_CONFIG(tooltip)
        allCheckBox->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Select all listed data boxes.", nullptr));
#endif // QT_CONFIG(tooltip)
        allCheckBox->setText(QCoreApplication::translate("DlgTimestampRenewal", "Select all", nullptr));
#if QT_CONFIG(tooltip)
        allStoredScanPushButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Scan selected data boxes for timestamp expiration dates of stored messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        allStoredScanPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Scan Expiring", nullptr));
#if QT_CONFIG(tooltip)
        allStoredScanInterruptPushButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Interrupt timestamp expiration scan.", nullptr));
#endif // QT_CONFIG(tooltip)
        allStoredScanInterruptPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Stop Scan", nullptr));
        allStoredGroupBox->setTitle(QCoreApplication::translate("DlgTimestampRenewal", "Which messages do you want to restamp?", nullptr));
#if QT_CONFIG(tooltip)
        onlyRestampableCheckBox->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "List only messages suitable for restamping.", nullptr));
#endif // QT_CONFIG(tooltip)
        onlyRestampableCheckBox->setText(QCoreApplication::translate("DlgTimestampRenewal", "Can be restamped", nullptr));
#if QT_CONFIG(tooltip)
        restampSelectedAllStoredPushButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Renew selected message's timestamp.", nullptr));
#endif // QT_CONFIG(tooltip)
        restampSelectedAllStoredPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp", nullptr));
#if QT_CONFIG(tooltip)
        restampListedAllStoredPushButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Renew listed message's timestamp.", nullptr));
#endif // QT_CONFIG(tooltip)
        restampListedAllStoredPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp Listed", nullptr));
#if QT_CONFIG(tooltip)
        allMessagesFilterLine->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Enter sought expression.", nullptr));
#endif // QT_CONFIG(tooltip)
        allMessagesFilterLine->setPlaceholderText(QCoreApplication::translate("DlgTimestampRenewal", "Type to filter.", nullptr));
#if QT_CONFIG(tooltip)
        closeAllFilterButton->setToolTip(QCoreApplication::translate("DlgTimestampRenewal", "Hide filter field.", nullptr));
#endif // QT_CONFIG(tooltip)
        closeAllFilterButton->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_1_all_stored), QCoreApplication::translate("DlgTimestampRenewal", "All Stored", nullptr));
        crntAcntLabel->setText(QCoreApplication::translate("DlgTimestampRenewal", "Data Box:", nullptr));
        srcZfoLabel->setText(QCoreApplication::translate("DlgTimestampRenewal", "Source ZFO:", nullptr));
        chooseSrcPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Choose ZFO", nullptr));
        chooseTgtPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Choose Target", nullptr));
        tgtZfoLabel->setText(QCoreApplication::translate("DlgTimestampRenewal", "Target ZFO:", nullptr));
        viewSrcPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "View Source", nullptr));
        restampExternalPushButton->setText(QCoreApplication::translate("DlgTimestampRenewal", "Restamp", nullptr));
        singleResultLabel->setText(QCoreApplication::translate("DlgTimestampRenewal", "n/a", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2_external), QCoreApplication::translate("DlgTimestampRenewal", "External", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgTimestampRenewal: public Ui_DlgTimestampRenewal {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_TIMESTAMP_RENEWAL_H
