/********************************************************************************
** Form generated from reading UI file 'dlg_preferences.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_PREFERENCES_H
#define UI_DLG_PREFERENCES_H

#include <QtCore/QLocale>
#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DlgPreferences
{
public:
    QVBoxLayout *verticalLayout_12;
    QTabWidget *prefWidget;
    QWidget *tab_0_download;
    QVBoxLayout *verticalLayout_8;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *downloadOnBackground;
    QHBoxLayout *horizontalLayout;
    QLabel *timerPreLabel;
    QSpinBox *timerSpinBox;
    QLabel *timerPostLabel;
    QSpacerItem *horizontalSpacer;
    QCheckBox *autoDownloadWholeMessages;
    QCheckBox *downloadAtStart;
    QCheckBox *downloadAfterSending;
    QGroupBox *groupBox_7;
    QVBoxLayout *verticalLayout_15;
    QHBoxLayout *horizontalLayout_4;
    QLabel *timeoutMinPreLabel;
    QSpinBox *timeoutMinSpinBox;
    QLabel *timeoutMinPostLabel;
    QSpacerItem *horizontalSpacer_4;
    QLabel *labelTimeoutNote;
    QGroupBox *enable_mark_msg_as_read;
    QVBoxLayout *verticalLayout_20;
    QHBoxLayout *horizontalLayout_11;
    QLabel *timeoutMarkMsgPreLabel;
    QSpinBox *timeoutMarkMsgSpinBox;
    QLabel *timeoutMarkMsgPostLabel;
    QSpacerItem *horizontalSpacer_6;
    QLabel *noteMarkMsgLabel;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QCheckBox *checkNewVersions;
    QSpacerItem *verticalSpacer_5;
    QWidget *tab_1_security;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_11;
    QLabel *storageOptionsLabel;
    QCheckBox *storeMessagesOnDisk;
    QCheckBox *storeAdditionalDataOnDisk;
    QLabel *storageNotificationLabel;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout_9;
    QRadioButton *certValidationDateNow;
    QRadioButton *certValidationDateDownload;
    QFrame *line;
    QCheckBox *checkCrl;
    QGroupBox *groupBox_10;
    QVBoxLayout *verticalLayout_26;
    QHBoxLayout *horizontalLayout_21;
    QLabel *timestampExpirPre;
    QSpinBox *timestampExpirSpinBox;
    QLabel *timestampExpirPost;
    QSpacerItem *horizontalSpacer_7;
    QLabel *timestampAutomaticTrackingLabel;
    QFormLayout *formLayout_01;
    QCheckBox *timestampTrackReceivedCheckBox;
    QCheckBox *timestampTrackSentCheckBox;
    QGroupBox *groupBox_11;
    QVBoxLayout *verticalLayout_21;
    QLabel *pinSettingsLabel;
    QHBoxLayout *horizontalLayout_13;
    QPushButton *setPinButton;
    QPushButton *changePinButton;
    QPushButton *clearPinButton;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *hspacer;
    QWidget *tab_2_navigation;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_10;
    QRadioButton *afterStartSelectNewest;
    QRadioButton *afterStartSelectLast;
    QRadioButton *afterStartSelectNothing;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_3_interface;
    QVBoxLayout *verticalLayout_19;
    QGroupBox *appThemeVerticalGroupBox;
    QVBoxLayout *applicationMode;
    QHBoxLayout *horizontalLayout_18;
    QLabel *appThemeLabel;
    QComboBox *appThemeComboBox;
    QSpacerItem *horizontalSpacer_5;
    QGroupBox *fontSizeGroupBox;
    QVBoxLayout *verticalLayout_24;
    QHBoxLayout *horizontalLayout_17;
    QLabel *fontScalePre;
    QSpinBox *fontScaleSpinBox;
    QLabel *fontScalePost;
    QSpacerItem *horizontalSpacer_10;
    QLabel *fontScaleNotificationLabel;
    QGroupBox *messageHighlighting;
    QVBoxLayout *verticalLayout_22;
    QHBoxLayout *horizontalLayout_14;
    QLabel *messageToBeDeletedPre;
    QSpinBox *messageToBeDeletedSpinBox;
    QLabel *messageToBeDeletedPost;
    QPushButton *messageToBeDeletedColour;
    QPushButton *changeMessageToBeDeletedColourButton;
    QSpacerItem *horizontalSpacer_9;
    QLabel *labelToBeDeletedHighlightNote;
    QSpacerItem *verticalSpacer_6;
    QWidget *tab_4_directories;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *enableGlobalPaths;
    QVBoxLayout *verticalLayout_14;
    QLabel *savePathLabel;
    QHBoxLayout *horizontalLayout_2;
    QLabel *savePath;
    QPushButton *savePathPushButton;
    QSpacerItem *horizontalSpacer_2;
    QLabel *addFilePathLabel;
    QHBoxLayout *horizontalLayout_3;
    QLabel *addFilePath;
    QPushButton *addFilePathPushButton;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_4;
    QWidget *tab_5_saving;
    QVBoxLayout *verticalLayout_13;
    QGroupBox *groupBox_8;
    QVBoxLayout *verticalLayout_16;
    QFormLayout *formLayout_02;
    QCheckBox *allAttachmentsSaveZfoMsg;
    QCheckBox *allAttachmentsSavePdfMsgEnvel;
    QCheckBox *allAttachmentsSaveZfoDelInfo;
    QCheckBox *allAttachmentsSavePdfDelInfo;
    QGroupBox *groupBox_9;
    QVBoxLayout *verticalLayout_17;
    QFormLayout *formLayout_03;
    QLabel *msgFileNameLabel;
    QHBoxLayout *horizontalLayout_8;
    QLineEdit *msgFileNameFmt;
    QLabel *suffLabel_1;
    QLabel *delInfoFileNameLabel;
    QLabel *attachFileNameLabel;
    QHBoxLayout *horizontalLayout_9;
    QLineEdit *delInfoFileNameFmt;
    QLabel *suffLabel_2;
    QHBoxLayout *horizontalLayout_10;
    QLineEdit *attachFileNameFmt;
    QCheckBox *delInfoForEveryFile;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_42;
    QLineEdit *allAttachDelInfoFileNameFmt;
    QLabel *characterReplacementNotificationLabel;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *showParamListButton;
    QPushButton *closeParamListButton;
    QGroupBox *enableBackupNotifications;
    QVBoxLayout *verticalLayout_25;
    QHBoxLayout *horizontalLayout_19;
    QLabel *backupMonthIntervalPreLabel;
    QSpinBox *backupMonthIntervalSpinBox;
    QLabel *backupMonthIntervalPostLabel;
    QSpacerItem *horizontalSpacer_11;
    QLabel *backupPathLabel;
    QHBoxLayout *horizontalLayout_20;
    QLabel *backupPath;
    QPushButton *backupPathPushButton;
    QSpacerItem *horizontalSpacer_12;
    QSpacerItem *verticalSpacer_3;
    QWidget *tab_6_language;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBox_6;
    QVBoxLayout *verticalLayout_3;
    QComboBox *langComboBox;
    QLabel *langNotificationLabel;
    QSpacerItem *verticalSpacer;
    QWidget *tab_7_data_collection;
    QVBoxLayout *verticalLayout_18;
    QGroupBox *groupBox_12;
    QVBoxLayout *verticalLayout_23;
    QLabel *dataCollectionDescriptionLabel;
    QHBoxLayout *horizontalLayout_6;
    QLabel *collectBuildReportLabel;
    QToolButton *buildReportToolButton;
    QHBoxLayout *horizontalLayout_15;
    QLabel *collectConfReportLabel;
    QToolButton *confReportToolButton;
    QLabel *optionalDataCollectionDescriptionLabel;
    QHBoxLayout *horizontalLayout_16;
    QCheckBox *collectUsageReport;
    QToolButton *usageReportToolButton;
    QSpacerItem *verticalSpacer_7;
    QDialogButtonBox *prefButtonBox;

    void setupUi(QDialog *DlgPreferences)
    {
        if (DlgPreferences->objectName().isEmpty())
            DlgPreferences->setObjectName("DlgPreferences");
        DlgPreferences->resize(680, 650);
        DlgPreferences->setMinimumSize(QSize(640, 520));
        DlgPreferences->setLocale(QLocale(QLocale::English, QLocale::UnitedKingdom));
        verticalLayout_12 = new QVBoxLayout(DlgPreferences);
        verticalLayout_12->setObjectName("verticalLayout_12");
        prefWidget = new QTabWidget(DlgPreferences);
        prefWidget->setObjectName("prefWidget");
        prefWidget->setElideMode(Qt::ElideNone);
        prefWidget->setDocumentMode(false);
        prefWidget->setTabsClosable(false);
        tab_0_download = new QWidget();
        tab_0_download->setObjectName("tab_0_download");
        verticalLayout_8 = new QVBoxLayout(tab_0_download);
        verticalLayout_8->setObjectName("verticalLayout_8");
        groupBox = new QGroupBox(tab_0_download);
        groupBox->setObjectName("groupBox");
        groupBox->setEnabled(true);
        groupBox->setBaseSize(QSize(0, 0));
        QFont font;
        font.setBold(true);
        groupBox->setFont(font);
        groupBox->setAutoFillBackground(false);
        groupBox->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        groupBox->setFlat(false);
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        downloadOnBackground = new QCheckBox(groupBox);
        downloadOnBackground->setObjectName("downloadOnBackground");
        QFont font1;
        font1.setBold(false);
        downloadOnBackground->setFont(font1);

        verticalLayout_2->addWidget(downloadOnBackground);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        timerPreLabel = new QLabel(groupBox);
        timerPreLabel->setObjectName("timerPreLabel");
        timerPreLabel->setFont(font1);

        horizontalLayout->addWidget(timerPreLabel);

        timerSpinBox = new QSpinBox(groupBox);
        timerSpinBox->setObjectName("timerSpinBox");
        timerSpinBox->setFont(font1);
        timerSpinBox->setMinimum(5);
        timerSpinBox->setMaximum(560);
        timerSpinBox->setSingleStep(5);

        horizontalLayout->addWidget(timerSpinBox);

        timerPostLabel = new QLabel(groupBox);
        timerPostLabel->setObjectName("timerPostLabel");
        timerPostLabel->setFont(font1);

        horizontalLayout->addWidget(timerPostLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        autoDownloadWholeMessages = new QCheckBox(groupBox);
        autoDownloadWholeMessages->setObjectName("autoDownloadWholeMessages");
        autoDownloadWholeMessages->setFont(font1);

        verticalLayout_2->addWidget(autoDownloadWholeMessages);

        downloadAtStart = new QCheckBox(groupBox);
        downloadAtStart->setObjectName("downloadAtStart");
        downloadAtStart->setFont(font1);

        verticalLayout_2->addWidget(downloadAtStart);

        downloadAfterSending = new QCheckBox(groupBox);
        downloadAfterSending->setObjectName("downloadAfterSending");
        downloadAfterSending->setFont(font1);

        verticalLayout_2->addWidget(downloadAfterSending);


        verticalLayout_8->addWidget(groupBox);

        groupBox_7 = new QGroupBox(tab_0_download);
        groupBox_7->setObjectName("groupBox_7");
        groupBox_7->setFont(font);
        verticalLayout_15 = new QVBoxLayout(groupBox_7);
        verticalLayout_15->setObjectName("verticalLayout_15");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        timeoutMinPreLabel = new QLabel(groupBox_7);
        timeoutMinPreLabel->setObjectName("timeoutMinPreLabel");
        timeoutMinPreLabel->setFont(font1);

        horizontalLayout_4->addWidget(timeoutMinPreLabel);

        timeoutMinSpinBox = new QSpinBox(groupBox_7);
        timeoutMinSpinBox->setObjectName("timeoutMinSpinBox");
        timeoutMinSpinBox->setFont(font1);
        timeoutMinSpinBox->setMaximum(120);
        timeoutMinSpinBox->setValue(15);

        horizontalLayout_4->addWidget(timeoutMinSpinBox);

        timeoutMinPostLabel = new QLabel(groupBox_7);
        timeoutMinPostLabel->setObjectName("timeoutMinPostLabel");
        timeoutMinPostLabel->setFont(font1);

        horizontalLayout_4->addWidget(timeoutMinPostLabel);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);


        verticalLayout_15->addLayout(horizontalLayout_4);

        labelTimeoutNote = new QLabel(groupBox_7);
        labelTimeoutNote->setObjectName("labelTimeoutNote");
        QFont font2;
        font2.setBold(false);
        font2.setItalic(true);
        labelTimeoutNote->setFont(font2);
        labelTimeoutNote->setText(QString::fromUtf8(""));
        labelTimeoutNote->setWordWrap(true);

        verticalLayout_15->addWidget(labelTimeoutNote);


        verticalLayout_8->addWidget(groupBox_7);

        enable_mark_msg_as_read = new QGroupBox(tab_0_download);
        enable_mark_msg_as_read->setObjectName("enable_mark_msg_as_read");
        enable_mark_msg_as_read->setFont(font);
        enable_mark_msg_as_read->setCheckable(false);
        verticalLayout_20 = new QVBoxLayout(enable_mark_msg_as_read);
        verticalLayout_20->setObjectName("verticalLayout_20");
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        timeoutMarkMsgPreLabel = new QLabel(enable_mark_msg_as_read);
        timeoutMarkMsgPreLabel->setObjectName("timeoutMarkMsgPreLabel");
        timeoutMarkMsgPreLabel->setFont(font1);

        horizontalLayout_11->addWidget(timeoutMarkMsgPreLabel);

        timeoutMarkMsgSpinBox = new QSpinBox(enable_mark_msg_as_read);
        timeoutMarkMsgSpinBox->setObjectName("timeoutMarkMsgSpinBox");
        timeoutMarkMsgSpinBox->setFont(font1);
        timeoutMarkMsgSpinBox->setMinimum(-1);
        timeoutMarkMsgSpinBox->setMaximum(30);
        timeoutMarkMsgSpinBox->setValue(5);

        horizontalLayout_11->addWidget(timeoutMarkMsgSpinBox);

        timeoutMarkMsgPostLabel = new QLabel(enable_mark_msg_as_read);
        timeoutMarkMsgPostLabel->setObjectName("timeoutMarkMsgPostLabel");
        timeoutMarkMsgPostLabel->setFont(font1);

        horizontalLayout_11->addWidget(timeoutMarkMsgPostLabel);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_6);


        verticalLayout_20->addLayout(horizontalLayout_11);

        noteMarkMsgLabel = new QLabel(enable_mark_msg_as_read);
        noteMarkMsgLabel->setObjectName("noteMarkMsgLabel");
        noteMarkMsgLabel->setFont(font2);
        noteMarkMsgLabel->setWordWrap(true);

        verticalLayout_20->addWidget(noteMarkMsgLabel);


        verticalLayout_8->addWidget(enable_mark_msg_as_read);

        groupBox_2 = new QGroupBox(tab_0_download);
        groupBox_2->setObjectName("groupBox_2");
        groupBox_2->setFont(font);
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        checkNewVersions = new QCheckBox(groupBox_2);
        checkNewVersions->setObjectName("checkNewVersions");
        checkNewVersions->setFont(font1);
        checkNewVersions->setChecked(true);

        verticalLayout->addWidget(checkNewVersions);


        verticalLayout_8->addWidget(groupBox_2);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_5);

        prefWidget->addTab(tab_0_download, QString());
        tab_1_security = new QWidget();
        tab_1_security->setObjectName("tab_1_security");
        verticalLayout_7 = new QVBoxLayout(tab_1_security);
        verticalLayout_7->setObjectName("verticalLayout_7");
        groupBox_3 = new QGroupBox(tab_1_security);
        groupBox_3->setObjectName("groupBox_3");
        groupBox_3->setFont(font);
        verticalLayout_11 = new QVBoxLayout(groupBox_3);
        verticalLayout_11->setObjectName("verticalLayout_11");
        storageOptionsLabel = new QLabel(groupBox_3);
        storageOptionsLabel->setObjectName("storageOptionsLabel");
        storageOptionsLabel->setFont(font2);
        storageOptionsLabel->setWordWrap(true);

        verticalLayout_11->addWidget(storageOptionsLabel);

        storeMessagesOnDisk = new QCheckBox(groupBox_3);
        storeMessagesOnDisk->setObjectName("storeMessagesOnDisk");
        storeMessagesOnDisk->setFont(font1);
        storeMessagesOnDisk->setChecked(true);

        verticalLayout_11->addWidget(storeMessagesOnDisk);

        storeAdditionalDataOnDisk = new QCheckBox(groupBox_3);
        storeAdditionalDataOnDisk->setObjectName("storeAdditionalDataOnDisk");
        storeAdditionalDataOnDisk->setFont(font1);
        storeAdditionalDataOnDisk->setChecked(true);

        verticalLayout_11->addWidget(storeAdditionalDataOnDisk);

        storageNotificationLabel = new QLabel(groupBox_3);
        storageNotificationLabel->setObjectName("storageNotificationLabel");
        storageNotificationLabel->setFont(font2);

        verticalLayout_11->addWidget(storageNotificationLabel);


        verticalLayout_7->addWidget(groupBox_3);

        groupBox_5 = new QGroupBox(tab_1_security);
        groupBox_5->setObjectName("groupBox_5");
        groupBox_5->setFont(font);
        verticalLayout_9 = new QVBoxLayout(groupBox_5);
        verticalLayout_9->setObjectName("verticalLayout_9");
        certValidationDateNow = new QRadioButton(groupBox_5);
        certValidationDateNow->setObjectName("certValidationDateNow");
        certValidationDateNow->setFont(font1);
        certValidationDateNow->setChecked(true);

        verticalLayout_9->addWidget(certValidationDateNow);

        certValidationDateDownload = new QRadioButton(groupBox_5);
        certValidationDateDownload->setObjectName("certValidationDateDownload");
        certValidationDateDownload->setFont(font1);

        verticalLayout_9->addWidget(certValidationDateDownload);

        line = new QFrame(groupBox_5);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_9->addWidget(line);

        checkCrl = new QCheckBox(groupBox_5);
        checkCrl->setObjectName("checkCrl");
        checkCrl->setFont(font1);
        checkCrl->setChecked(true);

        verticalLayout_9->addWidget(checkCrl);


        verticalLayout_7->addWidget(groupBox_5);

        groupBox_10 = new QGroupBox(tab_1_security);
        groupBox_10->setObjectName("groupBox_10");
        groupBox_10->setFont(font);
        verticalLayout_26 = new QVBoxLayout(groupBox_10);
        verticalLayout_26->setObjectName("verticalLayout_26");
        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName("horizontalLayout_21");
        timestampExpirPre = new QLabel(groupBox_10);
        timestampExpirPre->setObjectName("timestampExpirPre");
        timestampExpirPre->setFont(font1);

        horizontalLayout_21->addWidget(timestampExpirPre);

        timestampExpirSpinBox = new QSpinBox(groupBox_10);
        timestampExpirSpinBox->setObjectName("timestampExpirSpinBox");
        timestampExpirSpinBox->setFont(font1);
        timestampExpirSpinBox->setReadOnly(false);
        timestampExpirSpinBox->setMinimum(1);
        timestampExpirSpinBox->setMaximum(999);
        timestampExpirSpinBox->setValue(15);

        horizontalLayout_21->addWidget(timestampExpirSpinBox);

        timestampExpirPost = new QLabel(groupBox_10);
        timestampExpirPost->setObjectName("timestampExpirPost");
        timestampExpirPost->setFont(font1);

        horizontalLayout_21->addWidget(timestampExpirPost);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_7);


        verticalLayout_26->addLayout(horizontalLayout_21);

        timestampAutomaticTrackingLabel = new QLabel(groupBox_10);
        timestampAutomaticTrackingLabel->setObjectName("timestampAutomaticTrackingLabel");
        timestampAutomaticTrackingLabel->setFont(font1);

        verticalLayout_26->addWidget(timestampAutomaticTrackingLabel);

        formLayout_01 = new QFormLayout();
        formLayout_01->setObjectName("formLayout_01");
        formLayout_01->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        timestampTrackReceivedCheckBox = new QCheckBox(groupBox_10);
        timestampTrackReceivedCheckBox->setObjectName("timestampTrackReceivedCheckBox");
        timestampTrackReceivedCheckBox->setFont(font1);

        formLayout_01->setWidget(0, QFormLayout::ItemRole::LabelRole, timestampTrackReceivedCheckBox);

        timestampTrackSentCheckBox = new QCheckBox(groupBox_10);
        timestampTrackSentCheckBox->setObjectName("timestampTrackSentCheckBox");
        timestampTrackSentCheckBox->setFont(font1);

        formLayout_01->setWidget(0, QFormLayout::ItemRole::FieldRole, timestampTrackSentCheckBox);


        verticalLayout_26->addLayout(formLayout_01);


        verticalLayout_7->addWidget(groupBox_10);

        groupBox_11 = new QGroupBox(tab_1_security);
        groupBox_11->setObjectName("groupBox_11");
        groupBox_11->setFont(font);
        verticalLayout_21 = new QVBoxLayout(groupBox_11);
        verticalLayout_21->setObjectName("verticalLayout_21");
        pinSettingsLabel = new QLabel(groupBox_11);
        pinSettingsLabel->setObjectName("pinSettingsLabel");
        pinSettingsLabel->setFont(font2);
        pinSettingsLabel->setWordWrap(true);

        verticalLayout_21->addWidget(pinSettingsLabel);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        setPinButton = new QPushButton(groupBox_11);
        setPinButton->setObjectName("setPinButton");
        setPinButton->setFont(font1);

        horizontalLayout_13->addWidget(setPinButton);

        changePinButton = new QPushButton(groupBox_11);
        changePinButton->setObjectName("changePinButton");
        changePinButton->setFont(font1);

        horizontalLayout_13->addWidget(changePinButton);

        clearPinButton = new QPushButton(groupBox_11);
        clearPinButton->setObjectName("clearPinButton");
        clearPinButton->setFont(font1);

        horizontalLayout_13->addWidget(clearPinButton);

        horizontalSpacer_8 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_8);


        verticalLayout_21->addLayout(horizontalLayout_13);


        verticalLayout_7->addWidget(groupBox_11);

        hspacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(hspacer);

        prefWidget->addTab(tab_1_security, QString());
        tab_2_navigation = new QWidget();
        tab_2_navigation->setObjectName("tab_2_navigation");
        verticalLayout_6 = new QVBoxLayout(tab_2_navigation);
        verticalLayout_6->setObjectName("verticalLayout_6");
        groupBox_4 = new QGroupBox(tab_2_navigation);
        groupBox_4->setObjectName("groupBox_4");
        groupBox_4->setFont(font);
        verticalLayout_10 = new QVBoxLayout(groupBox_4);
        verticalLayout_10->setObjectName("verticalLayout_10");
        afterStartSelectNewest = new QRadioButton(groupBox_4);
        afterStartSelectNewest->setObjectName("afterStartSelectNewest");
        afterStartSelectNewest->setFont(font1);
        afterStartSelectNewest->setChecked(false);

        verticalLayout_10->addWidget(afterStartSelectNewest);

        afterStartSelectLast = new QRadioButton(groupBox_4);
        afterStartSelectLast->setObjectName("afterStartSelectLast");
        afterStartSelectLast->setFont(font1);

        verticalLayout_10->addWidget(afterStartSelectLast);

        afterStartSelectNothing = new QRadioButton(groupBox_4);
        afterStartSelectNothing->setObjectName("afterStartSelectNothing");
        afterStartSelectNothing->setFont(font1);
        afterStartSelectNothing->setChecked(true);

        verticalLayout_10->addWidget(afterStartSelectNothing);


        verticalLayout_6->addWidget(groupBox_4);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_2);

        prefWidget->addTab(tab_2_navigation, QString());
        tab_3_interface = new QWidget();
        tab_3_interface->setObjectName("tab_3_interface");
        verticalLayout_19 = new QVBoxLayout(tab_3_interface);
        verticalLayout_19->setObjectName("verticalLayout_19");
        appThemeVerticalGroupBox = new QGroupBox(tab_3_interface);
        appThemeVerticalGroupBox->setObjectName("appThemeVerticalGroupBox");
        appThemeVerticalGroupBox->setFont(font);
        applicationMode = new QVBoxLayout(appThemeVerticalGroupBox);
        applicationMode->setObjectName("applicationMode");
        applicationMode->setContentsMargins(9, 9, 9, 9);
        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        appThemeLabel = new QLabel(appThemeVerticalGroupBox);
        appThemeLabel->setObjectName("appThemeLabel");
        appThemeLabel->setFont(font1);

        horizontalLayout_18->addWidget(appThemeLabel);

        appThemeComboBox = new QComboBox(appThemeVerticalGroupBox);
        appThemeComboBox->setObjectName("appThemeComboBox");
        appThemeComboBox->setFont(font1);

        horizontalLayout_18->addWidget(appThemeComboBox);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_5);


        applicationMode->addLayout(horizontalLayout_18);


        verticalLayout_19->addWidget(appThemeVerticalGroupBox);

        fontSizeGroupBox = new QGroupBox(tab_3_interface);
        fontSizeGroupBox->setObjectName("fontSizeGroupBox");
        fontSizeGroupBox->setFont(font);
        verticalLayout_24 = new QVBoxLayout(fontSizeGroupBox);
        verticalLayout_24->setObjectName("verticalLayout_24");
        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        fontScalePre = new QLabel(fontSizeGroupBox);
        fontScalePre->setObjectName("fontScalePre");
        fontScalePre->setFont(font1);

        horizontalLayout_17->addWidget(fontScalePre);

        fontScaleSpinBox = new QSpinBox(fontSizeGroupBox);
        fontScaleSpinBox->setObjectName("fontScaleSpinBox");
        fontScaleSpinBox->setFont(font1);
        fontScaleSpinBox->setMinimum(80);
        fontScaleSpinBox->setMaximum(250);
        fontScaleSpinBox->setSingleStep(10);
        fontScaleSpinBox->setValue(100);

        horizontalLayout_17->addWidget(fontScaleSpinBox);

        fontScalePost = new QLabel(fontSizeGroupBox);
        fontScalePost->setObjectName("fontScalePost");
        fontScalePost->setFont(font1);
        fontScalePost->setText(QString::fromUtf8("%"));

        horizontalLayout_17->addWidget(fontScalePost);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_10);


        verticalLayout_24->addLayout(horizontalLayout_17);

        fontScaleNotificationLabel = new QLabel(fontSizeGroupBox);
        fontScaleNotificationLabel->setObjectName("fontScaleNotificationLabel");
        fontScaleNotificationLabel->setFont(font2);

        verticalLayout_24->addWidget(fontScaleNotificationLabel);


        verticalLayout_19->addWidget(fontSizeGroupBox);

        messageHighlighting = new QGroupBox(tab_3_interface);
        messageHighlighting->setObjectName("messageHighlighting");
        messageHighlighting->setFont(font);
        verticalLayout_22 = new QVBoxLayout(messageHighlighting);
        verticalLayout_22->setObjectName("verticalLayout_22");
        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        messageToBeDeletedPre = new QLabel(messageHighlighting);
        messageToBeDeletedPre->setObjectName("messageToBeDeletedPre");
        messageToBeDeletedPre->setFont(font1);

        horizontalLayout_14->addWidget(messageToBeDeletedPre);

        messageToBeDeletedSpinBox = new QSpinBox(messageHighlighting);
        messageToBeDeletedSpinBox->setObjectName("messageToBeDeletedSpinBox");
        messageToBeDeletedSpinBox->setFont(font1);
        messageToBeDeletedSpinBox->setMaximum(30);

        horizontalLayout_14->addWidget(messageToBeDeletedSpinBox);

        messageToBeDeletedPost = new QLabel(messageHighlighting);
        messageToBeDeletedPost->setObjectName("messageToBeDeletedPost");
        messageToBeDeletedPost->setFont(font1);

        horizontalLayout_14->addWidget(messageToBeDeletedPost);

        messageToBeDeletedColour = new QPushButton(messageHighlighting);
        messageToBeDeletedColour->setObjectName("messageToBeDeletedColour");
        messageToBeDeletedColour->setFont(font1);

        horizontalLayout_14->addWidget(messageToBeDeletedColour);

        changeMessageToBeDeletedColourButton = new QPushButton(messageHighlighting);
        changeMessageToBeDeletedColourButton->setObjectName("changeMessageToBeDeletedColourButton");
        changeMessageToBeDeletedColourButton->setFont(font1);

        horizontalLayout_14->addWidget(changeMessageToBeDeletedColourButton);

        horizontalSpacer_9 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_9);


        verticalLayout_22->addLayout(horizontalLayout_14);

        labelToBeDeletedHighlightNote = new QLabel(messageHighlighting);
        labelToBeDeletedHighlightNote->setObjectName("labelToBeDeletedHighlightNote");
        labelToBeDeletedHighlightNote->setFont(font2);
        labelToBeDeletedHighlightNote->setWordWrap(true);

        verticalLayout_22->addWidget(labelToBeDeletedHighlightNote);


        verticalLayout_19->addWidget(messageHighlighting);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_19->addItem(verticalSpacer_6);

        prefWidget->addTab(tab_3_interface, QString());
        tab_4_directories = new QWidget();
        tab_4_directories->setObjectName("tab_4_directories");
        verticalLayout_5 = new QVBoxLayout(tab_4_directories);
        verticalLayout_5->setObjectName("verticalLayout_5");
        enableGlobalPaths = new QGroupBox(tab_4_directories);
        enableGlobalPaths->setObjectName("enableGlobalPaths");
        enableGlobalPaths->setEnabled(true);
        enableGlobalPaths->setFont(font);
        enableGlobalPaths->setCheckable(true);
        enableGlobalPaths->setChecked(false);
        verticalLayout_14 = new QVBoxLayout(enableGlobalPaths);
        verticalLayout_14->setObjectName("verticalLayout_14");
        savePathLabel = new QLabel(enableGlobalPaths);
        savePathLabel->setObjectName("savePathLabel");
        savePathLabel->setFont(font1);

        verticalLayout_14->addWidget(savePathLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        savePath = new QLabel(enableGlobalPaths);
        savePath->setObjectName("savePath");
        QFont font3;
        font3.setBold(true);
        font3.setItalic(true);
        savePath->setFont(font3);

        horizontalLayout_2->addWidget(savePath);

        savePathPushButton = new QPushButton(enableGlobalPaths);
        savePathPushButton->setObjectName("savePathPushButton");
        savePathPushButton->setFont(font1);

        horizontalLayout_2->addWidget(savePathPushButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_14->addLayout(horizontalLayout_2);

        addFilePathLabel = new QLabel(enableGlobalPaths);
        addFilePathLabel->setObjectName("addFilePathLabel");
        addFilePathLabel->setFont(font1);

        verticalLayout_14->addWidget(addFilePathLabel);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        addFilePath = new QLabel(enableGlobalPaths);
        addFilePath->setObjectName("addFilePath");
        addFilePath->setFont(font3);

        horizontalLayout_3->addWidget(addFilePath);

        addFilePathPushButton = new QPushButton(enableGlobalPaths);
        addFilePathPushButton->setObjectName("addFilePathPushButton");
        addFilePathPushButton->setFont(font1);

        horizontalLayout_3->addWidget(addFilePathPushButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_14->addLayout(horizontalLayout_3);


        verticalLayout_5->addWidget(enableGlobalPaths);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_4);

        prefWidget->addTab(tab_4_directories, QString());
        tab_5_saving = new QWidget();
        tab_5_saving->setObjectName("tab_5_saving");
        verticalLayout_13 = new QVBoxLayout(tab_5_saving);
        verticalLayout_13->setObjectName("verticalLayout_13");
        groupBox_8 = new QGroupBox(tab_5_saving);
        groupBox_8->setObjectName("groupBox_8");
        groupBox_8->setFont(font);
        verticalLayout_16 = new QVBoxLayout(groupBox_8);
        verticalLayout_16->setObjectName("verticalLayout_16");
        formLayout_02 = new QFormLayout();
        formLayout_02->setObjectName("formLayout_02");
        formLayout_02->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        allAttachmentsSaveZfoMsg = new QCheckBox(groupBox_8);
        allAttachmentsSaveZfoMsg->setObjectName("allAttachmentsSaveZfoMsg");
        allAttachmentsSaveZfoMsg->setFont(font1);

        formLayout_02->setWidget(0, QFormLayout::ItemRole::LabelRole, allAttachmentsSaveZfoMsg);

        allAttachmentsSavePdfMsgEnvel = new QCheckBox(groupBox_8);
        allAttachmentsSavePdfMsgEnvel->setObjectName("allAttachmentsSavePdfMsgEnvel");
        allAttachmentsSavePdfMsgEnvel->setFont(font1);

        formLayout_02->setWidget(0, QFormLayout::ItemRole::FieldRole, allAttachmentsSavePdfMsgEnvel);

        allAttachmentsSaveZfoDelInfo = new QCheckBox(groupBox_8);
        allAttachmentsSaveZfoDelInfo->setObjectName("allAttachmentsSaveZfoDelInfo");
        allAttachmentsSaveZfoDelInfo->setFont(font1);

        formLayout_02->setWidget(1, QFormLayout::ItemRole::LabelRole, allAttachmentsSaveZfoDelInfo);

        allAttachmentsSavePdfDelInfo = new QCheckBox(groupBox_8);
        allAttachmentsSavePdfDelInfo->setObjectName("allAttachmentsSavePdfDelInfo");
        allAttachmentsSavePdfDelInfo->setFont(font1);

        formLayout_02->setWidget(1, QFormLayout::ItemRole::FieldRole, allAttachmentsSavePdfDelInfo);


        verticalLayout_16->addLayout(formLayout_02);


        verticalLayout_13->addWidget(groupBox_8);

        groupBox_9 = new QGroupBox(tab_5_saving);
        groupBox_9->setObjectName("groupBox_9");
        groupBox_9->setFont(font);
        verticalLayout_17 = new QVBoxLayout(groupBox_9);
        verticalLayout_17->setObjectName("verticalLayout_17");
        formLayout_03 = new QFormLayout();
        formLayout_03->setObjectName("formLayout_03");
        formLayout_03->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_03->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        msgFileNameLabel = new QLabel(groupBox_9);
        msgFileNameLabel->setObjectName("msgFileNameLabel");
        msgFileNameLabel->setFont(font1);

        formLayout_03->setWidget(0, QFormLayout::ItemRole::LabelRole, msgFileNameLabel);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        msgFileNameFmt = new QLineEdit(groupBox_9);
        msgFileNameFmt->setObjectName("msgFileNameFmt");
        msgFileNameFmt->setFont(font1);

        horizontalLayout_8->addWidget(msgFileNameFmt);

        suffLabel_1 = new QLabel(groupBox_9);
        suffLabel_1->setObjectName("suffLabel_1");
        suffLabel_1->setFont(font1);
        suffLabel_1->setText(QString::fromUtf8(".zfo | .pdf"));

        horizontalLayout_8->addWidget(suffLabel_1);


        formLayout_03->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_8);

        delInfoFileNameLabel = new QLabel(groupBox_9);
        delInfoFileNameLabel->setObjectName("delInfoFileNameLabel");
        delInfoFileNameLabel->setFont(font1);

        formLayout_03->setWidget(1, QFormLayout::ItemRole::LabelRole, delInfoFileNameLabel);

        attachFileNameLabel = new QLabel(groupBox_9);
        attachFileNameLabel->setObjectName("attachFileNameLabel");
        attachFileNameLabel->setFont(font1);

        formLayout_03->setWidget(2, QFormLayout::ItemRole::LabelRole, attachFileNameLabel);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        delInfoFileNameFmt = new QLineEdit(groupBox_9);
        delInfoFileNameFmt->setObjectName("delInfoFileNameFmt");
        delInfoFileNameFmt->setFont(font1);

        horizontalLayout_9->addWidget(delInfoFileNameFmt);

        suffLabel_2 = new QLabel(groupBox_9);
        suffLabel_2->setObjectName("suffLabel_2");
        suffLabel_2->setFont(font1);
        suffLabel_2->setText(QString::fromUtf8(".zfo | .pdf"));

        horizontalLayout_9->addWidget(suffLabel_2);


        formLayout_03->setLayout(1, QFormLayout::ItemRole::FieldRole, horizontalLayout_9);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        attachFileNameFmt = new QLineEdit(groupBox_9);
        attachFileNameFmt->setObjectName("attachFileNameFmt");
        attachFileNameFmt->setFont(font1);

        horizontalLayout_10->addWidget(attachFileNameFmt);


        formLayout_03->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout_10);


        verticalLayout_17->addLayout(formLayout_03);

        delInfoForEveryFile = new QCheckBox(groupBox_9);
        delInfoForEveryFile->setObjectName("delInfoForEveryFile");
        delInfoForEveryFile->setFont(font1);

        verticalLayout_17->addWidget(delInfoForEveryFile);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label_42 = new QLabel(groupBox_9);
        label_42->setObjectName("label_42");
        label_42->setFont(font1);

        horizontalLayout_7->addWidget(label_42);

        allAttachDelInfoFileNameFmt = new QLineEdit(groupBox_9);
        allAttachDelInfoFileNameFmt->setObjectName("allAttachDelInfoFileNameFmt");
        allAttachDelInfoFileNameFmt->setEnabled(true);
        allAttachDelInfoFileNameFmt->setFont(font1);

        horizontalLayout_7->addWidget(allAttachDelInfoFileNameFmt);


        verticalLayout_17->addLayout(horizontalLayout_7);

        characterReplacementNotificationLabel = new QLabel(groupBox_9);
        characterReplacementNotificationLabel->setObjectName("characterReplacementNotificationLabel");
        characterReplacementNotificationLabel->setFont(font2);
        characterReplacementNotificationLabel->setAlignment(Qt::AlignCenter);
        characterReplacementNotificationLabel->setWordWrap(true);

        verticalLayout_17->addWidget(characterReplacementNotificationLabel);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        showParamListButton = new QPushButton(groupBox_9);
        showParamListButton->setObjectName("showParamListButton");
        showParamListButton->setFont(font1);

        horizontalLayout_5->addWidget(showParamListButton);

        closeParamListButton = new QPushButton(groupBox_9);
        closeParamListButton->setObjectName("closeParamListButton");
        closeParamListButton->setFont(font1);

        horizontalLayout_5->addWidget(closeParamListButton);


        verticalLayout_17->addLayout(horizontalLayout_5);


        verticalLayout_13->addWidget(groupBox_9);

        enableBackupNotifications = new QGroupBox(tab_5_saving);
        enableBackupNotifications->setObjectName("enableBackupNotifications");
        enableBackupNotifications->setFont(font);
        enableBackupNotifications->setCheckable(true);
        enableBackupNotifications->setChecked(false);
        verticalLayout_25 = new QVBoxLayout(enableBackupNotifications);
        verticalLayout_25->setObjectName("verticalLayout_25");
        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        backupMonthIntervalPreLabel = new QLabel(enableBackupNotifications);
        backupMonthIntervalPreLabel->setObjectName("backupMonthIntervalPreLabel");
        backupMonthIntervalPreLabel->setFont(font1);

        horizontalLayout_19->addWidget(backupMonthIntervalPreLabel);

        backupMonthIntervalSpinBox = new QSpinBox(enableBackupNotifications);
        backupMonthIntervalSpinBox->setObjectName("backupMonthIntervalSpinBox");
        backupMonthIntervalSpinBox->setFont(font1);
        backupMonthIntervalSpinBox->setMinimum(1);
        backupMonthIntervalSpinBox->setValue(3);

        horizontalLayout_19->addWidget(backupMonthIntervalSpinBox);

        backupMonthIntervalPostLabel = new QLabel(enableBackupNotifications);
        backupMonthIntervalPostLabel->setObjectName("backupMonthIntervalPostLabel");
        backupMonthIntervalPostLabel->setFont(font1);

        horizontalLayout_19->addWidget(backupMonthIntervalPostLabel);

        horizontalSpacer_11 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_11);


        verticalLayout_25->addLayout(horizontalLayout_19);

        backupPathLabel = new QLabel(enableBackupNotifications);
        backupPathLabel->setObjectName("backupPathLabel");
        backupPathLabel->setFont(font1);

        verticalLayout_25->addWidget(backupPathLabel);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        backupPath = new QLabel(enableBackupNotifications);
        backupPath->setObjectName("backupPath");

        horizontalLayout_20->addWidget(backupPath);

        backupPathPushButton = new QPushButton(enableBackupNotifications);
        backupPathPushButton->setObjectName("backupPathPushButton");
        backupPathPushButton->setFont(font1);

        horizontalLayout_20->addWidget(backupPathPushButton);

        horizontalSpacer_12 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_12);


        verticalLayout_25->addLayout(horizontalLayout_20);


        verticalLayout_13->addWidget(enableBackupNotifications);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_13->addItem(verticalSpacer_3);

        prefWidget->addTab(tab_5_saving, QString());
        tab_6_language = new QWidget();
        tab_6_language->setObjectName("tab_6_language");
        verticalLayout_4 = new QVBoxLayout(tab_6_language);
        verticalLayout_4->setObjectName("verticalLayout_4");
        groupBox_6 = new QGroupBox(tab_6_language);
        groupBox_6->setObjectName("groupBox_6");
        groupBox_6->setFont(font);
        verticalLayout_3 = new QVBoxLayout(groupBox_6);
        verticalLayout_3->setObjectName("verticalLayout_3");
        langComboBox = new QComboBox(groupBox_6);
        langComboBox->setObjectName("langComboBox");
        langComboBox->setFont(font1);

        verticalLayout_3->addWidget(langComboBox);

        langNotificationLabel = new QLabel(groupBox_6);
        langNotificationLabel->setObjectName("langNotificationLabel");
        langNotificationLabel->setFont(font2);

        verticalLayout_3->addWidget(langNotificationLabel);


        verticalLayout_4->addWidget(groupBox_6);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);

        prefWidget->addTab(tab_6_language, QString());
        tab_7_data_collection = new QWidget();
        tab_7_data_collection->setObjectName("tab_7_data_collection");
        verticalLayout_18 = new QVBoxLayout(tab_7_data_collection);
        verticalLayout_18->setObjectName("verticalLayout_18");
        groupBox_12 = new QGroupBox(tab_7_data_collection);
        groupBox_12->setObjectName("groupBox_12");
        groupBox_12->setFont(font);
        verticalLayout_23 = new QVBoxLayout(groupBox_12);
        verticalLayout_23->setObjectName("verticalLayout_23");
        dataCollectionDescriptionLabel = new QLabel(groupBox_12);
        dataCollectionDescriptionLabel->setObjectName("dataCollectionDescriptionLabel");
        dataCollectionDescriptionLabel->setFont(font1);
        dataCollectionDescriptionLabel->setWordWrap(true);

        verticalLayout_23->addWidget(dataCollectionDescriptionLabel);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        collectBuildReportLabel = new QLabel(groupBox_12);
        collectBuildReportLabel->setObjectName("collectBuildReportLabel");
        collectBuildReportLabel->setFont(font1);
        collectBuildReportLabel->setWordWrap(true);

        horizontalLayout_6->addWidget(collectBuildReportLabel);

        buildReportToolButton = new QToolButton(groupBox_12);
        buildReportToolButton->setObjectName("buildReportToolButton");

        horizontalLayout_6->addWidget(buildReportToolButton);


        verticalLayout_23->addLayout(horizontalLayout_6);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        collectConfReportLabel = new QLabel(groupBox_12);
        collectConfReportLabel->setObjectName("collectConfReportLabel");
        collectConfReportLabel->setFont(font1);
        collectConfReportLabel->setWordWrap(true);

        horizontalLayout_15->addWidget(collectConfReportLabel);

        confReportToolButton = new QToolButton(groupBox_12);
        confReportToolButton->setObjectName("confReportToolButton");

        horizontalLayout_15->addWidget(confReportToolButton);


        verticalLayout_23->addLayout(horizontalLayout_15);

        optionalDataCollectionDescriptionLabel = new QLabel(groupBox_12);
        optionalDataCollectionDescriptionLabel->setObjectName("optionalDataCollectionDescriptionLabel");
        optionalDataCollectionDescriptionLabel->setFont(font1);

        verticalLayout_23->addWidget(optionalDataCollectionDescriptionLabel);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        collectUsageReport = new QCheckBox(groupBox_12);
        collectUsageReport->setObjectName("collectUsageReport");
        collectUsageReport->setFont(font1);

        horizontalLayout_16->addWidget(collectUsageReport);

        usageReportToolButton = new QToolButton(groupBox_12);
        usageReportToolButton->setObjectName("usageReportToolButton");

        horizontalLayout_16->addWidget(usageReportToolButton);


        verticalLayout_23->addLayout(horizontalLayout_16);


        verticalLayout_18->addWidget(groupBox_12);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_18->addItem(verticalSpacer_7);

        prefWidget->addTab(tab_7_data_collection, QString());

        verticalLayout_12->addWidget(prefWidget);

        prefButtonBox = new QDialogButtonBox(DlgPreferences);
        prefButtonBox->setObjectName("prefButtonBox");
        prefButtonBox->setOrientation(Qt::Horizontal);
        prefButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_12->addWidget(prefButtonBox);

        QWidget::setTabOrder(prefWidget, downloadOnBackground);
        QWidget::setTabOrder(downloadOnBackground, timerSpinBox);
        QWidget::setTabOrder(timerSpinBox, autoDownloadWholeMessages);
        QWidget::setTabOrder(autoDownloadWholeMessages, downloadAtStart);
        QWidget::setTabOrder(downloadAtStart, downloadAfterSending);
        QWidget::setTabOrder(downloadAfterSending, timeoutMinSpinBox);
        QWidget::setTabOrder(timeoutMinSpinBox, timeoutMarkMsgSpinBox);
        QWidget::setTabOrder(timeoutMarkMsgSpinBox, checkNewVersions);
        QWidget::setTabOrder(checkNewVersions, storeMessagesOnDisk);
        QWidget::setTabOrder(storeMessagesOnDisk, storeAdditionalDataOnDisk);
        QWidget::setTabOrder(storeAdditionalDataOnDisk, certValidationDateNow);
        QWidget::setTabOrder(certValidationDateNow, certValidationDateDownload);
        QWidget::setTabOrder(certValidationDateDownload, checkCrl);
        QWidget::setTabOrder(checkCrl, timestampExpirSpinBox);
        QWidget::setTabOrder(timestampExpirSpinBox, timestampTrackReceivedCheckBox);
        QWidget::setTabOrder(timestampTrackReceivedCheckBox, timestampTrackSentCheckBox);
        QWidget::setTabOrder(timestampTrackSentCheckBox, setPinButton);
        QWidget::setTabOrder(setPinButton, changePinButton);
        QWidget::setTabOrder(changePinButton, clearPinButton);
        QWidget::setTabOrder(clearPinButton, afterStartSelectNewest);
        QWidget::setTabOrder(afterStartSelectNewest, afterStartSelectLast);
        QWidget::setTabOrder(afterStartSelectLast, afterStartSelectNothing);
        QWidget::setTabOrder(afterStartSelectNothing, appThemeComboBox);
        QWidget::setTabOrder(appThemeComboBox, fontScaleSpinBox);
        QWidget::setTabOrder(fontScaleSpinBox, messageToBeDeletedSpinBox);
        QWidget::setTabOrder(messageToBeDeletedSpinBox, messageToBeDeletedColour);
        QWidget::setTabOrder(messageToBeDeletedColour, changeMessageToBeDeletedColourButton);
        QWidget::setTabOrder(changeMessageToBeDeletedColourButton, enableGlobalPaths);
        QWidget::setTabOrder(enableGlobalPaths, savePathPushButton);
        QWidget::setTabOrder(savePathPushButton, addFilePathPushButton);
        QWidget::setTabOrder(addFilePathPushButton, allAttachmentsSaveZfoMsg);
        QWidget::setTabOrder(allAttachmentsSaveZfoMsg, allAttachmentsSavePdfMsgEnvel);
        QWidget::setTabOrder(allAttachmentsSavePdfMsgEnvel, allAttachmentsSaveZfoDelInfo);
        QWidget::setTabOrder(allAttachmentsSaveZfoDelInfo, allAttachmentsSavePdfDelInfo);
        QWidget::setTabOrder(allAttachmentsSavePdfDelInfo, msgFileNameFmt);
        QWidget::setTabOrder(msgFileNameFmt, delInfoFileNameFmt);
        QWidget::setTabOrder(delInfoFileNameFmt, attachFileNameFmt);
        QWidget::setTabOrder(attachFileNameFmt, delInfoForEveryFile);
        QWidget::setTabOrder(delInfoForEveryFile, allAttachDelInfoFileNameFmt);
        QWidget::setTabOrder(allAttachDelInfoFileNameFmt, showParamListButton);
        QWidget::setTabOrder(showParamListButton, closeParamListButton);
        QWidget::setTabOrder(closeParamListButton, enableBackupNotifications);
        QWidget::setTabOrder(enableBackupNotifications, backupMonthIntervalSpinBox);
        QWidget::setTabOrder(backupMonthIntervalSpinBox, backupPathPushButton);
        QWidget::setTabOrder(backupPathPushButton, langComboBox);
        QWidget::setTabOrder(langComboBox, collectUsageReport);
        QWidget::setTabOrder(collectUsageReport, buildReportToolButton);
        QWidget::setTabOrder(buildReportToolButton, confReportToolButton);
        QWidget::setTabOrder(confReportToolButton, usageReportToolButton);

        retranslateUi(DlgPreferences);
        QObject::connect(prefButtonBox, &QDialogButtonBox::accepted, DlgPreferences, qOverload<>(&QDialog::accept));
        QObject::connect(prefButtonBox, &QDialogButtonBox::rejected, DlgPreferences, qOverload<>(&QDialog::reject));

        prefWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DlgPreferences);
    } // setupUi

    void retranslateUi(QDialog *DlgPreferences)
    {
        DlgPreferences->setWindowTitle(QCoreApplication::translate("DlgPreferences", "Datovka - Preferences", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgPreferences", "Message downloading", nullptr));
#if QT_CONFIG(tooltip)
        downloadOnBackground->setToolTip(QCoreApplication::translate("DlgPreferences", "When allowed, the application will automatically download\n"
"messages on background without blocking the user interface.\n"
"Download period can be selected below.", nullptr));
#endif // QT_CONFIG(tooltip)
        downloadOnBackground->setText(QCoreApplication::translate("DlgPreferences", "Automatically synchronise all in background", nullptr));
        timerPreLabel->setText(QCoreApplication::translate("DlgPreferences", "Check every", nullptr));
#if QT_CONFIG(tooltip)
        timerSpinBox->setToolTip(QCoreApplication::translate("DlgPreferences", "Sets a time interval for automatic synchronisation\n"
"of all accounts on background.", nullptr));
#endif // QT_CONFIG(tooltip)
        timerPostLabel->setText(QCoreApplication::translate("DlgPreferences", "minutes", nullptr));
#if QT_CONFIG(tooltip)
        autoDownloadWholeMessages->setToolTip(QCoreApplication::translate("DlgPreferences", "When accessing the data boxes, only envelopes of messages are downloaded at first.\n"
"You can then download the whole message, including any attachments, manually.\n"
"When this option is turned on, the application will perform automatic downloading\n"
"of complete messages. The only downside to this approach is a longer waiting time\n"
"on slower internet connections.", nullptr));
#endif // QT_CONFIG(tooltip)
        autoDownloadWholeMessages->setText(QCoreApplication::translate("DlgPreferences", "Automatically download whole messages (may be slow)", nullptr));
#if QT_CONFIG(tooltip)
        downloadAtStart->setToolTip(QCoreApplication::translate("DlgPreferences", "When allowed, the application will automatically synchronise data boxes\n"
"and will also download new messages on background at startup.", nullptr));
#endif // QT_CONFIG(tooltip)
        downloadAtStart->setText(QCoreApplication::translate("DlgPreferences", "Automatically synchronize all data boxes on startup", nullptr));
#if QT_CONFIG(tooltip)
        downloadAfterSending->setToolTip(QCoreApplication::translate("DlgPreferences", "Sets the default behaviour whether to download the signed content\n"
"of a message immediately after it has been sent.\n"
"\n"
"This option can be overridden when sending individual messages.\n"
"\n"
"Downloading the content of the sent message won't cause the acceptance\n"
"of any delivered message.", nullptr));
#endif // QT_CONFIG(tooltip)
        downloadAfterSending->setText(QCoreApplication::translate("DlgPreferences", "Download sent message immediately after sending", nullptr));
        groupBox_7->setTitle(QCoreApplication::translate("DlgPreferences", "Connection settings", nullptr));
        timeoutMinPreLabel->setText(QCoreApplication::translate("DlgPreferences", "Timeout for message downloading is set on", nullptr));
#if QT_CONFIG(tooltip)
        timeoutMinSpinBox->setToolTip(QCoreApplication::translate("DlgPreferences", "Allows to set a timeout interval for network connection.\n"
"If you have a slow network connection or you cannot\n"
"download complete messages, here you can increase\n"
"the connection timeout.", nullptr));
#endif // QT_CONFIG(tooltip)
        timeoutMinPostLabel->setText(QCoreApplication::translate("DlgPreferences", "minutes", nullptr));
        enable_mark_msg_as_read->setTitle(QCoreApplication::translate("DlgPreferences", "Mark message as read settings", nullptr));
        timeoutMarkMsgPreLabel->setText(QCoreApplication::translate("DlgPreferences", "Automatically mark message as read after", nullptr));
#if QT_CONFIG(tooltip)
        timeoutMarkMsgSpinBox->setToolTip(QCoreApplication::translate("DlgPreferences", "Allows to set a timeout interval for\n"
"automatic marking of messages as read.", nullptr));
#endif // QT_CONFIG(tooltip)
        timeoutMarkMsgPostLabel->setText(QCoreApplication::translate("DlgPreferences", "seconds", nullptr));
        noteMarkMsgLabel->setText(QString());
        groupBox_2->setTitle(QCoreApplication::translate("DlgPreferences", "New versions", nullptr));
#if QT_CONFIG(tooltip)
        checkNewVersions->setToolTip(QCoreApplication::translate("DlgPreferences", "When this option is active, the application will automatically\n"
"check for new application versions on startup.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkNewVersions->setText(QCoreApplication::translate("DlgPreferences", "Check for new application versions on startup", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_0_download), QCoreApplication::translate("DlgPreferences", "Downloading", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DlgPreferences", "Storage options", nullptr));
        storageOptionsLabel->setText(QCoreApplication::translate("DlgPreferences", "It migth be possible for a person with access to the configuration directory to modify data in the application databases with a malicious intent or to steal you data-box login credentials. Using the checkboxes below, you can influence which data the application stores on the disk.\n"
"Note: Password storing can be adjusted on a per-account basis in the credentials dialogue.", nullptr));
#if QT_CONFIG(tooltip)
        storeMessagesOnDisk->setToolTip(QCoreApplication::translate("DlgPreferences", "When allowed, messages are stored in database files. Such storage\n"
"might be compromised by an attacker and the content of messages\n"
"might be modified.\n"
"When turned off, messages are freshly downloaded each time you\n"
"start the application. In this case, messages older than 90 days\n"
"may not be available.", nullptr));
#endif // QT_CONFIG(tooltip)
        storeMessagesOnDisk->setText(QCoreApplication::translate("DlgPreferences", "Allow storing of messages", nullptr));
#if QT_CONFIG(tooltip)
        storeAdditionalDataOnDisk->setToolTip(QCoreApplication::translate("DlgPreferences", "The application stores some additional data outside the message\n"
"database. These contain information about the data boxes allowing\n"
"for notification about expiring passwords etc. These data might also\n"
"be changed by an attacker, but with a relative low impact.", nullptr));
#endif // QT_CONFIG(tooltip)
        storeAdditionalDataOnDisk->setText(QCoreApplication::translate("DlgPreferences", "Allow storing of additional data", nullptr));
        storageNotificationLabel->setText(QCoreApplication::translate("DlgPreferences", "Note: Storage settings will not be applied until you restart the application.", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("DlgPreferences", "Signing certificate validity", nullptr));
#if QT_CONFIG(tooltip)
        certValidationDateNow->setToolTip(QCoreApplication::translate("DlgPreferences", "Validity of the signing certificate will be checked against\n"
"the current date. This provides the highest protection against\n"
"compromised certificates.", nullptr));
#endif // QT_CONFIG(tooltip)
        certValidationDateNow->setText(QCoreApplication::translate("DlgPreferences", "Check against current date (safer)", nullptr));
#if QT_CONFIG(tooltip)
        certValidationDateDownload->setToolTip(QCoreApplication::translate("DlgPreferences", "The validity of the signing certificate is checked against\n"
"the message download date. This allows a long-term storing\n"
"of messages without generating false alarms about invalid\n"
"signatures. It is less safe, because an attacker could modify\n"
"download dates in the application databases.", nullptr));
#endif // QT_CONFIG(tooltip)
        certValidationDateDownload->setText(QCoreApplication::translate("DlgPreferences", "Check against the date of download", nullptr));
#if QT_CONFIG(tooltip)
        checkCrl->setToolTip(QCoreApplication::translate("DlgPreferences", "Check certificate revocation list (CRL).", nullptr));
#endif // QT_CONFIG(tooltip)
        checkCrl->setText(QCoreApplication::translate("DlgPreferences", "Check certificate revocation list (CRL)", nullptr));
        groupBox_10->setTitle(QCoreApplication::translate("DlgPreferences", "Message time stamp expiration", nullptr));
#if QT_CONFIG(tooltip)
        timestampExpirPre->setToolTip(QCoreApplication::translate("DlgPreferences", "Message time stamps expiring within the selected numer\n"
"of days will be included into the expiration notification summary.", nullptr));
#endif // QT_CONFIG(tooltip)
        timestampExpirPre->setText(QCoreApplication::translate("DlgPreferences", "Notify about time stamps expiring within", nullptr));
#if QT_CONFIG(tooltip)
        timestampExpirSpinBox->setToolTip(QCoreApplication::translate("DlgPreferences", "Allows to set an interval specifying how many days before\n"
"the time-stamp expiration date to notify about the upcoming\n"
"expiration.", nullptr));
#endif // QT_CONFIG(tooltip)
        timestampExpirPost->setText(QCoreApplication::translate("DlgPreferences", "days", nullptr));
#if QT_CONFIG(tooltip)
        timestampAutomaticTrackingLabel->setToolTip(QCoreApplication::translate("DlgPreferences", "We recommend to carefully select only those messages whose\n"
"timestamps you really want to track.\n"
"\n"
"Many messages don't really need to be restamped. Select only\n"
"important messages which you have evaluated to be worth\n"
"of being restampted to avoid unnecessary load on the application\n"
"and the ISDS system.", nullptr));
#endif // QT_CONFIG(tooltip)
        timestampAutomaticTrackingLabel->setText(QCoreApplication::translate("DlgPreferences", "Automatically turn on time stamp tracking for newly downloaded:", nullptr));
#if QT_CONFIG(tooltip)
        timestampTrackReceivedCheckBox->setToolTip(QCoreApplication::translate("DlgPreferences", "When enabled, time stamps of freshly added received messages\n"
"will automatically be added into time stamp tracking.\n"
"\n"
"This may cause unnecessary load on the application and\n"
"the ISDS system if many unimportant messages are going\n"
"to be restamped.", nullptr));
#endif // QT_CONFIG(tooltip)
        timestampTrackReceivedCheckBox->setText(QCoreApplication::translate("DlgPreferences", "Received messages", nullptr));
#if QT_CONFIG(tooltip)
        timestampTrackSentCheckBox->setToolTip(QCoreApplication::translate("DlgPreferences", "When enabled, time stamps of freshly added sent messages\n"
"will automatically be added into time stamp tracking.\n"
"\n"
"This may cause unnecessary load on the application and\n"
"the ISDS system if many unimportant messages are going\n"
"to be restamped.", nullptr));
#endif // QT_CONFIG(tooltip)
        timestampTrackSentCheckBox->setText(QCoreApplication::translate("DlgPreferences", "Sent messages", nullptr));
        groupBox_11->setTitle(QCoreApplication::translate("DlgPreferences", "PIN settings", nullptr));
        pinSettingsLabel->setText(QCoreApplication::translate("DlgPreferences", "When PIN (master password) is set then the application cannot be started without entering a valid PIN value. The PIN is also used to protect the stored data-box passwords. If an attacker gains the access to the application configuration he cannot easily recover the stored passwords as they are encrypted using the PIN value. The PIN does not protect any stored messages.\n"
"Note: If you forget the PIN then you'll loose all stored passwords. Other stored data won't be affected.", nullptr));
        setPinButton->setText(QCoreApplication::translate("DlgPreferences", "Set PIN", nullptr));
        changePinButton->setText(QCoreApplication::translate("DlgPreferences", "Change PIN", nullptr));
        clearPinButton->setText(QCoreApplication::translate("DlgPreferences", "Clear PIN", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_1_security), QCoreApplication::translate("DlgPreferences", "Security", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("DlgPreferences", "When data box is opened, select", nullptr));
#if QT_CONFIG(tooltip)
        afterStartSelectNewest->setToolTip(QCoreApplication::translate("DlgPreferences", "After a data box is selected,\n"
"select the newest message.", nullptr));
#endif // QT_CONFIG(tooltip)
        afterStartSelectNewest->setText(QCoreApplication::translate("DlgPreferences", "Newest message", nullptr));
#if QT_CONFIG(tooltip)
        afterStartSelectLast->setToolTip(QCoreApplication::translate("DlgPreferences", "After a data box is selected,\n"
"select the last displayed message for this data box.", nullptr));
#endif // QT_CONFIG(tooltip)
        afterStartSelectLast->setText(QCoreApplication::translate("DlgPreferences", "Last displayed message", nullptr));
#if QT_CONFIG(tooltip)
        afterStartSelectNothing->setToolTip(QCoreApplication::translate("DlgPreferences", "After a data box is selected,\n"
"no particular message will be selected.", nullptr));
#endif // QT_CONFIG(tooltip)
        afterStartSelectNothing->setText(QCoreApplication::translate("DlgPreferences", "Nothing", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_2_navigation), QCoreApplication::translate("DlgPreferences", "Navigation", nullptr));
        appThemeVerticalGroupBox->setTitle(QCoreApplication::translate("DlgPreferences", "Application Theme", nullptr));
        appThemeLabel->setText(QCoreApplication::translate("DlgPreferences", "Application theme:", nullptr));
        fontSizeGroupBox->setTitle(QCoreApplication::translate("DlgPreferences", "Font Size", nullptr));
        fontScalePre->setText(QCoreApplication::translate("DlgPreferences", "Font scaling:", nullptr));
        fontScaleSpinBox->setSuffix(QString());
        fontScaleNotificationLabel->setText(QCoreApplication::translate("DlgPreferences", "Note: In order to apply all size changes properly you will need to restart the application.", nullptr));
        messageHighlighting->setTitle(QCoreApplication::translate("DlgPreferences", "Highlighting", nullptr));
        messageToBeDeletedPre->setText(QCoreApplication::translate("DlgPreferences", "Highlight messages to be deleted from ISDS within", nullptr));
        messageToBeDeletedPost->setText(QCoreApplication::translate("DlgPreferences", "days using the colour", nullptr));
        messageToBeDeletedColour->setText(QString());
        changeMessageToBeDeletedColourButton->setText(QCoreApplication::translate("DlgPreferences", "Change", nullptr));
        labelToBeDeletedHighlightNote->setText(QString());
        prefWidget->setTabText(prefWidget->indexOf(tab_3_interface), QCoreApplication::translate("DlgPreferences", "Interface", nullptr));
#if QT_CONFIG(tooltip)
        enableGlobalPaths->setToolTip(QCoreApplication::translate("DlgPreferences", "When allowed, the application will use the following\n"
"directories as paths for loading and saving files for\n"
"all accounts. Per-account path remembering will\n"
"be disabled.", nullptr));
#endif // QT_CONFIG(tooltip)
        enableGlobalPaths->setTitle(QCoreApplication::translate("DlgPreferences", "Use global path settings", nullptr));
        savePathLabel->setText(QCoreApplication::translate("DlgPreferences", "Path for attachment saving is currently set to:", nullptr));
        savePath->setText(QString());
#if QT_CONFIG(tooltip)
        savePathPushButton->setToolTip(QCoreApplication::translate("DlgPreferences", "Select a new path for attachment saving.", nullptr));
#endif // QT_CONFIG(tooltip)
        savePathPushButton->setText(QCoreApplication::translate("DlgPreferences", "Change", nullptr));
        addFilePathLabel->setText(QCoreApplication::translate("DlgPreferences", "Path for adding files as attachments is currently set to:", nullptr));
        addFilePath->setText(QString());
#if QT_CONFIG(tooltip)
        addFilePathPushButton->setToolTip(QCoreApplication::translate("DlgPreferences", "Select a new path for adding attachment files.", nullptr));
#endif // QT_CONFIG(tooltip)
        addFilePathPushButton->setText(QCoreApplication::translate("DlgPreferences", "Change", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_4_directories), QCoreApplication::translate("DlgPreferences", "Directories", nullptr));
#if QT_CONFIG(tooltip)
        groupBox_8->setToolTip(QCoreApplication::translate("DlgPreferences", "These actions will be performed when saving all message attachments.", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_8->setTitle(QCoreApplication::translate("DlgPreferences", "When saving all attachments also", nullptr));
#if QT_CONFIG(tooltip)
        allAttachmentsSaveZfoMsg->setToolTip(QCoreApplication::translate("DlgPreferences", "When saving all attachments also save\n"
"signed message as ZFO to the same path.", nullptr));
#endif // QT_CONFIG(tooltip)
        allAttachmentsSaveZfoMsg->setText(QCoreApplication::translate("DlgPreferences", "Save signed message to ZFO", nullptr));
#if QT_CONFIG(tooltip)
        allAttachmentsSavePdfMsgEnvel->setToolTip(QCoreApplication::translate("DlgPreferences", "When saving all attachments also save\n"
"message envelope as PDF to the same path.", nullptr));
#endif // QT_CONFIG(tooltip)
        allAttachmentsSavePdfMsgEnvel->setText(QCoreApplication::translate("DlgPreferences", "Save message envelope to PDF", nullptr));
#if QT_CONFIG(tooltip)
        allAttachmentsSaveZfoDelInfo->setToolTip(QCoreApplication::translate("DlgPreferences", "When saving all attachments also save\n"
"signed acceptance info as ZFO to the same path.", nullptr));
#endif // QT_CONFIG(tooltip)
        allAttachmentsSaveZfoDelInfo->setText(QCoreApplication::translate("DlgPreferences", "Save signed acceptance info to ZFO", nullptr));
#if QT_CONFIG(tooltip)
        allAttachmentsSavePdfDelInfo->setToolTip(QCoreApplication::translate("DlgPreferences", "When saving all attachments also save\n"
"acceptance info as PDF to the same path.", nullptr));
#endif // QT_CONFIG(tooltip)
        allAttachmentsSavePdfDelInfo->setText(QCoreApplication::translate("DlgPreferences", "Save acceptance info to PDF", nullptr));
#if QT_CONFIG(tooltip)
        groupBox_9->setToolTip(QCoreApplication::translate("DlgPreferences", "Specifies the format of file names for attachment files\n"
"and other exported files. When saving attachment\n"
"files, the default value is %<FN> (i.e. saved file name will\n"
"match the original file name including its suffix).", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_9->setTitle(QCoreApplication::translate("DlgPreferences", "File name format of saved/exported files", nullptr));
        msgFileNameLabel->setText(QCoreApplication::translate("DlgPreferences", "Message filename:", nullptr));
        delInfoFileNameLabel->setText(QCoreApplication::translate("DlgPreferences", "Acceptance info filename:", nullptr));
        attachFileNameLabel->setText(QCoreApplication::translate("DlgPreferences", "Attachment filename:", nullptr));
#if QT_CONFIG(tooltip)
        delInfoForEveryFile->setToolTip(QCoreApplication::translate("DlgPreferences", "Saves acceptance info for every attachment file separately.\n"
"The acceptance info file name has to contain the original\n"
"attachment file name (parameter %<FN> is required).", nullptr));
#endif // QT_CONFIG(tooltip)
        delInfoForEveryFile->setText(QCoreApplication::translate("DlgPreferences", "Save acceptance info for every attachment file.", nullptr));
        label_42->setText(QCoreApplication::translate("DlgPreferences", "Format:", nullptr));
        characterReplacementNotificationLabel->setText(QCoreApplication::translate("DlgPreferences", "Note: Use predefined parameters in order to add additional information from message envelope into the exported filenames. The symbol \"/\" in the format string stands for a subdirectory. Illegal characters in the resulting filenames will be replaced.", nullptr));
        showParamListButton->setText(QCoreApplication::translate("DlgPreferences", "Show Parameter List", nullptr));
        closeParamListButton->setText(QCoreApplication::translate("DlgPreferences", "Close Parameter List", nullptr));
#if QT_CONFIG(tooltip)
        enableBackupNotifications->setToolTip(QCoreApplication::translate("DlgPreferences", "When allowed, the user will be notified in periodic intervals\n"
"after application start-up to perform a data backup into\n"
"a predefined path.\n"
"The interval for backups can be set.", nullptr));
#endif // QT_CONFIG(tooltip)
        enableBackupNotifications->setTitle(QCoreApplication::translate("DlgPreferences", "Enable Backup Notifications", nullptr));
        backupMonthIntervalPreLabel->setText(QCoreApplication::translate("DlgPreferences", "Notify every", nullptr));
#if QT_CONFIG(tooltip)
        backupMonthIntervalSpinBox->setToolTip(QCoreApplication::translate("DlgPreferences", "Interval between backup notifications.", nullptr));
#endif // QT_CONFIG(tooltip)
        backupMonthIntervalPostLabel->setText(QCoreApplication::translate("DlgPreferences", "months", nullptr));
        backupPathLabel->setText(QCoreApplication::translate("DlgPreferences", "Path for saving backups is currently set to:", nullptr));
        backupPath->setText(QString());
#if QT_CONFIG(tooltip)
        backupPathPushButton->setToolTip(QCoreApplication::translate("DlgPreferences", "Select a new path for saving backups.", nullptr));
#endif // QT_CONFIG(tooltip)
        backupPathPushButton->setText(QCoreApplication::translate("DlgPreferences", "Change", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_5_saving), QCoreApplication::translate("DlgPreferences", "Saving", nullptr));
        groupBox_6->setTitle(QCoreApplication::translate("DlgPreferences", "Application language", nullptr));
        langNotificationLabel->setText(QCoreApplication::translate("DlgPreferences", "Note: Language settings will not be applied until you restart the application.", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_6_language), QCoreApplication::translate("DlgPreferences", "Language", nullptr));
        groupBox_12->setTitle(QCoreApplication::translate("DlgPreferences", "Data Collection", nullptr));
        dataCollectionDescriptionLabel->setText(QCoreApplication::translate("DlgPreferences", "We value the privacy of your data. But we would like to gain some basic information about how this application is being deployed and configured so we can further improve it.\n"
"\n"
"Data are sent via an encrypted channel to a server where they are available for the developers. These data serve for the purpose of improving the application. No personal data such as data-box identifiers no login information nor information from data messages are being or will be sent away.\n"
"\n"
"List with descriptions of the collected data:", nullptr));
#if QT_CONFIG(tooltip)
        collectBuildReportLabel->setToolTip(QCoreApplication::translate("DlgPreferences", "Information like the name of the operating system, processor architecture or version of Qt libraries are reported.\n"
"\n"
"We need this information because it will helps us to identify dominant environments which we can focus on.\n"
"\n"
"(e.g. OS Windows 10 64-bit, CPU x86_64, Qt 6.6.2 or OS Linux Mint 64-bit, CPU x86_64, Qt 5.15.2)", nullptr));
#endif // QT_CONFIG(tooltip)
        collectBuildReportLabel->setText(QCoreApplication::translate("DlgPreferences", "- information about the application build", nullptr));
        buildReportToolButton->setText(QCoreApplication::translate("DlgPreferences", "?", nullptr));
#if QT_CONFIG(tooltip)
        collectConfReportLabel->setToolTip(QCoreApplication::translate("DlgPreferences", "Application version and available functionality are reported.\n"
"\n"
"We need this information because it shows us how package maintainers configure the application.", nullptr));
#endif // QT_CONFIG(tooltip)
        collectConfReportLabel->setText(QCoreApplication::translate("DlgPreferences", "- information how the application was configured when it was built", nullptr));
        confReportToolButton->setText(QCoreApplication::translate("DlgPreferences", "?", nullptr));
        optionalDataCollectionDescriptionLabel->setText(QCoreApplication::translate("DlgPreferences", "It is further possible to let us collect additional information about the configuration and the usage of this application.\n"
"We ask you politely to let us receive this additional information.", nullptr));
#if QT_CONFIG(tooltip)
        collectUsageReport->setToolTip(QCoreApplication::translate("DlgPreferences", "We ask you kindly to provide the information about:\n"
"- the number of data boxes\n"
"- an estimate of how much data the application stores\n"
"- whether tags are being used\n"
"- whether PIN is being used\n"
"- whether records management functionality is being used", nullptr));
#endif // QT_CONFIG(tooltip)
        collectUsageReport->setText(QCoreApplication::translate("DlgPreferences", "Collect information about the usage of the application.", nullptr));
        usageReportToolButton->setText(QCoreApplication::translate("DlgPreferences", "?", nullptr));
        prefWidget->setTabText(prefWidget->indexOf(tab_7_data_collection), QCoreApplication::translate("DlgPreferences", "Data Collection", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgPreferences: public Ui_DlgPreferences {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_PREFERENCES_H
