/********************************************************************************
** Form generated from reading UI file 'dlg_msg_search.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DLG_MSG_SEARCH_H
#define UI_DLG_MSG_SEARCH_H

#include <QtCore/QDate>
#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>
#include "src/views/lowered_table_view.h"

QT_BEGIN_NAMESPACE

class Ui_DlgMsgSearch
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *infoTextLabel;
    QFrame *line_2;
    QHBoxLayout *h6;
    QFormLayout *accountFormLayout;
    QLabel *crntAcntLabel;
    QComboBox *accountComboBox;
    QCheckBox *searchAllAcntCheckBox;
    QFrame *line;
    QGroupBox *groupBox;
    QHBoxLayout *horizontalLayout_3;
    QCheckBox *searchRcvdMsgCheckBox;
    QCheckBox *searchSntMsgCheckBox;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *h1;
    QFormLayout *formLayout_5;
    QLabel *msgIdLabel;
    QLineEdit *msgIdLine;
    QFormLayout *formLayout_2;
    QLabel *subjectLabel;
    QLineEdit *subjectLine;
    QHBoxLayout *h2;
    QFormLayout *formLayout;
    QLabel *sndrBoxIdLabel;
    QLineEdit *sndrBoxIdLine;
    QLabel *sndrNameLabel;
    QLineEdit *sndrNameLine;
    QLabel *sndrRefNumLabel;
    QLineEdit *sndrRefNumLine;
    QLabel *sndrFileMarkLabel;
    QLineEdit *sndrFileMarkLine;
    QFormLayout *formLayout_3;
    QLabel *rcpntNameLabel;
    QLineEdit *rcpntBoxIdLine;
    QLabel *rcpntBoxIdLabel;
    QLineEdit *rcpntNameLine;
    QLabel *rcpntRefNumLabel;
    QLineEdit *rcpntRefNumLine;
    QLabel *rcpntFileMarkLabel;
    QLineEdit *rcpntFileMarkLine;
    QHBoxLayout *h3;
    QFormLayout *formLayout_4;
    QLabel *addressLabel;
    QLineEdit *addressLine;
    QFormLayout *formLayout_7;
    QLabel *toHandsLabel;
    QLineEdit *toHandsLine;
    QFrame *line_5;
    QHBoxLayout *horizontalLayout_2;
    QFormLayout *formLayout_12;
    QLabel *fromdateLabel;
    QHBoxLayout *horizontalLayout_5;
    QCheckBox *fromDateCheckBox;
    QDateEdit *fromDateEdit;
    QFormLayout *formLayout_11;
    QLabel *todateLabel;
    QHBoxLayout *horizontalLayout_4;
    QCheckBox *toDateCheckBox;
    QDateEdit *toDateEdit;
    QFrame *line_3;
    QHBoxLayout *h4;
    QFormLayout *formLayout_10;
    QLabel *attachNameLabel;
    QLineEdit *attachNameLine;
    QFormLayout *formLayout_6;
    QLabel *tagLabel;
    QLineEdit *tagLine;
    QFrame *line_4;
    QLabel *tooManyFields;
    QHBoxLayout *h5;
    QCheckBox *logicalAndRelationCheckBox;
    QPushButton *searchPushButton;
    QHBoxLayout *horizontalLayout;
    QLabel *filterLabel;
    QLineEdit *filterLine;
    LoweredTableView *resultsTableView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DlgMsgSearch)
    {
        if (DlgMsgSearch->objectName().isEmpty())
            DlgMsgSearch->setObjectName("DlgMsgSearch");
        DlgMsgSearch->resize(610, 740);
        DlgMsgSearch->setMinimumSize(QSize(610, 725));
        verticalLayout = new QVBoxLayout(DlgMsgSearch);
        verticalLayout->setObjectName("verticalLayout");
        infoTextLabel = new QLabel(DlgMsgSearch);
        infoTextLabel->setObjectName("infoTextLabel");
        infoTextLabel->setText(QString::fromUtf8("n/a"));
        infoTextLabel->setAlignment(Qt::AlignJustify|Qt::AlignVCenter);
        infoTextLabel->setWordWrap(true);

        verticalLayout->addWidget(infoTextLabel);

        line_2 = new QFrame(DlgMsgSearch);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        h6 = new QHBoxLayout();
        h6->setObjectName("h6");
        accountFormLayout = new QFormLayout();
        accountFormLayout->setObjectName("accountFormLayout");
        crntAcntLabel = new QLabel(DlgMsgSearch);
        crntAcntLabel->setObjectName("crntAcntLabel");

        accountFormLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, crntAcntLabel);

        accountComboBox = new QComboBox(DlgMsgSearch);
        accountComboBox->setObjectName("accountComboBox");

        accountFormLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, accountComboBox);


        h6->addLayout(accountFormLayout);

        searchAllAcntCheckBox = new QCheckBox(DlgMsgSearch);
        searchAllAcntCheckBox->setObjectName("searchAllAcntCheckBox");

        h6->addWidget(searchAllAcntCheckBox);

        h6->setStretch(0, 1);

        verticalLayout->addLayout(h6);

        line = new QFrame(DlgMsgSearch);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line);

        groupBox = new QGroupBox(DlgMsgSearch);
        groupBox->setObjectName("groupBox");
        horizontalLayout_3 = new QHBoxLayout(groupBox);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        searchRcvdMsgCheckBox = new QCheckBox(groupBox);
        searchRcvdMsgCheckBox->setObjectName("searchRcvdMsgCheckBox");
        searchRcvdMsgCheckBox->setChecked(true);

        horizontalLayout_3->addWidget(searchRcvdMsgCheckBox);

        searchSntMsgCheckBox = new QCheckBox(groupBox);
        searchSntMsgCheckBox->setObjectName("searchSntMsgCheckBox");
        searchSntMsgCheckBox->setChecked(true);

        horizontalLayout_3->addWidget(searchSntMsgCheckBox);


        verticalLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(DlgMsgSearch);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        h1 = new QHBoxLayout();
        h1->setObjectName("h1");
        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName("formLayout_5");
        msgIdLabel = new QLabel(groupBox_2);
        msgIdLabel->setObjectName("msgIdLabel");

        formLayout_5->setWidget(0, QFormLayout::ItemRole::LabelRole, msgIdLabel);

        msgIdLine = new QLineEdit(groupBox_2);
        msgIdLine->setObjectName("msgIdLine");
        msgIdLine->setInputMethodHints(Qt::ImhDigitsOnly);

        formLayout_5->setWidget(0, QFormLayout::ItemRole::FieldRole, msgIdLine);


        h1->addLayout(formLayout_5);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName("formLayout_2");
        subjectLabel = new QLabel(groupBox_2);
        subjectLabel->setObjectName("subjectLabel");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::LabelRole, subjectLabel);

        subjectLine = new QLineEdit(groupBox_2);
        subjectLine->setObjectName("subjectLine");

        formLayout_2->setWidget(0, QFormLayout::ItemRole::FieldRole, subjectLine);


        h1->addLayout(formLayout_2);


        verticalLayout_2->addLayout(h1);

        h2 = new QHBoxLayout();
        h2->setObjectName("h2");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        sndrBoxIdLabel = new QLabel(groupBox_2);
        sndrBoxIdLabel->setObjectName("sndrBoxIdLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, sndrBoxIdLabel);

        sndrBoxIdLine = new QLineEdit(groupBox_2);
        sndrBoxIdLine->setObjectName("sndrBoxIdLine");
        sndrBoxIdLine->setMaxLength(7);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, sndrBoxIdLine);

        sndrNameLabel = new QLabel(groupBox_2);
        sndrNameLabel->setObjectName("sndrNameLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, sndrNameLabel);

        sndrNameLine = new QLineEdit(groupBox_2);
        sndrNameLine->setObjectName("sndrNameLine");

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, sndrNameLine);

        sndrRefNumLabel = new QLabel(groupBox_2);
        sndrRefNumLabel->setObjectName("sndrRefNumLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, sndrRefNumLabel);

        sndrRefNumLine = new QLineEdit(groupBox_2);
        sndrRefNumLine->setObjectName("sndrRefNumLine");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, sndrRefNumLine);

        sndrFileMarkLabel = new QLabel(groupBox_2);
        sndrFileMarkLabel->setObjectName("sndrFileMarkLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, sndrFileMarkLabel);

        sndrFileMarkLine = new QLineEdit(groupBox_2);
        sndrFileMarkLine->setObjectName("sndrFileMarkLine");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, sndrFileMarkLine);


        h2->addLayout(formLayout);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName("formLayout_3");
        formLayout_3->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        formLayout_3->setLabelAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        rcpntNameLabel = new QLabel(groupBox_2);
        rcpntNameLabel->setObjectName("rcpntNameLabel");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::LabelRole, rcpntNameLabel);

        rcpntBoxIdLine = new QLineEdit(groupBox_2);
        rcpntBoxIdLine->setObjectName("rcpntBoxIdLine");
        rcpntBoxIdLine->setMaxLength(7);

        formLayout_3->setWidget(0, QFormLayout::ItemRole::FieldRole, rcpntBoxIdLine);

        rcpntBoxIdLabel = new QLabel(groupBox_2);
        rcpntBoxIdLabel->setObjectName("rcpntBoxIdLabel");

        formLayout_3->setWidget(0, QFormLayout::ItemRole::LabelRole, rcpntBoxIdLabel);

        rcpntNameLine = new QLineEdit(groupBox_2);
        rcpntNameLine->setObjectName("rcpntNameLine");

        formLayout_3->setWidget(1, QFormLayout::ItemRole::FieldRole, rcpntNameLine);

        rcpntRefNumLabel = new QLabel(groupBox_2);
        rcpntRefNumLabel->setObjectName("rcpntRefNumLabel");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::LabelRole, rcpntRefNumLabel);

        rcpntRefNumLine = new QLineEdit(groupBox_2);
        rcpntRefNumLine->setObjectName("rcpntRefNumLine");

        formLayout_3->setWidget(2, QFormLayout::ItemRole::FieldRole, rcpntRefNumLine);

        rcpntFileMarkLabel = new QLabel(groupBox_2);
        rcpntFileMarkLabel->setObjectName("rcpntFileMarkLabel");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::LabelRole, rcpntFileMarkLabel);

        rcpntFileMarkLine = new QLineEdit(groupBox_2);
        rcpntFileMarkLine->setObjectName("rcpntFileMarkLine");

        formLayout_3->setWidget(3, QFormLayout::ItemRole::FieldRole, rcpntFileMarkLine);


        h2->addLayout(formLayout_3);


        verticalLayout_2->addLayout(h2);

        h3 = new QHBoxLayout();
        h3->setObjectName("h3");
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName("formLayout_4");
        addressLabel = new QLabel(groupBox_2);
        addressLabel->setObjectName("addressLabel");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::LabelRole, addressLabel);

        addressLine = new QLineEdit(groupBox_2);
        addressLine->setObjectName("addressLine");

        formLayout_4->setWidget(0, QFormLayout::ItemRole::FieldRole, addressLine);


        h3->addLayout(formLayout_4);

        formLayout_7 = new QFormLayout();
        formLayout_7->setObjectName("formLayout_7");
        toHandsLabel = new QLabel(groupBox_2);
        toHandsLabel->setObjectName("toHandsLabel");

        formLayout_7->setWidget(0, QFormLayout::ItemRole::LabelRole, toHandsLabel);

        toHandsLine = new QLineEdit(groupBox_2);
        toHandsLine->setObjectName("toHandsLine");

        formLayout_7->setWidget(0, QFormLayout::ItemRole::FieldRole, toHandsLine);


        h3->addLayout(formLayout_7);


        verticalLayout_2->addLayout(h3);

        line_5 = new QFrame(groupBox_2);
        line_5->setObjectName("line_5");
        line_5->setFrameShape(QFrame::Shape::HLine);
        line_5->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_5);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        formLayout_12 = new QFormLayout();
        formLayout_12->setObjectName("formLayout_12");
        fromdateLabel = new QLabel(groupBox_2);
        fromdateLabel->setObjectName("fromdateLabel");

        formLayout_12->setWidget(0, QFormLayout::ItemRole::LabelRole, fromdateLabel);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        fromDateCheckBox = new QCheckBox(groupBox_2);
        fromDateCheckBox->setObjectName("fromDateCheckBox");

        horizontalLayout_5->addWidget(fromDateCheckBox);

        fromDateEdit = new QDateEdit(groupBox_2);
        fromDateEdit->setObjectName("fromDateEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fromDateEdit->sizePolicy().hasHeightForWidth());
        fromDateEdit->setSizePolicy(sizePolicy);
        fromDateEdit->setDateTime(QDateTime(QDate(2009, 7, 1), QTime(0, 0, 0)));
        fromDateEdit->setMinimumDate(QDate(2009, 6, 1));
        fromDateEdit->setDisplayFormat(QString::fromUtf8("dd.MM.yyyy"));
        fromDateEdit->setCalendarPopup(true);
        fromDateEdit->setDate(QDate(2009, 7, 1));

        horizontalLayout_5->addWidget(fromDateEdit);


        formLayout_12->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_5);


        horizontalLayout_2->addLayout(formLayout_12);

        formLayout_11 = new QFormLayout();
        formLayout_11->setObjectName("formLayout_11");
        todateLabel = new QLabel(groupBox_2);
        todateLabel->setObjectName("todateLabel");

        formLayout_11->setWidget(0, QFormLayout::ItemRole::LabelRole, todateLabel);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        toDateCheckBox = new QCheckBox(groupBox_2);
        toDateCheckBox->setObjectName("toDateCheckBox");

        horizontalLayout_4->addWidget(toDateCheckBox);

        toDateEdit = new QDateEdit(groupBox_2);
        toDateEdit->setObjectName("toDateEdit");
        sizePolicy.setHeightForWidth(toDateEdit->sizePolicy().hasHeightForWidth());
        toDateEdit->setSizePolicy(sizePolicy);
        toDateEdit->setMinimumDate(QDate(2009, 7, 1));
        toDateEdit->setDisplayFormat(QString::fromUtf8("dd.MM.yyyy"));
        toDateEdit->setCalendarPopup(true);
        toDateEdit->setDate(QDate(2024, 1, 1));

        horizontalLayout_4->addWidget(toDateEdit);


        formLayout_11->setLayout(0, QFormLayout::ItemRole::FieldRole, horizontalLayout_4);


        horizontalLayout_2->addLayout(formLayout_11);


        verticalLayout_2->addLayout(horizontalLayout_2);

        line_3 = new QFrame(groupBox_2);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_3);

        h4 = new QHBoxLayout();
        h4->setObjectName("h4");
        formLayout_10 = new QFormLayout();
        formLayout_10->setObjectName("formLayout_10");
        attachNameLabel = new QLabel(groupBox_2);
        attachNameLabel->setObjectName("attachNameLabel");

        formLayout_10->setWidget(0, QFormLayout::ItemRole::LabelRole, attachNameLabel);

        attachNameLine = new QLineEdit(groupBox_2);
        attachNameLine->setObjectName("attachNameLine");

        formLayout_10->setWidget(0, QFormLayout::ItemRole::FieldRole, attachNameLine);


        h4->addLayout(formLayout_10);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName("formLayout_6");
        tagLabel = new QLabel(groupBox_2);
        tagLabel->setObjectName("tagLabel");

        formLayout_6->setWidget(0, QFormLayout::ItemRole::LabelRole, tagLabel);

        tagLine = new QLineEdit(groupBox_2);
        tagLine->setObjectName("tagLine");

        formLayout_6->setWidget(0, QFormLayout::ItemRole::FieldRole, tagLine);


        h4->addLayout(formLayout_6);


        verticalLayout_2->addLayout(h4);

        line_4 = new QFrame(groupBox_2);
        line_4->setObjectName("line_4");
        line_4->setFrameShape(QFrame::Shape::HLine);
        line_4->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line_4);

        tooManyFields = new QLabel(groupBox_2);
        tooManyFields->setObjectName("tooManyFields");
        tooManyFields->setEnabled(false);
        tooManyFields->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(tooManyFields);

        h5 = new QHBoxLayout();
        h5->setObjectName("h5");
        logicalAndRelationCheckBox = new QCheckBox(groupBox_2);
        logicalAndRelationCheckBox->setObjectName("logicalAndRelationCheckBox");
        logicalAndRelationCheckBox->setChecked(true);

        h5->addWidget(logicalAndRelationCheckBox);

        searchPushButton = new QPushButton(groupBox_2);
        searchPushButton->setObjectName("searchPushButton");
        searchPushButton->setEnabled(false);
        QFont font;
        font.setBold(true);
        searchPushButton->setFont(font);

        h5->addWidget(searchPushButton);

        h5->setStretch(1, 1);

        verticalLayout_2->addLayout(h5);


        verticalLayout->addWidget(groupBox_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        filterLabel = new QLabel(DlgMsgSearch);
        filterLabel->setObjectName("filterLabel");

        horizontalLayout->addWidget(filterLabel);

        filterLine = new QLineEdit(DlgMsgSearch);
        filterLine->setObjectName("filterLine");

        horizontalLayout->addWidget(filterLine);


        verticalLayout->addLayout(horizontalLayout);

        resultsTableView = new LoweredTableView(DlgMsgSearch);
        resultsTableView->setObjectName("resultsTableView");
        resultsTableView->setShowGrid(false);
        resultsTableView->setWordWrap(false);
        resultsTableView->horizontalHeader()->setHighlightSections(false);
        resultsTableView->horizontalHeader()->setProperty("showSortIndicator", QVariant(true));
        resultsTableView->horizontalHeader()->setStretchLastSection(true);
        resultsTableView->verticalHeader()->setVisible(false);
        resultsTableView->verticalHeader()->setHighlightSections(false);

        verticalLayout->addWidget(resultsTableView);

        buttonBox = new QDialogButtonBox(DlgMsgSearch);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(accountComboBox, searchAllAcntCheckBox);
        QWidget::setTabOrder(searchAllAcntCheckBox, searchRcvdMsgCheckBox);
        QWidget::setTabOrder(searchRcvdMsgCheckBox, searchSntMsgCheckBox);
        QWidget::setTabOrder(searchSntMsgCheckBox, msgIdLine);
        QWidget::setTabOrder(msgIdLine, subjectLine);
        QWidget::setTabOrder(subjectLine, sndrBoxIdLine);
        QWidget::setTabOrder(sndrBoxIdLine, rcpntBoxIdLine);
        QWidget::setTabOrder(rcpntBoxIdLine, sndrNameLine);
        QWidget::setTabOrder(sndrNameLine, rcpntNameLine);
        QWidget::setTabOrder(rcpntNameLine, sndrRefNumLine);
        QWidget::setTabOrder(sndrRefNumLine, rcpntRefNumLine);
        QWidget::setTabOrder(rcpntRefNumLine, sndrFileMarkLine);
        QWidget::setTabOrder(sndrFileMarkLine, rcpntFileMarkLine);
        QWidget::setTabOrder(rcpntFileMarkLine, addressLine);
        QWidget::setTabOrder(addressLine, toHandsLine);
        QWidget::setTabOrder(toHandsLine, fromDateCheckBox);
        QWidget::setTabOrder(fromDateCheckBox, fromDateEdit);
        QWidget::setTabOrder(fromDateEdit, toDateCheckBox);
        QWidget::setTabOrder(toDateCheckBox, toDateEdit);
        QWidget::setTabOrder(toDateEdit, attachNameLine);
        QWidget::setTabOrder(attachNameLine, tagLine);
        QWidget::setTabOrder(tagLine, logicalAndRelationCheckBox);
        QWidget::setTabOrder(logicalAndRelationCheckBox, searchPushButton);
        QWidget::setTabOrder(searchPushButton, resultsTableView);

        retranslateUi(DlgMsgSearch);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DlgMsgSearch, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DlgMsgSearch, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(DlgMsgSearch);
    } // setupUi

    void retranslateUi(QDialog *DlgMsgSearch)
    {
        DlgMsgSearch->setWindowTitle(QCoreApplication::translate("DlgMsgSearch", "Advanced Message Search", nullptr));
        crntAcntLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Data box:", nullptr));
#if QT_CONFIG(tooltip)
        searchAllAcntCheckBox->setToolTip(QCoreApplication::translate("DlgMsgSearch", "It may be slow.", nullptr));
#endif // QT_CONFIG(tooltip)
        searchAllAcntCheckBox->setText(QCoreApplication::translate("DlgMsgSearch", "Search also in other data boxes", nullptr));
        groupBox->setTitle(QCoreApplication::translate("DlgMsgSearch", "Message type", nullptr));
        searchRcvdMsgCheckBox->setText(QCoreApplication::translate("DlgMsgSearch", "Received", nullptr));
        searchSntMsgCheckBox->setText(QCoreApplication::translate("DlgMsgSearch", "Sent", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("DlgMsgSearch", "Fill in data according to which you want to search:", nullptr));
        msgIdLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Message ID:", nullptr));
        subjectLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Subject:", nullptr));
        sndrBoxIdLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Sender data-box ID:", nullptr));
        sndrBoxIdLine->setText(QString());
        sndrNameLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Sender name:", nullptr));
        sndrRefNumLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Sender ref. num.:", nullptr));
        sndrFileMarkLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Sender file mark:", nullptr));
        rcpntNameLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Recipient name:", nullptr));
        rcpntBoxIdLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Recipient data-box ID:", nullptr));
        rcpntRefNumLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Recipient ref. num.:", nullptr));
        rcpntFileMarkLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Recipient file mark:", nullptr));
        addressLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Address:", nullptr));
        toHandsLabel->setText(QCoreApplication::translate("DlgMsgSearch", "To hands:", nullptr));
        fromdateLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Delivery from:", nullptr));
#if QT_CONFIG(tooltip)
        fromDateCheckBox->setToolTip(QCoreApplication::translate("DlgMsgSearch", "Check this field if you want to restrict the delivery date\n"
"of the oldest sought message.", nullptr));
#endif // QT_CONFIG(tooltip)
        fromDateCheckBox->setText(QString());
        todateLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Delivery to:", nullptr));
#if QT_CONFIG(tooltip)
        toDateCheckBox->setToolTip(QCoreApplication::translate("DlgMsgSearch", "Check this field if you want to restrict the delivery date\n"
"of the youngest sought message.", nullptr));
#endif // QT_CONFIG(tooltip)
        toDateCheckBox->setText(QString());
        attachNameLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Attachment name:", nullptr));
        tagLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Tag text:", nullptr));
        tooManyFields->setText(QCoreApplication::translate("DlgMsgSearch", "Too many parameters to search for!", nullptr));
        logicalAndRelationCheckBox->setText(QCoreApplication::translate("DlgMsgSearch", "Search for messages matching all supplied criteria", nullptr));
        searchPushButton->setText(QCoreApplication::translate("DlgMsgSearch", "Search", nullptr));
        filterLabel->setText(QCoreApplication::translate("DlgMsgSearch", "Filter:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DlgMsgSearch: public Ui_DlgMsgSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DLG_MSG_SEARCH_H
